<?php

// This file has common functions for constructing the rows of the daily
// DPU and Vault status email for application types.  It is included in
// report_dpu.php and report_vault.php using a "require" statement.
// In order for this file to be interpreted as code to be executed, it
// must start and end with PHP start/end tags.
//

$tableCellHeight = 24;
$tableCellWidth = 30;

$bploc = trim(shell_exec('eval `grep BPDIR /etc/default/bp.ini`; echo $BPDIR'));
require("$bpdir/bin/report_style_funcs.php");

function get_client_sql_backups($dpu, $client, $end) {
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellWidth;
		
	$start = strtotime(date("Y-m-d", $end - 518400));
	$backups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client, "start_time" => $start, "end_time" => $end, "type" => array("mssql full", "mssql differential", "mssql transaction")));
	if (is_null($backups) || $backups === false || count($backups) == 0) {
		return false;
	}

	$sync_bkups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client, "start_time" => $start, "end_time" => $end, "type" => array("securesync mssql full", "securesync mssql diff", "securesync mssql trans")));

	$inst_names = array();
	$db_names = array();
	// sort the backups by server_instance_name, then database name
	foreach ($backups as $bkup) {
		$inst_names[$bkup["id"]] = $bkup["server_instance_name"];
		$db_names[$bkup["id"]] = $bkup["database_name"];
	}

	// Since backups may not sync at the same rate as they are 
	// created, there may be some instances &/or DBs that are
	// in the $sync_bkups array that aren't in $backups.
	$Sinst_names = array();
	$Sdb_names = array();
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			$Sinst_names[$sb["id"]] = $sb["server_instance_name"];
			$Sdb_names[$sb["id"]] = $sb["database_name"];
		}
	}

	// also make the sort case insensitive
	array_multisort(array_map('strtolower',$inst_names), SORT_ASC, SORT_STRING,
			array_map('strtolower',$db_names),SORT_ASC, SORT_STRING,$backups);

	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		array_multisort(array_map('strtolower',$Sinst_names), SORT_ASC, SORT_STRING,
				array_map('strtolower',$Sdb_names),SORT_ASC, SORT_STRING,$sync_bkups);
	}

	$row = array();
	foreach ($backups as $bkup) {
		// initialize the status arrays for each instance/database 
		// bs is backup status, ss is sync status 
		for ($i=0; $i<7; $i++) {
			$row[$bkup["server_instance_name"]][$bkup["database_name"]]["bs"][$i] = -1;
			$row[$bkup["server_instance_name"]][$bkup["database_name"]]["ss"][$i] = -1;
		}
	}

	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			for ($i=0; $i<7; $i++) {
				$row[$sb["server_instance_name"]][$sb["database_name"]]["bs"][$i] = -1;
				$row[$sb["server_instance_name"]][$sb["database_name"]]["ss"][$i] = -1;
			}
		}
	}

	// now get the real highest status code for each day
	for ($i=0; $i<7; $i++) {
		foreach ($backups as $bkup) {
			if (date("w", $bkup["start_time"]) ==  $i) {
				if ($bkup["complete"] == true) {
					if ($bkup["status"] > $row[$bkup["server_instance_name"]][$bkup["database_name"]]["bs"][$i]) {
						$row[$bkup["server_instance_name"]][$bkup["database_name"]]["bs"][$i] = $bkup["status"];
					}
				}
			} // end of if this backup is in the current day of the week
		} // end of looping thru backups
	
		if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
			foreach ($sync_bkups as $sb) {
				$sync_finished = $sb["start_time"] + $sb["elapsed_time"];
				if (date("w", $sync_finished) ==  $i) {
					if ($sb["complete"] == true) {
						if ($sb["status"] > $row[$sb["server_instance_name"]][$sb["database_name"]]["ss"][$i]) {
							$row[$sb["server_instance_name"]][$sb["database_name"]]["ss"][$i] = $sb["status"];
						}
					}
				} // end of if this sync finished in the current day of the week
			} // end of looping thru syncs
		}
	} // end of looping thru days

	return $row;
}

function dpu_summary_sql_backups($dpu_id, $client, $client_id) {
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellWidth;
	
	static $first_time = 0;

	$row = get_client_sql_backups($dpu_id, $client_id, time());
	if ($row === false || is_null($row) || count($row) == 0) {
		return false;
	}

	if ($first_time == 1) {
		// report_dpu.php has this function with a single arg, height
		print(build_filler(8,1));
	}
	else {
		$first_time = 1;
	}
	
	$appClientStyle = getAppClientStyle();
	$clientStyle = getClientStyle();

	$dummy = array(-2, -2, -2, -2, -2, -2, -2);
	$str = $client . ": ";
	foreach($row as $instance => $database) {
		$str .= $instance . ":  ";
		$results = "	<tr>\n";
		$results .= '		<td height="' . $tableCellHeight . '" style="' . $appClientStyle . '">' . $str . "</td>\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($dummy) .  "\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($dummy) .  "\n";
		$results .= "	</tr>\n";
		print($results);
		$nrows = count($database);
		$row_count = 1;
		foreach ($database as $db => $status) {
			$results = "	<tr>\n";
			$results .= '		<td height="' . $tableCellHeight . '" style="' . $clientStyle . '">' . $db . "</td>\n";
			$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
			$results .= '		' . create_client_table($status["bs"]) . "\n";
			$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
			$results .= '		' . create_client_table($status["ss"]) .  "\n";
			$results .= "	</tr>\n";
			print($results);
			// add whitespace between rows, but not after the last row
			if ($row_count != $nrows) {
				print(build_filler(8,1)); 
			}
			$row_count++;
		}
		$str = $client . ": ";
	}
	return true;
}

function get_client_exch_backups($dpu, $client, $end) {
	$start = strtotime(date("Y-m-d", $end - 518400));
	$backups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client, "start_time" => $start, "end_time" => $end, "type" => array("exchange full", "exchange differential", "exchange incremental")));
	if (is_null($backups) || $backups === false || count($backups) == 0) {
		return false;
	}

	$sync_bkups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client, "start_time" => $start, "end_time" => $end, "type" => array("securesync msexch full", "securesync msexch diff", "securesync msexch incr")));

	$db_names = array();
	// sort the backups by database name
	foreach ($backups as $bkup) {
		$db_names[$bkup["id"]] = $bkup["database_name"];
	}

	// Backups may not sync at the same rate that they were created,
	// so some DB names may occur in one array and not the other.
	$Sdb_names = array();
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			$Sdb_names[$sb["id"]] = $sb["database_name"];
		}
	}

	// also make the sort case insensitive
	array_multisort(array_map('strtolower',$db_names),SORT_ASC, SORT_STRING,
			$backups);

	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		array_multisort(array_map('strtolower',$Sdb_names),SORT_ASC, SORT_STRING,
			$sync_bkups);
	}


	$row = array();
	foreach ($backups as $bkup) {
		// initialize the status arrays for each database 
		// bs is backup status, ss is sync status 
		for ($i=0; $i<7; $i++) {
			$row[$bkup["database_name"]]["bs"][$i] = -1;
			$row[$bkup["database_name"]]["ss"][$i] = -1;
		}
	}
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			for ($i=0; $i<7; $i++) {
				$row[$sb["database_name"]]["bs"][$i] = -1;
				$row[$sb["database_name"]]["ss"][$i] = -1;
			}
		}
	}

	// now get the real highest status code for each day
	for ($i=0; $i<7; $i++) {
		foreach ($backups as $bkup) {
			if (date("w", $bkup["start_time"]) ==  $i) {
				if ($bkup["complete"] == true) {
					if ($bkup["status"] > $row[$bkup["database_name"]]["bs"][$i]) {
						$row[$bkup["database_name"]]["bs"][$i] = $bkup["status"];
					}
				}
			} // end of if this backup is in the current day of the week
		}// end of looping thru backups
		if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
			foreach ($sync_bkups as $sb) {
				$sync_finished = $sb["start_time"] + $sb["elapsed_time"];
				if (date("w", $sync_finished) ==  $i) {
					if ($sb["complete"] == true) {
						if ($sb["status"] > $row[$sb["database_name"]]["ss"][$i]) {
							$row[$sb["database_name"]]["ss"][$i] = $sb["status"];
						}
					}
				} // end of if this sync finished in the current day of the week
			} // end of looping thru syncs
		}
	} // end of looping thru days

	return $row;
}

function dpu_summary_exch_backups($dpu_id, $client, $client_id) {
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellWidth;
	
	static $first_time = 0;

	$row = get_client_exch_backups($dpu_id, $client_id, time());
	if ($row === false || is_null($row) || count($row) == 0) {
		return false;
	}

	if ($first_time == 1) {
		// report_dpu.php has this function with a single arg, height
		print(build_filler(8,1));
	}
	else {
		$first_time = 1;
	}
	
	$appClientStyle = getAppClientStyle();
	$clientStyle = getClientStyle();
	
	$dummy = array(-2, -2, -2, -2, -2, -2, -2);
	$str = $client . ": ";
	$results = "	<tr>\n";
	$results .= '		<td height="' . $tableCellHeight . '" style="' . $appClientStyle . '">' . $str . "</td>\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= "	</tr>\n";
	print($results);
	$nrows = count($row);
	$row_count = 1;
	foreach ($row as $db => $status) {
		$results = "	<tr>\n";
		$results .= '		<td height="' . $tableCellHeight . '" style="' . $clientStyle . '">' . $db . "</td>\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["bs"]) . "\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["ss"]) .  "\n";
		$results .= "	</tr>\n";
		print($results);
		// add whitespace between rows, but not after the last row
		if ($row_count != $nrows) {
			print(build_filler(8,1)); 
		}
		$row_count++;
	}
	return true;
}

function get_client_vmware_backups($dpu, $client, $end) {
	$start = strtotime(date("Y-m-d", $end - 518400));
	$backups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client, "start_time" => $start, "end_time" => $end, "type" => array("vmware full", "vmware differential", "vmware incremental")));
	if (is_null($backups) || $backups === false || count($backups) == 0) {
		return false;
	}
	$sync_bkups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client, "start_time" => $start, "end_time" => $end, "type" => array("securesync vmware full", "securesync vmware diff", "securesync vmware incr")));

	$server_names = array();
	$vm_names = array();
	// sort the backups by server_name, then VM name
	foreach ($backups as $bkup) {
		$server_names[$bkup["id"]] = $bkup["server_name"];
		$vm_names[$bkup["id"]] = $bkup["vm_name"];
	}

	// Backups may not sync at the same rate as they were
	// created, so some servers/VMs may be in one array
	// and not the other.
	$Sserver_names = array();
	$Svm_names = array();
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			$Sserver_names[$sb["id"]] = $sb["server_name"];
			$Svm_names[$sb["id"]] = $sb["vm_name"];
		}
	}


	// also make the sort case insensitive
	array_multisort(array_map('strtolower',$server_names), SORT_ASC, SORT_STRING, array_map('strtolower',$vm_names),SORT_ASC, SORT_STRING,$backups);

	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		array_multisort(array_map('strtolower',$Sserver_names), SORT_ASC, SORT_STRING,
				array_map('strtolower',$Svm_names),SORT_ASC, SORT_STRING, $sync_bkups);
	}

	$row = array();
	foreach ($backups as $bkup) {
		// initialize the status arrays for each server/VM 
		// bs is backup status, ss is sync status 
		for ($i=0; $i<7; $i++) {
			$row[$bkup["server_name"]][$bkup["vm_name"]]["bs"][$i] = -1;
			$row[$bkup["server_name"]][$bkup["vm_name"]]["ss"][$i] = -1;
		}
	}
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			for ($i=0; $i<7; $i++) {
				$row[$sb["server_name"]][$sb["vm_name"]]["bs"][$i] = -1;
				$row[$sb["server_name"]][$sb["vm_name"]]["ss"][$i] = -1;
			}
		}
	}

	// now get the real highest status code for each day
	for ($i=0; $i<7; $i++) {
		foreach ($backups as $bkup) {
			if (date("w", $bkup["start_time"]) ==  $i) {
				if ($bkup["complete"] == true) {
					if ($bkup["status"] > $row[$bkup["server_name"]][$bkup["vm_name"]]["bs"][$i]) {
						$row[$bkup["server_name"]][$bkup["vm_name"]]["bs"][$i] = $bkup["status"];
					}
				}
			} // end of if this backup is in the current day of the week
		} // end of looping thru backups

		if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
			foreach ($sync_bkups as $sb) {
				$sync_finished = $sb["start_time"] + $sb["elapsed_time"];
				if (date("w", $sync_finished) ==  $i) {
					if ($sb["complete"] == true) {
						if ($sb["status"] > $row[$sb["server_name"]][$sb["vm_name"]]["ss"][$i]) {
							$row[$sb["server_name"]][$sb["vm_name"]]["ss"][$i] = $sb["status"];
						}
					}
				} // end of if this sync finished in the current day of the week
			} // end of looping thru syncs
		}
	} // end of looping thru days

	return $row;
}

function dpu_summary_vmware_backups($dpu_id, $client_id) {
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellWidth;

	$row = get_client_vmware_backups($dpu_id, $client_id, time());
	if ($row === false || is_null($row) || count($row) == 0) {
		return false;
	}	
	
	$appClientStyle = getAppClientStyle();
	$clientStyle = getClientStyle();

	$dummy = array(-2, -2, -2, -2, -2, -2, -2);
	$first_time = true;
	foreach($row as $server => $virtual_mach) {
		$str = $server . ":  ";
		$results = "	<tr>\n";
		$results .= '		<td height="' . $tableCellHeight . '" style="' . $appClientStyle . '">' . $str . "</td>\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($dummy) .  "\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($dummy) .  "\n";
		$results .= "	</tr>\n";
		// add whitespace between ESX/VCenter servers 
		if ($first_time) {
			$first_time = false;
		}
		else {
			print(build_filler(8,1));
		}
		print($results);
		$nrows = count($virtual_mach);
		$row_count = 1;
		foreach ($virtual_mach as $vm => $status) {
			$results = "	<tr>\n";
			$results .= '		<td height="' . $tableCellHeight . '" style="' . $clientStyle . '">' . $vm . "</td>\n";
			$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
			$results .= '		' . create_client_table($status["bs"]) . "\n";
			$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
			$results .= '		' . create_client_table($status["ss"]) .  "\n";
			$results .= "	</tr>\n";
			print($results);
			// add whitespace between rows, but not after the last row
			if ($row_count != $nrows) {
				print(build_filler(8,1)); 
			}
			$row_count++;
		}
	}
	return true;
}

function get_client_hyperv_backups($dpu, $client, $end) {
	$start = strtotime(date("Y-m-d", $end - 518400));
	$backups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client,
				"start_time" => $start, "end_time" => $end,
				"type" => array("hyperv full", "hyperv differential", "hyperv incremental")));

	if (is_null($backups) || $backups === false || count($backups) == 0) {
		return false;
	}

	$sync_bkups = bp_get_backup_status(array("dpu_id" => $dpu,
			"client_id" => $client, "start_time" => $start, "end_time" => $end, 
			"type" => array("securesync hyperv full", "securesync hyperv diff",
				"securesync hyperv incr")));


	// sort the backups by VM name
	$vm_names = array();
	foreach ($backups as $bkup) {
		$vm_names[$bkup["id"]] = $bkup["vm_name"];
	}

	// Backups may not sync at the same rate that they were created,
	// so some VM names may occur in one array and not the other.
	$Svm_names = array();
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			$Svm_names[$sb["id"]] = $sb["vm_name"];
		}
	}

	// also make the sort case insensitive
	array_multisort(array_map('strtolower',$vm_names), SORT_ASC, SORT_STRING,
			$backups);

	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		array_multisort(array_map('strtolower',$Svm_names), SORT_ASC,
				SORT_STRING, $sync_bkups);
	}

	$row = array();
	foreach ($backups as $bkup) {
		// initialize the status arrays for each VM 
		// bs is backup status, ss is sync status 
		for ($i=0; $i<7; $i++) {
			$row[$bkup["vm_name"]]["bs"][$i] = -1;
			$row[$bkup["vm_name"]]["ss"][$i] = -1;
		}
	}
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			for ($i=0; $i<7; $i++) {
				$row[$sb["vm_name"]]["bs"][$i] = -1;
				$row[$sb["vm_name"]]["ss"][$i] = -1;
			}
		}
	}

	// now get the real highest status code for each day
	for ($i=0; $i<7; $i++) {
		foreach ($backups as $bkup) {
			if (date("w", $bkup["start_time"]) ==  $i) {
				if ($bkup["complete"] == true) {
					if ($bkup["status"] > $row[$bkup["vm_name"]]["bs"][$i]) {
						$row[$bkup["vm_name"]]["bs"][$i] = $bkup["status"];
					}
				}
			} // end of if this backup is in the current day of the week
		}// end of looping thru backups
		if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
			foreach ($sync_bkups as $sb) {
				$sync_finished = $sb["start_time"] + $sb["elapsed_time"];
				if (date("w", $sync_finished) ==  $i) {
					if ($sb["complete"] == true) {
						if ($sb["status"] > $row[$sb["vm_name"]]["ss"][$i]) {
							$row[$sb["vm_name"]]["ss"][$i] = $sb["status"];
						}
					}
				} // end of if this sync finished in the current day of the week
			} // end of looping thru syncs
		}
	} // end of looping thru days

	return $row;
}

function dpu_summary_hyperv_backups($dpu_id, $client, $client_id) {
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellWidth;
	
	static $first_time = 0;

	$row = get_client_hyperv_backups($dpu_id, $client_id, time());
	if ($row === false || is_null($row) || count($row) == 0) {
		return false;
	}

	if ($first_time == 1) {
		// report_dpu.php has this function with a single arg, height
		print(build_filler(8,1));
	}
	else {
		$first_time = 1;
	}
	
	$appClientStyle = getAppClientStyle();
	$clientStyle = getClientStyle();

	$dummy = array(-2, -2, -2, -2, -2, -2, -2);
	$str = $client . ": ";
	$results = "	<tr>\n";
	$results .= '		<td height="' . $tableCellHeight . '" style="' . $appClientStyle . '">' . $str . "</td>\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= "	</tr>\n";
	print($results);
	$nrows = count($row);
	$row_count = 1;
	foreach ($row as $vm => $status) {
		$results = "	<tr>\n";
		$results .= '		<td height="' . $tableCellHeight . '" style="' . $clientStyle . '">' . $vm . "</td>\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["bs"]) . "\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["ss"]) .  "\n";
		$results .= "	</tr>\n";
		print($results);
		// add whitespace between rows, but not after the last row
		if ($row_count != $nrows) {
			print(build_filler(8,1)); 
		}
		$row_count++;
	}
	return true;
}

function get_client_oracle_backups($dpu, $client, $end) {
	$start = strtotime(date("Y-m-d", $end - 518400));
	$backups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client,
				"start_time" => $start, "end_time" => $end,
				"type" => array("oracle full", "oracle incr")));
	if (is_null($backups) || $backups === false || count($backups) == 0) {
		return false;
	}

	$sync_bkups = bp_get_backup_status(array("dpu_id" => $dpu,
			"client_id" => $client, "start_time" => $start, "end_time" => $end, 
			"type" => array("securesync oracle full", "securesync oracle incr")));


	// sort the backups by VM name
	$db_names = array();
	foreach ($backups as $bkup) {
		$db_names[$bkup["id"]] = $bkup["oracle_sid"];
	}

	// Backups may not sync at the same rate that they were created,
	// so some VM names may occur in one array and not the other.
	$Sdb_names = array();
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			$Sdb_names[$sb["id"]] = $sb["oracle_sid"];
		}
	}

	// also make the sort case insensitive
	array_multisort(array_map('strtolower',$db_names), SORT_ASC, SORT_STRING,
			$backups);

	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		array_multisort(array_map('strtolower',$Sdb_names), SORT_ASC,
				SORT_STRING, $sync_bkups);
	}

	$row = array();
	foreach ($backups as $bkup) {
		// initialize the status arrays for each VM 
		// bs is backup status, ss is sync status 
		for ($i=0; $i<7; $i++) {
			$row[$bkup["oracle_sid"]]["bs"][$i] = -1;
			$row[$bkup["oracle_sid"]]["ss"][$i] = -1;
		}
	}
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			for ($i=0; $i<7; $i++) {
				$row[$sb["oracle_sid"]]["bs"][$i] = -1;
				$row[$sb["oracle_sid"]]["ss"][$i] = -1;
			}
		}
	}

	// now get the real highest status code for each day
	for ($i=0; $i<7; $i++) {
		foreach ($backups as $bkup) {
			if (date("w", $bkup["start_time"]) ==  $i) {
				if ($bkup["complete"] == true) {
					if ($bkup["status"] > $row[$bkup["oracle_sid"]]["bs"][$i]) {
						$row[$bkup["oracle_sid"]]["bs"][$i] = $bkup["status"];
					}
				}
			} // end of if this backup is in the current day of the week
		}// end of looping thru backups
		if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
			foreach ($sync_bkups as $sb) {
				$sync_finished = $sb["start_time"] + $sb["elapsed_time"];
				if (date("w", $sync_finished) ==  $i) {
					if ($sb["complete"] == true) {
						if ($sb["status"] > $row[$sb["oracle_sid"]]["ss"][$i]) {
							$row[$sb["oracle_sid"]]["ss"][$i] = $sb["status"];
						}
					}
				} // end of if this sync finished in the current day of the week
			} // end of looping thru syncs
		}
	} // end of looping thru days

	return $row;
}

function dpu_summary_oracle_backups($dpu_id, $client, $client_id) {
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellWidth;
	
	static $first_time = 0;

	$row = get_client_oracle_backups($dpu_id, $client_id, time());
	if ($row === false || is_null($row) || count($row) == 0) {
		return false;
	}

	if ($first_time == 1) {
		// report_dpu.php has this function with a single arg, height
		print(build_filler(8,1));
	}
	else {
		$first_time = 1;
	}
	
	$appClientStyle = getAppClientStyle();
	$clientStyle = getClientStyle();

	$dummy = array(-2, -2, -2, -2, -2, -2, -2);
	$str = $client . ": ";
	$results = "	<tr>\n";
	$results .= '		<td height="' . $tableCellHeight . '" style="' . $appClientStyle . '">' . $str . "</td>\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= "	</tr>\n";
	print($results);
	$nrows = count($row);
	$row_count = 1;
	foreach ($row as $db => $status) {
		$results = "	<tr>\n";
		$results .= '		<td height="' . $tableCellHeight . '" style="' . $clientStyle . '">' . $db . "</td>\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["bs"]) . "\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["ss"]) .  "\n";
		$results .= "	</tr>\n";
		print($results);
		// add whitespace between rows, but not after the last row
		if ($row_count != $nrows) {
			print(build_filler(8,1)); 
		}
		$row_count++;
	}
	return true;
}

function get_client_sharepoint_backups($dpu, $client, $end) {
	$start = strtotime(date("Y-m-d", $end - 518400));
	$backups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client,
				"start_time" => $start, "end_time" => $end,
				"type" => array("sharepoint full", "sharepoint diff")));
	echo "\n";
	if (is_null($backups) || $backups === false || count($backups) == 0) {
		return false;
	}

	$sync_bkups = bp_get_backup_status(array("dpu_id" => $dpu,
			"client_id" => $client, "start_time" => $start, "end_time" => $end, 
			"type" => array("securesync sharepoint full", "securesync sharepoint diff")));


	// sort the backups by VM name
	$farm_names = array();
	foreach ($backups as $bkup) {
		$farm_names[$bkup["id"]] = $bkup["sharepoint_guid"];
	}

	// Backups may not sync at the same rate that they were created,
	// so some VM names may occur in one array and not the other.
	$Sfarm_names = array();
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			$Sfarm_names[$sb["id"]] = $sb["sharepoint_guid"];
		}
	}

	// also make the sort case insensitive
	array_multisort(array_map('strtolower',$farm_names), SORT_ASC, SORT_STRING,
			$backups);

	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		array_multisort(array_map('strtolower',$Sfarm_names), SORT_ASC,
				SORT_STRING, $sync_bkups);
	}

	$row = array();
	foreach ($backups as $bkup) {
		// initialize the status arrays for each VM 
		// bs is backup status, ss is sync status 
		for ($i=0; $i<7; $i++) {
			$row[$bkup["sharepoint_guid"]]["bs"][$i] = -1;
			$row[$bkup["sharepoint_guid"]]["ss"][$i] = -1;
		}
	}
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			for ($i=0; $i<7; $i++) {
				$row[$sb["sharepoint_guid"]]["bs"][$i] = -1;
				$row[$sb["sharepoint_guid"]]["ss"][$i] = -1;
			}
		}
	}

	// now get the real highest status code for each day
	for ($i=0; $i<7; $i++) {
		foreach ($backups as $bkup) {
			if (date("w", $bkup["start_time"]) ==  $i) {
				if ($bkup["complete"] == true) {
					if ($bkup["status"] > $row[$bkup["sharepoint_guid"]]["bs"][$i]) {
						$row[$bkup["sharepoint_guid"]]["bs"][$i] = $bkup["status"];
					}
				}
			} // end of if this backup is in the current day of the week
		}// end of looping thru backups
		if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
			foreach ($sync_bkups as $sb) {
				$sync_finished = $sb["start_time"] + $sb["elapsed_time"];
				if (date("w", $sync_finished) ==  $i) {
					if ($sb["complete"] == true) {
						if ($sb["status"] > $row[$sb["sharepoint_guid"]]["ss"][$i]) {
							$row[$sb["sharepoint_guid"]]["ss"][$i] = $sb["status"];
						}
					}
				} // end of if this sync finished in the current day of the week
			} // end of looping thru syncs
		}
	} // end of looping thru days

	return $row;
}

function dpu_summary_sharepoint_backups($dpu_id, $client, $client_id) {
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellWidth;
	
	static $first_time = 0;

	$row = get_client_sharepoint_backups($dpu_id, $client_id, time());
	if ($row === false || is_null($row) || count($row) == 0) {
		return false;
	}

	if ($first_time == 1) {
		// report_dpu.php has this function with a single arg, height
		print(build_filler(8,1));
	}
	else {
		$first_time = 1;
	}
	
	$appClientStyle = getAppClientStyle();
	$clientStyle = getClientStyle();

	$dummy = array(-2, -2, -2, -2, -2, -2, -2);
	$str = $client . ": ";
	$results = "	<tr>\n";
	$results .= '		<td height="' . $tableCellHeight . '" style="' . $appClientStyle . '">' . $str . "</td>\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= "	</tr>\n";
	print($results);
	$nrows = count($row);
	$row_count = 1;
	foreach ($row as $vm => $status) {
		$results = "	<tr>\n";
		$results .= '		<td height="' . $tableCellHeight . '" style="' . $clientStyle . '">' . $vm . "</td>\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["bs"]) . "\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["ss"]) .  "\n";
		$results .= "	</tr>\n";
		print($results);
		// add whitespace between rows, but not after the last row
		if ($row_count != $nrows) {
			print(build_filler(8,1)); 
		}
		$row_count++;
	}
	return true;
}

function get_client_cisco_ucs_backups($dpu, $client, $end) {
	$instance_key = "database_name";
	$start = strtotime(date("Y-m-d", $end - 518400));
	$backups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client,
				"start_time" => $start, "end_time" => $end,
				"type" => array("ucs service profile full")));
	echo "\n";
	if (is_null($backups) || $backups === false || count($backups) == 0) {
		return false;
	}

	$sync_bkups = bp_get_backup_status(array("dpu_id" => $dpu,
			"client_id" => $client, "start_time" => $start, "end_time" => $end, 
			"type" => array("securesync ucs service profile full")));


	// sort the backups by instance name
	$profile_names = array();
	foreach ($backups as $bkup) {
		$profile_names[$bkup["id"]] = $bkup[$instance_key];
	}

	// Backups may not sync at the same rate that they were created,
	// so some instance names may occur in one array and not the other.
	$Sprofile_names = array();
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			$Sprofile_names[$sb["id"]] = $sb[$instance_key];
		}
	}

	// also make the sort case insensitive
	array_multisort(array_map('strtolower',$profile_names), SORT_ASC, SORT_STRING,
			$backups);

	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		array_multisort(array_map('strtolower',$Sprofile_names), SORT_ASC,
				SORT_STRING, $sync_bkups);
	}

	$row = array();
	foreach ($backups as $bkup) {
		// initialize the status arrays for each VM 
		// bs is backup status, ss is sync status 
		for ($i=0; $i<7; $i++) {
			$row[$bkup[$instance_key]]["bs"][$i] = -1;
			$row[$bkup[$instance_key]]["ss"][$i] = -1;
		}
	}
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			for ($i=0; $i<7; $i++) {
				$row[$sb[$instance_key]]["bs"][$i] = -1;
				$row[$sb[$instance_key]]["ss"][$i] = -1;
			}
		}
	}

	// now get the real highest status code for each day
	for ($i=0; $i<7; $i++) {
		foreach ($backups as $bkup) {
			if (date("w", $bkup["start_time"]) ==  $i) {
				if ($bkup["complete"] == true) {
					if ($bkup["status"] > $row[$bkup[$instance_key]]["bs"][$i]) {
						$row[$bkup[$instance_key]]["bs"][$i] = $bkup["status"];
					}
				}
			} // end of if this backup is in the current day of the week
		}// end of looping thru backups
		if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
			foreach ($sync_bkups as $sb) {
				$sync_finished = $sb["start_time"] + $sb["elapsed_time"];
				if (date("w", $sync_finished) ==  $i) {
					if ($sb["complete"] == true) {
						if ($sb["status"] > $row[$sb[$instance_key]]["ss"][$i]) {
							$row[$sb[$instance_key]]["ss"][$i] = $sb["status"];
						}
					}
				} // end of if this sync finished in the current day of the week
			} // end of looping thru syncs
		}
	} // end of looping thru days

	return $row;
}

function dpu_summary_cisco_ucs_backups($dpu_id, $client, $client_id) {
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellWidth;
	
	static $first_time = 0;

	$row = get_client_cisco_ucs_backups($dpu_id, $client_id, time());
	if ($row === false || is_null($row) || count($row) == 0) {
		return false;
	}

	if ($first_time == 1) {
		// report_dpu.php has this function with a single arg, height
		print(build_filler(8,1));
	}
	else {
		$first_time = 1;
	}
	
	$appClientStyle = getAppClientStyle();
	$clientStyle = getClientStyle();

	$dummy = array(-2, -2, -2, -2, -2, -2, -2);
	$str = $client . ": ";
	$results = "	<tr>\n";
	$results .= '		<td height="' . $tableCellHeight . '" style="' . $appClientStyle . '">' . $str . "</td>\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
	$results .= '		' . create_client_table($dummy) .  "\n";
	$results .= "	</tr>\n";
	print($results);
	$nrows = count($row);
	$row_count = 1;
	foreach ($row as $vm => $status) {
		$results = "	<tr>\n";
		$results .= '		<td height="' . $tableCellHeight . '" style="' . $clientStyle . '">' . $vm . "</td>\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["bs"]) . "\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($status["ss"]) .  "\n";
		$results .= "	</tr>\n";
		print($results);
		// add whitespace between rows, but not after the last row
		if ($row_count != $nrows) {
			print(build_filler(8,1)); 
		}
		$row_count++;
	}
	return true;
}

function get_client_xen_backups($dpu, $client, $end) {
	$start = strtotime(date("Y-m-d", $end - 518400));
	$backups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client, "start_time" => $start, "end_time" => $end, "type" => array("xen full")));
	if (is_null($backups) || $backups === false || count($backups) == 0) {
		return false;
	}
	$sync_bkups = bp_get_backup_status(array("dpu_id" => $dpu, "client_id" => $client, "start_time" => $start, "end_time" => $end, "type" => array("securesync xen full")));

	$server_names = array();
	$vm_names = array();
	// sort the backups by server_name, then VM name
	foreach ($backups as $bkup) {
		$server_names[$bkup["id"]] = $bkup["server_name"];
		$vm_names[$bkup["id"]] = $bkup["vm_name"];
	}

	// Backups may not sync at the same rate as they were
	// created, so some servers/VMs may be in one array
	// and not the other.
	$Sserver_names = array();
	$Svm_names = array();
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			$Sserver_names[$sb["id"]] = $sb["server_name"];
			$Svm_names[$sb["id"]] = $sb["vm_name"];
		}
	}


	// also make the sort case insensitive
	array_multisort(array_map('strtolower',$server_names), SORT_ASC, SORT_STRING, array_map('strtolower',$vm_names),SORT_ASC, SORT_STRING,$backups);

	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		array_multisort(array_map('strtolower',$Sserver_names), SORT_ASC, SORT_STRING,
				array_map('strtolower',$Svm_names),SORT_ASC, SORT_STRING, $sync_bkups);
	}

	$row = array();
	foreach ($backups as $bkup) {
		// initialize the status arrays for each server/VM 
		// bs is backup status, ss is sync status 
		for ($i=0; $i<7; $i++) {
			$row[$bkup["server_name"]][$bkup["vm_name"]]["bs"][$i] = -1;
			$row[$bkup["server_name"]][$bkup["vm_name"]]["ss"][$i] = -1;
		}
	}
	if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
		foreach ($sync_bkups as $sb) {
			for ($i=0; $i<7; $i++) {
				$row[$sb["server_name"]][$sb["vm_name"]]["bs"][$i] = -1;
				$row[$sb["server_name"]][$sb["vm_name"]]["ss"][$i] = -1;
			}
		}
	}

	// now get the real highest status code for each day
	for ($i=0; $i<7; $i++) {
		foreach ($backups as $bkup) {
			if (date("w", $bkup["start_time"]) ==  $i) {
				if ($bkup["complete"] == true) {
					if ($bkup["status"] > $row[$bkup["server_name"]][$bkup["vm_name"]]["bs"][$i]) {
						$row[$bkup["server_name"]][$bkup["vm_name"]]["bs"][$i] = $bkup["status"];
					}
				}
			} // end of if this backup is in the current day of the week
		} // end of looping thru backups

		if (isset($sync_bkups) && $sync_bkups !== false && count($sync_bkups) > 0) {
			foreach ($sync_bkups as $sb) {
				$sync_finished = $sb["start_time"] + $sb["elapsed_time"];
				if (date("w", $sync_finished) ==  $i) {
					if ($sb["complete"] == true) {
						if ($sb["status"] > $row[$sb["server_name"]][$sb["vm_name"]]["ss"][$i]) {
							$row[$sb["server_name"]][$sb["vm_name"]]["ss"][$i] = $sb["status"];
						}
					}
				} // end of if this sync finished in the current day of the week
			} // end of looping thru syncs
		}
	} // end of looping thru days

	return $row;
}


function dpu_summary_xen_backups($dpu_id, $client_id) {
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellWidth;

	$row = get_client_xen_backups($dpu_id, $client_id, time());
	if ($row === false || is_null($row) || count($row) == 0) {
		return false;
	}	
	
	$appClientStyle = getAppClientStyle();
	$clientStyle = getClientStyle();

	$dummy = array(-2, -2, -2, -2, -2, -2, -2);
	$first_time = true;
	foreach($row as $server => $virtual_mach) {
		$str = $server . ":  ";
		$results = "	<tr>\n";
		$results .= '		<td height="' . $tableCellHeight . '" style="' . $appClientStyle . '">' . $str . "</td>\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($dummy) .  "\n";
		$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
		$results .= '		' . create_client_table($dummy) .  "\n";
		$results .= "	</tr>\n";
		// add whitespace between ESX/VCenter servers 
		if ($first_time) {
			$first_time = false;
		}
		else {
			print(build_filler(8,1));
		}
		print($results);
		$nrows = count($virtual_mach);
		$row_count = 1;
		foreach ($virtual_mach as $vm => $status) {
			$results = "	<tr>\n";
			$results .= '		<td height="' . $tableCellHeight . '" style="' . $clientStyle . '">' . $vm . "</td>\n";
			$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
			$results .= '		' . create_client_table($status["bs"]) . "\n";
			$results .= '		<td width="10" height="' . $tableCellHeight . '"></td>' . "\n";
			$results .= '		' . create_client_table($status["ss"]) .  "\n";
			$results .= "	</tr>\n";
			print($results);
			// add whitespace between rows, but not after the last row
			if ($row_count != $nrows) {
				print(build_filler(8,1)); 
			}
			$row_count++;
		}
	}
	return true;
}


?>

