#!/usr/bin/perl

use strict;
use warnings;

use Fcntl ':mode';

# Output format: newline-delimited lists of comma-delimited fields:
# 0: client
# 1: type: Master|Differential|Incremental|BareMetal
#            |SQL <Full|Diff|Trans>|MSSQL|Exch|VMESX|HyperV <Full|Diff|Incr|Trans>|CEP
# 2: path (i.e. ~/clientname/backup_xxx)
# 3: timestamp
# 4: size, in bytes
# 5: compressed (y/n)
# 6: encrypted (y/n)
# 7: instance number
# 8: legacy sql instance (only for SQL Server)
# 9: legacy sql database (only for SQL Server)
#10: original pathname (only for localdir)

$\ = "\n";
$, = ' ';
my $bputil = "/usr/bp/bin/bputil";

if ($#ARGV < 2) {
	my $name = $0;
	$name =~ s/.*\/([^\/]*)/$1/ if $name =~ /\//;
	print "Usaege: ".$name." <path> <dpuname> <clients (delimited by space)>";
	exit 1;
}

my ($path, $dpuname, @clients) = (shift, shift, @ARGV);
my $dbdump = "$path/transform/dbdump.sql";
my %fileList = ();
my $index = 0;
print STDERR "Clients to transform: @clients";

# Don't die() on error; print to stdout and don't print the script line
if (!$path) {print "Error: an archive path must be supplied"; exit 1; }
if (! -x $path) {print "Error: cannot open archive path"; exit 1; }


&getBackupInfo(); 	# Get all backup types ( file-level, applications, legacy sql)
#&getLocalDirInfo(); 	# Get localdir
#&getCEPInfo();

print STDERR "Finished processing archive at $path";
exit 0;

# Print the list of backups for each selected client at $path. Corresponding information 
# should also be present in @dpuclientsrpt
sub getBackupInfo()
{
	#Extract dpu_clients.rpt from state
   	my @cmd = ("/usr/bp/bin/bkup -x -f $path/$dpuname/archives/lastDPUstate -zFLAT -zWHERE=$path/transform/ /usr/bp/reports.dir/dpu_clients.rpt 2>/dev/null 1>/dev/null");
	if (system(@cmd) != 0) {
		print "Failed to execute @cmd. Exit code: $?";
		exit 1;
	}
	my @dpuclientsrpt;
	my $fname = "$path/transform/dpu_clients.rpt";
	my @stat = stat("$fname");
	if (!@stat) { print "File $fname does not exist"; exit 1; }
	open (FILE, $fname)
		and push @dpuclientsrpt, <FILE>
		and close FILE
		and print STDERR "Loaded dpu clients report";
	@dpuclientsrpt = map {chomp; $_ } @dpuclientsrpt;
	#print map { "$_ \n" } @dpuclientsrpt;
	
	#Extract dbdump.sql from state
   	@cmd = ("/usr/bp/bin/bkup -x -f $path/$dpuname/archives/lastDPUstate -zFLAT -zWHERE=$path/transform/ /usr/bp/reports.dir/dbdump.sql 2>/dev/null 1>/dev/null");
	if (system(@cmd) != 0) {
		print "Failed to execute @cmd. Exit code: $?";
		exit 1;
	}
	@stat = stat("$dbdump");
	if (!@stat) { print "File $dbdump does not exist"; exit 1; }
	
	my $bkup = 1;
	foreach my $client (@clients) {
		next if (! -x "$path/$client");
		print STDERR "Found path for client $path/$client";

		#Read dpuclientsrpt to get data we need

		my $section = '';
		my @lines;
		my $masterBackup = 0;
		my $incrBackup = 0;
		for (my $i = 0; $i <= $#dpuclientsrpt; $i++) {
			my $line = $dpuclientsrpt[$i];
			my ($type, $file, $time, $size, $comp, $encr, $localType,$instInfo) = ('','');            # Required attributes
			my ($inst, $backupno, $depbackupno, $application)= ('','','','',''); # Some more attributes
			my $delta = 0;
			my $deltaMast = '';
			next if $line =~ /^\s*$/;
			if ($line =~ /^\[(.*)\]/) {
				$section = $1;
				next;
			}
			#my $qc = quotemeta $client;
			# get file-level backup information 
			if ($section eq $client and $line =~ /\s*Item[0-9]+=([^\s]+[^\,]+)/) {
				#print STDERR "$section: $client, value: $1";
				my @fields = split ',', $1;
				if ($#fields < 4) { print STDERR "Invalid line"; next; }
				$type = $fields[0];
				$localType = $type;
				if ($type eq 'Master') {
					$masterBackup = $backupno;
				}
				if ($type eq 'Incremental') {
					$depbackupno = $masterBackup;
				}
				$instInfo = getFilelevelInstanceInfo($client, $localType);
				#$origdir = $fields[1];
				#$origdev = $fields[2];
				if ($localType eq 'SQL') {
					$time = $fields[5];
					$type = "SQL".$fields[4];
					$backupno = 
					$instInfo = $instInfo."|". $fields[2] . "|". $fields[3]. "||"	
				} else {
					$time = $fields[4];
					$backupno = $bkup++;
				}
				$time =~ s/\s+$//;
				$application = 0;	
				$file = "$path/$client/archives/last$localType";

			} 
			# get applications backup information
			if ($section eq "$client-applications" and $line =~ /\s*Item[0-9]+=([^\s]+[^\,]+)/) {
				#print STDERR "$section: $client, value: $1";
				my @fields = split ',', $1;
				if ($#fields < 6) { print STDERR "Invalid line"; next; }

				$type = $fields[0];
				$backupno = $fields[1];
				#$origdir = $fields[2];
				#$origdev = $fields[3];
				$time = $fields[5];
				$inst = $fields[6];
				$inst =~ s/\s+$//;
				if ($type =~ /[^\s]+Full/) {
					$fname = 'lastFull';
				} elsif ($type =~ /[^\s]+Diff/) {
					$fname = 'lastDifferential';
				} elsif ($type =~ /[^\s]+Trans/) {
					$fname = "trans-$backupno";
				} elsif ($type =~ /[^\s]+Incr/) {
					$fname = "incr-$backupno";
				}
				$file = "$path/$client/appInstance/$inst/$fname"; 
				$application = 1;	
				# look at state db dump to find the instance name
				$instInfo = getApplicationInstanceInfo($inst);
			}
			if (!length($file)) { next; }
			#print STDERR "file: $file";
			my @stat = stat("$file");
			if (!@stat) { 
				print STDERR "File $file does not exist"; 
				if ($fname eq 'lastDifferential' or $fname eq 'lastIncremental') {
					# Try lastDifferential.delta
					if ($application) {
						$file = "$path/$client/appInstance/$inst/$fname.delta";
						$deltaMast = "$path/$client/appInstance/$inst/lastFull";
						$delta = 1;
					} else {
						$file = "$path/$client/archives/last$localType.delta";
						$deltaMast = "$path/$client/archives/lastMaster";
						$delta = 1;
					}
					@stat = stat("$file");
					if (!@stat) {
						print STDERR "File $file does not exist";
						next;
					}
				} else {
					next;
				}
			}
			#Add file to hash list. dpu_clients.rpt has sometimes multiple entires of same backup.
			if (addToHashList($file) != 0) {
				next; #Skip this, go find next
			}
			$size = $stat[7];
			$comp = 'n';
			$encr = is_encrypted("$file");

			my $str = "$client|$type|$file|$time|$size|$comp|$encr|$backupno|$depbackupno|$instInfo|$deltaMast"; 
			#print STDERR "OUTPUT: $str";
			print "$str";
		}
	}
}

sub addToHashList() {
	my $fileName = shift;
	my $found = 0;

	#search for value
	for my $i ( keys %fileList) {
		next if ($fileList{$i} ne $fileName);
		$found = 1;
	}
	# Add iff not found
	if (!$found) {
		$fileList{$index} = $fileName;
		$index++;
	}
	return $found;
}

sub is_encrypted {
	my $filename = shift;
	my $linkflag = `zcat -qf $filename 2>/dev/null | dd bs=1 skip=156 count=1 2>/dev/null`;
	#print STDERR "Linkflag for $filename is $linkflag";
	return ($linkflag eq 'E' ? 'y' : 'n');
}

# Returns node_no|app_id|instance_id|os|key1|key2|key3|key4 from database for all application backups
sub getApplicationInstanceInfo() {
	my $id = shift;
	my $name = "Instance ID $id";
	if (! open(FILE, "$dbdump")) {
		print STDERR "Cannot open state db dump $dbdump"; return $name;
	}

	# Read through the dump until we get to the application_instances table
	my $section = '';
	my @ldlines;
	my $found = 0;
	while (<FILE>) {
		chomp;
		if (!$found && m/^COPY application_instances /) {
			$found = 1; next;
		}
		next if !$found;
		if (m/^$id\s+/) {
			# This is the one, return the 5th,6th,7th,8th fields
			my @fields = split ' ';
			close FILE;
			#print STDERR "Found keys for id $id";
			if ($fields[5] =~ /^\\N/) {
				$fields[5] = '';
			}
			if ($fields[6] =~ /^\\N/) {
				$fields[6] = '';
			}
			if ($fields[7] =~ /^\\N/) {
				$fields[7] = '';
			}
			my $os = getClientOS($fields[1]);
			return $fields[1]."|".$fields[2]."|".$id. "|". $os."|".$fields[4]."|".$fields[5]."|". $fields[6]."|".$fields[7];
		}
		if (m/^\\\./ or m/^\s*$/) {
			print STDERR "Instance $id not found in dump";
			close FILE;
			return $name;
		}
	}
	print STDERR "Instance $id not found in dump (EOF)";
	close FILE;
	return $name;
}

sub getFilelevelInstanceInfo() {
	my $nodeName = shift;
	my $localType = shift;
	if (! open(FILE, "$dbdump")) {
		print STDERR "Cannot open state db dump $dbdump"; return '';
	}

	# Read through the dump until we get to the application_instances table
	my $section = '';
	my @ldlines;
	my $found = 0;
	my $appsFound = 0;
	my $infoFound = 0;;
	my $appid = 1;
	my $os = 0;
	my $nodeNo = 0;
	my $instanceId = 0;

	while (<FILE>) {
		chomp;
		if (!$found && m/^COPY nodes /) {
			$found = 1; next;
		}
		next if !$found;
		if (!$infoFound && m/^[0-9]+\s+$nodeName/) {
			my @fields = split ' ';
			$nodeNo = $fields[0];
			$os = $fields[3];
			#print STDERR "Node:: $nodeNo, $os";
			
			# Get instance_id from application_instances, file is not closed yet.
			$infoFound = 1;
		}
		if (!$infoFound && (m/^\\\./ or m/^\s*$/)) {
			print STDERR "Node $nodeName not found in dump";
			close FILE;
			return '';
		}
		#print "Line: $_\n";
		if ($infoFound && !$appsFound && m/^COPY application_instances/) {
			$appsFound = 1; next;
		}
		next if !$appsFound;
		if ($infoFound && m/^[0-9]+\s+$nodeNo/) {
			my @fields1 = split ' ';
			$instanceId = $fields1[0];
			close FILE;
			if ($localType eq 'SQL') {
				return $nodeNo."|".$appid."|".$instanceId."|".$os;
			} else {
				return $nodeNo."|".$appid."|".$instanceId."|".$os."||||";
			}
		}
	}
	print STDERR "Node $nodeName not found in dump (EOF)";
	close FILE;
	return '';
}

sub getClientOS() {
	my $id = shift;
	if (! open(FILE, "$dbdump")) {
		print STDERR "Cannot open state db dump $dbdump"; return '';
	}

	# Read through the dump until we get to nodes table
	my $section = '';
	my @ldlines;
	my $found = 0;
	while (<FILE>) {
		chomp;
		if (!$found && m/^COPY nodes /) {
			$found = 1; next;
		}
		next if !$found;
		if (m/^$id\s+/) {
			# This is the one, return the 4th field
			my @fields = split ' ';
			close FILE;
			#print STDERR "Found node for id $id";
			return $fields[3];
		}
		if (m/^\\\./ or m/^\s*$/) {
			print STDERR "Node $id not found in dump";
			close FILE;
			return '';
		}
	}
	print STDERR "Node $id not found in dump (EOF)";
	close FILE;
	return '';
}
