#!/usr/bin/php
<?php
// This script goes through all the systems in the systems table
// and updates the value of the role column.  It is normally called just
// once from setup_postgresql.sh during the postinstall of the first
// upgrade to Release 4.1 (but it can be called standalone at any time).
// The basic logic for determining the role of remote systems is:
//   if the host is found in the DPUuser's file: it's a vaulting DPU
//   else : it's a managed DPU
// The logic for determining the role of the local system:
//   if no remote systems, it's a DPU
//   else if at least one remote system is vaulting, it's a Vault
//   else (no remote systems are vaulting), it's a Manager
// Returns: 0 on success, 1 on failure.  
//   

$debug=false; // set this to true to see all queries printed
$dbconn_string = "port=5432 dbname=bpdb user=postgres";

$dbconn = pg_connect($dbconn_string);
if($dbconn === FALSE) {
	print "ERROR: Failed to connect to the database.\n";
	die(1);
}

// make sure the role column has been added
$query = "SELECT role FROM bp.systems";
if ($debug) print $query."\n";
$res = pg_query($dbconn, $query);
if ($res === false) {
	print "ERROR: failed to access the role column in the systems table:\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}

$update_str = "UPDATE bp.systems SET role = '";
$query = "SELECT system_id, host FROM bp.systems WHERE is_local_system = false";
if ($debug) print $query."\n";

$res = pg_query($dbconn, $query);
if ($res === false) {
	print "ERROR: failed to get host list from systems table:\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}

$nrows = pg_num_rows($res);
if ($nrows == 0) {
	if($debug) print "There are no remote systems...\n";
	$local_role = 'DPU';
}
else {
	if($debug) print "Processing remote systems...\n";
	$local_role = 'Manager';
	for ($i = 0; $i < $nrows; $i++) {
		$sysID = pg_fetch_result($res, $i, 0);
		$host = pg_fetch_result($res, $i, 1);
		system("[ -f /backups/.DPUusers ] && cat /backups/.DPUusers | awk '{print $3}' | grep -w $host >/dev/null", $ret);
		if ($ret == 0) {
			$local_role = 'Vault';
			$role = 'DPU Configured for Vaulting';
		}
		else {
			$role = 'Managed DPU';
		}
		if ($debug) {
			print "Role for $host is $role:\n";
		}
		$query = $update_str.$role."' WHERE system_id = $sysID";
		if ($debug) print $query."\n";
		$update_res = pg_query($dbconn, $query);
		if ($update_res === false) {
			print "ERROR: failed to update role to $role for system $host:\n";
			print pg_last_error($dbconn)."\n";
			die(1);
		}
	}
}

// Update the record for the local system
$query = $update_str.$local_role."' WHERE is_local_system = true";
if ($debug) print $query."\n";
$res = pg_query($dbconn, $query);
if ($res === false) {
	print "ERROR: failed to update role to $role for local system:\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}

$nrows++;
$query = "SELECT system_id, host, role FROM bp.systems ORDER BY system_id";
if ($debug) print $query."\n";
$res = pg_query($dbconn, $query);
$n = pg_num_rows($res);
if ($n !=  $nrows) {
	print "ERROR: the number of rows processed ($nrows) is different ";
	print "from the number of rows in the systems table ($n)!\n";
	die(1);
}
if ($res !== false) {
	printf("%d record%s processed:\n", $n, $n==1?"":"s");
	printf("ID  HOSTNAME                   ROLE\n");
	printf("--- -------------------------  ----------------------------\n");
	for ($i = 0; $i < $n; $i++) {
		$row = pg_fetch_assoc($res, $i);
		printf("%-3d %-25s  %s\n", $row["system_id"], $row["host"],
			$row["role"]);
	}
}
exit(0);

?>
