#!/bin/bash
#
# Generate TXT and PDF of all skipped files on failed or warning backups

RCPT=""
SUBJECT="Skipped file report for $(hostname) - $(date)"
BODY="The attached PDF contains a list of all the backups having skipped files for the past 24 hours as of $(date)"
REPORTS_DIR="/var/www/html/recoveryconsole/reports.dir/"
FILE="${REPORTS_DIR}/$(date +%Y_%m_%d)_$(hostname).txt"
FILEPDF="${REPORTS_DIR}/$(date +%Y_%m_%d)_$(hostname).pdf"
DAYS_TO_KEEP=30

if [ ! -d "${REPORTS_DIR}" ]; then
	mkdir "${REPORTS_DIR}"
fi

BACKUP_LIST=$(psql bpdb postgres --tuples-only -c "select bp.backups.backup_no from bp.backups where to_timestamp(bp.backups.start_time) >= (now() - INTERVAL '2 day') and bp.backups.status != 3168 and bp.backups.status != 512")

for BACKUP in ${BACKUP_LIST}; do
	CLIENT_NAME=$(psql bpdb postgres --tuples-only -c "select node_name from bp.nodes join bp.backups using (node_no) where backup_no = ${BACKUP}")
	TIME=$(psql bpdb postgres --tuples-only -c "select to_timestamp(start_time) from bp.backups where backup_no = ${BACKUP}")

	if [ -f "${REPORTS_DIR}/backup_msg.${BACKUP}" ]; then
		BACK2=$(sed -e '1,/----- Files Messages -----/d;/----- End Files Messages -----/,$d' "${REPORTS_DIR}/backup_msg.${BACKUP}")
	else
		BACKUP_MSG=$(psql bpdb postgres --no-align --tuples-only -c "select messages from bp.backup_msg join bp.backups using (backup_no) where backup_no = ${BACKUP}")
		BACK2=$(sed -e '1,/----- Files Messages -----/d;/----- End Files Messages -----/,$d' <<< "${BACKUP_MSG}")
	fi

	if [ -n "${BACK2}" ]; then
		echo -e "Backup Number:\t\t" ${BACKUP} >> "${FILE}"
		echo -e "Client Name:\t\t" ${CLIENT_NAME} >> "${FILE}"
		echo -e "Backup Start Time:\t" ${TIME} >> "${FILE}"
		echo -e "Backup Messages:" >> "${FILE}"
		echo -e "${BACK2}" >> "${FILE}"
		echo >> "${FILE}"
		echo >> "${FILE}"
	fi
done

#Send as an email
if [ -f "${FILE}" ]; then
	# Generate PDF and attach to an email
	enscript "${FILE}" -b "Missed File Report|%D{%F}|$%" -o - 2>/dev/null | ps2pdf - > "${FILEPDF}"
	echo "${BODY}" | mutt -s "${SUBJECT}" -a "${FILEPDF}" -- "${RCPT}"

	# Compress the backup messages
	for BACKUP in ${BACKUP_LIST}; do
		gzip -q -f --best "${REPORTS_DIR}/backup_msg.${BACKUP}"
	done

	# Cleanup files sent in email and older backup messages
	rm -f "${FILE}" "${FILEPDF}"
	find "${REPORTS_DIR}" -maxdepth 1 -mtime +${DAYS_TO_KEEP} -name backup_msg.*.gz -type f -exec rm -f {} \;
fi
