#!/usr/bin/php -q
<?php
# Change log
## Bug  Who  Description
## ---  ---  -----------
## 2952 MB   Updated bp_db_delete_unit and bp_db_delete_drive to close old alerts for deleted drives/units.
## 2932 MB   Alert script for Alertman [PCI, 3ware, License, Tasker, Clients]
## 2770 GDP  Escaped strings for database, fix php warning ouput if devices not present (3ware, PCI)
## 3594 GDP  Refactor raising/closing of alerts, fix 3ware alert severity, some general 
##               cleanup, add cryptoDaemon alerts, and fix mail alerts.
## 4618 ME   Added support for processing CEP error log alerts.
## 4647 YES  Strict licensing
## 4647 YES  Strict licensing
## 4695 ME   Added grace period alert and email when license not initialized
## 4745 LCG  Determine tasker status with "ps -C" and update library table if it's not running
## 5128 ME   Consider empty string return from get_masterini on previous 
##               grace period.
## 5015 ME   Send different string if grace period reaches 0.
## 5275 ME   Resolve CEP alerts older than one day if no alerts were found in this run.
## 5576 GDP  Add software update alert if customer has DA/DAP
## 9056 MC   Add new SNMP Traps for Support
#

# define severity levels to clarify code
define('S_FATAL', 1);
define('S_WARNING', 2);
define('S_NOTICE', 3);

#---------------------------------------------------
# master.ini Functions
#---------------------------------------------------
// Gets a mster.ini setting.  If $boolean is true, then we will parse any
// boolean-like values (On/on, true, yes, ...) and return a proper php boolean
function get_master_ini_setting($section, $item, $default, $boolean = false) {
	global $bploc;

	if (is_bool($default)) {
		$default = ($default === true ? "yes" : "no");
	}

	$master_ini = "$bploc/bpinit/master.ini";
	$bputil = "$bploc/bin/bputil";

	$cmd = "$bploc/bin/bputil -g \"$section\" \"$item\" \"$default\" $master_ini";
	$return_string = trim(shell_exec("$cmd"));

	if ($boolean == true) {
		$lower_return_string = strtolower($return_string);
		if ($lower_return_string{0} == 'y' ||
		    $lower_return_string{0} == 't' ||
		    $lower_return_string == 1 ||
		    $lower_return_string == 'on') {
			return true;
		} else {
			return false;
		}
	} else {
		return $return_string;
	}
}


// Puts a mster.ini setting.  If $boolean is true, then we will parse any
// boolean-like values (On/on, true, yes, ...) and return a proper php boolean
function put_master_ini_setting($section, $label, $string, $boolean = false) {
	global $bploc;

	$master_ini = "$bploc/bpinit/master.ini";
	$bputil = "$bploc/bin/bputil";

	$cmd = "$bploc/bin/bputil -p \"$section\" \"$label\" \"$string\" $master_ini";
	exec($cmd, $retval);

	return $retval;
}

#---------------------------------------------------
# Database Utility Functions
#---------------------------------------------------
function simple_email($from, $to, $subject, $body) {
        $boundary = md5(uniqid(time()));

        $email = 'From: ' . $from . "\n";
        $email .= 'To: ' . $to . "\n";
        $email .= 'Return-Path: ' . $from . "\n";
        $email .= 'MIME-Version: 1.0' . "\n";
        $email .= 'Content-Type: text/plain; boundary="' . $boundary . '"' . "\n\n";

        $email .= $body . "\n\n";
        $email .= '--' . $boundary . "--\n";
        $mail_status = mail('', $subject,'',$email);
        return $mail_status;
}

# writes an ini file, where contents is an array with keys of section
# names containing arrays of key/value pairs (see php.net documentation
# for parse_ini_file() for the format), e.g.:
#
#  $contents = array('section1' => array('key1' => 'value1', 'k2' => 'v2'))
#
function bppa_write_ini_file($filename, $contents) {
	$str = '';
	if (is_array($contents)) {
		foreach ($contents as $section => $kvs) {
			$str .= "\n[$section]\n";
			foreach ($kvs as $k => $v) {
				$str .= "  $k=$v\n";
			}
		}
	}
	return file_put_contents($filename, $str);
}

# If this function returns, it is guaranteed to return a good connection.
#
# If we get a good connection, clear the bad connection history.
# If we don't get a good connection: if it's been an hour and we haven't 
#   sent an email out, then send an email.
function bppa_db_connect() {
	global $bploc;

	$timefile = "$bploc/lists.dir/alertman_db_timespec";
	$sec = 'connectivity';
	$ini = array($sec => array(
				'status' => 'success',  # whether the last connection attempt was successful or not
				'failuretime' => 0,     # database failure epoch
				'emailtime' => 0,       # email epoch
				'updatetime' => 0,      # epoch when file was touched
				'lastfailuretime' => 0, # time of previous failure, set only when status is set to success
				'lastemailtime' => 0)); # time of email sent from previous failure, if any

	$conn = pg_connect("port=5432 dbname=bpdb user=postgres connect_timeout=30");

	# read the ini file
	$filearr = @parse_ini_file($timefile, true);

	# see if we need to write the defaults out
	if ($filearr === false or empty($filearr)) {
		$filearr = $ini;
	}

	$now = time();
	$filearr[$sec]['updatetime'] = $now;

	# test our connection
	if ($conn !== false) {
		# clear failure and email times
		if ($filearr[$sec]['status'] != 'success') {
			$filearr[$sec]['status'] = 'success';
			$filearr[$sec]['lastfailuretime'] = $filearr[$sec]['failuretime'];
			$filearr[$sec]['lastemailtime'] = $filearr[$sec]['emailtime'];
			$filearr[$sec]['failuretime'] = 0;
			$filearr[$sec]['emailtime'] = 0;
		}
	}
	else {
		# see if we've failed for $threshold_mins, if so send an email
		$status = $filearr[$sec]['status'];
		$emailtime = $filearr[$sec]['emailtime'];
		$failuretime = $filearr[$sec]['failuretime'];
		$threshmins = get_master_ini_setting('Alertman', 'DatabaseEmailTimeoutMins', 60);
		if ($threshmins < 0) $threshmins = 0;         # don't allow negative time
		$threshsecs = $threshmins * 60;

		if ($status == 'success') {
			# last time connection was successful, so record the failure
			$filearr[$sec]['status'] = 'failed';
			$filearr[$sec]['failuretime'] = $now;
			$failuretime = $now;  # this allows threshold_mins of 0 to work properly	
		}
		$downtime = $now - $failuretime;
		if ($emailtime == 0 and $downtime >= $threshsecs) {
			# no email has been sent and it's past the threshold
			if ($downtime > 0) {
				$minutes_down = floor($downtime / 60);
				$hours_down = floor($minutes_down / 60);
				$days_down = floor($hours_down / 24);
				$rem_hours_down = $hours_down % 24;
				$rem_minutes_down = $minutes_down % 60;

				if ($days_down > 0) {
					$str = "{$days_down}d {$rem_hours_down}h {$rem_minutes_down}m";
				} else {
					$str = "{$hours_down}h {$rem_minutes_down}m";
				}
				$timestr = " (down: $str)";
			} else {
				$timestr = "";
			}

			mail_alert("System Error",
				"Unable to process alerts: the alert manager was unable to connect\n" .
				"to the DPU database".$timestr.".");
			$filearr[$sec]['emailtime'] = $now;
		}
	}
	bppa_write_ini_file($timefile, $filearr);

	# either return or exit
	if ($conn !== false) {
		return $conn;
	} else {
		die(1);
	}
}

# Build a SQL insert statement out of an array.
function bppa_db_build_insert($table,$array)
{

	$str = "INSERT INTO $table ";
	$strn = "(";
	$strv = " VALUES (";

	$keys = array_keys($array);
	$values = array_values($array);

	for ($i=0; $i < count($array); $i++) {
		$name = $keys[$i];
		$value = $values[$i];

		if(is_bool($value)) {
			$strn .= "$name,";
			$strv .= ($value ? "true":"false") . ",";
			continue;
		};

		if(is_string($value)) {
			$strn .= "$name,";
			$strv .= "'" . pg_escape_string($value) . "',";
			continue;
		}
		if (!is_null($value) and ($value != "")) {
			$strn .= "$name,";
			$strv .= "$value,";
		continue;
		}
	}

	$strn[strlen($strn)-1] = ')';
	$strv[strlen($strv)-1] = ')';
	$str .= $strn . $strv;
	return $str;
}

function bppa_db_add_run($dbconn, $started, $ended) {
	$insert = bppa_db_build_insert('bp.alert_runs', array("started" => $started, "ended" => $ended));
	$retval = pg_query($dbconn, $insert);

	return $retval;
}

function bppa_db_last_run($dbconn) {
	$results = pg_query($dbconn, "SELECT * FROM bp.alert_runs ORDER BY run_id DESC LIMIT 1");
	$results = pg_fetch_all($results);
	if(empty($results) === TRUE) return array();

	return $results;
}

function bppa_trap_last_sent($dbconn, $trapType) {
	$results = pg_query($dbconn, "SELECT time_sent FROM bp.snmp_history WHERE traptype = '$trapType' ORDER BY time_sent DESC LIMIT 1");
	$results = pg_fetch_all($results);
	if(empty($results) === TRUE) return array();

	return $results;
}

function license_has_expired() {
	global $bploc;

	exec($bploc . '/bin/bptool -v 2>/dev/null 1>&2', $output, $retval);
	if ($retval == 50) {
		return "true";
	} else if ($retval == 0) {
		return "false";
	} else {
		return "unknown";
	}
}

#---------------------------------------------------
# PCI Functions
#---------------------------------------------------

# Get the current PCI table from the running system.
function bppa_get_pci() {
        exec('lspci', $pciArray, $retval);
	if ($retval != 0) return $retval;
        exec("lspci -n | sed 's/Class //'", $pciArrayN, $retval);
        if ($retval != 0) return $retval;

	$pciTable = array();

	for($i=0; $i < count($pciArray) - 1; $i++) {
		$pciArray[$i] = explode(" ", $pciArray[$i], 2);
		$pciArrayN[$i] = explode(" ", $pciArrayN[$i]);

		$device_array = split(":", $pciArrayN[$i][2]);
		$class_id = rtrim($pciArrayN[$i][1], ":");
		$pciEntry = array("bus_id" => $pciArray[$i][0], "class_id" => hexdec($class_id), "vendor_id" => hexdec($device_array[0]), "device_id" => hexdec($device_array[1]), "device_name" => $pciArray[$i][1]);
		array_push($pciTable, $pciEntry);
	}

	return $pciTable;
}

# Get the PCI table from the database.
function bppa_db_get_pci($dbconn) {
	$results = pg_query($dbconn, "SELECT * FROM bp.pci_table");
	$results = pg_fetch_all($results);
	if(empty($results) === TRUE) return array();

	return $results;
}

# Update the PCI table.
function bppa_db_set_pci($dbconn, $pcitable) {
	$retval = pg_query($dbconn, 'BEGIN');
	if ($retval == FALSE) {
		return FALSE;
	}
	
	$retval = pg_query($dbconn, 'DELETE FROM bp.pci_table');
	if ($retval == FALSE) {
		pg_query($dbconn, 'ROLLBACK');
		return FALSE;
	}

	for ($i=0; $i < count($pcitable); $i++) {
		$insert = bppa_db_build_insert('bp.pci_table', $pcitable[$i]);
		$retval = pg_query($dbconn, $insert);
		if ($retval == FALSE) {
			pg_query($dbconn, 'ROLLBACK');
			return FALSE;
		}
	}

	$retval = pg_query($dbconn, 'COMMIT');
	return ($retval !== FALSE);
}


#---------------------------------------------------
# Client Functions
#---------------------------------------------------

# Check to see if a client is enabled, returns true or false
function bppa_client_is_enabled($name) {
	$enabled = false;
	$id = bp_get_client_id($name);
	if ($id !== false) {
		$info = bp_get_client_info($id);
		if ($info['is_enabled'] == true) {
			$enabled = true;
		}
	}
	return $enabled;
}

# Check to see if a client is active, returns true or false
# Bug 8817: If the volume list has changed, send a notification.
function bppa_client_is_active($name) {
	global $dbconn;
	global $bploc;

	$active = false;
	$id = bp_get_client_id($name);

	# Skip clients added as a Hyper-V cluster IP (they aren't real systems)
	# Also skip the DPU-client
	# Also skip the RAE clients with NT_GENERIC os 
	$query = "SELECT (properties & 16) = 16, options, os FROM bp.nodes WHERE node_no = $id";
	$res = pg_query($dbconn, $query);
	if (pg_fetch_result($res, 0, 0) == "t") {
		# Issue ReqCapabilities anyway, so agent version is updated in the
		# nodes table.  Also return active status, since we do backups for
		# this type of client, and should raise an alert if the cluster IP
		# is down.  Skip the volume processing below.
		$active = bp_save_client_info(array('id'=>$id));
		return $active;
	}
	if (pg_fetch_result($res, 0, 1) == 1) {
		return true;
	}
	if (pg_fetch_result($res, 0, 2) == 40) {
		return true;
	}

	if ($id !== false) {
		$query = "SELECT volume FROM bp.node_volumes WHERE node_no = $id ORDER BY volume";
		$res = pg_query($dbconn, $query);
		$vols_before = pg_fetch_all_columns($res);
		$active = bp_save_client_info(array('id'=>$id));
	}
	if ($active == true) {
		# bp_get_client_files doesn't connect to the client if the client has no volumes (@@@:),
		# so use bp_save_client_info instead, since it always sends a ReqCapabilities request.
		# We still call bp_get_client_files though, to update the client volume list.
		$files = bp_get_client_files($id, '', '', 0);
		if ($files === false) {
			return $active;
		}
		$res = pg_query($dbconn, $query);
		$vols_after =  pg_fetch_all_columns($res);
		if (!empty($vols_before) and ($vols_before != $vols_after)) {
			// If the client is included in any file-level schedules,
			// raise alert & send email...
			$q = "SELECT DISTINCT(name), enabled::int from bp.schedules JOIN ";
			$q .= "bp.schedule_client_assoc USING(schedule_id) WHERE ";
			$q .= "instance_id = ";
			$q .= "(SELECT instance_id FROM bp.application_instances ";
			$q .= "WHERE node_no = $id AND app_id = 1) ORDER BY enabled DESC";
			$res = pg_query($dbconn, $q);
			if (pg_num_rows($res) == 0) {
				// There are no file-level schedules for this client
				return $active;
			}
			$before_str = implode(" ", $vols_before);
			$after_str = implode(" ", $vols_after);
			$scheds = "\n";
			while ($row = pg_fetch_assoc($res)) {
				$scheds .= " ".$row["name"];
				if ($row["enabled"] == false) {
					$scheds .= " (disabled)";
				}
				$scheds .= "\n";
			}
			bp_send_notification(106, $name."-vols", $name, $before_str,
					$after_str, $scheds);
		}
		else {
			// see if there's an open alert & get its details
			$alert = get_alert('Clients', $name."-vols", 106);
			if ($alert == false) {
				// There's no open alert about vols changed for this client
				return $active;
			}
			// The alert exists
			if (($alert["created"] + 604800) <= time()) {
				// The alert is over 7 days old, close it
				bp_send_notification(107, $name."-vols", $name,
						"the notice period has expired");
				return $active;
			}
			// Get list of profile_paths for the enabled schedules that
			// include this client.
			$q = "SELECT profile_path FROM bp.schedule_profiles JOIN ";
			$q .= "bp.schedules USING(schedule_id) WHERE enabled=true AND ";
			$q .= "instance_id = ";
			$q .= "(SELECT instance_id FROM bp.application_instances ";
			$q .= "WHERE node_no = $id AND app_id = 1)";
			$res = pg_query($dbconn, $q);
			if (pg_num_rows($res) == 0) {
				// No enabled file-level schedules found with this client;
				// they must have been disabled since the alert was raised?
				bp_send_notification(107, $name."-vols", $name,
						"no enabled file-level schedules include this client");
				return $active;
			}
			$profiles = pg_fetch_all_columns($res);
			$nprofs = count($profiles);
			$found_allvols = 0;
			$close_alert = false;
			// Get the Drives setting from each profile and compare it to
			// the volume list.  If they are equal, assume that means
			// someone updated the schedule, and close the alert.
			foreach ($profiles as $fname) {
				$path = $bploc."/profiles.dir/".$fname;
				$cmd = "$bploc/bin/bputil -g Header Drives NotFound $path";
				$drive_str = trim(shell_exec($cmd));
				if ($drive_str == "NotFound") {
					continue;
				}
				if ($drive_str == "Allvolumes:") {
					$found_allvols++;
					continue;
				}
				$drives = explode('|', $drive_str);
				sort($drives);
				if ($drives == $vols_after) {
					$close_alert = true;
					break;
				}
			}
			if ($close_alert == true) {
				bp_send_notification(107, $name."-vols", $name,
						"file-level schedule(s) appear to have been updated");
				return $active;
			}
			if ($found_allvols == $nprofs) {
				bp_send_notification(107, $name."-vols", $name,
						"all file-level schedule(s) include all volumes");
				return $active;
			}
		}
	}
	return $active;
}

# Get the new inactive client list.
function bppa_get_inactive_clients() {
	$clientList = bp_get_client_list();
	$inactiveList = array();
	# save count, otherwise our count will shrink as we pop clients off the array
	foreach ($clientList as $name) {
		if (bppa_client_is_active($name) == false) {
			$inactiveList[] = $name;
		}
	}

	return $inactiveList;
}

# Query the database to get the inactive client list.
function bppa_db_get_inactive_clients($dbconn) {
	$resultset = pg_query($dbconn, "SELECT * FROM bp.inactive_clients");
	$results = pg_fetch_all($resultset);
	$list = array();
	if (!empty($results)) {
		foreach ($results as $row) {
			$list[$row['name']] = $row;
		}
	}
	return $list;
}


# If using fca or beta repo and it is EOL, turn it off 
function eol_off() {
   $command = <<<EOF
#!/bin/sh
repos=`ls /etc/yum.repos.d/*nitrends*.repo 2>/dev/null`
for repo in \$repos
do
   grep -q "^include=" \$repo 
   if [ $? -eq 0 ]; then
     eol=`grep "^include=" \$repo|head -n1|cut -f2 -d'=' |sed -e 's/repo$/eol/'`
   else
     bpath=`grep "^baseurl=" \$repo|head -n1|cut -f2 -d'='`
     eol=\${bpath}/Unitrends.eol
   fi
   wget --spider -v \$eol 2>&1 |grep -q "Remote file exists"
   if [ $? -eq 0 ]; then
         func=fca
         echo \$eol |grep -q beta
         if [ $? -eq 0 ]; then
            func=beta
         fi

         echo \$eol |grep -q ce

 	 if [ $? -eq 0 ]; then
		func=ce
	 fi

	 echo "Turning off \$func for repo \$repo"
         /usr/local/bin/dpu \$func off 
   fi
done
EOF;
   $result = shell_exec($command);
   print "$result \n";
}

#---------------------------------------------------
# 4618
#
# CEP Error Log Functions
#---------------------------------------------------

# Do the CEP alerts.  These come from the bp_get_exchange_info and
# bp_get_workspace_info API's. We only want to show errors and we do not
# want to show an error more than once.
function processCEPAlerts($dbconn) {

	global $started;
	global $ui_type;
	$lastrun = bppa_db_last_run($dbconn);
	# Check when to process these--we don't do it every alertman run
	$processCEPAlerts = false;
	$CEPCheck = get_master_ini_setting('Alertman', 'CEPCheckErrorsPerDay', '1');
	if ($CEPCheck > 0) {
		$countLastRun = count($lastrun);
		if (count($lastrun) > 0) {
			$checkTime = get_master_ini_setting('Alertman', 'ClientConnectCheckBaseTime', '23:00');
			$checkTimestamp = strtotime($checkTime);
			$lastStarted = $lastrun[0]["started"];
			/* How many seconds from when we should have started a CEP check? */
			$CEPCheckInterval = 86400 / $CEPCheck;
			$thisOffsetFromCheck = (($started + 86400 - $checkTimestamp) % 86400) % $CEPCheckInterval;
			$lastOffsetFromCheck = (($lastStarted + 86400 - $checkTimestamp) % 86400) % $CEPCheckInterval;
			if ($thisOffsetFromCheck <= $lastOffsetFromCheck) {
				$processCEPAlerts = true;
			}
		} else {
			$processCEPAlerts = true;
		}

		// 5275
		$alertFound = false;
		if ($processCEPAlerts == true) {
			$CEPWorkSpaces = bp_get_workspace_list();
			foreach ($CEPWorkSpaces as $k => $v) {
				$CEPWorkspaceLog = bp_get_workspace_log($k, "error", 10);
				foreach ($CEPWorkspaceLog as $onewslog) {
					$Host = $onewslog["host"];
					// notify_id 50: CEP workspace %s has errors today.
					if (!alert_exists('CEP', $k, 50)) {
						switch ($ui_type) {
							case 'CE':
								// placeholder for notification
								break;
							case 'Satori':
								// placeholder for notification
								break;
							case 'RRC':
								bp_send_notification(50, $k, $k);
								break;
						}
					}
					// 5275
					$alertFound = true;
				}
			}

			$CEPExchangeLog = bp_get_exchange_log("error", 10);
			foreach ($CEPExchangeLog as $onelog) {
				$Host = $onelog["host"];
				// notify_id 51: Global Exchange log has errors today.
				if (!alert_exists('CEP', $Host, 51)) {
					switch ($ui_type) {
						case 'CE':
							// placeholder for notification
							break;
						case 'Satori':
							// placeholder for notification
							break;
						case 'RRC':
							bp_send_notification(51, $Host);
							break;
					}
				}
				// 5275
				$alertFound = true;
			}
		}
		// Search through old CEP alerts and close any that have been open longer
		// than 1 week.
		// One week in EPOCH time is 604800.
		// One day in EPOCH time is 86400.
                // 5275
		if ($alertFound === true) {
			$oneWeekAgo = $started - 604800;
			bppa_db_close_old_CEP_alerts($dbconn, $oneWeekAgo);
		} else {
			$oneDayAgo = $started - 86400;
			bppa_db_close_old_CEP_alerts($dbconn, $oneDayAgo);
		}
	}
}

#---------------------------------------------------
# 3ware Functions
#---------------------------------------------------

function bppa_get_3ware_controllers() {
	exec("tw_cli info", $output_array, $retval);

	if ($retval != 0) return $retval;

	$controller_list = array();
	for ($i=3; $i < count($output_array) -1; $i++) {
		$cur_controller = preg_split("/[\s]+/", $output_array[$i]);
		array_push($controller_list, ltrim($cur_controller[0], 'c'));
	}

	return $controller_list;
}

function bppa_get_3ware_units($controller) {
	exec("tw_cli info c" . $controller . " unitstatus", $output_array, $retval);

	if ($retval != 0) return $retval;

	$unit_list = array();
	for ($i=3; $i < count($output_array) - 1; $i++) {
		$cur_unit = preg_split("/[\s]+/", $output_array[$i]);
		if (strcmp($cur_unit[0], "u?") != 0) array_push($unit_list, array("unit" => ltrim($cur_unit[0], 'u'), "controller" => $controller, "type" => $cur_unit[1], "status" => $cur_unit[2]));
	}

	return $unit_list;
}

function bppa_get_3ware_model($controller) {
	$model_ver = shell_exec('tw_cli info c' . $controller . ' model');

	return $model_ver;
}

function bppa_get_3ware_driver($controller) {
	$driver_ver = shell_exec('tw_cli info c' . $controller . ' driver');

	return $driver_ver;
}

function bppa_get_3ware_firmware($controller) {
	$firmware_ver = shell_exec('tw_cli info c' . $controller . ' firmware');

	return $firmware_ver;
}

function bppa_get_3ware_drives($controller) {
	exec("tw_cli info c" . $controller . " drivestatus", $output_array, $retval);

	if ($retval != 0) return $retval;

	$port_list = array();
	for ($i=3; $i < count($output_array) - 1; $i++) {
		$cur_port = preg_split("/[\s]+/", $output_array[$i]);
		if (strncmp($cur_port[2], '-', 1) != 0 && strncmp($cur_port[2], 'u?', 2) != 0) {
			array_push($port_list, array("port" => ltrim($cur_port[0], 'p'), "unit" => ltrim($cur_port[2], 'u'), "controller" => $controller, "status" => $cur_port[1]));
		}
	}

	return $port_list;
}

function bppa_db_get_3ware_units($dbconn) {
	$results = pg_query($dbconn, "SELECT * FROM bp.unit_table INNER JOIN bp.disk_status ON bp.unit_table.status_id = bp.disk_status.status_id");

	return pg_fetch_all($results);
}

function bppa_db_get_3ware_drives($dbconn) {
	$results = pg_query($dbconn, "SELECT * FROM bp.drive_table INNER JOIN bp.disk_status ON bp.drive_table.status_id = bp.disk_status.status_id");

	return pg_fetch_all($results);
}

function bppa_db_close_old_CEP_alerts($dbconn, $cutoff) {
	$retval = pg_query($dbconn, "UPDATE bp.alerts SET closed = 't' WHERE created < " . pg_escape_string($cutoff) . " AND source_id = 7");

	return $retval;
}
function bppa_db_update_unit($dbconn, $unit) {
	$retval = pg_query($dbconn, "UPDATE bp.unit_table SET status_id = (SELECT status_id FROM bp.disk_status WHERE status = '" . pg_escape_string($unit["status"]) . "') WHERE controller = '" . pg_escape_string($unit["controller"]) . "' AND unit = '" . pg_escape_string($unit["unit"]) . "'");

	return $retval;
}

function bppa_db_update_drive($dbconn, $drive) {
	$retval = pg_query($dbconn, "UPDATE bp.drive_table SET status_id = (SELECT status_id FROM bp.disk_status WHERE status = '" . pg_escape_string($drive["status"]) . "'), unit = '" . pg_escape_string($drive["unit"]) . "' WHERE controller = '" . pg_escape_string($drive["controller"]) . "' AND port = '" . pg_escape_string($drive["port"]) . "'");

	return $retval;
}

function bppa_db_add_unit($dbconn, $unit) {
	$insert = "INSERT INTO bp.unit_table (controller,unit,type,status_id) VALUES ('" . pg_escape_string($unit["controller"]) . "', '" . pg_escape_string($unit["unit"]) . "', '" . pg_escape_string($unit["type"]) . "', (SELECT status_id FROM bp.disk_status WHERE status = '" . pg_escape_string($unit["status"]) . "'))";

	$retval = pg_query($dbconn, $insert);
	return $retval;
}

function bppa_db_delete_unit($dbconn, $unit) {
	$retval = pg_query($dbconn, "DELETE FROM bp.unit_table WHERE controller = '" . pg_escape_string($unit["controller"]) . "' AND unit = '" . pg_escape_string($unit["unit"]) . "'");
	$retval = pg_query($dbconn, "UPDATE bp.alerts SET updated = " . time() . ", closed = TRUE WHERE target = '" . pg_escape_string("c".$unit["controller"]." u".$unit["unit"]) . "' AND closed = FALSE");
	return $retval;
}

function bppa_db_add_drive($dbconn, $drive) {
	$insert = "INSERT INTO bp.drive_table (controller,unit,port,status_id) VALUES ('" . pg_escape_string($drive["controller"]) . "', '" . pg_escape_string($drive["unit"]) . "', '" . pg_escape_string($drive["port"]) . "', (SELECT status_id FROM bp.disk_status WHERE status = '" . pg_escape_string($drive["status"]) . "'))";

	$retval = pg_query($dbconn, $insert);
	return $retval;
}

function bppa_db_delete_drive($dbconn, $drive) {
	$retval = pg_query($dbconn, "DELETE FROM bp.drive_table WHERE controller = '" . pg_escape_string($drive["controller"]) . "' AND port = '" . pg_escape_string($drive["port"]) . "'");
	$retval = pg_query($dbconn, "UPDATE bp.alerts SET updated = " . time() . ", closed = TRUE WHERE target = '" . pg_escape_string("c".$drive["controller"]." p".$drive["port"]) . "' AND closed = FALSE");
	return $retval;
}

function bppa_db_unit_status($dbconn, $unit) {
	$results = pg_query($dbconn, "SELECT status FROM bp.unit_table INNER JOIN bp.disk_status ON bp.unit_table.status_id = bp.disk_status.status_id WHERE controller = '" . pg_escape_string($unit["controller"]) . "' AND unit = '" . pg_escape_string($unit["unit"]) . "'");

	return pg_fetch_all($results);
}

function bppa_db_drive_status($dbconn, $drive) {
	$results = pg_query($dbconn, "SELECT status FROM bp.drive_table INNER JOIN bp.disk_status ON bp.drive_table.status_id = bp.disk_status.status_id WHERE controller = '" . pg_escape_string($drive["controller"]) . "' AND port = '" . pg_escape_string($drive["port"]) . "'"); 
	return pg_fetch_all($results);
}

function bppa_diff_units($unit_array_1, $unit_array_2, $do_status) {

	$unit_list_1 = array();
	for ($i=0; $i < count($unit_array_1); $i++) {
		if ($do_status === TRUE) {
			array_push($unit_list_1, $unit_array_1[$i]["controller"] . " " . $unit_array_1[$i]["unit"] . " " . $unit_array_1[$i]["status"]);
		} else {
		        array_push($unit_list_1, $unit_array_1[$i]["controller"] . " " . $unit_array_1[$i]["unit"]);
		}
	}

	$unit_list_2 = array();
	for ($i=0; $i < count($unit_array_2); $i++) {
		if ($do_status === TRUE) {
		        array_push($unit_list_2, $unit_array_2[$i]["controller"] . " " . $unit_array_2[$i]["unit"] . " " . $unit_array_2[$i]["status"]);
		} else {
			array_push($unit_list_2, $unit_array_2[$i]["controller"] . " " . $unit_array_2[$i]["unit"]);
		}
	}

	$unit_diff = array_diff($unit_list_1, $unit_list_2);
	if (empty($unit_diff) === TRUE) return array();

	$unit_diff_keys = array_keys($unit_diff);
	$unit_diff_array = array();

	for ($i=0; $i < count($unit_diff_keys); $i++) {
		$unit_diff_array = $unit_diff_array + array($unit_diff_keys[$i] => $unit_array_1[$unit_diff_keys[$i]]);
	}

	return $unit_diff_array;
}

function bppa_diff_drives($drive_array_1, $drive_array_2, $do_status) {
	$drive_list_1 = array();
	for ($i=0; $i < count($drive_array_1); $i++) {
		if ($do_status === TRUE) {
			array_push($drive_list_1, $drive_array_1[$i]["controller"] . " " . $drive_array_1[$i]["port"] . " " . $drive_array_1[$i]["status"]);
		} else {
			array_push($drive_list_1, $drive_array_1[$i]["controller"] . " " . $drive_array_1[$i]["port"]);
		}
	}

	$drive_list_2 = array();
	for ($i=0; $i < count($drive_array_2); $i++) {
		if ($do_status === TRUE) {
			array_push($drive_list_2, $drive_array_2[$i]["controller"] . " " . $drive_array_2[$i]["port"] . " " . $drive_array_2[$i]["status"]);
		} else {
			array_push($drive_list_2, $drive_array_2[$i]["controller"] . " " . $drive_array_2[$i]["port"]);
		}
	}

	$drive_diff = array_diff($drive_list_1, $drive_list_2);
	if (empty($drive_diff) === TRUE) return array();

	$drive_diff_keys = array_keys($drive_diff);
	$drive_diff_array = array();

	for ($i=0; $i < count($drive_diff_keys); $i++) {
		$drive_diff_array = $drive_diff_array + array($drive_diff_keys[$i] => $drive_array_1[$drive_diff_keys[$i]]);
	}

	return $drive_diff_array;
}

#---------------------------------------------------
# MegaRaiD Functions
#---------------------------------------------------
function bppa_get_device_drive_counts() {
	$id_list = array();
	$pos_list = array();
	$enc_devices = array();

	exec("MegaCli64 -pdlist -aall | grep 'Enclosure position:'", $position_array, $retval);
	if ($retval != 0) return $retval;

	exec("MegaCli64 -pdlist -aall | grep 'Enclosure Device ID'", $id_array, $retval);
	if ($retval != 0) return $retval;

	foreach ($position_array as $pos) {
		$pos_num = preg_split("/[:]+/", $pos);
		array_push($pos_list, $pos_num[1]);
	}

	foreach ($id_array as $id) {
		$id_num = preg_split("/[:]+/", $id);
		array_push($id_list, $id_num[1]);
	}

	$dev_disk_num = array_values(array_count_values($id_list));
	$pos_list = array_values(array_unique($pos_list));
	$id_list = array_values(array_unique($id_list));

	for ($i=0; $i<3; $i++) {
		array_push($enc_devices, array("enc_id" => $id_list[$i], "enc_pos" => $pos_list[$i], "drive_count" => $dev_disk_num[$i]));
	}

	return $enc_devices;

}

function bppa_get_status() {
	$unit_list = array();

	for ($i=0; $i<2; $i++) {
		exec("MegaCli64 -ldinfo -l$i -aall | grep 'Virtual Drive:\|State'", $output_array, $retval);
		if ($retval != 0) return $retval;

		$vd_num = preg_split("/[\s]+/", $output_array[0]);
		$vd_state = preg_split("/[:]+/", $output_array[1]);
		array_push($unit_list, array("virt_num" => $vd_num[2], "state" => $vd_state[1]));

		unset($output_array);
	}

	return $unit_list;
}

function bppa_get_drives($dev_id, $position, $drive_count) {
	$drive_list = array();	
	$db_drive_list = array();	
	$slots = array();

 	exec("MegaCli64 -pdlist -aall | grep 'Enclosure position'", $enc_array, $retval);
        if ($retval != 0) return $retval;

        exec("MegaCli64 -pdlist -aall | grep 'Slot Number:'", $slot_array, $retval);
        if ($retval != 0) return $retval;

        for ($i=0; $i<count($enc_array); $i++) {
	        $enc_pos = preg_split("/[:]+/", $enc_array[$i]);
                $slot_num = preg_split("/[:]+/", $slot_array[$i]);
                array_push($db_drive_list, array("enc_pos" => $enc_pos[1], "slot_num" => $slot_num[1]));
        }

        foreach ($db_drive_list as $drives) {
        	if ($drives['enc_pos'] == $position) {
                	array_push($slots, array("slot_num" => $drives['slot_num']));
                }
        }

	for ($i=0; $i<$drive_count; $i++) {
		$slot = $slots[$i]['slot_num'];

		if ($position == ' N/A') {
			exec("MegaCli64 -pdinfo -physdrv [:$slot] -aall | grep 'Slot Number:\|Enclosure position\|Firmware state:\|Inquiry Data:'", $output_array, $retval);
		} else {
			exec("MegaCli64 -pdinfo -physdrv [$dev_id:$slot] -aall | grep 'Slot Number:\|Enclosure position\|Firmware state:\|Inquiry Data:'", $output_array, $retval);
		}
		if ($retval != 0) return $retval;

		$slot_num = preg_split("/[:]+/", $output_array[0]);
		$enc_pos = preg_split("/[:]+/", $output_array[1]);
		$state = preg_split("/[:]+/", $output_array[2]);
		$serial = preg_split("/[\s]+/", $output_array[3]);
		array_push($drive_list, array("enc_pos" => $enc_pos[1], "slot_num" => $slot_num[1], "state" => $state[1], "serial" => $serial[2]));
		unset($output_array);
	}

	return $drive_list;
}

#-------------------
# mdraid functions
#-------------------

function bppa_diff_raid_units($unit_array_1, $unit_array_2, $do_status) {
	# return drives in unit_array_1 that are not in unit_array_2
	$debug = 0;

	$unit_list_1 = array();
	for ($i=0; $i < count($unit_array_1); $i++) {
		if ($do_status === TRUE) {
			array_push($unit_list_1, $unit_array_1[$i]["controller"] . " " . $unit_array_1[$i]["unit"] . " " . $unit_array_1[$i]["status"]);
		} else {
			array_push($unit_list_1, $unit_array_1[$i]["controller"] . " " . $unit_array_1[$i]["unit"]);
		}
	}

	$unit_list_2 = array();
	for ($i=0; $i < count($unit_array_2); $i++) {
		if ($do_status === TRUE) {
			array_push($unit_list_2, $unit_array_1[$i]["controller"] . " " . $unit_array_2[$i]["unit"] . " " . $unit_array_2[$i]["status"]);
		} else {
			array_push($unit_list_2, $unit_array_2[$i]["controller"] . " " . $unit_array_2[$i]["unit"]);
		}
	}

	$unit_diff = array_diff($unit_list_1, $unit_list_2);
	if (empty($unit_diff) === TRUE) return array();

	$unit_diff_keys = array_keys($unit_diff);
	$unit_diff_array = array();

	for ($i=0; $i < count($unit_diff_keys); $i++) {
		$unit_diff_array = $unit_diff_array + array($unit_diff_keys[$i] => $unit_array_1[$unit_diff_keys[$i]]);
	}

	if ($debug) {
		print "bppa_diff_raid_units returning:\n";
		var_dump($unit_diff_array);
	}

	return $unit_diff_array;
}

function bppa_diff_mdraid_drives($drive_array_1, $drive_array_2, $do_status) {
	# return drives in $drive_array_1 that are not in $drive_array_2
	$debug = 0;

	if ($debug) {
		print "bppa_diff_mdraid_drives called with:\n";
		var_dump($drive_array_1);
	
		print "array_2:\n";
		var_dump($drive_array_2);
	}

	$drive_list_1 = array();
	for ($i=0; $i < count($drive_array_1); $i++) {
		if ($do_status === TRUE) {
			array_push($drive_list_1, $drive_array_1[$i]["controller"] . " " . $drive_array_1[$i]["status"] . " " . $drive_array_1[$i]["partition"] . " " . $drive_array_1[$i]["serial"]);
		} else {
			array_push($drive_list_1, $drive_array_1[$i]["controller"] . " " . $drive_array_1[$i]["partition"] . " " . $drive_array_1[$i]["serial"]);
		}
	}

	$drive_list_2 = array();
	for ($i=0; $i < count($drive_array_2); $i++) {
		if ($do_status === TRUE) {
			array_push($drive_list_2, $drive_array_2[$i]["controller"] . " " . $drive_array_2[$i]["status"] . " " . $drive_array_2[$i]["partition"] . " " . $drive_array_2[$i]["serial"]);
		} else {
			array_push($drive_list_2, $drive_array_2[$i]["controller"] . " " . $drive_array_2[$i]["partition"] . " " . $drive_array_2[$i]["serial"]);
		}
	}

	if ($debug) {
		print "drive_list_1:\n";
		var_dump($drive_list_1);
		print "drive_list_2:\n";
		var_dump($drive_list_2);
	}

	$drive_diff = array_diff($drive_list_1, $drive_list_2);
	if (empty($drive_diff) === TRUE) {
		if ($debug) {
			print "diff is empty\n";
		}
		return array();
	}

	$drive_diff_keys = array_keys($drive_diff);
	$drive_diff_array = array();

	for ($i=0; $i < count($drive_diff_keys); $i++) {
		$drive_diff_array = $drive_diff_array + array($drive_diff_keys[$i] => $drive_array_1[$drive_diff_keys[$i]]);
	}

	if ($debug) {
		print "diff:\n";
		var_dump($drive_diff_array);
	}

	return $drive_diff_array;
}

function bppa_db_get_mdraid_units($dbconn) {
	$results = pg_query($dbconn, "SELECT * FROM bp.raid_units");
	return pg_fetch_all($results);
}

function bppa_db_get_mdraid_drives($dbconn) {
	$results = pg_query($dbconn, "SELECT * from bp.raid_drives");

	$results = pg_fetch_all($results);

	return $results === FALSE ? array() : $results;
}

function bppa_db_add_raid_unit($dbconn, $unit) {
	$debug = 0;
	if ($debug) {
		print "bp_db_add_raid_unit called with\n";
		var_dump($unit);
	}

	$insert = "INSERT INTO bp.raid_units (controller,unit,type,status,uuid) VALUES ('" .
			pg_escape_string($unit["controller"]) . "', '" .
			pg_escape_string($unit["unit"]) . "', '" .
			pg_escape_string($unit["type"]) .
			"', (SELECT status_id FROM bp.disk_status WHERE status = '" .
				pg_escape_string($unit["status"]) . "') , '" .
			pg_escape_string($unit["uuid"]) .
			"')";

	$retval = pg_query($dbconn, $insert);
	return $retval;
}

function bppa_db_delete_raid_unit($dbconn, $unit) {
	$retval = pg_query($dbconn, "DELETE FROM bp.raid_units WHERE controller = '" . pg_escape_string($unit["controller"]) . "' AND unit = '" . pg_escape_string($unit["unit"]) . "'");
	$retval = pg_query($dbconn, "UPDATE bp.alerts SET updated = " . time() . ", closed = TRUE WHERE target = '" . pg_escape_string("c".$unit["controller"]." u".$unit["unit"]) . "' AND closed = FALSE");
	return $retval;
}

function bppa_db_add_raid_drive($dbconn, $drive) {
	# add a drive to the database

	$debug = 0;

	if ($debug) {
		print "bppa_db_add_raid_drive called with:\n";
		var_dump($drive);
	}
	$insert = "INSERT INTO bp.raid_drives (controller,device,status,serial,partition) VALUES ('" .
		pg_escape_string($drive["controller"])  . "', '" .
		pg_escape_string($drive["device"])      . "', " .
		(int) $drive["status"]                  . ", '" .
		pg_escape_string($drive["serial"])      . "', '" .
		(int) $drive["partition"]               . "')" ;

	$retval = pg_query($dbconn, $insert);

	return $retval;
}

function bppa_db_delete_raid_drive($dbconn, $drive) {
	# remove drive from database and open a fatal alert for it
	$debug = 0;

	$target = $drive['device'];
	$severity = S_FATAL;
	$closed = false;

	if ($debug) {
		print "bppa_db_delete_raid_drive called for " . $drive['device'] . "\n";
		print "target: $target\n";
	}

	system("grep -qvsE 'md_drives_internal' /var/opt/unitrends/platform_capabilities", $md_drives_internal);
	if ($md_drives_internal) {
		$md_internal_message = "(Internal Drive)";
	} else {
		$md_internal_message = "";
	}

	$serialnumber = $drive['serial'] . $md_internal_message;
	$drive['portno'] = bppa_get_drive_portno($drive['device']);

	if (isset($drive['portno']) && ($drive['portno'] != NULL)) {
		$suppltext = " (Port: " . $drive['portno'] . ", Serial: $serialnumber)";
	} else {
		$suppltext = " (Serial: $serialnumber)";
	}

	// notify_id 41: "RAID drive status changed to offline. %s"
	if (bp_send_notification(41, $target, $suppltext) === false) {
		print "bp_send_notification for bppa_db_delete_raid_drive failed.\n";
	}

	$delete = "DELETE FROM bp.raid_drives WHERE controller = '" .
		pg_escape_string($drive["controller"]) .
		"' AND device = '" .
		pg_escape_string($drive["device"]) .
		"';";

	$retval = pg_query($dbconn, $delete);

	return $retval;
}

function bppa_db_raid_unit_status($dbconn, $unit) {
	# returns text-style status

	$query = "SELECT status from bp.disk_status WHERE status_id = (SELECT status FROM bp.raid_units WHERE unit = '" . $unit . "');";

	$result = pg_query($dbconn, $query);

	$result = pg_fetch_all($result);

	if ($result == NULL or $result === FALSE) {
		return '';
	} else {
		return $result[0]['status'];
	}
}

function bppa_db_update_raid_unit($dbconn, $unit) {
	# takes text-style status and sets numeric status in database
	if (0) {
		print "bppa_db_update_raid_unit:\n";
		var_dump($unit);
	}

	$retval = pg_query($dbconn, "UPDATE bp.raid_units SET status = (SELECT status_id FROM bp.disk_status WHERE status = '" . pg_escape_string($unit['status']) . "') WHERE controller = '" . pg_escape_string($unit['controller']) . "' AND unit = '" . pg_escape_string($unit['unit']) . "'");

	return $retval;
}

function bppa_db_update_raid_drive($dbconn, $drive) {
	# takes text-style status and sets numeric status in database

	$debug = 0;

	if ($debug) {
		print "bppa_db_update_raid_drive called with:\n";
		var_dump($drive);
	}

	$retval = pg_query($dbconn, "UPDATE bp.raid_drives SET status = " .
	"'" . pg_escape_string($drive['status']) .
	"' WHERE controller = '" .  pg_escape_string($drive['controller']) .
	"' AND device = '" .  pg_escape_string($drive['device']) . "'");

	return $retval;
}

function bppa_db_raid_drive_status($dbconn, $drive) {
	$debug = 0;

	# takes text-style status and sets numeric status in database
	$query = "SELECT status FROM bp.disk_status WHERE status_id = (SELECT status FROM bp.raid_drives WHERE device = '" . pg_escape_string($drive['device']) . "')";

	$result = pg_query($dbconn, $query);

	$result = pg_fetch_all($result);

	if ($debug) {
		print "bppa_db_raid_drive_status\n";
		var_dump($result);
		var_dump($result[0]);
	}

	if ($result === FALSE or $result == NULL) {
		return '';
	} else {
		return $result[0]['status'];
	}
}

function bppa_get_type_for_unit($unit) {
	$command = <<<EOF
	mdadm --detail $unit | grep "Raid Level" | sed "s/^[ \t]*Raid Level : //"
EOF;
	exec($command, $output_array, $retval);

	return $output_array[0];
}

function bppa_get_uuid_for_unit($unit) {
	$command = <<<EOF
	mdadm --detail $unit | grep UUID | sed "s/^[ \t]*UUID : //"
EOF;
	exec($command, $output_array, $retval);

	return $output_array[0];
}

function bppa_get_mdraid_units($dbconn) {
	$debug = 0;

	exec("mdadm --detail --scan 2>/dev/null | grep ARRAY", $output_array, $retval);

	if ($retval != 0) return array();

	$unit_list = array();

	for ($i=0; $i < count($output_array); $i++) {
		$cur_unit = preg_split("/[\s]+/", $output_array[$i]);
		array_push($unit_list, $cur_unit[1]);
	}

	$units_array = array();

	for ($i=0; $i < count($unit_list); $i++) {
		$cur_unit = $unit_list[$i];
		$cur_controller = "mdraid";
		$cur_type = bppa_get_type_for_unit($cur_unit);
		$cur_uuid = bppa_get_uuid_for_unit($cur_unit);
		$cur_status = bppa_get_mdraid_unit_status($cur_unit);

		$cur_array['unit'] = $cur_unit;
		$cur_array['controller'] = $cur_controller;
		$cur_array['type'] = $cur_type;
		$cur_array['uuid'] = $cur_uuid;
		$cur_array['status'] = $cur_status;

		array_push($units_array, $cur_array);
	}

	sort($units_array);

	if ($debug) {
		print "get_mdraid_units var_dump:\n";
		var_dump($units_array);
	}

	return $units_array;
}

function bppa_get_mdraid_drive_list($dbconn) {
	# parameter: database connection
	# return a list of drives/partitions in mdraid arrays

	# find all block devices attached to MD arrays
	# indenting the closing heredoc marker seems to break vim syntax highlighting
	$command = <<<EOF
	find /sys/block/md*/slaves/ -type l 2>/dev/null | grep sd | cut -d'/' -f 6 | awk '{ print "/dev/"$1 }'
EOF;

	exec($command, $output_array, $retval);
	if ($retval != 0) return $retval;

	$drive_list = array();

	for ($i=0; $i < count($output_array); $i++) {
		$cur_unit = preg_split("/[\s]+/", $output_array[$i]);
		# throw out those which are broken symlinks
		# (device is part of an MD array, but not present)
		# this should handle components that show up as sdd2[2](F) in /proc/mdstat
		if (file_exists($cur_unit[0])) {
			array_push($drive_list, $cur_unit[0]);
		}
	}

	sort($drive_list);

	#print "bppa_get_mdraid_drive_list:\n";
	#var_dump($drive_list);

	return $drive_list;
}

function bppa_get_sys_path_for_drive($drive) {
	# input: /dev/sda5
	# output: /sys/block/md3/md/dev-sda5

	#print "bppa_get_sys_path_for_drive called for $drive\n";

	$t = substr($drive, 1) . "\n";
	$drive = preg_replace("/\//", "-", $t);

	$command = "find /sys/block/md* -type d -name $drive";
	exec($command, $output_array, $retval);

	if ($retval != 0) {
		return False;
	}

	return $output_array[0];
}

function bppa_status_id_lookup($dbconn, $status) {
	$results = pg_query($dbconn, "SELECT status_id FROM bp.disk_status WHERE status = '$status'");

	if ($results === false or pg_num_rows($results) == 0) {
		return false;
	}

	$arr = pg_fetch_assoc($results, 0);

	return $arr['status_id'];
}

function bppa_status_lookup($dbconn, $status_id) {
	$results = pg_query($dbconn, "SELECT status FROM bp.disk_status WHERE status_id = '$status_id'");

	if ($results === false or pg_num_rows($results) == 0) {
		return false;
	}

	$arr = pg_fetch_assoc($results, 0);

	return $arr['status'];
}

function bppa_get_mdraid_drive_status($dbconn, $drive) {
	# Returns entry from disk_status table of
	# OK, INOPERABLE, or NOT-PRESENT

	# OK = drive is in an array
	# INOPERABLE = drive is present but not in an array
	# NOT-PRESENT = drive is missing

	$debug = 0;

	$sys_path = bppa_get_sys_path_for_drive($drive);
	if ($debug) print "sys_path: " . $sys_path . "\n";

	if ($sys_path == False) {
		$status = 'NOT-PRESENT';
	} else {
		# read state from $sys_path/state

		$state = trim(file_get_contents("$sys_path/state"));
		if ($debug) print $state;

		switch ($state) {
			case 'in_sync':
				$status = 'OK';
				break;
			case 'faulty':
				$status = 'INOPERABLE';
				break;
			case 'spare':
				$status = 'REBUILDING';
				break;
			default:
				$status = 'NOT-PRESENT';
				break;
		}
	}

	if ($debug) print "status: $status\n";

	return $status;
}

function bppa_get_mdraid_unit_status($unit) {
	# input: unit is /dev/md<x>
	# returns OK, DEGRADED, INOPERABLE, REBUILDING, or VERIFYING

	# in /sys/block/md<x>/md/ ...
	#  checked in this order
	#  not present = INOPERABLE
	#  sync_action = resync / recover = REBUILDING
	#  sync_action = check / repair = VERIFYING
	#  degraded = 1 = DEGRADED
	#  array_state = clean / active / active-idle = OK
	#  else DEGRADED

	$splitname = split("/", $unit);
	$shortname = $splitname[count($splitname)-1];
	
	#print "shortname: $shortname\n";

	$mdpath = '/sys/block/' . $shortname . '/md';
	#print "mdpath: $mdpath\n";

	if (!file_exists($mdpath)) {
		return 'INOPERABLE';
	}

	$sync_action_path = $mdpath . '/sync_action';
	$degraded_path = $mdpath . '/degraded';

	if (!file_exists($sync_action_path)) {
		# doesn't exist for RAID-0 arrays
		$sync_action = 'N/A';
	} else {
		$sync_action = trim(file_get_contents($sync_action_path));
	}

	if (!file_exists($degraded_path)) {
		# doesn't exist for RAID-0 arrays
		$degraded = 0;
	} else {
		$degraded = trim(file_get_contents($degraded_path));
	}

	# array_state should always exist
	$array_state = trim(file_get_contents($mdpath . '/array_state'));

	if ($sync_action == 'resync' or $sync_action == 'recover') {
		return 'REBUILDING';
	}

	if ($sync_action == 'check' or $sync_action == 'repair') {
		return 'VERIFYING';
	}

	# Need to look with mdadm, since /sys/block/mdx/md/degraded does not exist
	#  on 2.6.18 kernels.
	$command = <<<EOF
	mdadm --detail $unit | grep "State : " | grep -q degraded
EOF;
	exec($command, $output_array, $retval);
	if ($retval == 0) {
		#echo "$unit is degraded (second test)\n";
		$degraded = 1; 
	}

	if ($degraded == 1) {
		return 'DEGRADED';
	}

	if ($array_state == 'clean' or $array_state == 'active' or $array_state =='active-idle') {
		return 'OK';
	}

	# we don't know, so claim INOPERABLE (worst-case)
	return 'INOPERABLE';
}

function bppa_get_drive_serial($drive) {
	#print "bppa_get_drive_serial called with $drive\n";
	$command = <<<EOF
	hdparm -i $drive | grep -o -E "SerialNo.+" | sed s/SerialNo=//g | sed "s/^[ \t]*//g"
EOF;
	exec($command, $output_array, $retval);
	if ($retval != 0) return $retval;

	return $output_array[0];
}

function bppa_get_drive_portno($drive) {
	$debug = 0;

	if ($debug) {
		print "bppa_get_drive_portno called with $drive\n";
	}

	$shortname = preg_split('/\//', $drive);
	$shortname = $shortname[2];
	$shortname = preg_split('/[0-9]/', $shortname);
	$shortname = $shortname[0];
	
	$command = <<<EOF
	readlink -f /sys/block/$shortname/device | grep -E host[0-9] -o | grep -o [0-9]
EOF;
	exec($command, $output_array, $retval);
	
	$portno = $output_array[0];

	if ($debug) {
		print "bppa_get_drive_portno returning $portno\n";
	}

	return $portno;
}

function bppa_get_mdraid_drives($dbconn) {
	# returns controller, device, status (numeric), and serial for all drives
	# If we're on a Recovery-610/710 or 300, return 'portno' field with the port number of the drive.

	$debug = 0;

	$drive_list = bppa_get_mdraid_drive_list($dbconn);
	if ($debug) {
		print "bppa_get_mdraid_drives list:\n";
		var_dump($drive_list);
	}

	$drives = array();
	
	system("grep -qvsE 'Recovery-610|Recovery-300|Recovery-710|Recovery-720|Recovery-730' /etc/unitrends-release", $know_port_num);

	foreach ($drive_list as $drive) {
		$d = array();
		$d['controller'] = 'mdraid';
		$d['device'] = $drive;
		pg_trace('/tmp/trace.log', 'w', $dbconn);
		$d['status'] = bppa_status_id_lookup($dbconn, bppa_get_mdraid_drive_status($dbconn, $drive));
		pg_untrace($dbconn);
		$d['serial'] = bppa_get_drive_serial($drive);
		if ($know_port_num) {
			$d['portno'] = bppa_get_drive_portno($drive);
		}
		
		if (preg_match("/(\/dev\/sd[a-z])([0-9]+)/", $d['device'], $device_parts) == 1) {
			$d['partition'] = $device_parts[2];
		} else {
			$d['partition'] = 0;
		}

		array_push($drives, $d);
	}

	if ($debug) {
		print "bppa_get_mdraid_drives returning:\n";
		var_dump($drives);
	}

	return $drives;
}

#--------------------------
# DPU Utility Functions
#--------------------------

function bppa_get_appliance_id() {
	$rc_string = false;
	$vc_string = false;

	// Parse license type for DEMO licenses
	system('/usr/bp/bin/lmmgr -info /usr/bp/bin/dispatch 2>/dev/null | grep DEMO 2>/dev/null 1>&2', $type_ret);

	if ($type_ret != 0) {
		// Parse the license Feature string
		$feature_output = shell_exec('/usr/bp/bin/lmmgr -info /usr/bp/bin/dispatch 2>/dev/null | grep Feature');
		$feature_array = explode(",", $feature_output, 10);

		foreach($feature_array as $feature_string) {
			// Matches a non-zero number
			if (preg_match("/RC=[1-9][0-9]*[0-9]*[0-9]*/i", $feature_string)) {
				$rc_string = true;
			}
			if (preg_match("/VC=[1-9][0-9]*[0-9]*[0-9]*/i", $feature_string)) {
				$vc_string = true;
			}
		}

		if ($rc_string == true && $vc_string == false) {
			return "DPU";
		} else if ($rc_string == false && $vc_string == true) {
			return "DPV";
		} else if ($rc_string == true && $vc_string == true) { 
			/* As of 5.0.2 all licenses will have non-zero values
			 * for RC and VC (licensed as cross-vault). 
			 * Have to check nvp table to find appliance ID.
			*/
			$nvp_list = bp_get_nvp_list('RRC', 'configuration');

			if ($nvp_list != false) {
				/* If Identity field is set, then appliance ID
  				 * is equal to that field.  If not set, then
				 * this is a licensed cross-vault pre-5.0.2
				*/
				if (isset($nvp_list['Identity'])) {
					return $nvp_list['Identity'];
				} else {
					return "DPXV";
				}
			} else {
				return;
			}
		}
	} else {
		return "DEMO";
	}
}

#----------------------------------------
# Alert processing for Virtual Clients
#----------------------------------------
# Processed with same frequency as client alerts (clientCheckInterval)
# times per day, called from processClientAlerts().
#
function processVirtualClientAlerts($dbconn) {
	global $started;

	$ret = array();
	exec("grep '^Unitrends' /etc/unitrends-release | awk '{print $3; print $2}'", $ret);

	if (empty($ret)) {
		$model = "this appliance";
	}
	else {
 		$model = "a ";
		if ($ret[0] === "-") {
			$model .= $ret[1];
		}
		else {
			$model .= $ret[0];
		}
	}

	$two_weeks = $started - (14 * 86400);
	$query = "SELECT CASE WHEN s.system_id IS NULL THEN n.node_name ";
	$query .= "ELSE n.gcname || ' (source ' || s.name || ')' END AS node_name ";
	$query .= "FROM bp.virtual_clients JOIN bp.nodes n USING(node_no) ";
	$query .= "LEFT OUTER JOIN bp.systems s USING(system_id) ";
	$query .= "WHERE live_time > 0 AND live_time < $two_weeks ";
	$query .= "AND vm_guid IS NULL";
	$res = pg_query($dbconn, $query);
	if ($res == false) {
		return;
	}

	# These are closed when the virtual client is deleted.
	while ($row = pg_fetch_row($res)) {
		bp_send_notification(112, $row[0] . "-virtual", $row[0], $model);
	}
}

#--------------------------------------------
# Alert processing for Virtual Failover Area
#--------------------------------------------
#
function processVirtualFailoverAlerts($dbconn) {
	# Get the failover space currently in use
	$query = "SELECT sum(vf_size) FROM bp.virtual_reservation";
	$res = pg_query($dbconn, $query);
	if ($res == false) {
		return;
	}
	$res = pg_fetch_result($res, 0, 0);

	# Get the total failover space available
	$vf_array = bp_get_virtual_failover();
	if($vf_array == false)
	{
		return;
	}
	$total_in_mb = $vf_array["allowed"] * 1024;
	if($total_in_mb == 0)
	{
		return;
	}

	$in_use_percent = (int)(($res * 100) / $total_in_mb);

	# Raise alert when in use percentage is above threshold from ini
	$threshold = get_master_ini_setting('Alertman', 'VFDiskSpaceAlertPercentageUsed', 95); #get from master ini
	if($in_use_percent >= $threshold)
	{
		#raise alert
		bp_send_notification(117, 'VF', $threshold, $in_use_percent);
	}
	else
	{
		#close alert
		bp_send_notification(11, 'VF');
	}
}

#----------------------------------------
# Alert processing for Virtual Clients
#----------------------------------------
# Processed with same frequency as client alerts (clientCheckInterval)
# times per day, called from processClientAlerts().
#
function processScheduleAlerts($dbconn)
{
	$query = "SELECT name, enabled::int FROM bp.schedules ";
	$query .= "WHERE run_only=false AND schedule_id NOT IN ";
	$query .= "(SELECT DISTINCT schedule_id FROM bp.schedule_client_assoc)";

	$res = pg_query($dbconn, $query);
	if ($res == false) {
		# Error
		return;
	}

	$nrows = pg_num_rows($res);

	if ($nrows < 0) {
		# Error
		return;
	}

	if ($nrows > 0) {
		# Empty schedule(s) found, send notification
 		$scheds = "";
		for ($i = 0; $i < $nrows; $i++) {
			$row = pg_fetch_assoc($res, $i);
			if ($i > 0) {
				if ($i == ($nrows - 1)) {
					$scheds .= ", and ";
				}
				else {
					$scheds .= ", ";
				}
			}
			$scheds .= $row["name"];
			if ($row["enabled"] == false) {
				$scheds .= " (disabled)";
			}
		}
		$scheds .= ".";
		bp_send_notification(167, "EmptySchedule", $scheds);
		return;
	}
	else {
		# No rows returned, close alert if it exists (notification ID 8:
		# resolve/close alerts for Clients source + target=EmptySchedule)
		bp_send_notification(8, "EmptySchedule");
		return;
	}
	return;
}

#----------------------------------------
# Alert processing for each category
#----------------------------------------
# Do the client alerts.  These are checked for (clientCheckInterval) times
# per day for active clients.  Inactive clients are scanned each alert run
# until they are active.
function processClientAlerts($dbconn) {
	global $started;

# Previously, this function did a check on remote managed systems.
# That functionality has been moved into sync_summary.php.

	# Check when to process these--we don't do it every alertman run
	$processClientAlerts = false;
	$lastrun = bppa_db_last_run($dbconn);
	$clientChecks = get_master_ini_setting('Alertman', 'ClientConnectChecksPerDay', '1');
	if ($clientChecks > 0) {
		/* if we've run before, only check if we're supposed to, otherwise (1st alert run) we check */
		if (count($lastrun) > 0) {
			$checkTime = get_master_ini_setting('Alertman', 'ClientConnectCheckBaseTime', '23:00');
			$checkTimestamp = strtotime($checkTime);
			$lastStarted = $lastrun[0]["started"];
			/* How many seconds from when we should have started a client check? */
			$clientCheckInterval = 86400 / $clientChecks;
			$thisOffsetFromCheck = (($started + 86400 - $checkTimestamp) % 86400) % $clientCheckInterval;
			$lastOffsetFromCheck = (($lastStarted + 86400 - $checkTimestamp) % 86400) % $clientCheckInterval;
			if ($thisOffsetFromCheck <= $lastOffsetFromCheck) {
				$processClientAlerts = true;
			}
		} else {
			$processClientAlerts = true;
		}
	}

	$inactive_client_table = bppa_db_get_inactive_clients($dbconn);
	$previously_inactive_clients = array_keys($inactive_client_table);
	if (empty($previously_inactive_clients)) {
		$previously_inactive_clients = array();
	}

	# always check inactive clients for reconnection
	foreach ($previously_inactive_clients as $client) {
		$client_down = $inactive_client_table[$client]['time_stamp'];
		$seconds_down = time() - $client_down;
		$minutes_down = floor($seconds_down / 60);
		$hours_down = floor($minutes_down / 60);
		$days_down = floor($hours_down / 24);
		$rem_hours_down = $hours_down % 24;
		$rem_minutes_down = $minutes_down % 60;

		if ($days_down > 0) {
			$str = "{$days_down}d {$rem_hours_down}h {$rem_minutes_down}m";
		} else {
			$str = "{$hours_down}h {$rem_minutes_down}m";
		}

		if (bppa_client_is_active($client) == true) {
			# close previous alert, create new notice that client is up
			$retval = pg_query($dbconn, "DELETE FROM bp.inactive_clients WHERE name = '" . pg_escape_string($client) . "'");

			# notify_id 8: Resolve/close alerts for Clients source + target
			bp_send_notification(8, $client);

			# notify_id 10: "%s: client is now listening to connections. %s"
			bp_send_notification(10, $client, $client, "(Down: $str)");
		} else {
			# update the timestamp
			# notify_id 9: "%s: client is no longer listening to connections. %s"
			bp_send_notification(9, $client,$client, "(Down: $str)");
		}
	}

	if ($processClientAlerts == true) {
		# this list contains all clients that were previously active, so only check them for disconnection
		$clients_to_check = array_diff(bp_get_client_list(), $previously_inactive_clients);
		foreach ($clients_to_check as $client) {
			if (bppa_client_is_enabled($client) == true && bppa_client_is_active($client) == false) {
				$inactive_insert = bppa_db_build_insert('bp.inactive_clients', array("name" => $client, "time_stamp" => time()));
				$retval = pg_query($dbconn, $inactive_insert);
				# notify_id 9: "%s: client is no longer listening to connections. %s"
				# Bug 8009: garbage in message; add empty string arg for format.
				bp_send_notification(9, $client, $client, "");
			}
		}
		# handle virtual client alerts with same frequency as for clients
		processVirtualClientAlerts($dbconn);
		processScheduleAlerts($dbconn);
	}
}

function processPCIAlerts($dbconn) {

	$oldpcitable = bppa_db_get_pci($dbconn);
	$newpcitable = bppa_get_pci();

	$oldpcilist = array();
	foreach ($oldpcitable as $pci) {
		$oldpcilist[] = "{$pci['bus_id']} {$pci['vendor_id']} {$pci['device_id']}";
	}

	$newpcilist = array();
	foreach ($newpcitable as $pci) {
		$newpcilist[] = "{$pci['bus_id']} {$pci['vendor_id']} {$pci['device_id']}";
	}

	$disconnected_list = array_diff($oldpcilist, $newpcilist);
	$newconnected_list = array_diff($newpcilist, $oldpcilist);

	foreach ($disconnected_list as $pci) {
		$device = explode(" ", $pci, 2);
		$bus_id = $device[0];

		foreach ($oldpcitable as $oldpci) {
			if ($oldpci['bus_id'] == $bus_id) {
				# notify_id 17: "PCI device has been disconnected: %s"
				bp_send_notification(17, $bus_id, $oldpci['device_name']);
				break;
			}
		}
	}

	foreach ($newconnected_list as $pci) {
		$device = explode(" ", $pci, 2);
		$bus_id = $device[0];

		foreach ($newpcitable as $newpci) {
			if ($newpci['bus_id'] == $bus_id) {
				# notify_id 18: "PCI device has been installed: %s"
				bp_send_notification(18, $bus_id, $newpci['device_name']);
				break;
			}
		}
	}

	if (count($newconnected_list) > 0 || count($disconnected_list) > 0) {
		$retval = bppa_db_set_pci($dbconn, bppa_get_pci());
	}
}

function process3wareAlerts($dbconn) {
	# test to see if tw_cli is present, if not, don't run
	exec("which tw_cli 2>&1 1>/dev/null", $output, $retval);
	if ($retval != 0) {
		return;
	}

	$old_unit_array = bppa_db_get_3ware_units($dbconn);
	if (!is_array($old_unit_array)) {
		$old_unit_array = array();
	}
	$cur_unit_array = array();
	$cur_controller_array = bppa_get_3ware_controllers();
	if (!is_array($cur_controller_array)) {
		$cur_controller_array = array();
	}

	for ($i=0; $i < count($cur_controller_array); $i++) {
		$these_units = bppa_get_3ware_units($cur_controller_array[$i]);
		if (is_array($these_units) and count($these_units) > 0) {
			$cur_unit_array = array_merge($cur_unit_array, $these_units);
		}
	}

	
	## Start looking for new and lost units.

	if (empty($old_unit_array) === TRUE) {
		$new_units = $cur_unit_array;
		$lost_units = array();
	} else {
		$new_units = bppa_diff_units($cur_unit_array, $old_unit_array, FALSE);
		$lost_units = bppa_diff_units($old_unit_array, $cur_unit_array, FALSE);
	}

	$new_units_keys = array_keys($new_units);
	$lost_units_keys = array_keys($lost_units);

	for ($i=0; $i < count($new_units_keys); $i++) {
		$retval = bppa_db_add_unit($dbconn, $cur_unit_array[$new_units_keys[$i]]);
	}

	for ($i=0; $i < count($lost_units_keys); $i++) {
		$retval = bppa_db_delete_unit($dbconn, $old_unit_array[$lost_units_keys[$i]]);
	}


	## Start looking for units that have changed status.

	$old_unit_array = bppa_db_get_3ware_units($dbconn);
	if (!is_array($old_unit_array)) {
		$old_unit_array = array();
	}
	$cur_unit_array = array();
	$cur_controller_array = bppa_get_3ware_controllers();
	if (!is_array($cur_controller_array)) {
		$cur_controller_array = array();
	}

	for ($i=0; $i < count($cur_controller_array); $i++) {
		$these_units = bppa_get_3ware_units($cur_controller_array[$i]);
		if (is_array($these_units) and count($these_units) > 0) {
			$cur_unit_array = array_merge($cur_unit_array, $these_units);
		}
	}

	if (empty($old_unit_array) === FALSE) {
		$changed_units = bppa_diff_units($cur_unit_array, $old_unit_array, TRUE);
	}

	if (empty($changed_units) === FALSE) {
		$changed_units_keys = array_keys($changed_units);
	} else {
		$changed_units_keys = array();
	}

	for ($i=0; $i < count($changed_units_keys); $i++) {
		$cur_unit = $changed_units[$changed_units_keys[$i]];
		$target = 'c' . $cur_unit["controller"] . " u" . $cur_unit["unit"];
		$old_status = bppa_db_unit_status($dbconn, $cur_unit);
		$retval = bppa_db_update_unit($dbconn, $cur_unit);

		# notify_id 1: Resolve/Close alerts for 3ware source + target
		bp_send_notification(1, $target);
		if ($cur_unit['status'] == 'OK') {
			# notify_id 6: "3ware %s unit status changed %s"
			# severity = S_NOTICE, closed = true
			$notify_id = 6;
		} elseif ($cur_unit['status'] == 'INOPERABLE') {
			# notify_id 2: "3ware %s unit status changed %s"
			# severity = S_FATAL, closed = false
			$notify_id = 2; 
		} else {
			# notify_id 4: "3ware %s unit status changed %s"
			# severity = S_WARNING, closed = false
			$notify_id = 4; 
		}
		bp_send_notification($notify_id, $target, $target,
			"from {$old_status[0]['status']} to {$cur_unit['status']}");

	}

	$old_drive_array = bppa_db_get_3ware_drives($dbconn);
	if (!is_array($old_drive_array)) {
		$old_drive_array = array();
	}
	$cur_drive_array = array();

	for ($i=0; $i < count($cur_controller_array); $i++) {
		$these_drives = bppa_get_3ware_drives($cur_controller_array[$i]);
		if (is_array($these_drives) and count($these_drives) > 0) {
			$cur_drive_array = array_merge($cur_drive_array, $these_drives);
		}
	}


	## Start looking for new and lost drives.


	if (empty($old_drive_array) === TRUE) {
		$new_drives = $cur_drive_array;
		$lost_drives = array();
	} else {
		$new_drives = bppa_diff_drives($cur_drive_array, $old_drive_array, FALSE);
		$lost_drives = bppa_diff_drives($old_drive_array, $cur_drive_array, FALSE);
	}

	$new_drives_keys = array_keys($new_drives);
	$lost_drives_keys = array_keys($lost_drives);

	for ($i=0; $i < count($new_drives_keys); $i++) {
		$retval = bppa_db_add_drive($dbconn, $cur_drive_array[$new_drives_keys[$i]]);
	}

	for ($i=0; $i < count($lost_drives_keys); $i++) {
		$retval = bppa_db_delete_drive($dbconn, $old_drive_array[$lost_drives_keys[$i]]);
	}


	## Start looking for drives that have changed status.

	$old_drive_array = bppa_db_get_3ware_drives($dbconn);
	if (!is_array($old_drive_array)) {
		$old_drive_array = array();
	}
	$cur_drive_array = array();

	for ($i=0; $i < count($cur_controller_array); $i++) {
		$these_drives = bppa_get_3ware_drives($cur_controller_array[$i]);
		if (is_array($these_drives) and count($these_drives) > 0) {
			$cur_drive_array = array_merge($cur_drive_array, $these_drives);
		}
	}

	if (empty($old_drive_array) === FALSE) {
		$changed_drives = bppa_diff_drives($cur_drive_array, $old_drive_array, TRUE);
	}

	if (empty($changed_drives) === FALSE) {
		$changed_drives_keys = array_keys($changed_drives);
	} else {
		$changed_drives_keys = array();
	}

	for ($i=0; $i < count($changed_drives_keys); $i++) {
		$cur_drive = $changed_drives[$changed_drives_keys[$i]];
		$target = 'c' . $cur_drive["controller"] . ' p' . $cur_drive["port"];
		$old_status = bppa_db_drive_status($dbconn, $cur_drive);
		$retval = bppa_db_update_drive($dbconn, $cur_drive);
		# notify_id 1: Resolve/Close alerts for 3ware source + target
		bp_send_notification(1, $target);

		if ($cur_drive['status'] == 'OK') {
			# notify_id 7: "3ware %s drive status changed %s"
			# severity = S_NOTICE, closed = true
			$notify_id = 7;
		} elseif ($cur_drive['status'] == 'INOPERABLE') {
			# notify_id 3: "3ware %s drive status changed %s"
			# severity = S_FATAL, closed = false
			$notify_id = 3; 
		} else {
			# notify_id 5: "3ware %s drive status changed %s"
			# severity = S_WARNING, closed = false
			$notify_id = 5; 
		}
		bp_send_notification($notify_id, $target, $target,
		  	"from {$old_status[0]['status']} to {$cur_drive['status']}");
	}

	return 0;
}

function bppa_fixup_raid_drives($dbconn) {
	# Put correct partition numbers in database where partition = 0
	$results = pg_query($dbconn, "SELECT * FROM bp.raid_drives WHERE partition = 0");
	if ($results == false) {
		print "pg_query failed\n";
		return false;
	}

	$raid_drives = pg_fetch_all($results);

	if (empty($raid_drives)) {
		return;
	}

	$i = 0;

	while ($i < pg_num_rows($results)) {
		$device = $raid_drives[$i]['device'];
		$partition = 0;

		if (preg_match("/(\/dev\/sd[a-z])([0-9]+)/", $device, $device_parts) == 1) {
			$partition = $device_parts[2];
			$result = pg_query($dbconn, "UPDATE bp.raid_drives SET partition = $partition WHERE device =  '$device'");
			if ($result == false) {
				print "pg_query failed\n";
				return false;
			}
		}

		$i++;
	}
}

function processMegaraidAlerts() {
	exec("which MegaCli64 2>&1 1>/dev/null", $output, $retval);
	if ($retval != 0) {
		return;
	}

	$vd_status = bppa_get_status();
	
	foreach ($vd_status as $status) {
		$target = 'Virtual Drive:' . $status['virt_num'];

		if ($status['state'] == ' Optimal') {
			bp_send_notification(1, $target);
		} elseif (($status['state'] == ' Failed') || ($status['state'] == ' Degraded')) {
			if (!alert_exists('3ware', $target, 137)) {
				#Close process + alert
				bp_send_notification(1, $target);
				bp_send_notification(137, $target, $target, $status['state']);
			}
			bp_send_notification(137, $target, $target, $status['state']);
		} else {
			if (!alert_exists('3ware', $target, 139)) {
				#Close process + alert
				bp_send_notification(1, $target);
				bp_send_notification(139, $target, $target, $status['state']);
			}
			bp_send_notification(139, $target, $target, $status['state']);
		}
	}

	$device_info = bppa_get_device_drive_counts();

	foreach ($device_info as $dev) {
		$enc_id = $dev['enc_id'];
		$enc_position = $dev['enc_pos'];
		$drive_count = $dev['drive_count'];
		$enc_alias = array(' 1' => 'Front panel', ' 2' => 'Back panel', ' N/A' => 'Internal DB trays');

		$phys_drives = bppa_get_drives($enc_id, $enc_position, $drive_count);
		foreach ($phys_drives as $drives) {
			$position = $drives['enc_pos'];
			if ($drives['enc_pos'] == ' 2') {
				$slot_alias = $drives['slot_num']+24;
			} else {
				$slot_alias = $drives['slot_num'];
			}

			$target = 'Slot:' . $slot_alias . ' located in the ' . $enc_alias[$position] . ',';

			if ($drives['state'] == ' Online, Spun Up') {
				#Close process + alert
				bp_send_notification(1, $target);
			} elseif ($drives['state'] == ' Failed') {
				#throw alert
				if (!alert_exists('3ware', $target, 138)) {
					#Close process + alert
					bp_send_notification(1, $target);
					bp_send_notification(138, $target, $target, $drives['state'], $drives['serial']);
				}
				bp_send_notification(138, $target, $target, $drives['state'], $drives['serial']);
			} else {
				#throw alert
				if (!alert_exists('3ware', $target, 140)) {
					#Close process + alert
					bp_send_notification(1, $target);
					bp_send_notification(140, $target, $target, $drives['state'], $drives['serial']);
				} 
				bp_send_notification(140, $target, $target, $drives['state'], $drives['serial']);
			}
		}
	}
}

function processMDraidAlerts($dbconn) {
	$debug = 0;

	if ($debug) {
		print "processMDraidAlerts()\n";
	}

	if (!file_exists("/sbin/mdadm")) {
		# Don't do anything if mdadm is missing.
		return 0;
	}

	$internal_found = 0;
	system("grep -qvsE 'md_drives_internal' /var/opt/unitrends/platform_capabilities", $internal_found); /* 1 if found */
	if ($internal_found != 1) {
		# Didn't find in platform_capabilities, also check here.
		system("grep -qvsE 'Recovery-720|Recovery-730' /etc/unitrends-release", $internal_found); /* 1 if found */
	}
	$md_drives_internal = $internal_found;

	bppa_fixup_raid_drives($dbconn);

	$old_unit_array = bppa_db_get_mdraid_units($dbconn);
	if (!is_array($old_unit_array)) {
		$old_unit_array = array();
	}

	$cur_unit_array = bppa_get_mdraid_units($dbconn);

	# Start looking for new and lost units.

	if (empty($old_unit_array) === TRUE) {
		$new_units = $cur_unit_array;
		$lost_units = array();
	} else {
		$new_units = bppa_diff_raid_units($cur_unit_array, $old_unit_array, FALSE);
		$lost_units = bppa_diff_raid_units($old_unit_array, $cur_unit_array, FALSE);
	}

	if (0) {
		print "new units:\n";
		foreach ($new_units as $unit) {
			print $unit['unit'];
			print "\n";
		}
		print "lost units:\n";
		var_dump($lost_units);
		foreach ($lost_units as $unit) {
			print $unit['unit'];
			print "\n";
		}
	}

	$new_units_keys = array_keys($new_units);
	$lost_units_keys = array_keys($lost_units);

	foreach ($new_units as $unit) {
		$retval = bppa_db_add_raid_unit($dbconn, $unit);
		if ($retval === FALSE) {
			return FALSE;
		}
	}

	foreach ($lost_units as $unit) {
		$retval = bppa_db_delete_raid_unit($dbconn, $unit);
	}

	# Start looking for units that have changed status.

	$old_unit_array = bppa_db_get_mdraid_units($dbconn);
	if (!is_array($old_unit_array)) {
		$old_unit_array = array();
	}
	$cur_unit_array = bppa_get_mdraid_units($dbconn);
	if (!is_array($cur_unit_array)) {
		$cur_unit_array = array();
	}

	if (empty($old_unit_array) === FALSE) {
		$changed_units = bppa_diff_raid_units($cur_unit_array, $old_unit_array, TRUE);
	} else {
		$changed_units = array();
	}

	if ($debug) {
		print "changed_units:\n";
		var_dump($changed_units);
	}

	foreach ($changed_units as $cur_unit) {
		$target = $cur_unit['unit'];
		$old_status = bppa_db_raid_unit_status($dbconn, $cur_unit['unit']);
		$retval = bppa_db_update_raid_unit($dbconn, $cur_unit);

		if ($debug) {
			print "unit is:\n";
			var_dump($cur_unit);
			print "old_status is: $old_status\n";
		}

		# notify_id 40: Resolve/Close alerts for RAID source + target
		bp_send_notification(40, $target);
		if ($cur_unit['status'] == 'OK') {
			# notify_id 48: "RAID %s: Array status %s"
			# severity = S_NOTICE, closed = true
			$notify_id = 48;
		} elseif ($cur_unit['status'] == 'INOPERABLE') {
			# notify_id 43: "RAID %s: Array status %s"
			# severity = S_FATAL, closed = false
			$notify_id = 43;
		} elseif ($cur_unit['status'] == 'DEGRADED') {
			# notify_id 43: "RAID %s: Array status %s"
			# severity = S_FATAL, closed = false
			$notify_id = 43;
		} else {
			# notify_id 45: "RAID %s: Array status %s"
			# severity = S_WARNING, closed = false;
			$notify_id = 45;
		}

		$do_alert = FALSE;
		if ($old_status == 'OK' or $old_status == '') {
			# close previous alert if we are OK now, raise new
			# otherwise do not alert for OK
			if (alert_exists('RAID', $target)) {
				if ($debug) {
					print "closing alert, old_status = $old_status";
				}
				# notify_id 40: Resolve/Close alerts for RAID source + target
				bp_send_notification(40, $target);
				$do_alert = TRUE;
			}
		} else {
			$do_alert = TRUE;
		}

		if ($do_alert) {
			if ($old_status == '' or $old_status == $cur_unit['status']) {
				$suppl_txt = "is " . $cur_unit['status'] . ".";
			} else {
				$suppl_txt = "changed from " . $old_status . " to " . $cur_unit['status'] . ".";
			}
			bp_send_notification($notify_id, $target, $target, $suppl_txt);
		}
	}


	# Look for new and lost drives.

	$old_drive_array = bppa_db_get_mdraid_drives($dbconn);
	if ($old_drive_array === FALSE) {
		print "bppa_db_get_mdraid_drives error\n";
		return 1;
	}
	if (!is_array($old_drive_array)) {
		$old_drive_array = array();
	}

	if ($debug) {
		print "old_drive_array:\n";
		var_dump($old_drive_array);
	}

	$cur_drive_array = bppa_get_mdraid_drives($dbconn);
	if (!is_array($cur_drive_array)) {
		$cur_drive_array = array();
	}

	if ($debug) {
		print "cur_drive_array:\n";
		var_dump($cur_drive_array);
	}

	if (empty($old_drive_array)) {
		$new_drives = $cur_drive_array;
		$lost_drives = array();
	} else {
		$new_drives = bppa_diff_mdraid_drives($cur_drive_array, $old_drive_array, FALSE);
		$lost_drives = bppa_diff_mdraid_drives($old_drive_array, $cur_drive_array, FALSE);
	}

	foreach ($new_drives as $new_drive) {
		if ($debug) {
			print "new_drive:\n";
			var_dump($new_drive);
		}
		$retval = bppa_db_add_raid_drive($dbconn, $new_drive);
	}

	foreach ($lost_drives as $lost_drive) {
		$retval = bppa_db_delete_raid_drive($dbconn, $lost_drive);
	}

	
	# Look for drives that have changed status

	$cur_drive_array = bppa_get_mdraid_drives($dbconn);
	if (!is_array($cur_drive_array)) {
		$cur_drive_array = array();
	}

	if (empty($old_drive_array) === FALSE) {
		$changed_drives = bppa_diff_mdraid_drives($cur_drive_array, $old_drive_array, TRUE);
	} else {
		$changed_drives = array();
	}

	if ($debug >= 5) {
		print "changed_drives:\n";
		var_dump($changed_drives);
	}

	foreach ($changed_drives as $cur_drive) {
		$target = $cur_drive['device'];
		$old_status = bppa_db_raid_drive_status($dbconn, $cur_drive);
		$retval = bppa_db_update_raid_drive($dbconn, $cur_drive);

		if ($debug >= 5) {
			print "changed drive:\n";
			print "target: " . $target . "\n";
			var_dump($cur_drive);
		}

		$cur_drive['portno'] = bppa_get_drive_portno($cur_drive['device']);

		if ($md_drives_internal) {
			$md_internal_msg = "(Internal Drive)";
		} else {
			$md_internal_msg = "";
		}

		if (isset($cur_drive['portno']) && ($cur_drive['portno'] != NULL)) {
				# include port number if we know it (610/300)
				$serialnumber = " (Port: " . $cur_drive['portno'] . ", Serial: " . $cur_drive['serial'] . $md_internal_msg . ")";
		}
		else {
			$serialnumber = " (Serial: " . $cur_drive['serial'] . ")";
		}

		switch (bppa_status_lookup($dbconn, $cur_drive['status'])) {
			case 'OK':
				# notify_id 47: "RAID %s: Drive status %s"
				# severity = S_NOTICE, closed = true
				$notify_id = 47;
				break;
			case 'REBUILDING':
				# notify_id 46: "RAID %s: Drive status %s"
				# severity = S_NOTICE, closed = false
				$notify_id = 46;
				break;
			case 'INOPERABLE':
				# notify_id 42: "RAID %s: Drive status %s"
				# severity = S_FATAL, closed = false
				$notify_id = 42;
				break;
			default:
				# notify_id 44: "RAID %s: Drive status %s"
				# severity = S_WARNING, closed = false
				$notify_id = 44;
				break;
		}

		$do_alert = FALSE;
		if ($old_status == 'OK') {
			# close previous alert if we are OK now
			# otherwise do not alert for OK
			if (alert_exists('RAID', $target)) {
				print "closing alert for $target.\n";
				# notify_id 40: Resolve/Close alerts for RAID source + target
				bp_send_notification(40, $target);
				$do_alert = TRUE;
			}
		} else {
			$do_alert = TRUE;
		}

		if ($do_alert) {
			$new_status = bppa_status_lookup($dbconn, $cur_drive['status']);
			if ($old_status == '' or $old_status == $new_status) {
				if ($debug) {
					print "raising alert for $target (no old)\n";
				}
				$suppl_txt = "is " . $new_status . "." . $serialnumber;
			} else {
				if ($debug) {
					print "raising alert for $target\n";
				}
				$suppl_txt = "changed from $old_status to " . $new_status . "." . $serialnumber;
			}
			bp_send_notification($notify_id, $target, $target, $suppl_txt);
		}
	}

	if ($debug) {
		print "processMDraidAlerts() returning 0\n";
	}
	return 0;
}


function processDiskAlerts($dbconn) {
	global $bploc;
	global $started;
	global $ui_type;
	$msg = "";

	# disk space
	$output = array();
	# get size remaining in MB
	$df = exec('df -PB1048576 2>/dev/null | grep -Ev "^/dev/loop|^tmpfs|/media/cdrecorder"', $output, $ret);
	if ($ret == 0 and count($output) > 1) {
		$percentage = get_master_ini_setting('Alertman', 'DiskSpaceAlertPercentageUsed', 85);
		$freespace = get_master_ini_setting('Alertman', 'DiskSpaceAlertMegabytesFree', 35840);

		unset($output[0]);
		foreach ($output as $line) {
			$matches = preg_split('/\s+/', $line);
			# skip invalid output format (should have 6 columns)
			if (!is_array($matches) or count($matches) != 6) {
				continue;
			}
			$MBfree = $matches[3];
			$percentused = str_replace('%', '', $matches[4]);
			$name = $matches[5];

			# skip partitions that aren't a disk or aren't mounted anywhere
			if ($name != '/_Stateless') {
			  if (substr($matches[0], 0, 4) != '/dev' or substr($name, 0, 1) != '/') {
				continue;
			  }
			}

			if ((bp_is_virtual()) and ($name == "/backups" || $name == "/_Stateless")) {
				$msg = "Please expand your storage.  Add a new virtual SCSI disk to this Virtual Appliance, then expand your storage in the User Interface.";
			} else {
				$msg = "";
			}

			# open an alert for partitions 95%+ (85%+ if UEB) full with <= 10GB (<= 20GB if UEB) space (by default)
			if ($percentused >= $percentage and $MBfree <= $freespace) {
				# notify_id 12:
				# "Disk space usage threshold exceeded. Disk % is %s full (%s MB free).i  %s"
				switch ($ui_type) {
					case 'CE':
						bp_send_notification(182, $name, $name, $matches[4], $MBfree, $msg);
						break;
					case 'Satori':
						bp_send_notification(12, $name, $name, $matches[4], $MBfree, $msg);
						break;
					case 'RRC':
						bp_send_notification(12, $name, $name, $matches[4], $MBfree, $msg);
						break;
				}
			} else {
				# notify_id 11: Resolve/close alerts for Disk source + target
				bp_send_notification(11, $name);
			}
		}
	}
}


function processProcessAlerts($dbconn) {
	global $bploc;
	global $started;
	global $ui_type;

	# license expiration
	$expired = license_has_expired();
	if ($expired == "true") {
		if (!alert_exists('Process', 'license', 20)) {
			if (!bp_is_demo()) {
				$license_info = bp_get_license_info();
				if ($license_info['class'] == "TRIAL") {
					$license = "Trial license";
					$message = "Please upgrade the system by clicking Upgrade button in the user interface.";
				} else if ($license_info['class'] == "NFR") {
					$license = "NFR license";
					$message = "Please upgrade the system by clicking Upgrade button in the user interface.";
				} else if ($license_info['class'] == "UNREG") {
					$license = "license";
					if (bp_get_default_ui() == "rrc") {
						$message = "Please register your system by clicking Register button in the upper right corner of the user interface.";
					} else {
						$message = "Please register your system by going to the Configure page, clicking Edit Appliance, then clicking the Register button on the License tab.";
					}
				} else if (($license_info['class'] == "ENTRR") || ($license_info['class'] == "ENTRB")) {
					$license = "license";
					$message = "Please upgrade the system by clicking Upgrade button in the user interface.";
				} else {
					$license = "license";
					$message = "Please renew your license by calling 888-374-6124.";
				}
			} else {
				$license = "license";
				$message = "Please renew your license by calling 888-374-6124.";
			}
			bp_send_notification(20, 'license', $license, $message);
		}
	} else if ($expired == "false") {
		# notify_id 19: Resolve/close alerts for Process source + target
		bp_send_notification(19, 'license');
	}

	# tasker status
	$tasker_up = false;
	system('ps -C tasker >/dev/null', $ret);
	if ($ret == 0) {
		$tasker_up = true;
		# notify_id 38: "Process: Tasker has been started."
		bp_send_notification(38, 'tasker');
	} else {
		# notify_id 26: "Process: Tasker has been stopped."
		bp_send_notification(26, 'tasker');
		// Update the library if it doesn't accurately reflect the status of tasker.
		// This enables the bp_get_tasker_status BPL API to query the library table for status.
		$query = pg_query($dbconn, "SELECT dispatcher, dispatch_pid FROM bp.library WHERE library_no = 1");
		$row = pg_fetch_row($query);
		if ($row[0] == 1 or $row[1] > 0) {
			$query = pg_query($dbconn, "UPDATE bp.library SET dispatcher=0, dispatch_pid=0 WHERE library_no = 1");
		}
	}

	# archive-daemon
	system('ps -C uarchive > /dev/null', $uarchive_ret);
	if ($uarchive_ret == 0) {
		#notify_id 80: "Process: archiveDaemon is running."
		bp_send_notification(80, 'archiveDaemon');
	} else {
		#notify_id 79: "Process: archiveDaemon is not running."
		bp_send_notification(79, 'archiveDaemon');
	}	

	# cryptodaemon
	if ($tasker_up == true) {
		## check cryptoDaemon status... he needs to be happy if any nodes have encryption enabled
		$vault_alerted = false;
		if( bp_local_system_is_vault() == true)
		{
			$enabled = false;
			$active = false;
			$me = bp_get_local_system_id();
			$myCrypt = bp_get_crypt_info($me);
			/* We can't connect to Non-managed Replication Sources.
			 * Original code was also performing this check on managed
			 * systems, but there should be no reason for that to require
			 * cryptoDaemon to run on the target.
			 */
			$res=pg_query($dbconn, "SELECT system_id FROM bp.systems WHERE role = 'Replication Source'");

			while ($row = pg_fetch_row($res)) {
				$id = $row[0];
				$crypt = bp_get_crypt_info($id);
				$enabled = $crypt["enabled"] || $enabled;
				$active = $crypt["active"] || $active;
			}
			if ($enabled and !$myCrypt["enabled"]){
				# notify_id 21: Process: cryptoDaemon is required but not running.
				$vault_alerted = true;
				bp_send_notification(21, 'cryptoDaemon');
			}

		}

		/* UNIBP-5889 : instance-level encryption support.
		 * Look at all local instances to determine if any have
		 * encryption enabled (instead of just the file-level
		 * instances).
		*/
		$res=pg_query($dbconn, "SELECT crypt FROM bp.application_instances WHERE crypt = true");
		if ($res and pg_num_rows($res) > 0) {
			/* at least one instance is encrypted */
			$cd_output = exec($bploc . '/bin/cryptoDaemon --status 2>&1', $output, $retval);

			if (isset($output[0])) {
				if (strpos($output[0], 'no running cryptoDaemon') !== false) {
					# notify_id 21: Process: cryptoDaemon is required but not running.
					bp_send_notification(21, 'cryptoDaemon');
				} else if (strpos($output[0], 'master key: notset') !== false) {
					# notify_id 22: Process: cryptoDaemon is required and running but has no master key.
					bp_send_notification(22, 'cryptoDaemon');
				} else if (strpos($output[0], 'master key: set') !== false) {
					# notify_id 39: Process: cryptoDaemon has been started.
					bp_send_notification(39, 'cryptoDaemon');
				}
			}

		} else if ($vault_alerted == false){
			# notify_id 19: Resolve/close alerts for Process source + target
			bp_send_notification(19, 'cryptoDaemon');
		}

	}

	# 5.0.2 Database Migration
	$query = pg_query($dbconn, "SELECT status FROM bp.operation_status WHERE operation_id = 2");
	if ($query and pg_num_rows($query) > 0) {
		$row = pg_fetch_row($query);
		$status = $row[0];
		if ($status != "done") {
			bp_send_notification(82, 'Migration', 'in progress');
		}
		else if (alert_exists('Process', 'Migration', 82)) {
			# Update alert to say it's complete and resolved
			bp_send_notification(82, 'Migration', 'complete');

			# Then close the alert with notify_id 19
			bp_send_notification(19, 'Migration');
		}
		else if (file_exists('/backups/db_migrate_dump.sql.gz')) {
			# After 2 weeks, clean up the backups of the old database
			$stat = stat('/backups/db_migrate_dump.sql.gz');
			if ($started >= $stat['mtime'] + (86400 * 14)) {
				$dbpath = "$bpdir/data";
				while (is_link($dbpath)) {
					$dbpath = readlink($dbpath);
				}
				system("rm -rf $dbpath.dbm_save.* 2>/dev/null", $ret);
				unlink("/backups/db_migrate_dump.sql.gz");
			}
		}
	}


	# retention period
	# Check when to process this--we don't do it every alertman run
	$processRetentionAlerts = false;
	$lastrun = bppa_db_last_run($dbconn);
	/* if we've run before, only check if we're supposed to, otherwise (1st alert run) we check */
	if (count($lastrun) > 0) {
		$checkTime = '06:45';
		$checkTimestamp = strtotime($checkTime);
		$lastStarted = $lastrun[0]["started"];
		/* How many seconds from when we should have started a retention check? */
		if ($lastStarted < $checkTimestamp and $started >= $checkTimestamp) {
			$processRetentionAlerts = true;
		}
	} else {
		$processRetentionAlerts = true;
	}
	# if tool wasn't initialized last time, scan again
	if ($processRetentionAlerts == false) {
		if (alert_exists('Process', 'retention', 31) ||
				alert_exists('Process', 'retention', 32))
		{
			$processRetentionAlerts = true;
		}
	}

	$lic_info = bp_get_license_info();
	if (($lic_info !== false) && isset($lic_info['class'])) {
		$lic_class =  $lic_info['class'];
	}
	else {
		$lic_class = "unknown";
	}

	# Only perform these checks for physical units and
	# UEBs with Enterprise capacity licenses.
	if ($processRetentionAlerts &&
			($lic_class == "PHY" || $lic_class == "ENTRB" || $lic_class == "CE")) {
		$rt_output = exec($bploc . '/bin/retention_validation -alerts -f /tmp/retret.dat', $output, $retval);
		# returns 0 on no violation, 1 for violation, 2 for not initialized, 3 for failure
		# 7,8,9 -70,80,90 percent of license usage.
		# don't send alerts if it's an SFF
		$send_alert=1;
		system("grep -qEs 'Small Form Factor|Recovery-710|Recovery-720|Recovery-730' /etc/unitrends-release", $send_alert);
		if ($send_alert == 0) {
			$retval = 0;
		}
		
		$fp = fopen ('/tmp/retret.dat','r');
		if ($fp) {
			$lic_allowed = (int)trim(fgets($fp));
			$lic_used = (int)trim(fgets($fp));
			$lic_grace = (int)trim(fgets($fp));
			fclose ($fp);
   		}
   		exec ('rm -f /tmp/retret.dat');		

		if ($lic_class == "ENTRB") {
			$lic_msg = "Please upgrade your license by clicking the Upgrade button in the user interface.";
		}

		switch ($retval) {
			case 0:
				# notify_id 19: Resolve/Close alerts for Process source + target
				bp_send_notification(19, 'retention');
				put_master_ini_setting('Alertman', 'LicenseGracePeriod', '99999');
				put_master_ini_setting('Alertman', 'LicenseSetupGracePeriod', '99999');
				break;
			case 1:
				$previous_lic_grace = get_master_ini_setting('Alertman', 'LicenseGracePeriod', '');
				// 5128
				if ($previous_lic_grace === '' or $lic_grace < $previous_lic_grace) {
					if ($lic_class == "PHY") {
						$lic_msg = "Please contact your authorized Unitrends partner or Unitrends representative immediately at 888-374-6124 to increase your licensed capacity and restart the process of protecting your data.";
					}
					if ($lic_grace > 0) {
						if (!alert_exists('Process', 'retention', 23)) {
							# notify_id 19: Resolve/Close alerts for Process + target
							bp_send_notification(19, 'retention');
						}
						# notify_id 23: License Grace Period: The licensed capacity
						# of this DPU has been exceeded.  (This notification also
						# sends email ($email_text_grace).
						bp_send_notification(23, 'retention', $lic_grace, $lic_msg);
					}
					else { # $lic_grace==0
						if (!alert_exists('Process', 'retention', 24)) {
							# notify_id 19: Resolve/Close alerts for Process + target
							bp_send_notification(19, 'retention');
						}
						# notify_id 24: License Failure Notification: This DPU cannot
						# perform backups because your data has exceeded the licensed
						# capacity. (This notification also sends email, $email_text_1)
						bp_send_notification(24, 'retention', $lic_allowed, $lic_msg);
					}
					put_master_ini_setting('Alertman', 'LicenseGracePeriod', $lic_grace);
				}
				put_master_ini_setting('Alertman', 'LicenseSetupGracePeriod', '99999');
				break;
			case 2:
				$previous_lic_setup_grace = get_master_ini_setting('Alertman', 'LicenseSetupGracePeriod', '');
				// 5128
				if ($previous_lic_setup_grace === '' or $lic_grace < $previous_lic_setup_grace) {
					put_master_ini_setting('Alertman', 'LicenseSetupGracePeriod', $lic_grace);
					if ($lic_grace === 0) {
						if (!alert_exists('Process', 'retention', 32)) {
							// close non-initialization retention alerts
							# notify_id 19: Resolve/Close alerts for Process + target
							bp_send_notification(19, 'retention');
						}
						# notify_id 32: Retention: The DPU retention tool
						# initialization grace period has expired. (This notification
						# also sends email, $email_text_setup_expire)
						bp_send_notification(32, 'retention');
					} else {
						if (!alert_exists('Process', 'retention', 31)) {
							# notify_id 19: Resolve/Close alerts for Process + target
							bp_send_notification(19, 'retention');
						}
						# notify_id 31: Retention: The DPU retention tool has not
						# been initialized.  # (This notification also sends email,
						# $email_text_setup_grace)
						bp_send_notification(31, 'retention', $lic_grace);
					}
				}
				break;
			case 7:
				if (!alert_exists('Process', 'retention', 27)) {
					# notify_id 19: Resolve/Close alerts for Process + target
					bp_send_notification(19, 'retention');

					# notify_id 27: License Alert: This DPU is at 70%% of its
					# licensed capacity. (This notification also sends email_text_7)
					if ($lic_class == "PHY") {
						$lic_msg = "You can upgrade your license by contacting your authorized Unitrends partner or Unitrends representative.";
					}
					switch ($ui_type) {
						case 'CE':
							bp_send_notification(184, 'retention', $lic_allowed, $lic_used, $lic_msg);
							break;
						case 'Satori':
							bp_send_notification(27, 'retention', $lic_allowed, $lic_used, $lic_mesg);
							break;
						case 'RRC':
							bp_send_notification(27, 'retention', $lic_allowed, $lic_used, $lic_msg);
							break;
					}
				}
				put_master_ini_setting('Alertman', 'LicenseGracePeriod', '99999');
				put_master_ini_setting('Alertman', 'LicenseSetupGracePeriod', '99999');
				break;
			case 8:
				if (!alert_exists('Process', 'retention', 28)) {
					# notify_id 19: Resolve/Close alerts for Process + target
					bp_send_notification(19, 'retention');

					# notify_id 28: License Warning (Yellow Alert): This DPU is at
					# 80%% of its licensed capacity. (This also sends email_text_8)
					if ($lic_class == "PHY") {
						$lic_msg = "You can upgrade your license now by contacting your authorized Unitrends partner or Unitrends representative (at 888-374-6124).";
					}
					switch ($ui_type) {
						case 'CE':
							bp_send_notification(185, 'retention', $lic_allowed, $lic_used, $lic_msg);
							break;
						case 'Satori':
							bp_send_notification(28, 'retention', $lic_allowed, $lic_used, $lic_msg);
							break;
						case 'RRC':
							bp_send_notification(28, 'retention', $lic_allowed, $lic_used, $lic_msg);
							break;
					}
				}
				put_master_ini_setting('Alertman', 'LicenseGracePeriod', '99999');
				put_master_ini_setting('Alertman', 'LicenseSetupGracePeriod', '99999');
				break;
			case 9:
				if (!alert_exists('Process', 'retention', 29)) {
					# notify_id 19: Resolve/Close alerts for Process + target
					bp_send_notification(19, 'retention');

					# notify_id 29: 'Urgent License Warning (Red Alert): This DPU
					# is at 90%% of its licensed capacity.  Backup failures are
					# imminent.  (This notification also sends email_text_9)
					if ($lic_class == "PHY") {
						$lic_msg = "Please contact your authorized Unitrends partner or Unitrends representative immediately at 888-374-6124 to avoid preventable backup failures.";
					}
					switch ($ui_type) {
						case 'CE':
							bp_send_notification(186, 'retention', $lic_allowed, $lic_used, $lic_msg);
							break;
						case 'Satori':
							bp_send_notification(29, 'retention', $lic_allowed, $lic_used, $lic_msg);
							break;
						case 'RRC':
							bp_send_notification(29, 'retention', $lic_allowed, $lic_used, $lic_msg);
							break;
					}
				}
				put_master_ini_setting('Alertman', 'LicenseGracePeriod', '99999');
				put_master_ini_setting('Alertman', 'LicenseSetupGracePeriod', '99999');
				break;
			default:
				# notify_id 30: Retention: The DPU retention tool was unable to
				# calculate the retention period.
				bp_send_notification(30, 'retention');
				break;
		}
	}

	# figure out if we need to process the update alerts
	do {
		# process if we haven't run before
		$lastrun = bppa_db_last_run($dbconn);
		if (count($lastrun) == 0) {
			$processUpdateAlerts = true;
			//echo "?: first run\n";
			break;
		}

		# process if a new rpm was installed since the last run
		$output = null;
		$retval = 0;
		$installed = trim(exec('rpm -q unitrends-rr --qf %{INSTALLTIME}', $output, $retval));
		if ($retval != 0) {
			//echo "?: could not determine install date\n";
			$processUpdateAlerts = false;
		}
		$lastStarted = $lastrun[0]["started"];
		if ($installed >= $lastStarted) {
			$processUpdateAlerts = true;
			//echo "?: new version install\n";
			break;
		}

		# process if we crossed the UpdateCheckTime
		$checkTime = get_master_ini_setting('Alertman', 'UpdateCheckTime', '03:00');
		$checkTimestamp = strtotime($checkTime);
		//echo "?: last started: $lastStarted\ncheck time: $checkTimestamp\nthis run: $started\n";
		if ($lastStarted < $checkTimestamp and $started >= $checkTimestamp) {
			$processUpdateAlerts = true;
			//echo "?: crossed check time\n";
			break;
		}

		# If none of the above, don't process
		//echo "?: not running...\n";
		$processUpdateAlerts = false;
	} while(false);

	if ($processUpdateAlerts) {
		$valid_da_file = "$bploc/lists.dir/.da_valid";
		$retval = 0;
		$output = null;

		if ($lic_class == "PHY") {
			$lic_msg = "To renew it, please call the Unitrends Support Center or your Authorized Partner.";
		}
		else {
			$lic_msg = "To renew it, please click on the Upgrade button in the user interface.";
		}

		# ensure customer has DA/DAP
		# Download the DA status file for all licenses.
		$cmd = '/usr/local/bin/dpu dastatus ts';
		$str = trim(exec($cmd, $output, $retval));

		# if DA expires in less than 30 days, double check the expiration
		# to avoid raising alerts after DA renewal
		if ($retval == 0 and ($str - $started) <= (86400 * 30)) {
			$str = trim(exec($cmd . ' force', $output, $retval));
		}

		if ($retval != 0 and $ui_type != 'CE') {
			# close alert if DA has expired or if we couldn't acquire the status
			//echo "P: closing alert due to retval of $retval from dpu dastatus\n";
			# notify_id 19: Resolve/Close alerts for Process source + target
			bp_send_notification(19, 'update');

			if ($retval == 10 and file_exists($valid_da_file)) {
				# DA was valid but now isn't, so show the expiration alert
				$ts = trim(file_get_contents($valid_da_file));
				# notify_id 19: Resolve/Close alerts for Process source + target
				bp_send_notification(19, 'DA');
				if ($ts <= $started) {
					$exp = date('F j, Y', $ts);
					$text = "expired on $exp";
				} else {
					$text = "has expired";
				}
				# Only raise alert for PHY/ENTRB/ENTRR license types,
				# not for NFR/TRIAL/UNREG/FREE licenses.
				if ($lic_class=="PHY" || $lic_class=="ENTRB" || $lic_class=="ENTRR") {
					# notify_id 36: "Data Assurance: Your Data Assurance %s."
					bp_send_notification(36, 'DA', $text, $lic_msg);
				}
				unlink($valid_da_file);
			}
			if ($retval != 10) {
				# error, so close open DA alerts
				# notify_id 19: Resolve/Close alerts for Process source + target
				bp_send_notification(19, 'DA');
			}
		} else {
			# customer has DA, so store the state, check expiration, and look for updates
			if ($ui_type != 'CE') {
			file_put_contents($valid_da_file, $str);

			# write expiration timestamp to a file and possibly raise an alert about expiration
			//echo "D: ts is $str, started $str, difference: ".($str - $started)."\n";
			if (($str - $started) <= (86400 * 30)) {
				$exp = date('F j, Y', $str);
				//echo "D: expiration alert: $exp\n";
				# Only raise alert for PHY/ENTRB/ENTRR license types,
				# not for NFR/TRIAL/UNREG/FREE licenses.
				if ($lic_class=="PHY" || $lic_class=="ENTRB" || $lic_class=="ENTRR") {
					# notify_id 37: "Data Assurance: Your Data Assurance expires on %s."
					bp_send_notification(37, 'DA', $exp, $lic_msg);
				}
			} else {
				# notify_id 19: Resolve/Close alerts for Process source + target
				bp_send_notification(19, 'DA');
			}}

			# check for updates
			
			#get the whitelist file from the ftp server
			$whitelist = "$bploc/lists.dir/.wlstatus" ;
			$remoteloc = " ftp://ftp.unitrends.com/utilities/tmp/.wlstatus";
			if (file_exists($whitelist)){
			        $cmd = "wget -qN $remoteloc";
			}else{
			        $cmd = "wget -q $remoteloc";
			}
			$str = trim(exec($cmd,$output, $retval));
            rename("./.wlstatus" , $whitelist);
			chmod($whitelist, 0444);
			#first get the asset tag and check if the asset is in the whitelist

			unset($output);
			$asset = trim( file_get_contents("/etc/unitrends-asset"));
			
			$retval = 0;
			$cmd = "grep $asset $whitelist";
			$str = trim(exec($cmd, $output, $retval));
			if ( file_exists($whitelist) and ((filesize($whitelist) == 0  or ($retval == 0 and !empty($str))))) {
			
				#output appends when exec is called if it is not cleared; clearing varibles
				$retval = 0;
				unset($output);
			
				$cmd = "$bploc/bin/cmc_updates list | grep 'unitrends-rr[ \\t]'"; #look for /unitrends-rr\s/
				$str = trim(exec($cmd, $output, $retval));
				//echo "P: got $retval returned from cmc_updates list|grep, str: $str\n";
				if ($retval == 0 and !empty($str)) {
                    # create software is available
                    $update = preg_split('/\s+/', $str);
				    $version = $update[2];

                    $alertId=($ui_type=='RRC') ? 25 : 187;
                    $alert=get_alert('Process', 'update');
                    if ($alert) {
                        $alert_text =$alert['alert_text'];
                        if (strpos($alert_text,$version) == false) {   # should specify false due to valid return 0
                            bp_send_notification(19, 'update'); # available version is different then close it for new one
                        }
                    }                

                    bp_send_notification($alertId, 'update', $version);
                    updateSyncSummaryTable($dbconn, "update_available");

				} else {
					# close alert if no software available
					//echo "P: closing alert because no update was available\n";
			
					# notify_id 19: Resolve/Close alerts for Process source + target
					bp_send_notification(19, 'update');
					updateSyncSummaryTable($dbconn, "latest");
				}
			} else {
				# the user is not in the whitelist and it exists
				# close alert until the use is in the whitelist or the whitelist is gone
				# notify_id 19: Resolve/Close alerts for Process source + target
				bp_send_notification(19, 'update');
			}

		}
	}

	# Inventory sync abnormal termination.
	# A subsequently successful sync will clear this alert.
	$q = "select final_status, last_message from bp.inventory_sync";
	$res = pg_query($dbconn, $q);
	if ($res !== false) {
		$row = pg_fetch_assoc($res);
		if ($row["final_status"] == -1) {
			$msg =  $row["last_message"];
			bp_send_notification(174, 'Inventory Sync', $msg);
		}
		pg_free_result($res);
	}

	return 0;
}

function updateSyncSummaryTable($dbconn, $status)
{
	/*
	 * Update the version status in the sync summary table. (This info
	 * is used during pull requests from targets to managed sources.)
	 * Version status values can be either "latest", or "update_available".
	 * The sync_summary.php (which runs frequently) also updates the version
	 * status field but excludes the check for "update_available" as this
	 * is network intensive. Therefore we continue to use the logic within
	 * populate_alerts to check for these on a daily basis.
	 */

	$system_id = bp_get_local_system_id();

	/* Try Update */
	$update = "UPDATE bp.sync_summary SET version_status = '$status' WHERE system_id = $system_id";
	$retval = pg_query($dbconn, $update);
	if (pg_affected_rows($retval) == 0) {
		/* Try Insert */
		$insert = "INSERT INTO bp.sync_summary (system_id, version_status) VALUES ($system_id, '$status')";
		$retval = pg_query($dbconn, $insert);
	}
	return $retval;
}

function processRebootAlerts($dbconn) {
	/* Closes a "need reboot" which has been satisfied.
	 * This is implemented for the case of a kernel update being installed.
	 * When the update is installed, an alert is inserted into bp.alerts with source_id "Process" and target "reboot".
	 * This alert should be closed after a reboot has occured.
	 * This is detected by looking at the current system uptime vs. the creation time of the alert. */

	if (alert_exists('Process', 'reboot')) {
		$alert = get_alert('Process', 'reboot');
		if ($alert == false) {
			return false;
		}

		$created = $alert['created'];

		exec("cut -f 1 -d' ' /proc/uptime | cut -f 1 -d.", $uptime_result, $retval);
		if ($retval != 0) return false;

		$uptime = $uptime_result[0];

		$currenttime = time();

		if (($currenttime - $uptime) > $created) {
			# We must have rebooted
			# notify_id 19: Resolve/Close alerts for Process source + target
			bp_send_notification(19, 'reboot');
		}
	}
}

function processVersionTrap($dbconn) {
	global $hostname;

	exec("rpm -qa --queryformat '%{NAME}\n' unitrends*", $name_list, $ret);
	$num_name_list = array_count_values($name_list);

	foreach ($num_name_list as $key => $num) {
		if ($num > 1) {
			exec("rpm -qa --queryformat '%{VERSION}-%{RELEASE}\n' $key", $ver_list, $ret);
			arsort($ver_list);
			$q_history = pg_query($dbconn, "SELECT * FROM bp.snmp_history WHERE object = '$key-$ver_list[0]'");
			$q_nrows = pg_num_rows($q_history);
		
			if ($q_nrows == 0) {
				bp_send_notification(168, "$key-$ver_list[0]", $key);
			}
			return;
		}
	}

	exec("rpm -qa --queryformat '%{VERSION}-%{RELEASE}\n' unitrends-ipmi unitrends-monitoring unitrends-rr unitrends-snmp unitrends-vprotect unitrends-winbm", $ver_list, $ret);

	$old_ver = 0;
	foreach ($ver_list as $new_ver) {
		if (version_compare($old_ver, $new_ver, '<')) {
			$old_ver = $new_ver;
		}
	}

	if (!file_exists("/usr/archive/greenbutton/current_vers")) {
		if (!is_dir("/usr/archive/greenbutton")) {
			shell_exec("mkdir -p /usr/archive/greenbutton");
		}
		shell_exec("echo $old_ver > /usr/archive/greenbutton/current_vers");
		bp_send_notification(71, $hostname, $old_ver);
	} else {
		$cur_ver = trim(shell_exec("cat /usr/archive/greenbutton/current_vers"));
		if (version_compare($cur_ver, $old_ver, '<')) {
			shell_exec("echo $old_ver > /usr/archive/greenbutton/current_vers");
			bp_send_notification(71, $hostname, $old_ver);
		}
	}
}

function processSmartInfoTraps() {
	# VMs do not support SMART
	if (bp_is_virtual()) {
		return;
	}

	$drives = explode("\n", trim(shell_exec("ls -l /sys/block | grep -o 'sd[a-z]'")));
	$joined_info = "";
	$lsi_processed = false;
	$send_trap = false;
	$log_path = "/var/log/monitoring_disks";

	if (!file_exists($log_path)) {
		mkdir($log_path);
	}

	$smart_trap_log = "$log_path/smart_trap.log";
	$handle = fopen($smart_trap_log, 'a+');

	for ($i=0; $i < count($drives); $i++){
		$drive_type = trim(shell_exec("cat /sys/block/$drives[$i]/device/vendor"));

		if ($drive_type === "LSI") {
			continue;
		} elseif ($drive_type === "ATA") {
			$smart_cmd = "/dev/$drives[$i]";
			system("smartctl -i $smart_cmd | grep 'Sector Size' | grep 4096 2>/dev/null 1>&2", $retval);
			if ($retval != 0) {
				$reallocated_min = 500;
			} else {
				$reallocated_min = 5000;
			}

			$process_realloc = false;
			system("grep '$drives[$i]:Reallocated_Sector_Ct' $smart_trap_log 2>/dev/null 1>&2", $check_ret);
			if ($check_ret != 0) { 
				$realloc_smart_list = explode(" ", exec("smartctl -a -s on $smart_cmd | grep 'Reallocated_Sector_Ct'", $temp_realloc_list, $realloc_ret));
				if ($realloc_ret == 0) {
					$realloc_raw = trim($realloc_smart_list[35]);
					$process_realloc = true;
				}
			}

			$process_uncorrect = false;
			system("grep '$drives[$i]:Offline_Uncorrectable' $smart_trap_log 2>/dev/null 1>&2", $uncorrect_check_ret);
			if ($uncorrect_check_ret != 0) {
				$uncorrect_smart_list = explode(" ", exec("smartctl -a -s on $smart_cmd | grep 'Offline_Uncorrectable'", $temp_uncorrect_list, $uncorrect_ret));
				if ($uncorrect_ret == 0) {
					$uncorrect_raw = trim($uncorrect_smart_list[33]);
					$process_uncorrect = true;
				}
			}

			$raw_info = "";
			$process_drive = false;
			if ($process_realloc && ($realloc_raw > $reallocated_min)) {
				$raw_info .= $realloc_smart_list[3] . "($realloc_raw)" . " :: ";
				fwrite($handle, "$drives[$i]:$realloc_smart_list[3]\n");
				$process_drive = true;
			}

			if ($process_uncorrect && ($uncorrect_raw > 0)) {
				$raw_info .= $uncorrect_smart_list[1] . "($uncorrect_raw)" . " :: ";
				fwrite($handle, "$drives[$i]:$uncorrect_smart_list[1]\n");
				$process_drive = true;
			}
			
			if ($process_drive) {
				$joined_info .= " // " . $drives[$i] . ":" . $raw_info;
				$send_trap = true;
			}
		} elseif ($drive_type === "AMCC") {
			unset($ports);
			$unit_list = explode(" ", shell_exec("tw_cli info | grep '^c[0-9]\{1,\}'"), 2);
			$unit = $unit_list[0];
			$ports = explode("\n", trim(shell_exec("tw_cli info $unit | grep -o '^p[0-9]\{1,\}'")));
			
			for ($x=0; $x < count($ports); $x++) {
				$smart_cmd = "-d 3ware,$x /dev/twa0";
				$drive_name = "port[$x]";
			
				system("smartctl -i $smart_cmd | grep 'Sector Size' | grep 4096 2>/dev/null 1>&2", $retval);
				if ($retval != 0) {
					$reallocated_min = 500;
				} else {
					$reallocated_min = 5000;
				}

				$process_realloc = false;
				system("grep '$ports[$x]:Reallocated_Sector_Ct' $smart_trap_log 2>/dev/null 1>&2", $check_ret);
				if ($check_ret != 0) { 
					$realloc_smart_list = explode(" ", exec("smartctl -a -s on $smart_cmd | grep 'Reallocated_Sector_Ct'", $temp_realloc_list, $realloc_ret));
					if ($realloc_ret == 0) {
						$realloc_raw = trim($realloc_smart_list[35]);
						$process_realloc = true;
					}
				}

				$process_uncorrect = false;
				system("grep '$ports[$x]:Offline_Uncorrectable' $smart_trap_log 2>/dev/null 1>&2", $uncorrect_check_ret);
				if ($uncorrect_check_ret != 0) {
					$uncorrect_smart_list = explode(" ", exec("smartctl -a -s on $smart_cmd | grep 'Offline_Uncorrectable'", $temp_uncorrect_list, $uncorrect_ret));
					if ($uncorrect_ret == 0) {
						$uncorrect_raw = trim($uncorrect_smart_list[33]);
						$process_uncorrect = true;
					}
				}

				$raw_info = "";
				$process_drive = false;
				if ($process_realloc && ($realloc_raw > $reallocated_min)) {
					$raw_info .= $realloc_smart_list[3] . "($realloc_raw)" . " :: ";
					fwrite($handle, "$ports[$x]:$realloc_smart_list[3]\n");
					$process_drive = true;
				}

				if ($process_uncorrect && ($uncorrect_raw > 0)) {
					$raw_info .= $uncorrect_smart_list[1] . "($uncorrect_raw)" . " :: ";
					fwrite($handle, "$ports[$x]:$uncorrect_smart_list[1]\n");
					$process_drive = true;
				}
	
				if ($process_drive) {
					$joined_info .= " // " . $drive_name . ":" . $raw_info;
					$send_trap = true;
				}
			}
		}
	}
	fclose($handle);
	if ($send_trap) {
		bp_send_notification(81, "SMART", $joined_info);
	}
}

function processFileLevelRecoveryAlerts($dbconn) {
	$disk_status = bp_get_disk_image_status();
	foreach ($disk_status as $status) {
		if ($status['status'] == 'available') {
			$current_time = time();
			$time_online = $current_time - $status['available_since'];
			if ($time_online >= (86400 * 3)) {  // 86400 seconds = 1 day
				if (!alert_exists('Disk', 'FLR', 92)) {
					bp_send_notification(11, 'FLR');
					#send alert to unmount file system and log out of iSCSI session and Tear Down the image
					bp_send_notification(92, 'FLR');
					break;
				}
			} 
		}
	}

	if (count ($disk_status) == 0) {
		#close alerts
		bp_send_notification(11, 'FLR');
	}

	/* bug 15644 get the creation date of the share */
	$share_status = bp_get_application_share_status();
	foreach ($share_status as $status) {
		$current_time = time();
		$time_online = $current_time - $status['creation_date'];
		if ($time_online >= (86400 * 3)) {  // 86400 seconds = 1 day
			if (!alert_exists('Backup', strval($status['appinst_id']), 148)) {
				#send alert to unmount file system and log out of iSCSI session and Tear Down the image
				$query = "SELECT node_name, key1 FROM bp.nodes, bp.application_instances WHERE application_instances.node_no = nodes.node_no AND application_instances.instance_id = ".$status['appinst_id'];				

				$result = pg_query($dbconn, $query);

				$result = pg_fetch_all($result);

				if ($result != NULL && $result !== FALSE) 
				{
					$client_name = $result[0]['node_name'];
					$inst_name = $result[0]['key1'];
					bp_send_notification(148, strval($status['appinst_id']),$client_name,$inst_name);
					break;
				}

			}
		} 
	}

	if (count ($share_status) == 0) {
		#close alerts
		pg_query($dbconn, "UPDATE bp.alerts SET closed = 't' WHERE notification_id = 148 AND closed = 'f'");
	}
	/* end of bug 15644 get the creation date of the share */
}

function processVMwareIRAlerts() {
	$disk_status = bp_vmware_ir_status();
	foreach ($disk_status as $status) {
		$current_time = time();
		if ($current_time - $status['time'] >= (86400 * 3)) {  // 3 days
			if (!alert_exists('Disk', 'VIR', 114)) {
				bp_send_notification(114, 'VIR');
			}
		}
	}

	if (count ($disk_status) == 0) {
		# close alert
		bp_send_notification(11, 'VIR');
	}
}

function processHyperVIRAlerts() {
	$disk_status = bp_hyperv_ir_status();
	foreach ($disk_status as $status) {
		$current_time = time();
		if ($current_time - $status['time'] >= (86400 *3)) {  // 3 days
			if (!alert_exists('Disk', 'HIR', 171)) {
				bp_send_notification(171, 'HIR');
			}
		}
	}

	if (count ($disk_status) == 0) {
		# close alert
		bp_send_notification(11, 'HIR');
	}
}

function processCallHomeTrap() {
	if (bp_is_virtual()) {
		$vrec_model = shell_exec("cat /etc/unitrends-release | cut -d' ' -f 2");
		$vrec_nvp_type = "system";
		$vrec_nvp_name = "vRecovery";
		$trap_destination = "notifications.unitrends.com";

		//Check nvp table to see if trap has already been sent.	
		//This trap is sent only once.
		$vrec_nvp_list = bp_get_nvp_list($vrec_nvp_type, $vrec_nvp_name);

		/* If there is no entry in nvp table, then trap has never been sent.
		*  Check that Unitrends' trap destination is configured/enabled, then
		*  check to make sure that snmp can connect by pinging notifications.unitrends.com .
		*  If successful, add entry to database then send trap.
		*/
		if ($vrec_nvp_list == NULL) {
			$vrec_snmp_list = bp_get_snmp_config();
			foreach ($vrec_snmp_list as $snmp_destination) {
				if ($snmp_destination['destination'] == $trap_destination) {
					if ($snmp_destination['enabled'] == true) {
						system("ping -c 1 $trap_destination 2>/dev/null 1>&2", $ping_retval);
						if ($ping_retval == 0) {
							/* Add entry to database before sending trap in case
							*  the entry is not able to be added, do not send trap.
							*  Wait to send trap until entry can be added to database.
							*/ 
							$vrec_nvp_array = array("isAlive" => "true");
							$nvp_saved = bp_save_nvp_list($vrec_nvp_type, $vrec_nvp_name, $vrec_nvp_array);
							if ($nvp_saved) {
								bp_send_notification(96, 'Call Home', $vrec_model);
							}
						}
					}
				}
			}
		}
	}
}		

function processLicenseExpAlerts() {
	global $is_metal;
	#do not run if virtual-demo
	if (!bp_is_demo()) {
		$license_info = bp_get_license_info();
		if (($license_info != NULL) && isset($license_info['expiration_date'])) {
			if ($license_info['expiration_date'] != "never") {

				$current_date = date("U");
				$exp_date = strtotime($license_info['expiration_date']);
				$real_left = $exp_date - $current_date;

				if ($license_info['class'] == "UNREG") {
					# Notifications for Unregistered vRecovery Appliance.
					# -- Notify to Register on the deployment.
					# -- Notify to Register, if 4 days passed, and not registered/licnesed.  
					if (!alert_exists('Process', 'license', 20)) {
						if ($real_left <= 86400) { // 1 day left. 
							if (!alert_exists('Process', 'License warning', 127)) {
								#close/resolve existing alerts
								bp_send_notification(19, 'License warning');

								bp_send_notification(127, 'License warning','Unitrends Enterprise Backup','1');
							}
						} else if ($real_left <= 1296000) { // 15 days left. 
							if (!alert_exists('Process', 'License warning', 126)) {
								#close/resolve existing alerts
								bp_send_notification(19, 'License warning');

								bp_send_notification(126, 'License warning','Unitrends Enterprise Backup','15');
							}
						} else if ($real_left <= 86400 && $is_metal == "Metal") { // 1 days left
								if (!alert_exists('Process', 'License warning', 192)) {
								bp_send_notification(19, 'License warning');
								bp_send_notification(192, 'License warning');
							}
						} else if ($real_left <= 1296000 && $is_metal == "Metal") { // 15 days left
								if (!alert_exists('Process', 'License warning', 193)) {
								bp_send_notification(19, 'License warning');
								bp_send_notification(193, 'License warning');
							}
						} else if ($real_left <= 2592000 && $is_metal == "Metal") { // 30 days left
								if (!alert_exists('Process', 'License warning', 194)) {
								bp_send_notification(19, 'License warning');
								bp_send_notification(194, 'License warning');
							}
						} else {
							bp_send_notification(19, 'License warning');
						}
					} else {
						bp_send_notification(19, 'License warning');
					}

				} else if (($license_info['class'] == "TRIAL") || ($license_info['class'] == "NFR")) {
					# Notifications for NFR and TRIAL license expiry.
					# -- Notify to when 30 days left.
					# -- Notify to when 15 days left.  
					# -- Notify to when 7 days left.  
					# -- Notify to when 3 days left.  
					# -- Notify to when 2 days left.  
					# -- Notify to when 1 day left.  
					# -- Notify once that you are in the grace license period.  

					$grace_time =  259200; // 3 days trial grace.
					$time_left = $real_left - $grace_time;
					#if license has not expired
					if (!alert_exists('Process', 'license', 20)) {
						if (($time_left <= 0) && $real_left > 0) { // Grace Alert. 
							if (!alert_exists('Process', 'License warning', 134)) {
								#close/resolve existing alerts
								bp_send_notification(19, 'License warning');

								bp_send_notification(134, 'License warning', $license_info['class'], $license_info['class']);
							}
						} else if ($time_left <= 86400) { // 1 day left
							if (!alert_exists('Process', 'License warning', 128)) {
								#close/resolve existing alerts
								bp_send_notification(19, 'License warning');

								bp_send_notification(128, 'License warning', $license_info['class'], $license_info['class']);
							}
						} else if ($time_left <= 172800) { // 2 days left
							if (!alert_exists('Process', 'License warning', 129)) {
								#close/resolve existing alerts
								bp_send_notification(19, 'License warning');

								bp_send_notification(129, 'License warning', $license_info['class'], $license_info['class']);
							}
						} else if ($time_left <= 259200) { // 3 days left
							if (!alert_exists('Process', 'License warning', 130)) {
								#close/resolve existing alerts
								bp_send_notification(19, 'License warning');

								bp_send_notification(130, 'License warning', $license_info['class'], $license_info['class']);
							}
						} else if ($time_left <= 604800) { // 7 days left
							if (!alert_exists('Process', 'License warning', 131)) {
								#close/resolve existing alerts
								bp_send_notification(19, 'License warning');

								bp_send_notification(131, 'License warning', $license_info['class'], $license_info['class']);
							}
						} else if ($time_left <= 1296000) { // 15 days left
							if (!alert_exists('Process', 'License warning', 132)) {
								#close/resolve existing alerts
								bp_send_notification(19, 'License warning');
		
								bp_send_notification(132, 'License warning', $license_info['class'], $license_info['class']);
							}
						} else if ($time_left <= 2592000) { // 30 days left
							if ((!alert_exists('Process', 'License warning', 133) && $license_info['class'] == "TRIAL")) {
								# close/resolve existing alerts
								# Thank you note for trail.
								bp_send_notification(19, 'License warning');
			
								bp_send_notification(133, 'License warning');
							}
							if ((!alert_exists('Process', 'License warning', 135) && $license_info['class'] == "NFR")) {
								# close/resolve existing alerts
								# License Warning. 
								bp_send_notification(19, 'License warning');
			
								bp_send_notification(135, 'License warning');
							}
						} else { 
							#time left is > 30 days
							#close/resolve existing alerts
							bp_send_notification(19, 'License warning');
						}
					} else { 
						#close/resolve existing alerts
						bp_send_notification(19, 'License warning');
						# All the grace period is over. Now, 'TRIAL' or 'NFR' license has expired, 
						# Drop the appliance license to unlimited 'FREE', i.e it will never expire. 
						if ($real_left <= 0) {  
							bp_revert_license_to_free();
						}
					}
				} else {
					#close/resolve existing alerts
					bp_send_notification(19, 'License warning');
				}
			} else {
				#close/resolve existing alerts
				bp_send_notification(19, 'License warning');
			}
		}
	}
}

function processStorageStatusAlert($dbconn) {

	$storage_list = bp_get_storage_list();

	if (count($storage_list) != 0) {
		foreach($storage_list as $storage_name) {
			system("php /usr/bp/bin/cmc_storage status $storage_name", $status_retval);
		
			if($status_retval != 0) {
				pg_query($dbconn, "UPDATE bp.storage SET is_online = true WHERE storage_name = '$storage_name' AND is_online = false");

				if(!alert_exists('Disk', $storage_name, 83)) {
					bp_send_notification(83, $storage_name, $storage_name);
				}
			} else {
				pg_query($dbconn, "UPDATE bp.storage SET is_online = false WHERE storage_name = '$storage_name' AND is_online = true");

				if(!alert_exists('Disk', $storage_name, 84)) {
					bp_send_notification(84, $storage_name, $storage_name);			
				}
			}
		}
	}
}

function get_instance_fullname($dbconn, $instance_id) {
	#Get Node No, Application Name, Appliacation Instance name
	$instance_res = pg_query($dbconn, "SELECT node_no, key1, key2 from bp.application_instances where instance_id='$instance_id'");
	$instance_row = pg_fetch_row($instance_res);
	$node_no = $instance_row[0];
	$key_1 = $instance_row[1];
	$key_2 = $instance_row[2];
        		
	#Get Node Name
	$node_res = pg_query($dbconn, "SELECT CASE WHEN (properties & 32)=0 THEN node_name ELSE gcname END from bp.nodes where node_no='$node_no'");
	$node_row = pg_fetch_row($node_res);
	$node_name = $node_row[0];
	
	#Form a fully qualified name.
	if (($key_2 === NULL) && ($key_1 === NULL)) {
		$full_name = "$node_name";
	} else if (($key_2 === NULL)) {
		$full_name = "$node_name::$key_1";
	} else {
		$full_name = "$node_name::$key_1::$key_2";
	}
	return $full_name;
}

function processRetentionLimitsAlerts($dbconn) {
	#Get all intances from database.
        $query = pg_query($dbconn, "SELECT DISTINCT instance_id from bp.application_instances WHERE current=TRUE");
        for($i = 0; $i < pg_num_rows($query); $i++) {
                $row = pg_fetch_row($query);
                $instance_id = $row[0];

                #Find min and max retention limits for the instance.  
                $res = pg_query($dbconn, "SELECT min_limit, max_limit from bp.retention_limits WHERE instance_id = '$instance_id'");
                if ($res and pg_num_rows($res) > 0) {
			$res_row = pg_fetch_row($res);
			#Convert limits in days to secs.
			$min_limit = $res_row[0];
			$max_limit = $res_row[1]; 
                } else {
			$min_limit = 0;
			$max_limit = 0;
                }

                #Skip when retention limits are not configured.
                if (($min_limit == 0) && ($max_limit == 0)) {
			if (alert_exists('Retention', $instance_id, 101)) {
				#Close min alert.
				bp_send_notification(100, $instance_id);
			}
			if (alert_exists('Retention', $instance_id, 102)) {
				#Close max alert.
				bp_send_notification(100, $instance_id);
			}
			continue;
                }
		
		#Find actual retention for the instance.
                $rec = bp_get_oldest_backup($instance_id);
                if ($rec == -1) {
			continue;
		}
		#Calculate Age of oldest backup.
                $now = time();
                if (count($rec) == 0) {
			$age = 0;
                } else {
                	$age = $now - $rec['0']['start_time'];
		}
                
		if (($max_limit != 0) && ($age > ($max_limit*86400))) {
			if (!alert_exists('Retention', $instance_id, 102)) {
				$full_name = get_instance_fullname($dbconn, $instance_id);
				bp_send_notification(102, $instance_id, $full_name, $max_limit);
			}
			continue;
		} else {
			if (alert_exists('Retention', $instance_id, 102)) {
				#Close max alert.
				bp_send_notification(100, $instance_id);
			}
		}

		if ($age < ($min_limit*86400)) {
			#Calculate age of the instance, and  
			#Make sure instance age is not less than min limit.
                	$instance_res = pg_query($dbconn, "SELECT  discovery_time from bp.application_instances WHERE instance_id = '$instance_id'");
                	if ($instance_res and pg_num_rows($instance_res) > 0) {
				$instance_row = pg_fetch_row($instance_res);
				$instance_age = $now - $instance_row[0];
				if ($instance_age < ($min_limit*86400)) {
					continue;
				}
			} 
			if (!alert_exists('Retention', $instance_id, 101)) {
				$full_name = get_instance_fullname($dbconn, $instance_id);
				bp_send_notification(101, $instance_id, $full_name, $min_limit);
			}
		} else {
			if (alert_exists('Retention', $instance_id, 101)) {
				#Close min alert.
				bp_send_notification(100, $instance_id);
			}
		}
	}
}

function processVaultSpaceAlerts($dbconn) {
	global $started;
	$dpu_id = bppa_get_appliance_id();
	
	if ($dpu_id == "DPV" || $dpu_id == "DPXV") {
		system("php /var/www/html/recoveryconsole/bpl/dpvlicense.php -c > /tmp/tempVC.xml", $vc_ret);

		$lic_info = bp_get_license_info();
		if (($lic_info !== false) && isset($lic_info['class'])) {
			$lic_class =  $lic_info['class'];
		}
		else {
			$lic_class = "unknown";
		}

		# Only perform these checks for physical units and
		# UEBs with Enterprise capacity licenses.
		if ($vc_ret == 0 && ($lic_class == "PHY" || $lic_class == "ENTRB")) {
			if (file_exists('/tmp/tempVC.xml')) {
				$processVaultUsage = false;
				$today = getdate();

				#Check if it is the 1st of the month or the 1st day of the week(Monday)
				#Then check if it has already run today (only run once)
				if (($today['mday'] == 1) || ($today['wday'] == 1)) {
					# Check when to process these--we don't do it every alertman run
					$lastrun = bppa_db_last_run($dbconn);
					$versChecks = get_master_ini_setting('Alertman', 'VersionInfoChecksPerDay', '1');
					if ($versChecks > 0) {
						/* if we've run before, only check if we're supposed to, otherwise (1st alert run) we check */ 
						if (count($lastrun) > 0) {
							$checkTime = get_master_ini_setting('Alertman', 'VersionInfoCheckBaseTime', '23:00');
							$checkTimestamp = strtotime($checkTime);
							$lastStarted = $lastrun[0]["started"];
							/* How many seconds from when we should have started a version check? */
							$versCheckInterval = 86400 / $versChecks;
							$thisOffsetFromCheck = (($started + 86400 - $checkTimestamp) % 86400) % $versCheckInterval;
							$lastOffsetFromCheck = (($lastStarted + 86400 - $checkTimestamp) % 86400) % $versCheckInterval;
							if ($thisOffsetFromCheck <= $lastOffsetFromCheck) {
								$processVaultUsage = true;
							}
						} else {
							$processVaultUsage = true;
						}
					}
				}

				$dpu_used_list = array();
				$total_used = 0;
				$xml = simplexml_load_file('/tmp/tempVC.xml');
				$dpv_total = $xml->DPV['Available'];

				foreach ($xml->DPV->DPU as $DPU) {
					$dpu_used_list[] = $DPU->Used;

					#Send vaulted client storage usage for each client.			
					if ($processVaultUsage) {
						bp_send_notification(111, 'Vault Client Usage', $DPU->Name, $DPU->Used);
					} 
				}

				foreach ($dpu_used_list as $value) {
					$total_used += $value;
				}

				$temp_num = $total_used / $dpv_total;
				$percentage = number_format($temp_num * 100, 0);

				shell_exec("rm -f /tmp/tempVC.xml");

				/**Code for Vault Enforcement**
				** To be implemented at a later time
				//Grabs the list of valid user home directories in /backups
				exec('ls /backups', $dir_list);
				foreach ($dir_list as $DIR) {
					system("grep -w $DIR /backups/.DPUusers 2>/dev/null 1>&2", $users_ret);
					if ($users_ret == 0) { #finds 
						$user_list[] = $DIR;
					}
				}
				**/
					
				if ($lic_class == "PHY") {
					$lic_msg = "Please contact your Vault Service Provider for further information.";
				}
				else { #ENTRB
					$lic_msg = "Please upgrade your license by clicking the Upgrade button in the user interface.";
				}

				if ($percentage >= 110) {
					#VC has been exceeded (110% of VC)
					if (!alert_exists('Disk', 'Vault License', 86)) {
						#Close process + alert
						bp_send_notification(85, 'Vault License');
						bp_send_notification(86, 'Vault License', $dpv_total, $total_used, $lic_msg);
					}
					
					/**Code for Grace Period/Vault Enforcement**
					** To be implemented at a later time
					**
					# Grace Period
					$q = "SELECT created FROM bp.alerts WHERE notification_id = 86";
					$result = pg_query($dbconn, $q);
					$nrows = pg_num_rows($result);

					if ($nrows != 0) {
						/* Uses the epoch time at which the alert was created
						 * and grabs the current epoch time. Gets the difference
						 * and activates vault enforcement after 6 days from creation
						 * of the alert. 
						*/

					/*****
						$time_created = pg_fetch_all_columns($result);
						$current_time = time();
						$time_difference = $current_time - $time_created[0];

						# Enforces vaulting restrictions and sends an alert/trap/e-mail stating that vaulting has been disabled
						if ($time_difference >= 518400) { // 518400 = 6days
							/* Vaulting enforcement
							 * Creates .noSync file to user's home directory
 							 * to disable vaulting operations 
							*/
					/*****

							foreach ($user_list as $users) {
								if (!file_exists("/backups/$users/.noSync")) {
									shell_exec("echo 'Vaulting has been disabled due to space limitations on the vault, Please contact your Vault Service Provider for further information.' > /backups/$users/.noSync");
								}
							}

							if (!alert_exists('Disk', 'Vault License', 90)) {
								# Close process + alert
								bp_send_notification(85, 'Vault License');
	
								# Vault has exceeded 110% and grace period (6 days) is over
								bp_send_notification(90, 'Vault License');
							}
						} else {
							$grace_used = number_format($time_difference / 86400, 0); //86400 = 1day
							$grace_initial = 6; // 6 days
							$grace_previous = get_master_ini_setting('Alertman', 'VaultGracePeriod', '');
							$grace_left = $grace_initial - $grace_used;

							# E-mail sent counts down the number of days left in the grace period
							if($grace_left != $grace_previous) {
								#VC has been exceeded (110% of VC)
								if (!alert_exists('Disk', 'Vault License', 86)) {
									#Close process + alert
									bp_send_notification(85, 'Vault License');
								}
								bp_send_notification(86, 'Vault License', $dpv_total, $total_used, $grace_left);

							}
							# Stores the previous value of the number of days left in grace period
							put_master_ini_setting('Alertman', 'VaultGracePeriod', $grace_left);
						}
					} 
					*****/
				} else if ($percentage >= 80) {
					#Capacity is at 80% of VC
					if (!alert_exists('Disk', 'Vault License', 87)) {
						#Close process + alert
						bp_send_notification(85, 'Vault License');
					
						bp_send_notification(87, 'Vault License', $dpv_total, $total_used, $percentage, $lic_msg);
					}
					
					/**Code to be implemented at later time**
					#Vaulting enforcement removal
					foreach ($user_list as $users) {
						if (file_exists("/backups/$users/.noSync")) {
							shell_exec("rm -f /backups/$users/.noSync");
						}
					}
					put_master_ini_setting('Alertman', 'VaultGracePeriod', '');
 					**/
				} else {
					#Capacity is below 80% of VC
					#Closing process + alerts
					bp_send_notification(85, 'Vault License');


					/**Code to be implemented at later time**				
					#Vaulting enforcement removal
					foreach ($user_list as $users) {
						if (file_exists("/backups/$users/.noSync")) {
							shell_exec("rm -f /backups/$users/.noSync");
						}
					}
					put_master_ini_setting('Alertman', 'VaultGracePeriod', '');
					**/
				}
			}
		}
	}
}

function processReplicationAlerts($dbconn) {
	global $started;
	global $ui_type;

	// Get the max backlog number of days
	$BLDays = get_master_ini_setting('Replication', 'NotifyBacklogDays', 5);
	
	// Subtract this many days from now
	$age = $started - ($BLDays * 86400);

	$query = "SELECT count(*) FROM bp.replication_queue r JOIN bp.backups ";
	$query.= "USING(backup_no) WHERE r.status = 1 ";
	$query.= "AND (start_time + elapsed_time) < $age";
	$res = pg_query($dbconn, $query);
	if ($res == false) {
		return;
	}

	$nbackups = pg_fetch_result($res, 0, 0);
	if ($nbackups > 10) {
		switch ($ui_type) {
			case 'CE':
				// placeholder for notification
				break;
			case 'Satori':
				bp_send_notification(145, "Replication", $BLDays);
				break;
			case 'RRC':
				bp_send_notification(145, "Replication", $BLDays);
				break;
		}
	}
	else {
		// Clear Process alert for Replication target
		bp_send_notification(19, "Replication");
	}
	return;
}


# This trap is inefficient, re-code. {Shutoff}
function processOpenVPNTrap($dbconn) {
	global $hostname;

	$q = "SELECT properties from bp.nodes ";
	$result = pg_query($dbconn, $q);
	$nrows = pg_num_rows($result);
	$vault_enabled = false;

	if($nrows !=0) {
		$vault_status_list = pg_fetch_all_columns($result);

		for($i=0; $i < $nrows; $i++) {
		
			#Find at least 1 client with vaulting enabled
			if(($vault_status_list[$i] & 1) == 1) {
				$vault_enabled = true;
				break 1;
			} 
		}	

		if($vault_enabled) {
			
			#check if openvpn is running (check ifconfig for tunnel device)
			system('/sbin/ifconfig | grep tun 2>/dev/null 1>&2', $retval);

				/* sends trap only if openvpn is not running and there
				 * is at least 1 client with vaulting enabled 
				*/ 
				if($retval != 0) {
					bp_send_notification(77, $hostname);
				}
		} 
	}		
}

function processGoodBackups($dbconn, $begin, $end) {
	global $snmp_debug;
	global $bploc;

	/* This function finds good backups in the time range specified by the
	 * $begin and $end parameters, which should be the start of the
	 * previous run and the start of this one.  If any are found, it sends
	 * a notification. It does NOT raise alerts, since that is handled 
	 * elsewhere.
	 */
	$q = "SELECT node_name, b.properties, b.type, b.instance_id FROM bp.backups AS b ";

	// JOB_FINAL=32856 - these are non-active (complete) backups.
	// (ST_JOB_FINAL|ST_CANCELLED|ST_CANCELLED_SENT|ST_PROC_DIED)
	$q .= "JOIN bp.nodes n USING (node_no) WHERE (b.status & 32856) > 0 AND ";
	$q .= "(n.properties & 32) = 0 AND "; // not P_IMPORTED

	// P_SYNTHETIC=1  - these are synthetic backups.
	// TAPE_PURGE_NUMBER=4294967295  - these are logically deleted backups.
	$q .= "(b.properties & 1) = 0 AND tape_no != 4294967295 AND ";

	// ST_SUCCESS=2048 Minimum for success, may have others like ST_WARNINGS
	// (ST_SUCCESS|ST_COMPLETE|ST_JOB_FINAL|ST_SUMMARY_RECVD)=3168
	// Bug 7625: look for backups that completed in this time range, not
	// those that started in the window.
	$q .= "(b.status & 2048) = 2048 AND (start_time + elapsed_time) >= " . $begin;
	$q .= " AND (start_time + elapsed_time) < " . $end . " order by backup_no ";
	if ($snmp_debug) {
		print "processGoodBackups: query is\n $q\n";
	}
	$result = pg_query($dbconn, $q);
	if ($result === false) {
		return false;
	}

	$nrows = pg_num_rows($result);
	if ($nrows == 0) {
		if ($snmp_debug) {
			print "processGoodBackups: No good backups found.\n";
		}
		return true;
	}
	else {
		if ($snmp_debug) {
			print "processGoodBackups: $nrows good backup(s) found.\n";
		}
	}

	while ($row = pg_fetch_row($result)) {
		$node = $row[0];
		if (is_null($node)) {
			$node = '(unknown)';
		}
		$object = $node;
		$prop = $row[1];
		$btype = $row[2];
		$inst_id = $row[3];
		$instname = bp_get_appinst_name($inst_id);
		switch ($btype) {
			case 1: // BT_MAST
				$btypestr = "Master backup";
				break;
			case 2: // BT_DIFF
				$btypestr = "Differential backup";
				break;
			case 4: // BT_SELE
				if ($prop & 128) { // P_SQL=128
					$btypestr = "Selective SQL Server backup";
				} else {
					$btypestr = "Selective client backup";
				}
				break;
			case 16: // BT_RESTORE
			case 32: // BT_RINC
			case 64: // BT_RSOME
				$btypestr = "client Restore";
				break;
			case 96: // BT_VIRT
				$btypestr = "Virtual restore";
				$object = $node . " (WIR)";
				break;
			case 128: // BT_VERIFY
			case 512: // BT_SIMPLE_VERIFY
				$btypestr = "Backup verify";
				break;
			case 1024: // BT_IMAGE_BACKUP
				$btypestr = "BareMetal backup";
				break;
			case 2048: // BT_IMAGE_RESTORE
				$btypestr = "BareMetal restore";
				break;
			case 3: // BT_INC4EVER
				$btypestr = "Incremental forever backup";
				$object = $node . " (Incremental)";
				break;
			case 5:    // BT_EXCHANGE_FULL
			case 6:    // BT_EXCHANGE_DIFF
			case 7:    // BT_EXCHANGE_INCR
				$btypestr = "Exchange backup";
				$object = $node . "(Exchange $instname)";
				break;
			case 9:    // BT_MSSQL_FULL
			case 10:   // BT_MSSQL_DIFF
			case 11:   // BT_MSSQL_TRANS:
				$btypestr = "MSSQL backup";
				$object = $node . " (MSSQL $instname)";
				break;
			case 12:   // BT_VMESX_FULL
			case 13:   // BT_VMESX_DIFF
			case 14:   // BT_VMESX_INCR
			case 15:   // BT_VMESX_CONFIG
				$btypestr = "VMware backup";
				$object = $node . " (VMware $instname)";
				break;
			case 1031: // BT_HYPERV_FULL
			case 1032: // BT_HYPERV_DIFF
			case 1033: // BT_HYPERV_INCR
				$btypestr = "Hyper-V backup";
				$object = $node . " (Hyper-V $instname)";
				break;
			case 1034: // BT_ORACLE_FULL
			case 1035: // BT_ORACLE_INCR
				$btypestr = "Oracle backup";
				$object = $node . " (Oracle $instname)";
				break;
			case 1036: // BT_SHAREPT_FULL
			case 1037: // BT_SHAREPT_DIFF
				$btypestr = "SharePoint backup";
				$object = $node . " (SharePoint $instname)";
				break;
			case 1038: // BT_APPLN_META
				$btypestr = "System Metadata backup";
				$object = $node . " (System Metadata)";
				break;
			case 4101: // BT_XEN_FULL
				$btypestr = "Xen backup";
				$object = $node . " (Xen $instname)";
				break;
			default: // just in case...
				$btypestr = "Backup operation";
				$object = $node . " ($instname)";
				break;
		}
		$trapstr = "Successful " . $btypestr . " $object";
		if ($snmp_debug) {
			print "node: $node, string: $trapstr\n";
		}
		$ret = bp_send_notification(176 , $node, $trapstr);
		if ($snmp_debug) {
			// Continue anyway if this fails 
			print "Return value from bp_send_notification: $ret\n";
		}
	}
	return true;
}

function processFailedBackups($dbconn, $begin, $end) {
	global $snmp_debug;
	global $bploc;
	global $ui_type;

	/* This function finds failed backups in the time range specified by the
	 * $begin and $end parameters, which should be the start of the
	 * previous run and the start of this one.  If any are found, it sends
	 * an SNMP trap and a PSA ticket (if either are configured). It does NOT
	 * raise alerts, this info is available in the RRC via front-page status.
	 */


	$q = "SELECT node_name, b.properties, b.type, b.instance_id FROM bp.backups AS b ";

	// JOB_FINAL=32856 - these are non-active (complete) backups.
	// (ST_JOB_FINAL|ST_CANCELLED|ST_CANCELLED_SENT|ST_PROC_DIED)
	$q .= "JOIN bp.nodes n USING (node_no) WHERE (b.status & 32856) > 0 AND ";
	$q .= "(n.properties & 32) = 0 AND "; // not P_IMPORTED

	// P_SYNTHETIC=1  - these are synthetic backups.
	// TAPE_PURGE_NUMBER=4294967295  - these are logically deleted backups.
	$q .= "(b.properties & 1) = 0 AND tape_no != 4294967295 AND ";

	// ST_SUCCESS=2048 Minimum for success, may have others like ST_WARNINGS
	// (ST_SUCCESS|ST_COMPLETE|ST_JOB_FINAL|ST_SUMMARY_RECVD)=3168
	// Bug 7625: look for backups that completed in this time range, not
	// those that started in the window.
	$q .= "(b.status & 2048) = 0 AND (start_time + elapsed_time) >= " . $begin;
	$q .= " AND (start_time + elapsed_time) < " . $end . " order by backup_no ";

	if ($snmp_debug) {
		print "processFailedBackups: query is\n $q\n";
	}

	$result = pg_query($dbconn, $q);
	if ($result === false) {
		return false;
	}

	$nrows = pg_num_rows($result);
	if ($nrows == 0) {
		if ($snmp_debug) {
			print "processFailedBackups: No failed backups found.\n";
		}
		return true;
	}
	else {
		if ($snmp_debug) {
			print "processFailedBackups: $nrows failed backup(s) found.\n";
		}
	}

	while ($row = pg_fetch_row($result)) {
		$node = $row[0];
		if (is_null($node)) {
			$node = '(unknown)';
		}
		$object = $node;
		$prop = $row[1];
		$btype = $row[2];
		$inst_id = $row[3];
		$instname = bp_get_appinst_name($inst_id);

		if ($snmp_debug) {
			print "node: $node, properties: $prop, type: $btype\n";
			print "instance_id: $inst_id, name: $instname\n";
		}

		switch ($btype) {
			case 1: // BT_MAST
				# notify_id 54: Master backup for client %s failed.
				switch ($ui_type) {
					case 'CE':
						$notify_id = 178;
						break;
					case 'Satori':
						$notify_id = 54;
						break;
					case 'RRC':
						$notify_id = 54;
						break;
				}
				break;
			case 2: // BT_DIFF
				# notify_id 55: Differential backup for client %s failed.
				switch ($ui_type) {
					case 'CE':
						$notify_id = 179;
						break;
					case 'Satori':
						$notify_id = 55;
						break;
					case 'RRC':
						$notify_id = 55;
						break;
				}
				break;
			case 4: // BT_SELE
				if ($prop & 128) { // P_SQL=128
					# notify_id 57: SQL Server backup for client %s failed.
					$notify_id = 57;
				}
				else {
					# notify_id 56: Selective backup for client %s failed.
					switch ($ui_type) {
						case 'CE':
							$notify_id = 180;
							break;
						case 'Satori':
							$notify_id = 56;
							break;
						case 'RRC':
							$notify_id = 56;
							break;
					}
				}
				break;
			case 16: // BT_RESTORE
			case 32: // BT_RINC
			case 64: // BT_RSOME
				# notify_id 58: Restore for client %s failed.
				$notify_id = 58;
				break;
			case 96: // BT_VIRT
				$notify_id = 58;
				$object = $node . " (WIR)";
				break;
			case 128: // BT_VERIFY
			case 512: // BT_SIMPLE_VERIFY
				# notify_id 59: Backup verify for client %s failed.
				$notify_id = 59;
				break;
			case 1024: // BT_IMAGE_BACKUP
				# notify_id 60: BareMetal backup for client %s failed.
				$notify_id = 60;
				break;
			case 2048: // BT_IMAGE_RESTORE
				# notify_id 61: BareMetal restore for client %s failed.
				$notify_id = 61;
				break;
			case 3: // BT_INC4EVER
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " (Incremental)";
						break;
					case 'Satori':
						$notify_id = 62;
						$object = $node . " (Incremental)";
						break;
					case 'RRC':
						$notify_id = 62;
						$object = $node . " (Incremental)";
						break;
				}
				break;
			case 5:    // BT_EXCHANGE_FULL
			case 6:    // BT_EXCHANGE_DIFF
			case 7:    // BT_EXCHANGE_INCR
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " (Exchange $instname)";
						break;
					case 'Satori':
						$notify_id = 62;
						$object = $node . " (Exchange $instname)";
						break;
					case 'RRC':
						$object = $node . " (Exchange $instname)";
						$notify_id = 62;
						break;
				}
				break;
			case 9:    // BT_MSSQL_FULL
			case 10:   // BT_MSSQL_DIFF
			case 11:   // BT_MSSQL_TRANS:
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " (MSSQL $instname)";
						break;
					case 'Satori':
						$object = $node . " (MSSQL $instname)";
                                		$notify_id = 62;
                                		break;
					case 'RRC':
						$object = $node . " (MSSQL $instname)";
                                		$notify_id = 62;
                                		break;
				}
				break;
			case 12:   // BT_VMESX_FULL
			case 13:   // BT_VMESX_DIFF
			case 14:   // BT_VMESX_INCR
			case 15:   // BT_VMESX_CONFIG
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " (VMware $instname)";
						break;
					case 'Satori':
						$notify_id = 62;
						$object = $node . " (VMware $instname)";
						break;
					case 'RRC':
						$object = $node . " (VMware $instname)";
						$notify_id = 62;
						break;
				}
				break;
			case 1031: // BT_HYPERV_FULL
			case 1032: // BT_HYPERV_DIFF
			case 1033: // BT_HYPERV_INCR
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " (Hyper-V $instname)";
						break;
					case 'Satori':
						$notify_id = 62;
						$object = $node . " (Hyper-V $instname)";
						break;
					case 'RRC':
						$object = $node . " (Hyper-V $instname)";
						$notify_id = 62;
						break;
				}
				break;
			case 1034: // BT_ORACLE_FULL
			case 1035: // BT_ORACLE_INCR
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " (Oracle $instname)";
						break;
					case 'Satori':
						$notify_id = 62;
						$object = $node . " (Oracle $instname)";
						break;
					case 'RRC':
						$object = $node . " (Oracle $instname)";
						$notify_id = 62;
						break;
				}
				break;
			case 1036: // BT_SHAREPT_FULL
			case 1037: // BT_SHAREPT_DIFF
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " (SharePoint $instname)";
						break;
					case 'Satori':
						$notify_id = 62;
						$object = $node . " (SharePoint $instname)";
						break;
					case 'RRC':
						$object = $node . " (SharePoint $instname)";
						$notify_id = 62;
						break;
				}
				break;
			case 1038: // BT_APPLN_META
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " (System Metadata)";
						break;
					case 'Satori':
						$notify_id = 62;
						$object = $node . " (System Metadata)";
						break;
					case 'RRC':
						$object = $node . " (System Metadata)";
						$notify_id = 62;
						break;
				}
				break;
			case 4101:   // BT_XEN_FULL
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " (Xen $instname)";
						break;
					case 'Satori':
						$notify_id = 62;
						$object = $node . " (Xen $instname)";
						break;
					case 'RRC':
						$object = $node . " (Xen $instname)";
						$notify_id = 62;
						break;
				}
				break;
			default: // just in case...
				# notify_id 62: Backup operation for client %s failed.
				switch ($ui_type) {
					case 'CE':
						$notify_id = 181;
						$object = $node . " ($instname)";
						break;
					case 'Satori':
						$notify_id = 62;
						$object = $node . " ($instname)";
						break;
					case 'RRC':
						$object = $node . " ($instname)";
						$notify_id = 62;
						break;
				}
				break;
		}
		if ($snmp_debug) {
			 print "notify_id: $notify_id, node: $node, object: $object\n";
		}

		$ret = bp_send_notification($notify_id, $node, $object);
		if ($snmp_debug) {
			// Continue if this "fails" because it will return an error
			// code if it couldn't send to any of the destinations,
			// although it may have succeeded sending to others.
			print "Return value from bp_send_notification: $ret\n";
		}
	}
	
	return true;
}

/* Sends notifications for completed archiving jobs */
function processArchivingAlerts($dbconn) {
	global $bploc;
	/* Only one archive job can be running at once, so close all alerts except 
	 * one that may be running, as found in /usr/bp/lists.dir/.archivecurrent
	 */
	$statusfile="$bploc/lists.dir/.archivecurrent";
	$source_id = get_source_id('Archiving');
	$query = pg_query($dbconn, "SELECT target FROM bp.alerts WHERE source_id = $source_id AND closed = FALSE");
	if ($query and pg_num_rows($query)) {
		# Some alerts may need to be closed
        $job = trim(file_get_contents("$bploc/lists.dir/.archivecurrent"));
		if ($job != '') {
			$arr = explode(' ', $job);
			$profile = $arr[1];
			$pid = $arr[0];
		} else {
			$profile = '';
			$pid = 0;
		}
		$alerts = pg_fetch_all($query);
		foreach ($alerts as $alert) {
			# Check every alert to see if the referenced job is running
			if ($alert['target'] == $profile) {
				system("pgrep archive | grep $pid", $ret);
				if ($ret != 0) {
					bp_send_notification(66, $alert['target'], $alert['target']);
					unlink($statusfile);
				}
			} else {
				bp_send_notification(66, $alert['target'], $alert['target']);
			}
		}
	}
}

/* Sends notifications for updated storage space */
function processStorageAllocationAlerts($dbconn) {
        $query = pg_query($dbconn, "select notification_id from bp.audit_history where message like 'Storage allocation for Backup has changed from 0.00%'");
        $e = pg_fetch_assoc($query);
        $f = $e['notification_id'];
        if ($f) {
                $q = pg_query($dbconn, "select * from bp.nvp where item_name='SCN'");
                if(!$q or pg_num_rows($q) == 0) {
                        bp_send_notification(188, 'SCN');
                        pg_query($dbconn, "insert into bp.nvp (nvp_type, nvp_name, item_name, item_value) values('system', 'configuration', 'SCN', true)");
                }
        }
}

function processOverloadAlert($last_start, $started) {

	// Do the check once a day, at 7am.
	$checkTimestamp = strtotime("07:00");
	if ($started < $checkTimestamp or $last_start >= $checkTimestamp) {
		return;
	}

	$intervalDays = (int)get_master_ini_setting("OverloadStats",
			"IntervalDays", 7, false);
	if ($intervalDays < 1 ) {
		// Change to miminum value of 1.
		$intervalDays = 1;
	}

	// The string in the message that describes the interval.
	if ($intervalDays > 1) {
		$day_str = $intervalDays." days";
	}
	else {
		$day_str = "day";
	}
			
	$threshold = (int)get_master_ini_setting("OverloadStats",
			"PercentageThreshold", 50, false);

	$intervalEnd = time();
	$intervalSecs = ((60 * 60 * 24) * $intervalDays);
	$intervalStart = $intervalEnd - $intervalSecs;

	// The API requires an array of arrays (it supports multiple intervals)
	// -- this is just sending one interval.
	$inputInterval = array();
	$inputInterval[0] = array("start_time" => $intervalStart,
			"end_time" => $intervalEnd);

	$ret = bp_get_overload_stats($inputInterval);
	if ($ret === false) {
		echo "bp_get_overload_stats FAILED: ".bp_error()."\n";
		return;
	}

	// Compute the percentage amount of time that was spent waiting for the
	// system load level to be acceptable for resuming work.  Multiply the
	// interval by the number of processors, since in theory, it could have
	// performed that many seconds of work.
	$numProcessors = (int)`cat /proc/cpuinfo | grep "^processor" | wc -l`; 
	$delay_sum = $ret[0];
	$delayPercentage = ($delay_sum * 100) / ($numProcessors * $intervalSecs);
	$delayPercentage = round($delayPercentage);

	# default alert text for physical system
	$msg = "purchasing another system to accommodate this workload";

	if (bp_is_virtual()) {
		# check for Linux Integration Components to verify a Hyper-V UEB
		system("lsmod | grep -w '^vmbus' >/dev/null 2>&1", $hyperv_ret);

		# Hyper-V has a 4 vCPU limit.  Once reached, no more vCPUs may be added.
		if ($hyperv_ret == 0 and $numProcessors >= 4) {
			$msg = "deploying another system to distribute the load";
		} else { // applies to both a VMware UEB and a Hyper-V UEB with < 4 vCPUs
			$msg = "adding more resources (virtual processors/memory) to the system";
		}
	}

	if ($delayPercentage >= $threshold) {
		// Raise the alert/trap
		bp_send_notification(93, 'Overload', $day_str, $msg);
	}
	else {
		if (alert_exists('Process', 'Overload')) {
			// notify_id 19: Resolve/close alerts for Process source
			bp_send_notification(19, 'Overload');
		}
	}
	return;
}

/* Deletes alerts that are now obsolete (e.g. alerts for removed clients, etc.) */
function resolveObsoleteAlerts($dbconn) {
	# deleted clients
	$source_id = get_source_id('Clients');
	if ($source_id !== false) {
		pg_query($dbconn, "DELETE FROM bp.inactive_clients WHERE name NOT IN (SELECT node_name FROM bp.nodes)");
		# Bug 8817: notification_id 106 uses <node_name>-vols for target,
		# notification_id uses 'BP Version' for target,
		# notification_id 112, 113, 120 use <node_name>-virtual for target,
		# notification_id 167 uses 'EmptySchedule'
		# don't close them here.
		pg_query($dbconn, "UPDATE bp.alerts SET closed = TRUE WHERE closed = FALSE AND source_id = $source_id" .
		                  " AND notification_id NOT IN (106, 112, 113, 88, 120, 141, 167, 190)".
		                  " AND target NOT IN (SELECT node_name FROM bp.nodes)");
	}

	# deleted DPUs
	$source_id = get_source_id('DPU');
	if ($source_id !== false) {
		pg_query($dbconn, "UPDATE bp.alerts SET closed = TRUE WHERE closed = FALSE AND source_id = $source_id" .
		                  " AND notification_id NOT IN (203,205,207,175)" .
		                  " AND target NOT IN (SELECT name FROM bp.systems)");
	}

	# deleted external storage
	$source_id = get_source_id('Disk');
	if ($source_id !== false) {
		pg_query($dbconn, "UPDATE bp.alerts SET closed = TRUE WHERE closed = FALSE AND notification_id = 84" .
				  " AND target NOT IN (SELECT storage_name FROM bp.storage)");
		
		pg_query($dbconn, "UPDATE bp.alerts SET closed = TRUE WHERE closed = FALSE AND notification_id = 12" .
				  " AND target LIKE '/media/cdrecorder%'");
	}
	
	# deprecated 3ware/MegaRAID alerts
	# Current RAID alerts reside in alert_source_id = "Disk Health"
	$source_id = get_source_id('3ware');
	if ($source_id !== false) {
		pg_query($dbconn, "UPDATE bp.alerts SET closed = TRUE WHERE closed = FALSE AND source_id = $source_id" .
				  " AND notification_id IN (2, 3, 4, 5, 6, 7, 137, 138)");
	}

	# deprecated Software RAID alerts
	# Current Software RAID alerts reside in alert_source_id = "Disk Health"
	$source_id = get_source_id('RAID');
	if ($source_id !== false) {
		pg_query($dbconn, "UPDATE bp.alerts SET closed = TRUE WHERE closed = FALSE AND source_id = $source_id" .
				  " AND notification_id IN (41, 42, 43, 44, 45, 46, 47, 48)");
	}
	
	# Delete min-max retention alerts for non-existing instances.
	$q = "DELETE FROM bp.alerts WHERE notification_id IN (101, 102) AND target NOT IN (SELECT DISTINCT instance_id::text FROM bp.application_instances WHERE current='t')";
	pg_query($dbconn, $q);
}

function r300_kernel_update($dbconn) {
	# run the script to install unitrends-drivers on the Recovery-300 if it hasn't been run in 24h
	$debug = 0;

	system("grep -qvs 'Recovery-300' /etc/unitrends-release", $r300);
	if ($r300 == 0) {
		return false;
	}

	$logfile = "/var/log/drivers-update-4.1.log";

	if (file_exists($logfile)) {
		$logstat = stat($logfile);
		if ($logstat != FALSE) {
			$mtime = $logstat['mtime'];
		} else {
			$mtime = 0;
		}
	} else {
		$mtime = 0;
	}
	$now = time();

	if ($debug) {
		print "mtime: $mtime\n";
	}

	if ((($now - $mtime) > 60*60*24) or $mtime = 0) {
		# has not been run in 24 hours, or never run
		exec("/usr/bp/bin/drivers-update-4.1", $return, $retval);

		if ($retval == 0) {
			# successfully updated
			# close failed alert
			# notify_id 19: Resolve/close alerts for Process source + target
			bp_send_notification(19, 'drivers-update-4.1');
			# notify_id 33: A software update requires the DPU to be rebooted. 
			switch ($ui_type) {
				case 'CE':
					// placeholder for alert notification
					break;
				case 'Satori':
					bp_send_notification(33, 'reboot');
					break;
				case 'RRC':
					bp_send_notification(33, 'reboot');
					break;
			}
		} else if ($retval == 1) {
			# update failed, will be retried
		} else if ($retval == 2) {
			# no update needed, close alert
			# notify_id 19: Resolve/close alerts for Process source + target
			bp_send_notification(19, 'drivers-update-4.1');
		}
	}
}

#----------------------------------------
# Alert raise/close/mail functions
#----------------------------------------

/* Mails an alert.  If bputil can't give us the email address we try to get it
 * using the php ini parsing functions, since the database might be down.
 */
function mail_alert($subject, $body) {
	global $bploc;

	exec("uname -n", $hostname, $retval);
	$hostname = $hostname[0];

	$email_addr = get_master_ini_setting('Unattended Backup Information', 'MailTo', '', false);

	// if we didn't get the email address, try to get it ourselves
	if (!isset($email_addr[0]) || strlen($email_addr[0]) < 1) {
		$i = parse_ini_file($bploc.'/bpinit/master.ini', true);
		if (isset($i['Unattended Backup Information']['MailTo'])) {
			$mail_addr = $i['Unattended Backup Information']['MailTo'];
		}
	}
	// we still don't have the email address -- just send it to root
	if (!isset($email_addr[0]) || strlen($email_addr[0]) < 1) {
		$email_addr = 'root';
	}

	$headers = "From: reports@" . $hostname;

	mail($email_addr, $hostname . ": $subject", $body, $headers);
	return 0;
}


/* Returns the source id corresponding to the given source name, or false on failure */
function get_source_id($source) {
	global $dbconn;
	# get source id
	$query = pg_query($dbconn, "SELECT source_id FROM bp.alert_source WHERE source_name = '$source'");
	if (!$query or pg_num_rows($query) == 0) {
		return false;
	}
	$row = pg_fetch_row($query);
	$source_id = $row[0];
	return $source_id;
}

/* NOTE: this functionality is now implemented in the BPL library as a
 * subset of the bp_send_notification API.
 */
/* Raises an alert: If the source, target, text, and severity are the
 * same as an existing alert, the timestamp is updated.  Otherwise,
 * any existing alerts with the same source and target are closed and
 * the new alert is raised.
 *
 * Note: alert text substrings inside curly braces (e.g. {text}) will
 * be converted to .* in a search regular expression, so if there are
 * parts of an alert string that might change (e.g. how long a client
 * has been down), put it in braces so the text will match prior alerts
 * of the same type.
 */
/*function raise_alert($source, $target, $text, $severity, $closed = false) {
}
*/

/* NOTE: this functionality is now implemented in the BPL library as a
 * subset of the bp_send_notification API.  There is an internal
 * notification type for each alert_source with severity=0 that is used to
 * close any open alerts for that source and target.  The equivalent call
 * is:    bp_sendnotification($notification_id, $target).
function close_alert($source, $target) {
}
 */

// alert_exists and get_alert have moved inside alert_funcs.php
require("/usr/bp/bin/alert_funcs.php");

/*
 * Name: AgentDownload
 *
 * Description: Downloads and installs new agent installers from the Unitrends repo.
 * 
 * Returns: nothing
 */
function AgentDownload()
{
	global $dbconn;
	global $started;
	
	$runThisIteration = false;
	$retval = 0;
	
	$methodStarted = time();
	
	# process if we haven't run before
	$lastrun = bppa_db_last_run($dbconn);
	if (count($lastrun) == 0)
	{
		$runThisIteration = true;
		#echo "AgentDownload: Run this time! Lastrun count 0.\n";
	}

	# process if a new rpm was installed since the last run
	$output = null;
	$installed = trim(exec('rpm -q unitrends-rr --qf %{INSTALLTIME}', $output, $retval));

	$lastStarted = $lastrun[0]["started"];
	if ($installed >= $lastStarted)
	{
		$runThisIteration = true;
		#echo "AgentDownload: Run this time! Installed: $installed >= lastStarted: $lastStarted\n";
	}

	if($runThisIteration == false)
	{
		# process if we crossed the UpdateCheckTime
		$checkTime = get_master_ini_setting('Alertman', 'UpdateCheckTime', '03:00');
		$checkTimestamp = strtotime($checkTime);
		#echo "AgentDownload: if last started: $lastStarted < check timestamp: $checkTimestamp.\n";
		#echo "AgentDownload: and started: $started >= check timestamp: $checkTimestamp.\n";
		if ($lastStarted < $checkTimestamp and $started >= $checkTimestamp)
		{
			$runThisIteration = true;
			#echo "AgentDownload: Run this time! UpdateCheckTime.\n";
		}
	}
		
	if($runThisIteration == false)
	{
		#echo "AgentDownload: Do not run this iteration.\n";
		return;
	}
	
	# check and turn off EOL fca/beta repo if needed 
	eol_off();

	# check for new updates
	$cmd = "/usr/bp/bin/cmc_updates list | grep '^unitrends-agentpush'";
	$retval = 0;
	exec($cmd, $updateListArray, $retval);
	if ($retval == 0 and count($updateListArray) > 0)
	{
		foreach($updateListArray as $line)
		{
			$updateArray = preg_split('/\s+/', $line);
			$version = $updateArray[2];
			#echo "AgentDownload: update available for $updateArray[0]: $version. Installing.\n";
			# now install it
			$cmd = "/usr/bp/bin/cmc_updates install $updateArray[0]";
			exec($cmd, $installOutput, $retval);
			#if ($retval != 0)
			#{
			#	echo "AgentDownload: update installation failed for $updateArray[0]: $version\n";
			#}
		}
	}
	#else
	#{
	#	echo "AgentDownload: No updates found.\n";
	#}
	
	# Update agent push alerts
	AgentAlerts();
}

# Detect if there are multiple replication sources that have the same
# identity.  UEBs in the field have the same UUID, and could have been
# configured as replication sources. See bugs 17818 and 18049.  Send a
# trap once per day if the situation is detected, until it no longer
# exists. Uses the ClientConnectCheckBaseTime setting in master.ini as
# the time of day to perform this check.
#
function processDuplicateReplicationSourcesTrap($dbconn)
{
	global $started;
	$processDuplicatesTrap = false;

	$lastrun = bppa_db_last_run($dbconn);
	if (count($lastrun) > 0) {
		$checkTime = get_master_ini_setting('Alertman', 'ClientConnectCheckBaseTime',
						'23:00');
		$checkTimestamp = strtotime($checkTime);
		$lastStarted = $lastrun[0]["started"];
		$thisOffsetFromCheck = (($started + 86400 - $checkTimestamp) % 86400);
		$lastOffsetFromCheck = (($lastStarted + 86400 - $checkTimestamp) % 86400);
		if ($thisOffsetFromCheck <= $lastOffsetFromCheck) {
			$processDuplicatesTrap = true;
		}
	}
	else {
		$processDuplicatesTrap = true;
	}

	if ($processDuplicatesTrap == false) {
			return;
	}
	
	# Count how many replicating sources are in the systems table.
	$q = "SELECT COUNT(*) FROM bp.systems WHERE identity IS NOT NULL";
	$r = pg_query($dbconn, $q);
	$Nsrc = pg_fetch_result($r, 0, 0);
	if ($Nsrc == 0) {
		# No replication sources
		return;
	}

	# Count how many have unique identity values.
	$q = "SELECT COUNT(DISTINCT(identity)) FROM bp.systems WHERE identity IS NOT NULL";
	$r = pg_query($dbconn, $q);
	$Nuniq = pg_fetch_result($r, 0, 0);

	if ($Nsrc > $Nuniq) {
		bp_send_notification(166, 'DuplicateReplicationSources');
	}
	return;
}

#-----------------------------------------
# Begin processing
# main program starts here
#-----------------------------------------
# Only run populate_alerts if Initial Setup Wizard has been completed
$htaccess = '/var/www/html/.htaccess';
if (file_exists($htaccess)) {
	$ui_output = exec("grep -q ui $htaccess", $output, $ui_retval);
	if ($ui_retval == 0) {
	        # satori UI
	        $ui = "satori";
	        $wizard = bp_get_nvp_list('user_preference','root');
	        $popup = "show_setup_wizard";
	} else {
	        $ui = "rc";
	        $wizard = bp_get_nvp_list('preferences','root');
	        $popup = "SetupWizardPopup";
	}

	if ($wizard != NULL) {
	        if (isset($wizard[$popup])) {
	                if ($wizard[$popup] == 'true') { //rc initial setup wizard has not been completed
	                        exit (0);
	                }
	        } else if ($ui == "satori") { //satori initial setup wizard has not been completed
	                exit (0);
	        }
	} else if ($ui == "satori") { //satori eula/wizard has never been run
	        exit (0);
	}
} else {
	exit (1);
}

$hostname = trim(shell_exec('hostname'));
$snmp_debug = 0;
$started = time();
$bploc = trim(shell_exec('eval `grep BPDIR /etc/default/bp.ini`; echo $BPDIR'));
if (empty($bploc) === true) exit(1);

$dbconn = bppa_db_connect();

// $argv[1] is appname ("alertman")
if (isset($argv[1])) {
	bp_bypass_cookie(3, $argv[1]);
}
else {
	bp_bypass_cookie(3);
}

$lastrun = bppa_db_last_run($dbconn);

# Put /usr/local/bin into path if missing, for tw_cli
#if (strrpos($_ENV['PATH'], "/usr/local/bin") === FALSE) {
#	putenv("PATH=" . $_ENV['PATH'] . ":/usr/local/bin");
#}

# Check the notification_mgmt table to see if there has been a change in
# the revision, and if any migration should be performed.  There should
# only be one record in the table, but use ORDER & LIMIT in case there
# are more.  If migration is needed, call migrate_alerts.php.  If none is
# needed, finalize the notification_mgmt record by setting upgrade_from to
# be the same as the current revision.
$q = "SELECT * FROM bp.notification_mgmt ORDER BY revision DESC LIMIT 1";
$res = pg_query($dbconn, $q);
if ($res and pg_num_rows($res)) {
	$row = pg_fetch_assoc($res);
	if ($row['status'] == 'needs migration') {
		exec($bploc . '/bin/migrate_alerts.php', $out, $ret);
		if ($ret != 0) {
			print "Data migration for notification management failed.\n";
		}
		# print any message
		foreach($out as $line) {
			print "$line\n";
		}
	}
	else {
		if ($row['revision'] != $row['upgrade_from']) {
			$q = "UPDATE bp.notification_mgmt SET upgrade_from = ";
			$q .= $row['revision'];
			pg_query($dbconn, $q);
		}
	}
}
else {
	print "Failed to access notification_mgmt table!\n";
	print pg_last_error($dbconn)."\n";
}

$res = pg_query($dbconn, "SELECT status FROM bp.operation_status WHERE operation_id = 8");
if ($res and pg_num_rows($res) > 0) {
	$status = pg_fetch_result($res, 0, 0);
	if ($status != "done") {
		$release=shell_exec("rpm -q --queryformat '%{version}' unitrends-rr");
		$msg = "Migrated on upgrade to release ".$release." on ". date('D M j H:i');
		if (bp_migrate_legacy_replication_config($msg) !== false) {
			$res = pg_query($dbconn, "UPDATE bp.operation_status SET status = 'done' WHERE operation_id = 8");
		}
	}
}

$res = pg_query($dbconn, "SELECT status FROM bp.operation_status WHERE operation_id = 10");
if ($res and pg_num_rows($res) > 0) {
	$status = pg_fetch_result($res, 0, 0);
	if ($status != "done") {
		$cmd = $bploc . '/bin/fixEvents.php | tee -a ';
		$cmd .= $bploc. '/logs.dir/fixEvents.log';
		exec($cmd, $out, $ret);
		if ($ret != 0) {
			print "Failed to detect/fix events with negative dtstart values.\n";
		}
		# print any message
		foreach($out as $line) {
			print "$line\n";
		}
	}
}

$send_traps = 0;
$query = "SELECT COUNT(*) FROM bp.snmp_config WHERE enabled = true";
$snmp_result = pg_query($dbconn, $query);
if ($snmp_result !== false) {
	$send_traps = pg_fetch_result($snmp_result, 0, 0);
}
if ($snmp_debug) {
	echo "SNMP: send_traps = $send_traps\n";
}

if (!empty($lastrun)) {
	$last_start = $lastrun[0]["started"];
}
else {
	// go back 15 min
	$last_start = $started - 54000;
}

# UNIBP-4759 set ui_type before processFailedBackups
$ui = exec('cmc_capabilities get ui');
switch ($ui) {
	case 'satori':
		$ce = exec('cmc_capabilities get CE');
		if ($ce === 'CE') {
			$ui_type = 'CE';
		}
		else {
			$ui_type = 'Satori';
		}
		break;
	default:
		$ui_type = 'RRC';
		break;
}
$is_metal = exec('echo $(cat /etc/unitrends-asset | cut -b 1-5)');

if ($send_traps) {
	#processSmartInfoTraps();
	processCallHomeTrap();
	#processOpenVPNTrap($dbconn); //re-code
	processDuplicateReplicationSourcesTrap($dbconn);
	processVersionTrap($dbconn);
}

/* Notifications 54-62 (failed backups) may produce PSA tickets,
 * in addition to SNMP traps.
 */
processFailedBackups($dbconn, $last_start, $started);
$goodtraps = get_master_ini_setting('Configuration Options', 'GoodBackupTraps', 0);
if ($goodtraps == 1) {
   processGoodBackups($dbconn, $last_start, $started);
}

# 9056
processStorageStatusAlert($dbconn);
processVaultSpaceAlerts($dbconn);
#processMegaraidAlerts();
processFileLevelRecoveryAlerts($dbconn);
processVMwareIRAlerts();
processHyperVIRAlerts();
processClientAlerts($dbconn);
processPCIAlerts($dbconn);
#process3wareAlerts($dbconn);
#processMDraidAlerts($dbconn);
processDiskAlerts($dbconn);
processProcessAlerts($dbconn);
processVirtualFailoverAlerts($dbconn);
processLicenseExpAlerts();
processReplicationAlerts($dbconn);

AgentDownload();

# 4618
processRetentionLimitsAlerts($dbconn); 
processCEPAlerts($dbconn);
processRebootAlerts($dbconn);
processArchivingAlerts($dbconn);
processOverloadAlert($last_start, $started);
processStorageAllocationAlerts($dbconn);
resolveObsoleteAlerts($dbconn);

r300_kernel_update($dbconn);

# Delete 'deleted' storage entries.
bp_remove_storage_records();

# Delete old alert entries.
if (count($lastrun) > 0) {
	// %z is the day of the year
	if (date("%z", $started) != date("%z", $lastrun[0]["started"])) {
		$drop_off = $started - 604800;
		pg_query($dbconn, "DELETE FROM bp.alerts WHERE created < " . $drop_off . " AND closed = TRUE");
		pg_query($dbconn, "DELETE FROM bp.alert_runs WHERE ended < " . $drop_off);
	}
}

// Remove older SNMP trap history records.
$trap_history = get_master_ini_setting('Alertman', 'TrapHistoryDays', 31);
$q = "DELETE FROM bp.snmp_history WHERE time_sent < (now() - ";
$q .= "interval '".$trap_history." days') AND object not LIKE 'Ana%'";
pg_query($dbconn, $q);

$audit_history = get_master_ini_setting('Alertman', 'AuditHistoryDays', 180);
$q = "DELETE FROM bp.audit_history WHERE notification_id != 137 AND notification_id != 138 AND event_time < (now() - ";
$q .= "interval '".$audit_history." days')";
pg_query($dbconn, $q);

$sched_history = get_master_ini_setting('Alertman', 'ScheduleHistoryDays', 180);
// The status_time is an epoch
$sched_history = time() - ($sched_history * 86400);
$q = "DELETE FROM bp.schedule_history WHERE status_time < ".$sched_history;
pg_query($dbconn, $q);

// Retain statsRetentionDays of records for data reduction report.
// The sample_time value is an epoch.
$dr_days = get_master_ini_setting('fileDedup', 'statsRetentionDays', 31);
$drop_off = time() - ($dr_days * 86400);
$q = "DELETE FROM bp.data_reduction_ratios WHERE sample_time < ".$drop_off;
pg_query($dbconn, $q);

// Prune records from overload_stats table
$ovr = get_master_ini_setting('OverloadStats', 'OverloadHistoryDays', 30);
$drop_off = time() - ($ovr * 86400);
$q = "DELETE FROM bp.overload_stats WHERE start_time < ".$drop_off;
pg_query($dbconn, $q);

$sds_days = get_master_ini_setting('Replication', 'statusRetentionDays', 31);
$drop_off = time() - ($sds_days * 86400);
$q = "DELETE FROM bp.sds_stats WHERE done_epoch < ".$drop_off;
pg_query($dbconn, $q);

$psa_history = get_master_ini_setting('Alertman', 'PSA_HistoryDays', 180);
$q = "DELETE FROM bp.psa_history WHERE time_sent < (now() - ";
$q .= "interval '".$psa_history." days')";
pg_query($dbconn, $q);

$scd = get_master_ini_setting('Alertman', 'SummaryChartDays', 31);
$q = "DELETE FROM bp.summary_charts WHERE insert_date < (now()::date - $scd)";
pg_query($dbconn, $q);

$ended = time();
# Adding the last run must be after all alert processing for timed alerts to work properly
bppa_db_add_run($dbconn, $started, $ended);
bp_destroy_cookie();
pg_close($dbconn);
?>
