<?php

//a utility method that will log the error in the log and also on the standard error. It also closes the handles and return exit status of 1 to indicate error.
function writeErrorAndExit($error) {
    global $logHandle;

    fwrite(STDERR, $error);
    fwrite($logHandle, $error);
    closeDB();
    fclose(STDERR);
    fclose($logHandle);
    exit(1);
}

function okExit() {
    global $logHandle;

    closeDB();
    fclose(STDERR);
    fwrite($logHandle, "exit ok\n");
    fclose($logHandle);
    exit(0);
}

function logError($error) {
    global $logHandle;
    fwrite($logHandle, $error);
}

// a utility method to get the handler for logging. 
function getLoggingHandler() {

    // get the location of the bpdir.
    $bpdir = trim(shell_exec('eval `grep BPDIR /etc/default/bp.ini`; echo $BPDIR'));

    $logname = $bpdir . "/logs.dir/psa.log";

    # Open a single log file, append 
    $logFile = fopen($logname, "a");
    if ($logFile == false) {
        fwrite($stderr, "Could not open " . $logname . " for writing.\n");
        exit(1);
    }
    set_exception_handler('psa_exception_handler');
    return $logFile;
}

// A utiltiy method that opens and returns back a database connection.
$dbconn = false;
function openDB() {
    global $dbconn;
    global $logHandle;

    if ($dbconn != false) {
        fwrite($logHandle, "openDB: already open dbconn = " . $dbconn . "\n");
	return $dbconn;
    }
    $dbconn_string = "host=localhost port=5432 dbname=bpdb user=postgres";
    // $dbconn_string = "port=5432 dbname=bpdb user=postgres";
    $dbconn = pg_connect($dbconn_string);
    fwrite($logHandle, "openDB: dbconn = " . $dbconn . "\n");
    if ($dbconn == false) {
        $message = "Error connecting to database.\n";
        writeErrorAndExit($message);
    } else {
        return $dbconn;
    }
}

function closeDB() {
    global $dbconn;
    global $logHandle;
    if ($dbconn != false) {
       pg_close($dbconn);
       $dbconn = false;
    }
    fwrite($logHandle, "closeDB called\n");
}

/* A utility to run a query
 * @param query : the string to query the database
 * @param returnResult: Determines if the result should be passed back as is or if the values should be passed back as an associative array
 *
 */
function executeQuery($querystr, $returnResult = false) {
    global $dbconn;
    global $logHandle;

    if (!$dbconn) {
        $dbconn = openDB();
    }
    $stat = pg_connection_status($dbconn);
    $busy = pg_connection_busy($dbconn);
    if ($busy) {
        fwrite($logHandle, "DB connection busy, query in progress\n");
        return false;
    }
    $result = pg_query($dbconn, $querystr);
    // pg_send_query/pg_get_result is regarded as better at php.net
    // pg_send_query($dbconn, $querystr);
    // $result = pg_get_result($dbconn);
    // $error = pg_result_error($result);
    // if ($error) {

    // if pg_query fails, log the error, return false.
    if (!$result) {
	$error = pg_last_error($dbconn);
        $msg ="Execution of the following query failed:\n" . $querystr . 
		"\nReason: " . $error . "\n";
	//$edetail = pg_result_error_field($result, PGSQL_DIAG_MESSAGE_DETAIL);
	//$ehint = pg_result_error_field($result, PGSQL_DIAG_MESSAGE_HINT);
        //$msg .=" Detail: ".$edetail." Hint: ".$ehint."\n";
        fwrite($logHandle,$msg);
        // throw new Exception($msg);
        return false;
    }

    // if the caller needs just the result return back the result. do not process. helpful when inserting into a table
    if ($returnResult) {
        return $result;
    } else {
        // throw an exception if zero rows are returned. let the caller handle the exception
        if (pg_num_rows($result) == 0) {
            throw new ZeroRowsException("\nThe following query returned back zero rows:\n" . $querystr . "\n");
        }
        // if there are more than zero rows, return an associative array
	$arr = pg_fetch_assoc($result);
	// pg_free_result($result);
        return $arr;
    }
}

/* return the severity based on notification_id
 * this function also trnaslates the severity into priority which is used when creating a ticket
 * returns back an array with severity and priority information
 */
function getSeverity($notification_id) {
    global $logHandle;
    $query = "SELECT severity FROM bp.notifications WHERE notification_id=" . $notification_id;
    $arr = executeQuery($query);
    $sev = $arr["severity"];
    //pg_free_result($arr);
    //$sev = 2;
    return $sev; 
}

/*
 * This function returns back the correpsonding PSA tool name for a given psa tool id.
 */
function getPSAToolName($psa_tool_id) {
    global $logHandle;
    $query = "SELECT psa_tool_name from bp.psa_tools where psa_tool_id =" . $psa_tool_id;
    $arr = executeQuery($query);
    $tname = $arr["psa_tool_name"];
    // pg_free_result($arr);
    return $tname; 
}

/*
 * this function returns back short and long descritpion for a given notification id. if the notify_psa table doesn't
 * contain text for the given notificaiton id, the function gets the information from the notifications table
 */
function getShortAndLongDescription($notification_id) {
    $query = "SELECT short_description, long_description FROM bp.notify_psa WHERE notification_id =" . $notification_id;
    try {
        return executeQuery($query);
    } catch (ZeroRowsException $zre ) {
        $query = "SELECT short_description, primary_text as long_description FROM bp.notifications WHERE notification_id =" . $notification_id;
        return executeQuery($query);
    }
}

/*
 * This functions gets the psa configuration
 */
function getPSAConfiguration($psa_tool_name) {
    if ( $psa_tool_name == "default"){
        $query = "SELECT psa_tool_name, url, company_id, credential_id FROM bp.psa_config, bp.psa_tools WHERE  bp.psa_tools.psa_tool_id = bp.psa_config.psa_tool_id AND is_default";            
    } else {
        $query = "SELECT url, company_id, credential_id FROM bp.psa_config where psa_tool_id = (SELECT psa_tool_id from bp.psa_tools where psa_tool_name='$psa_tool_name')";                
    }
    
    try{
        return executeQuery($query);        
    }catch (ZeroRowsException $zre ) {
	if ( $psa_tool_name == "default" ){
		$errorMessage = "There is no default PSA tool. Add a default PSA tool and try again.\n";
	} else {
        	$errorMessage = $psa_tool_name." is not a valid PSA tool name. Correct the name and try again.\n";
	}
        writeErrorAndExit($errorMessage);        
    } 
}

/*
 * this function gets the PSA credentials
 */
function getPSACredentials($credential_id) {
    $query = "SELECT  username, password FROM bp.client_credentials WHERE credential_id =" . $credential_id;
    return executeQuery($query);
}

function str_num_matches($instr, $match)
{
   $len  = strlen($instr);
   $idx = strpos($instr,$match);
   if ( $idx == FALSE ) { $n = 0; }
   else {
      $n = 1;
      $pstr = $instr;
      while ( $idx < $len ) {
         $idx++;
         $pstr = substr("$pstr",$idx,($len - $idx));
         $idx = strpos($pstr,$match);
         if ( $idx == FALSE ) { break; }
         $n++;
      }
   }
   return $n;
}

function str_check_quotes($instr)
{
   global $logHandle;
   $squote = "'";  // single quote
   $dquote = "\""; // double quote
   $limit = 400; // was 316
   $qstring = $instr;
   // check string length against limit, but just log warning.
   $len  = strlen($instr);
   if ( $len >= $limit ) {
      fprintf($logHandle, "Note: message string length %d >= %d\n",$len,$limit);
      $qstring = substr($instr, 0, $limit);
   }
   // check for unterminated quotes in instring
   $n = str_num_matches(" ".$qstring, $squote);
   if ( $n & 1 ) {
      fprintf($logHandle, "Note: Odd number of single quotes (%d)\n",$n);
      $qstring = $qstring . $squote;  // fix it
   }
   $n = str_num_matches(" ".$qstring, $dquote);
   if ( $n & 1 ) {
      fprintf($logHandle, "Note: Odd number of double quotes (%d)\n",$n);
      $qstring = $qstring . $dquote;  // fix it
   }
   return $qstring;
}

function psa_check_string($instring)
{
   $qstring = str_check_quotes($instring);
   $qstring = utf8_encode($qstring); // verify UTF-8
   if ( 1 ) {   // only if executeQuery method
      // escape any special pg characters
      $qstring = pg_escape_string($qstring); 
   }
   return($qstring);
}

/*
 * This function writes to the history table when a ticket is created or when a ticket cannot be created.
 */
function writeToHistoryTable($notification_id, $severity, $tool_name, 
				$ticket_id, $description) 
{
    global $logHandle;
    global $dbconn;
    $skipit = 0;
    $query = "INSERT INTO bp.psa_history (notification_id, severity, tool_name, ticket_id, description) VALUES (";
    $query .= $notification_id . "," . $severity . ",'" . 
	 	pg_escape_string($tool_name) . "','" . 
	 	pg_escape_string($ticket_id) . "','" . 
	 	psa_check_string($description) . "')";
    fwrite($logHandle, "writeToHistory for query: " . $query . "\n");
    if ($skipit) {
       fwrite($logHandle, "insert skipped\n");
       return true;
    } 
if ( 0 ) {   // executeQuery is not as robust
    $str = executeQuery($query, true);
    // was getting error "Reason: SSL error: tlsv1 alert protocol version"
}
if ( 1 ) {   // use psql shell method
    $q1 = str_replace(" - "," \- ",$query);  // escape for shell arg error
    $cmd = 'psql -d bpdb -U postgres -c "' .$q1. '" ';
    // fwrite($logHandle, "shell_exec command: " . $cmd . "\n");  //DEBUG+++
    $str = shell_exec( $cmd ); 
}
    if (!$str) {
        fprintf($logHandle, "insert error\n");
	return false;
    } else {
        // usually "INSERT 0 1"
        fprintf($logHandle, "result: %s\n", $str );
	return $str;
    }
}

/*
 * function that creates ConnectWise priorities based on the severity
 * Deprecated - see getConnectWiseConfiguration instead
 */
function priorityForSeverity($severity) {
    // if ($arr["severity"] == "0" ) {
    // return "Priority 1 - Emergency Response";
    // } else
    if ($severity == "1") {
        return "Priority 2 - Quick Response";
    } else if ($severity == "2") {
        return "Priority 3 - Normal Response";
    } else if ($severity == "3") {
        return "Priority 4 - Scheduled Maintenance";
    }
}

function psa_exception_handler($exception) {
    fwrite($logHandle, "PSA exception: \n". $exception->getMessage(). "\n");
}


/*
 * A subclass of exception class that is used when zero rows are returned for a query.
 */
class ZeroRowsException extends Exception {
}

// Gets a mster.ini setting.  If $boolean is true, then we will parse any
// boolean-like values (On/on, true, yes, ...) and return a proper php boolean
function get_master_ini_setting($section, $item, $default) {
    $bploc="/usr/bp/";
    $master_ini = "$bploc/bpinit/master.ini";
    $bputil = "$bploc/bin/bputil";

    if (is_bool($default)) {
        $default = ($default === true ? "yes" : "no");
    }

    $cmd = "$bploc/bin/bputil -g \"$section\" \"$item\" \"$default\" $master_ini";
    $retval = trim(shell_exec("$cmd"));
    if (empty($retval)) {
	$retval = $default;
    }
    return $retval;
}

// Puts a mster.ini setting.  
function put_master_ini_setting($section, $label, $string) {
    $bploc="/usr/bp/";
    $master_ini = "$bploc/bpinit/master.ini";
    $bputil = "$bploc/bin/bputil";

    $cmd = "$bploc/bin/bputil -p \"$section\" \"$label\" \"$string\" $master_ini";
    exec($cmd, $retval);

    return $retval;
}

?>
