# Create (or update) symlink to last exchange master and differential backup

CRON_JOB="/etc/cron.hourly/bpexchsetlast"

SMB_CONF="/etc/samba/smb.conf"
SMBSHARE_PATTERN='^[[:space:]]*path[[:space:]]*=[[:space:]]*'
EXCHSETLAST=/var/lock/exchsetlast


LASTMASTER_FOLDER="LASTMASTER"
LASTDIFF_FOLDER="LASTDIFF"

MAGIC_FILE="UniExch.magic"
INI_SECTION="Unitrends"
INI_MAGIC_KEY="Magic"
INI_MAGIC_VAL="Unitrends_Exchange"
INI_LAST_MASTER="Last_Master"
INI_LAST_DIFF="Last_Diff"
STATUS_SECTION="Status"


RecreateSymlink() {	
	STATUSGETTER="$BP_BINDIR/bputil -g "
	lastBackupName="$1"
	lastBackupFolder="$2"
	clientRoot="$3"
	# 5543 - also make sure Status.ini has Status=success
	status=`$STATUSGETTER $STATUS_SECTION Status "" $clientRoot/$lastBackupName/Status.ini`
	if [ "$status" != "Success" ]
	then
		return
	fi

	# 3642 - make sure log file indicates success before creating
	#	symlink to LAST directory.
	JobStatus=`grep BPEXCH_Job_Status $lastBackupName/log.txt 2> /dev/null | awk '{print $2}' | awk -F"
" '{print $1}'`
	for stat in $JobStatus
	do
		if [ "$stat" != "Success" ]
		then
			return
		fi 
	done
	if [ $lastBackupFolder"Z" = "Z" ]; then 
	    echo "RecreateSymlink with source not set (for $lastBackupName), skipping" >> $LOG
	else
	# if symlink already exist - remove it	
	    if [ -L "$lastBackupFolder" ]; then
		echo "removing "`pwd`"/$lastBackupFolder symlink" >> $LOG
		ln -fs $lastBackupFolder
		rm -f $lastBackupFolder
	# 4591
	    elif [ -d "$lastBackupFolder" ]
	    then
	        echo "$lastBackupFolder was a directory instead of symlink. Removing." >> $LOG
		rm -rf $lastBackupFolder
	    fi

	    # if source directory exists - create symlink
	    if [ -d "$lastBackupName" ]; then
		echo "Creating $lastBackupName -> $lastBackupFolder symlink" >> $LOG
		ln -s $lastBackupName $lastBackupFolder
	    fi
	fi
}

# process single samba share
ProcessShare() {
    for fname in `find $1 -name $MAGIC_FILE` 
    do
	echo "Analysing " $fname >> $LOG
	magic=`$INIGETTER $INI_MAGIC_KEY "" $fname`
	if [ "Z$magic" = "Z$INI_MAGIC_VAL" ]; then 
    	    clientroot=`echo $fname | sed "s/$MAGIC_FILE//"`
	    echo "Client root is "$clientroot >> $LOG
	    lastmaster=`$INIGETTER $INI_LAST_MASTER "" $fname`
	    lastdiff=`$INIGETTER $INI_LAST_DIFF "" $fname`
	    echo "last master is "$lastmaster  >> $LOG
	    echo "last differ is "$lastdiff >> $LOG
	
			chmod 777 $clientroot
	    cd $clientroot
	    if [ -d "$lastmaster" ]
	    then
	    	RecreateSymlink $lastmaster $LASTMASTER_FOLDER $clientroot
	    fi
	    if [ -d "$lastdiff" ]
	    then
	    	RecreateSymlink $lastdiff $LASTDIFF_FOLDER $clientroot
	    fi
	else 
	    echo $fname " has wrong magic, skipping" >> $LOG
	fi
    done
}

# 3642
if ! lockfile -r 0 $EXCHSETLAST 2> /dev/null
then
	echo "bpexchsetlast process already running."
	exit 0
fi

# Determine where BP is installed
# 7838 The brackets contain a space and a tab
eval `egrep '^[ 	]*BPDIR=' /etc/default/bp.ini 2>/dev/null`

# Set basic environment variables
eval `${BPDIR}/bin/bputil -e -c`

INIGETTER="$BP_BINDIR/bputil -g $INI_SECTION "

LOG=$BPDIR/logs.dir/setlast.log
echo `date` > $LOG

# Install this script as a cron job
if [ ! -e $CRON_JOB ]; then
    echo "installing $CRON_JOB" >> $LOG
    ln -s ${BP_BINDIR}/setlast.sh $CRON_JOB
fi

# Loop through samba shares
for sharepath in `grep -i $SMBSHARE_PATTERN $SMB_CONF | grep -E -v "/backups/vmware_flr/|/backups/backups_restore_share/" | awk -F = '{ print $2 }' `
do
    if [ -d "${sharepath}" ]; then
	echo "Using share:"$sharepath >> $LOG
	ProcessShare $sharepath
    fi
done

# 3642
rm -f $EXCHSETLAST 2> /dev/null
