#!/bin/sh -u

# Query for information about a tape or changer device:
# @param $1 - command
# @param $2 - device node, such as "/dev/sg5"
# @param $3 - required for a few commands, such as a certain slot number, or a certain barcode

get_drive_count()
{
	dcount=`mtx -f $1 status|grep "Data Transfer Element"|wc -l`
	if [ $dcount -ne 0 ]; then
		echo $dcount
	else
		echo "Wrong parameters"
		exit 1
	fi
}

get_slot_count()
{
	scount=`mtx -f $1 status|grep "Storage Element"|grep -v "Loaded"|grep -v "IMPORT"|wc -l`
	if [ $scount -ne 0 ]; then
		echo $scount
	else
		echo "Wrong parameters"
		exit 1
	fi
}

scan_drive()
{
	temp=/tmp/tmp_alldrives.info
	mtx -f $1 status|grep "Data Transfer Element"  >$temp
	lines=`cat $temp|wc -l`

	for (( i=0; i < $lines; i++)); do
		status=`cat $temp|sed -n $(($i + 1))p |awk '{print $4}'|cut -d ":" -f 2`
		if [ "$status" = "Full" ]; then
			ds=`cat $temp|sed -n $(($i + 1))p` 
			if [[ "$ds" =~ "VolumeTag" ]]; then
				barcode=`echo $ds|tail -c 9`
			else
				barcode=none
			fi
			loadedslot=`echo $ds|awk '{print $7}'`
		else
			loadedslot=-1
			barcode=none
		fi

		echo "$loadedslot,$status,$barcode"
	done
	rm -f $temp 2>/dev/null
}


scan_slot()
{
	temp=/tmp/tmp_allslots.info
	mtx -f $1 status|grep "Storage Element"|grep -v "Loaded"|grep -v "IMPORT" >$temp
	lines=`cat $temp|wc -l`

#	The above command cannot go wrong at this step, not checking 
#	scnt=`get_slot_count $1`
#	if [ $lines -ne $scnt ]; then
#		echo "Wrong command"
#		exit 1
#	fi
	for (( i=0; i < $lines; i++)); do
		status=`cat $temp|sed -n $(($i + 1))p |awk '{print $3}'|cut -d ":" -f 2`
		if [ "$status" = "Full" ]; then
			ds=`cat $temp|sed -n $(($i + 1))p |awk '{print $4}'`
			if [[ "$ds" =~ "VolumeTag" ]]; then
				barcode=`echo $ds|tail -c 9`
			else
				barcode=none
			fi

		else
			barcode=none
		fi

		echo "$status,$barcode"
	done
	rm -f $temp 2>/dev/null
	
}

get_barcode_by_slot()
{
	devstr=`mtx -f $1 status|grep "Storage Element $2"|head -n 1`
	if [ "$devstr" = "" ]; then
		echo "No match found slot $2"
		exit 1
	fi	
	dtype=`echo $devstr|awk '{print $1}'`
	if [ "$dtype" = "Storage" ]; then
		if [[ "$devstr" =~ "VolumeTag" ]]; then
			bcode=`echo $devstr|awk '{print $NF}'|tail -c 9`
		else
			echo "n/a"
			exit 1
		fi
	fi

	if [ "$dtype" = "Data" ]; then
		if [[ "$devstr" =~ "VolumeTag" ]]; then
			bcode=`echo $devstr|awk '{print $NF}'`
		else
			echo "n/a"
			exit 1
		fi
	fi
	echo $bcode
}

get_slot_by_barcode()
{
	devstr=`mtx -f $1 status |grep $2`
	if [ $? -ne 0 ]; then
		echo "No result: wrong command or invalid barcode pattern"
		exit 2
	fi

	wcr=`mtx -f $1 status |grep $2|wc -l`
	if [ $wcr -gt 1 ]; then
		echo "multiple results, probably caused by partial barcode"
		exit 1
	fi
	
	dtype=`echo $devstr|awk '{print $1}'`

	if [ "$dtype" = "Storage" ]; then
		if [[ "$devstr" =~ "VolumeTag" ]]; then
			sn=`echo $devstr|awk '{print $3}'|cut -d ":" -f 1`
			status=`echo $devstr|awk '{print $3}'|cut -d ":" -f 2`
		else
			echo "No match found from barcode $2"
			exit 1
		fi
	fi

	if [ "$dtype" = "Data" ]; then
		if [[ "$devstr" =~ "VolumeTag" ]]; then
			sn=`echo $devstr|awk '{print $7}'|cut -d ":" -f 1`
			status=Loaded
		else
			echo "No match found from barcode $2"
			exit 1
		fi
	fi

	if [ "$sn" = "" ]; then
		echo "No match found from barcode $2"
		exit 1
	else
		echo $sn
	fi
}

case $1 in
	'countdrive')
		get_drive_count $2	
		;;
	'countslot')
		get_slot_count $2	
		;;
	'driveinfo')
		scan_drive $2 
		;;
	'slotinfo')
		scan_slot $2 
		;;
	'getslot')
		get_slot_by_barcode $2 $3
		;;
	'getbarcode')
		get_barcode_by_slot $2 $3
		;;
esac

exit 0;

