#!/bin/sh
#   Parse a UCS SP backup into several SP files
# 12/16/2014 acress v1.4 - save last input file in /tmp/ucs_parsexml.in
# 01/26/2014 acress v1.5 - use xmlin_dir=/var/unitrends
ver=1.5
xmlfile=$1
xmlfmt=${xmlfile}-fmt
eval `grep BPDIR /etc/default/bp.ini`
log=$BPDIR/logs.dir/ucs_parsexml.log
splist=/tmp/sp.list$$
dnlist=/tmp/dn.list$$
classlist=/tmp/class.list$$
xmltmp=/tmp/xml.tmp$$
xmlin_dir=/var/unitrends
xmlin_file=$xmlin_dir/ucs_parsexml.in   

echo "`date` -- Started $*" >$log
if [ "x$xmlfile" = "x" ]; then
   echo "Usage $0 <XML file>"  |tee -a $log
   exit 1
fi 
if [ ! -f $xmlfile ]; then
   echo "$xmlfile not found"  |tee -a $log
   exit 1
fi
cat - <<EOF >$classlist
aaaEpAuthProfile
adaptorHostEthIfProfile
adaptorHostFcIfProfile
adaptorHostIscsiIfProfile
biosVProfile
callhomeEp
computeAutoconfigPolicy
computeBladeInheritPolicy
computeChassisConnPolicy
computeChassisDiscPolicy
computePool
computePoolingPolicy
computePsuPolicy
computeQual
computeScrubPolicy
computeServerDiscPolicy
computeServerMgmtPolicy
epqosDefinition
fabricEp
fabricOrgVlanPolicy
fabricVConProfile
fabricMulticastPolicy
faultHolder
fcpoolInitiators
firmwareCatalogPack
firmwareComputeHostPack
ippoolPool
iqnpoolPool
iscsiAuthProfile
lsAgentPolicy
lsbootPolicy
lsmaintMaintPolicy
lsServer
macpoolPool
mgmtBackupPolicy
mgmtCfgExportPolicy
nwctrlDefinition
powerPolicy
powerMgmtPolicy
solPolicy
statsHolder
statsThresholdPolicy
storageLocalDiskConfigPolicy
sysdebugMEpLogPolicy
uuidpoolPool
vnicVnicBehPolicy
vnicVhbaBehPolicy
vmLifeCyclePolicy
orgOrg
topSystem
EOF

nchars=`cat $xmlfile |wc -c`
if [ $nchars -lt 250 ]; then
   echo "Input XML file too small ($nchars bytes)"  |tee -a $log
   cat $xmlfile >>$log
   exit 1
fi
cat $xmlfile |  xmlstarlet fo >$xmlfmt
nlines=`cat $xmlfmt |wc -l`
echo "Input XML: $xmlfmt  ($nlines lines)"  |tee -a $log
if [ $nlines -lt 3 ]; then
   echo "Invalid XML format: $xmlfmt"  |tee -a $log
   exit 1
fi

# check for restoreable attributes (see ucs_restorefilter.sh)

for cl in `cat $classlist`
do
   rn=0
   org=0
   if [ "$cl" = "orgOrg" ]; then
      org=1
   fi
   xmlstarlet sel -t -m //$cl  -s A:T:U @dn -v @dn  -n $xmlfmt >$dnlist
   n=`cat $dnlist |wc -l`
   if [ $n -eq 0 ]; then
      xmlstarlet sel -t -m //$cl  -s A:T:U @rn -v @rn  -n $xmlfmt >$dnlist
      n=`cat $dnlist |wc -l`
      if [ $n -gt 0 ]; then
         echo "Warning: Found rn instead of dn for $cl" | tee -a $log
         cat $dnlist >>$log
      fi
      # Leave rn disabled for now
      # rn=1
      >$dnlist
   fi
   for dn in `cat $dnlist`
   do 
      dndir=`dirname $dn`
      dnfile=${dn}.orig
      echo "isolating $cl : $dnfile ..."  |tee -a $log
      mkdir -p $dndir
      echo "<inConfig>" >$dnfile
      if [ $rn -eq 1 ]; then
         xmlstarlet sel -t -c "//$cl[@rn='${dn}']" $xmlfmt >>$dnfile
      else
         xmlstarlet sel -t -c "//$cl[@dn='${dn}']" $xmlfmt >>$dnfile
      fi
      if [ $org -eq 1 ]; then
         # only include the orgOrg class, not its contained classes
	 head -n2 $dnfile >$xmltmp
	 mv -f $xmltmp  $dnfile
         grep -q '/>' $dnfile
         if [ $? -ne 0 ]; then
            # if not already terminated, do so
            echo "</orgOrg>" >>$dnfile
         fi
      fi
      n=`cat $dnfile |wc -l`
      echo "</inConfig>" >>$dnfile
      if [ $n -lt 2 ]; then
         echo "Warning: XML parsed only $n lines for $cl : $dn"  |tee -a $log
         cat $dnfile >>$log
         rm -f $dnfile
         exit 1
      fi
   done
done

mkdir -p $xmlin_dir
cp -f $xmlfile  $xmlin_file
mv -f $xmlfmt $xmlfile  2>>$log
rm -f $splist $dnlist $classlist $xmltmp 2>/dev/null
echo "`date` -- Done " >>$log
