-- This script is only run on a fresh install!
-- To update or add views, add statements to the update_bpdb.sql script.

set search_path to bp,public;

CREATE VIEW privileges AS
SELECT user_id, priv_level, customer_id, location_id, CAST(system_id AS integer) FROM
 (SELECT user_id, priv_level, customer_id, null as location_id, null as system_id FROM customer_priv
  UNION ALL
  SELECT user_id, priv_level, null, location_id, null FROM location_priv
 ) AS cl
UNION ALL
  SELECT user_id, priv_level, null, null, system_id FROM system_priv;


CREATE VIEW ownership AS
SELECT * FROM
  (SELECT system_id, location_id FROM systems) AS s
FULL OUTER JOIN
  (SELECT location_id, customer_id FROM locations) AS l
USING (location_id)
FULL OUTER JOIN
  (SELECT customer_id FROM customers) AS c
USING (customer_id);


CREATE VIEW inherited_privileges AS
SELECT user_id, priv_level, o.customer_id, o.location_id, o.system_id FROM customer_priv AS p INNER JOIN ownership AS o USING (customer_id)
UNION ALL
SELECT user_id, priv_level, null, o.location_id, o.system_id FROM location_priv AS p INNER JOIN ownership AS o USING (location_id)
UNION ALL
SELECT user_id, priv_level, null, null, system_id FROM system_priv WHERE system_id IS NOT NULL;


CREATE VIEW viewables AS
SELECT user_id, priv_level, o.*
  FROM privileges AS p
  INNER JOIN ownership AS o
  ON (o.customer_id = p.customer_id OR o.location_id = p.location_id OR o.system_id = p.system_id)
WHERE (priv_level >= 1);

set search_path to bp;
-------------------------------------------------------------------------------------------------------

--------------------------------------------------------------------------------------------------------------
-- datawidth
--------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE VIEW bloat_datawidth_bpdb AS
SELECT ns.nspname AS schemaname, tbl.oid AS relid, tbl.relname,
  CASE WHEN every(avg_width IS NOT NULL)
  THEN round((SUM((1-null_frac)*avg_width) + MAX(null_frac) * 24)::NUMERIC, 2) ELSE NULL END AS datawidth
  FROM pg_attribute att JOIN pg_class tbl ON att.attrelid = tbl.oid
  JOIN pg_namespace ns ON ns.oid = tbl.relnamespace LEFT JOIN pg_stats s
  ON s.schemaname=ns.nspname AND s.tablename = tbl.relname /*AND s.inherited=false*/ AND
  s.attname=att.attname AND s.schemaname = 'bp'
  WHERE att.attnum > 0 AND tbl.relkind in ('r', 'i') GROUP BY 1,2,3;


--------------------------------------------------------------------------------------------------------------
-- bloat
--------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE VIEW bloat_tables_bpdb AS
SELECT bdw.schemaname, bdw.relname, bdw.datawidth,
  cc.reltuples::bigint, cc.relpages::bigint,
  ceil(cc.reltuples*bdw.datawidth/current_setting('block_size')::NUMERIC)::bigint AS expectedpages,
  --100 - (cc.reltuples*100*bdw.datawidth)/(current_setting('block_size')::NUMERIC*cc.relpages) AS bloatpct
  (100 - (cc.reltuples*100*bdw.datawidth)/(current_setting('block_size')::NUMERIC*cc.relpages))::NUMERIC(5,2) AS bloatpct
  FROM bloat_datawidth_bpdb bdw JOIN pg_class cc ON cc.oid = bdw.relid
  AND bdw.schemaname = 'bp' AND bdw.datawidth IS NOT NULL AND cc.relpages > 1 ORDER BY bloatpct DESC;


--------------------------------------------------------------------------------------------------------------
-- table size
--------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE VIEW size_tables_bpdb AS
SELECT schemaname, tablename,
  pg_relation_size(schemaname || '.' || tablename) AS size_p,
  pg_total_relation_size(schemaname || '.' || tablename) AS siz,
  pg_total_relation_size(schemaname || '.' || tablename) AS total_size_p,
  pg_total_relation_size(schemaname || '.' || tablename) - pg_relation_size(schemaname || '.' || tablename) AS index_size,
  (100*(pg_total_relation_size(schemaname || '.' || tablename) - pg_relation_size(schemaname || '.' || tablename))) /
  CASE WHEN pg_total_relation_size(schemaname || '.' || tablename) = 0 THEN 1
  ELSE pg_total_relation_size(schemaname || '.' || tablename) END AS index_pct
  FROM pg_tables WHERE schemaname='bp' ORDER BY siz DESC LIMIT 10;
