<?php
/**
 * Created by PhpStorm.
 * User: Sonja12
 * Date: 4/10/2015
 * Time: 8:30 PM
 */

class Catalogs
{
    private $BP;

    public function __construct($BP = NULL)
    {
        $this->BP = $BP;

    }


    public function get_catalog($type, $filter, $sid, $systems)
    {
        $allCatalogs = array();

        switch ($type[0]) {
            case 'backups':
                $allCatalogs = $this->getBackupCatalog($filter, $sid, $systems);
                break;
            case 'archives':
                $allCatalogs = $this->getArchiveCatalog($filter, $sid, $systems);
                break;
            default:
                $backupCatalog = $archiveCatalog = $replicationCatalog = array();
                $b = $a = array();
                $typeToken = $type[0];
                if ($typeToken !== NULL && $typeToken !== '' && $typeToken[0] !== '?') {
                    if ($typeToken === 'all' || strpos($typeToken, 'b') !== false) {
                        $backupCatalog = $this->getBackupCatalog($filter, $sid, $systems);
                    }
                    if ($typeToken === 'all' || strpos($typeToken, 'a') !== false) {
                        $archiveCatalog = $this->getArchiveCatalog($filter, $sid, $systems);
                    }
                } else {
                    // default is backups only.
                    $backupCatalog = $this->getBackupCatalog($filter, $sid, $systems);
                }

                if (array_key_exists('catalog', $backupCatalog)) {
                    $b = $backupCatalog['catalog'];
                }

                if (array_key_exists('catalog', $archiveCatalog)) {
                    $a = $archiveCatalog['catalog'];
                }
                $view = isset($_GET['view']) ? $_GET['view'] : "system";

                $combined = array();

                if ($view == "day"){
                    foreach ($b as $backup){
                     //   $backup = $this->mergeDayView($backup, $a);
                        $combined[] = $this->mergeDayView($backup, $a);
                    }
                    foreach($a as $archive) {
                        $catalogInstance = $this->findArchiveDayNoBackups($archive, $b);
                        if($catalogInstance !== false && $catalogInstance !== NULL) {
                            $combined[] = $catalogInstance;
                        }
                    }

                    $combined = $this->sortCombined($combined);
                }
                if ($view == "instance") {
                    foreach ($b as $backup){
                        $combined[] = $this->mergeInstanceView($backup, $a);
                    }
                    foreach($a as $archive) {
                        $catalogInstance = $this->findArchiveInstanceNoBackups($archive, $b);
                        if($catalogInstance !== false && $catalogInstance !== NULL) {
                            $combined[] = $catalogInstance;
                        }
                    }
                }
                if ($view == "system") {
                  //  var_dump($view);
                    foreach ($a as $backup){
                        $combined[] = $this->mergeSystemView($backup, $b);


                    }
                    foreach($b as $archive) {
                        $catalogInstance = $this->findArchiveInstanceNoBackups($archive, $a);
                        if($catalogInstance !== false) {
                            $combined[] = $catalogInstance;
                        }
                    }
                }
                $allCatalogs['catalog'] = $combined;
                break;

        }
        return $allCatalogs;
    }

    function getBackupCatalog($filter, $sid, $systems)
    {
        require_once('backups.php');
        $backups = new Backups($this->BP, $sid);
        $backupCatalog = $backups->get("catalog", $filter, $sid, $systems);

        return $backupCatalog;
    }

    function getArchiveCatalog($filter, $sid, $systems)
    {
        $which = array();
        $which[] = "catalog";

        require_once('archive.php');
        $archive = new Archive($this->BP, $sid);
        $archiveCatalog = $archive->get($which, $filter, $sid, $systems);

        return $archiveCatalog;
    }

    function mergeDayView($backup, $archives, $areBackups = false){
        $itemsProperty = $areBackups ? 'backups' : 'archives';
         foreach($archives as $archive){
                if(array_key_exists("day", $archive)){
                    $archiveDay = $archive['day'];
                }
                if(isset($archiveDay) && ($archiveDay == $backup['day'])){
                    $backupInstances = $backup['instances'];
                    $archiveInstances = $archive['instances'];
                    for ($i=0; $i < count($backupInstances); $i++){
                        for ($j=0; $j < count($archiveInstances); $j++){
                        if($backupInstances[$i]['system_id'] == $archiveInstances[$j]['system_id'] &&
                            $backupInstances[$i]['instance_name'] == $archiveInstances[$j]['instance_name']){
                            $archives = $archiveInstances[$j][$itemsProperty];
                                $backupsArray = array_merge($backup['instances'][$i]['backups'], $archives);
                                $backups = $this->sortBackups($backupsArray);
                                $backup['instances'][$i]['backups'] = $backups;
                            }
                        }
                    }
                }
            }

        return $backup;
    }


    function findArchiveDayNoBackups($archive, $backups, $areBackups = false, $matchSID = true)
    {
        $result = false;
        $hasMatch = false;
        $itemsProperty = $areBackups ? 'backups' : 'archives';

        $count = count($backups);
        for ($i = 0; $i < $count && !$hasMatch; $i++) {
            $backup = $backups[$i];
            if (isset($archive['day']) && ($archive['day'] == $backup['day'])) {
                $backupInstances = $backup['instances'];
                $archiveInstances = $archive['instances'];
                for ($n=0; $n < count($backupInstances); $n++){
                    for ($j=0; $j < count($archiveInstances); $j++){
                        if ($matchSID && ($backupInstances[$n]['system_id'] != $archiveInstances[$j]['system_id'])) {
                            continue;
                        }
                        if($archiveInstances[$j]['instance_name'] == $backupInstances[$n]['instance_name']){
                             $hasMatch = true;
                            break;
                         }
                     }
                }
            }
        }
        if (!$hasMatch && isset($archive['instances'])) {
                $instances = $archive['instances'];
                $count = count($instances);
                $new = 0;
                for ($i = 0; $i < $count; $i++) {
                    $app = $archive['instances'][$i]['app_name'];
                    if ($app != 'System Metadata') {
                        $archive['instances'][$new] = $archive['instances'][$i];
                    $archive['instances'][$new]['backups'] = $archive['instances'][$i][$itemsProperty];
                    if ($itemsProperty != 'backups') {
                        unset($archive['instances'][$i][$itemsProperty]);
                    }
                        if ($i != $new) {
                            unset($archive['instances'][$i]);
                        }
                        $new++;
                    } else {
                        unset($archive['instances'][$i]);
                    }
                }
                if ($new == 0) {
                    $archive = false;
                }
                $result = $archive;
            }


        return $result;

    }

    function mergeInstanceView($backup, $archives)
    {
        foreach ($archives as $archive) {
            if ($backup['system_id'] == $archive['system_id'] && $backup['instance_name'] == $archive['instance_name']
                && $archive['app_name'] !== "System Metadata" ) {
                $instanceArchives = $archive['archives'];
                if ($backup['instance_name'] == "file-level"){
                    if ($backup['client_id'] !== $archive['client_id']){
                        continue;
                    }
                } else if ($backup['instance_name'] == "Farm"){
                    if ($backup['client_id'] !== $archive['client_id']){
                        continue;
                    }
                }
                $backupsArray = array_merge($backup['backups'], $instanceArchives);
                $backups = $this->sortBackups($backupsArray);
                $backup['backups'] = $backups;
                break;
            }
        }
        return $backup;
    }

    function findArchiveInstanceNoBackups($archive, $backups, $areBackups = false, $matchSID = true) {

        $result = false;
        $itemsProperty = $areBackups ? 'backups' : 'archives';

        if ($archive["app_name"] !== "System Metadata") {
        $hasMatch = false;
        foreach($backups as $backup) {
                if ($matchSID && ($backup['system_id'] != $archive['system_id'])) {
                    continue;
                }
            if($archive['instance_name'] == $backup['instance_name']){
                $hasMatch = true;
                    break;
            }
        }

            if($hasMatch == false) {
                $archives = $archive[$itemsProperty];
            $archives = $this->sortBackups($archives);
                unset($archive[$itemsProperty]);
            $archive['backups'] = $archives;
            $result = $archive;
        } else {
            $result = false;
        }
        }
        return $result;
    }

    function mergeSystemView($backup, $archives)
    {
      // var_dump($archives);
        foreach ($archives as $archive) {
           // var_dump("bu instance:  " . $backup['instance_name'] . "archive  instance:  " . $archive['instance_name']);
            if ($backup['instance_name'] == $archive['instance_name']) {

                $archives = $archive['archives'];
                $backupsArray = array_merge($backup['backups'], $archives);
                $backups = $this->sortBackups($backupsArray);
                $backup['backups'] = $backups;

            }
        }
        return $backup;
    }

    function sortBackups($backupArr) {
        $backups = $backupArr;
        $orderByStartDate = array();
        foreach ($backups as $key => $row) {
            $orderByStartDate[$key] = strtotime($row['start_date']);
        }
        array_multisort($orderByStartDate, SORT_DESC, $backups);

        return $backups;
    }

    function sortCombined($combined){
        $orderByDay = array();
        foreach($combined as $key => $row){
            $orderByDay[$key] = $row['day'];
        }
        array_multisort($orderByDay, SORT_DESC, $combined);

        return $combined;
    }
}
