<?php

define("VCENTER_RRC", "vCenter-RRC");
define("PROTECT_GENERIC_UCSM", "Cisco UCS Manager");
define("PROTECT_GENERIC_UCSC","Cisco UCS Central");
define("PROTECT_GENERIC_NAS","NAS NDMP Client");
define("ALL_OTHER_OS","All other OS");

class Clients
{
    private $BP;
     
    public function __construct($BP)
    {
		$this->BP = $BP;

        require_once('function.lib.php');
        $this->functions = new Functions($this->BP);
    }

    public function get($which, $filter, $data, $sid, $systems)
    {
        switch($which){
            case "config":
                $allClients = $this->getConfig();
                break;

            case "files":
                $allClients = $this->getFiles($filter);
                break;

            case "filtered":
                $allClients = $this -> getFilteredClients($data, $sid, $systems);
                break;

            case "agent-check":
                $allClients = $this -> AgentCheck($filter);
                break;

            default:
                $allClients = $this -> getClients($which);
                break;
        }

	return($allClients);
    }

    public function put($which, $inputArray, $sid){
        switch($which){
            case 'agent-push':
                $result = $this->agentPush($inputArray);
                break;
            default:
                $result = $this -> putClient($which,$inputArray, $sid);
                break;
        }
        return($result);
    }


    public function delete($which,$sid)
    {
       $result = $this-> deleteClient($which,$sid);
        return $result;
    }

    public function add($which, $inputArray, $sid)
    {
        $result = $this-> addClient($which, $inputArray, $sid);
        return $result;
    }




    function getConfig(){
        $cid = isset($_GET['cid']) ? (int)$_GET['cid'] : false;
        $sid = isset($_GET['sid']) ? (int)$_GET['sid'] : false;
        $bid = isset($_GET['bid']) ? (int)$_GET['bid'] : false;
        $bValid = true;
        $grandClientView = false;
        $localID = 1;
        if(isset($sid)){
            $dpuID=$sid;
        }else{
            $dpuID=1;
        }
        if ($grandClientView) {
            $clients = $this->BP->get_grandclient_list($dpuID);
            // working on the local system.
            $dpuID = $localID;
        } else {
            $clients = $this->BP->get_client_list($dpuID);
        }
        foreach ($clients as $id => $name){
            if($cid!==false and ($id === $cid)){
                $clientConfig=$this->BP->get_client_config($cid,$dpuID);
            }
            if($bid!==false){
                $clientConfig=$this->BP->get_client_config_from_backup($bid,$dpuID);
            }
        }
        if($clientConfig === false){
            $result['error'] = 500;
            $result['message'] = $this->BP->getError();;
            return $result;
        }
        $allClients = array('data'=>$clientConfig);
        return $allClients;
    }



    function getFiles($filter){
        $cid = isset($filter) ? $filter : false;
        $sid = isset($_GET['sid']) ? (int)$_GET['sid'] : false;
        $directory = isset($_GET['dir']) ?  $_GET['dir'] : "";
        $last = isset($_GET['last']) ?  $_GET['last'] : "";
        $count = isset($_GET['count']) ?  (int)$_GET['count'] : 0;
        $rootlist = $this->BP->get_client_files($cid, $directory, $last, $count, $sid);
        $allClients['data'] = array();
        $realcount = 0;
        $data = array();
        foreach($rootlist as $key => $val){
            $data[$key]['directory'] = $rootlist[$key]['directory'];
            $data[$key]['name'] = $rootlist[$key]['name'];
            $data[$key]['id'] = $rootlist[$key]['id'];
            $data[$key]['type'] = $rootlist[$key]['type'];
            if($data[$key]['type'] === "file"){
                $data[$key]['isBranch'] = false;
            }
            else $data[$key]['isBranch'] = true;
            $realcount++;
        }
        $allClients['start'] = 1;
        $allClients['count'] = $realcount;
        $allClients['total'] = $realcount;
        $allClients['data'] = $data;
        return $allClients;
    }




    function AgentCheck($filter){
        $cid = isset($_GET['cid']) ? (int)$_GET['cid'] : false;
        $sid = isset($_GET['sid']) ? (int)$_GET['sid'] : $this->BP->get_local_system_id();
        $data1 = array();
        $clients = $this->BP->get_client_list($sid);
        foreach($clients as $key => $val){
            if(is_numeric($cid) and $cid !== $key) {
                continue;
            } else {
                $clientInfo = $this->BP->get_client_info($key, $sid);
                $hostInfo = $this->BP->get_host_info($clientInfo['name'], $sid);
                $data['address'] = $hostInfo['ip'];
                $data['id'] = $key;
                $data['name'] = $clientInfo['name'];
                $data['version'] = $clientInfo['version'];
                if($this->functions->getOSFamily($clientInfo['os_type_id']) == Constants::OS_FAMILY_WINDOWS) {
                    $updateAvailable = $this->BP->agent_update_available($key, $sid);
                    if(is_array($updateAvailable)) {
                        $data['update_available'] = $updateAvailable[0]['isUpdateAvailable'];
                    } else {
                        $data['update_available'] = false;
                    }
                } else {
                    $data['update_available'] = false;
                    $data['message'] = "This client does not support agent check and agent push";
                }

                $data1[]=$data;
            }
        }
        $allClients = array('clients'=>$data1);
        return $allClients;
    }

    private function getFilteredClients($data, $sid, $systems) {
        $os_id = NULL;
        $os_family = NULL;
        $app_id = NULL;
        $app_type = NULL;
        $get_all_apps = false;
        $showSystemClient = false;
        if ( $data !== NULL ) {
            if ( array_key_exists('os_id', $data) ) {
                $os_id = (int)$data['os_id'];
            }
            if ( array_key_exists('os_family', $data) ) {
                $os_family = $data['os_family'];
            }
            if ( array_key_exists('app_id', $data) ) {
                $app_id = (int)$data['app_id'];
            }
            if ( array_key_exists('app_type', $data) ) {
                if ( $data['app_type'] === 'SQL' ) {
                    $app_type = Constants::APPLICATION_TYPE_NAME_SQL_SERVER;
                } else {
                    $app_type = $data['app_type'];
                }
            }
            if ( array_key_exists('all_apps', $data) ) { //1 for true, 0 for false
                $get_all_apps = ( (int)$data['all_apps'] === 1 );
            }
            if ( array_key_exists('showSystemClient', $data) )
            {
                $showSystemClient = ($data['showSystemClient'] === '1');
            }
        }

        $returnClients = array();

        foreach ( $systems as $systemID => $systemName ) {

            $clients = $this->BP->get_client_list($systemID);
            if ( $clients !== false ) {
                foreach ($clients as $clientID => $clientName) {
                    $returnThisClient = true;
                    $isSystemClient = ($clientName === $systemName);

                    $clientInfo = $this->BP->get_client_info($clientID, $systemID);
                    if ($clientInfo !== false and ($showSystemClient === true or $isSystemClient === false) ) {  //Always exclude the system client
                        if ($os_id !== NULL and $clientInfo['os_type_id'] !== $os_id) {
                            $returnThisClient = false;
                        }
                        if ($returnThisClient === true and $os_family !== NULL and $clientInfo['os_family'] !== $os_family) {
                            $returnThisClient = false;
                        }
                        $applicationsForThisClient = array();
                        if ( $get_all_apps === true or $app_id !== NULL or $app_type !== NULL ) {
                            foreach ($clientInfo['applications'] as $applicationID => $applicationArray) {
                                if ( $get_all_apps === true or
                                     ( $app_id !== NULL and $applicationID === $app_id  ) or
                                     ( $app_type !== NULL and $applicationArray['type'] === $app_type ) ) {
                                    $applicationsForThisClient[$applicationID] = $applicationArray['name'];
                                }
                            }
                            if ( $get_all_apps === false and count($applicationsForThisClient) === 0 ) {
                                $returnThisClient = false;
                            }
                        }
                        if ( $returnThisClient === true or ($showSystemClient === true and $isSystemClient === true) ) {
                            $returnClients[] = array(
                                'system_id'=>$systemID,
                                'system_name'=>$systemName,
                                'client_id'=>$clientID,
                                'client_name'=>$clientName,
                                'os_type'=>$clientInfo['os_type'],
                                'applications'=>$applicationsForThisClient);
                        }
                    }
                }
            }
        }

        return array( 'data'=>$returnClients);
    }



    function getClients($which){
        $bValid = true;
        $grandClientView = false;
        $dpuID = 1;
        $localID = 1;

        $dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $this->BP->get_local_system_id();

        if ($grandClientView) {
            $clients = $this->BP->get_grandclient_list($dpuID);
            // working on the local system.
            $dpuID = $localID;
        } else {
            $clients = $this->BP->get_client_list($dpuID);
        }

        $systemList = $this->BP->get_system_list();
        foreach($systemList as $key => $val){
            if($dpuID===$key){
                $system_id = $key;
                $system_name = $val;
            }
        }
        $includeVCenter = true;
        if(!isset($_GET['include_vmware_client'])){
            $includeVCenter = false;
        }
        else if($_GET['include_vmware_client'] !== "true"){
            $includeVCenter = false;
        }

        $allClients = array();
        if ($clients === false) {
            $bValid = false;
        } else {
            foreach ($clients as $id => $name) {
                $isSystem=false;
                if ($which > 0 and $id != $which) {
                    continue;
                }
                if(!$includeVCenter and $name === VCENTER_RRC){
                    continue;
                }
                $clientInfo = $this->BP->get_client_info($id, $dpuID);
                $hostInfo = $this->BP->get_host_info($clientInfo['name']);
                foreach($systemList as $system){
                    if($system===$name){
                        $isSystem=true;
                        continue;
                    }
                }
                if($isSystem){
                    continue;
                }

                if ($clientInfo === false) {
                    $bValid = false;
                    break;
                } else {
                    $networkID = sprintf("%d.%d", $dpuID, $id);
                    $Credentials = NULL;
                    if (   $clientInfo['os_type_id'] == Constants::OS_WIN_16
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_95
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2000
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_XP
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2003
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_VISTA
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2008
                        or $clientInfo['os_type_id'] == Constants::OS_WIN_2008_R2
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_7
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_8
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2012
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_8_1
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2012_R2
                        or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_10)
                    {
                        $clientInfo['supports_agent_push']=true;
                    }
                    else{
                        $clientInfo['supports_agent_push']=false;
                    }
                    $clientInfo['asset_type'] = Constants::ASSET_TYPE_PHYSICAL;
                    $clientInfo['system_id'] = $system_id;
                    $clientInfo['system_name'] = $system_name;
                    $clientInfo['ip'] = $hostInfo['ip'];
                    $allClients[] = $clientInfo;
                }
            }
            // add vCenters if present

            $result = $this->BP->get_vcenter_list($dpuID);
            if ($result !== false) {
                foreach ($result as $id => $name) {
                    if( $id != $which and $which > 0){
                        continue;
                    }
                    $appInfo = $this->BP->get_vcenter_info($id, $dpuID);
                    $appInfo['asset_type'] = Constants::ASSET_TYPE_VMWARE_HOST;
                    $appInfo['system_id'] = $system_id;
                    $appInfo['system_name'] = $system_name;
                    $allClients[] = $appInfo;
                }
            }
        }
        $allClients = array('data'=>$allClients);
        return $allClients;
    }


    function putClient($which, $inputArray, $sid){
        $cid=$which;
        if($sid===false) {
            $sid = $this->BP->get_local_system_id();
        }

        if(ctype_digit($cid)) {
            $clientInfo = $this->BP->get_client_info($cid,$sid);
        }
        else{
            $vcenterInfo = $this->BP->get_vcenter_info($cid,$sid);
            if($vcenterInfo === false){
                return false;
            }
            else {
                $hostInfo = $inputArray['host_info'];
                $credentials = $inputArray['credentials'];
                $newvcenterInfo['id'] = $cid;
                $newvcenterInfo['address'] = isset($hostInfo['ip']) ? $hostInfo['ip'] : $vcenterInfo['ip'];
                $newvcenterInfo['login'] = isset($credentials['username']) ? $credentials['username'] : $vcenterInfo['username'];
                $newvcenterInfo['password'] = isset($credentials['password']) ? $credentials['password'] : $vcenterInfo['password'];
                $result = $this->BP->save_vcenter_info($newvcenterInfo, $sid);
                return $result;
            }
        }

        $newClientInfo['id']= $clientInfo['id'];

        if(isset($inputArray['host_info']) or isset($inputArray['name'])){
            $oldHostInfo=$this->BP->get_host_info($clientInfo['name']);
            $newHostInfo['original_ip'] = $oldHostInfo['ip'];
            $newHostInfo['ip'] = $oldHostInfo['ip'];                // New IP will be reset later on line 386
            if(isset($inputArray['name'])) {
                $newClientInfo['name'] = $inputArray['name'];
                $newHostInfo['name'] = $inputArray['name'];
            }
            if(isset($inputArray['host_info']) && !isset($clientInfo['remote_address'])) {          //special condition for Xen
                $hostInfo = $inputArray['host_info'];
                $newHostInfo['ip'] = $hostInfo['ip'];
            }
            if($newHostInfo['ip']!=="127.0.0.1" and $newHostInfo['name']===$inputArray['name']){
                $alias = false;
                $saveHost=$this->BP->save_host_info($newHostInfo);
            }
            else if($newHostInfo['name'] !== $clientInfo['name']){
                $alias = true;
                $saveHost = $this->addHostAlias($this->BP, $oldHostInfo['name'], $newHostInfo['name'], $sid);       //Add new alias
            }
            else{
                $saveHost = true;
                $alias = false;
            }

            //aliases should be updated in hostname.php
            if($saveHost===false){
                return false;
            }
            else if ($alias){
                $removeHost = $this->removeHostAlias($this->BP, $newHostInfo['name'], $clientInfo['name'], $sid);      //Remove old alias.
                if($removeHost===false){
                    global $Log;
                    $msg= "could not update host file".$this->BP->getError();
                    $Log->writeError($msg, true);
                }
            }
        }
        //credential changes

        if(isset($inputArray['credential_id'])and $inputArray['credential_id'] !== -1){
            $newClientInfo['credential_id']=$inputArray['credential_id'];
        }
        elseif($inputArray['is_auth_enabled']===false or $inputArray['credential_id'] === -1){
            $newClientInfo['no_credentials']=true;
        }
        elseif(isset($inputArray['credentials'])){
            $credInfo = $inputArray['credentials'];
            if(isset($credInfo['display_name']) && ($credInfo['display_name']) !== "" ){
                $newCredInfo['display_name'] = $credInfo['display_name'];
            }
            if(!isset($credInfo['is_default'])){
                $newCredInfo['is_default'] = false;
            } else {
                $newCredInfo['is_default'] = $credInfo['is_default'];
            }
            $newCredInfo['username'] = $credInfo['username'];
            $newCredInfo['password'] = $credInfo['password'];
            $newCredInfo['domain'] = $credInfo['domain'];
            $saveCred = $this->BP->save_credentials($newCredInfo,$sid);
            if($saveCred === false){
                return false;//$this->BP->getError();
            }
            else{
                $newClientInfo['credential_id'] = $saveCred;
            }
        }

        //priority changes
        $newClientInfo['is_enabled'] = isset($inputArray['is_enabled']) ? $inputArray['is_enabled'] : $clientInfo['is_enabled'];
        $newClientInfo['is_synchable'] = isset($inputArray['is_synchable']) ? $inputArray['is_synchable'] : $clientInfo['is_synchable'];
        $newClientInfo['is_encrypted'] = isset($inputArray['is_encrypted']) ? $inputArray['is_encrypted'] :$clientInfo['is_encrypted'];
        $newClientInfo['priority'] = isset($inputArray['priority']) ? $inputArray['priority'] : $clientInfo['priority'];
        $newClientInfo['use_ssl'] = isset($inputArray['use_ssl']) ? $inputArray['use_ssl'] : $clientInfo['use_ssl'];

        //For NAS NDMP clients remote IP change

        if ( isset ($clientInfo['os_type_id'])
            && $clientInfo['os_type_id'] == 40
            && isset($clientInfo['generic_property'])
            && $clientInfo['generic_property'] !== 4
            && $clientInfo['generic_property'] !== Constants::GENERIC_PROPERTY_XEN ) {
            $useRaeSave = true;
        }
        else{
            $useRaeSave= false;
        }
        if(isset($inputArray['remote_port']) or isset($clientInfo['remote_port'])){         //For NDMP special case
            $newClientInfo['remote_address']=isset($inputArray['remote_address']) ? $inputArray['remote_address'] : $clientInfo['remote_address'];
            $newClientInfo['remote_port'] = isset($inputArray['remote_port']) ? $inputArray['remote_port'] : $clientInfo['remote_port'];
        }
        if(isset($clientInfo['generic_property']) && $clientInfo['generic_property'] === Constants::GENERIC_PROPERTY_XEN){         //For Xen special case
            $newClientInfo['remote_address']=isset($inputArray['host_info']['ip']) ? $inputArray['host_info']['ip'] : $clientInfo['remote_address'];
        }
        //Works for regular clients if added "os" Info in newClientInfo Array
        if(isset($clientInfo['os_type_id'])){
            $newClientInfo['os']= $clientInfo['os_type_id'];
            $newClientInfo['os_type_id'] = $clientInfo['os_type_id'];
            $newClientInfo['generic_property'] = $clientInfo['generic_property'];
        }
        //finally save client info with changes
        if ($useRaeSave) {
            $result = $this->BP->save_rae_client_info($newClientInfo, $sid);
        } else {
            $result = $this->BP->save_client_info($newClientInfo, $sid);

        }

        if($result === false && (isset($inputArray['host_info']) or isset($inputArray['name'])) && (!isset($clientInfo['remote_address']) or (isset($clientInfo['generic_property']) && $clientInfo['generic_property'] === Constants::GENERIC_PROPERTY_XEN) )){                   //If saving client info fails roll back to old host info.

            $HostInfo = $this->BP->get_host_info($newClientInfo['name']);
            if($HostInfo['ip']==="127.0.0.1"){
                $saveHost = $this->addHostAlias($this->BP, $newClientInfo['name'], $clientInfo['name'], $sid);       //Add old alias back in alias list
                if($saveHost===false){
                    global $Log;
                    $msg= "could not update host file".$this->BP->getError();
                    $Log->writeError($msg, true);
                }
                else{
                    if ( $clientInfo['name'] !== $newClientInfo['name'] ) {  //If the alias has not changed, don't remove it from the host file.
                        $removeHost = $this->removeHostAlias($this->BP, $clientInfo['name'], $newClientInfo['name'], $sid);      //Remove new alias
                        if ($removeHost === false) {
                            global $Log;
                            $msg = "could not update host file" . $this->BP->getError();
                            $Log->writeError($msg, true);
                        }
                    }
                }
            }
            else{
                $tempHostInfo['original_ip'] = $HostInfo['ip'];
                $tempHostInfo['ip'] = $newHostInfo['original_ip'];
                $saveHost=$this->BP->save_host_info($tempHostInfo);
                if($saveHost===false){
                    global $Log;
                    $msg= "could not update host file".$this->BP->getError();
                    $Log->writeError($msg, true);
                }
            }
        }
        return $result;
    }



    function deleteClient($which, $sid){
        $isVirtual=false;
        if($sid===false) {
            $sid = $this->BP->get_local_system_id();
        }

        $result = $this->BP->get_vcenter_list($sid);

        foreach($result as $id=>$name){
            if($which==$id){
                $isVirtual=true;
                $vClientDeleted= $this->BP->delete_vcenter_info($which, $sid);
                if($vClientDeleted===false){
                    return false;
                }
            }
        }

        //Check if client to be deleted is virtual(VMware or ESX)

        if($isVirtual===false){
            $clientInfo=$this->BP->get_client_info((int)$which,$sid);

            $isNAS = (($temp = strlen($clientInfo['name']) - strlen(Constants::NAS_POSTFIX)) >= 0 && strpos($clientInfo['name'], Constants::NAS_POSTFIX, $temp) !== FALSE);

            if($isNAS) {
                //route through delete storage function
                $storageName = substr($clientInfo['name'], 0, $temp);
                $storageID = $this->BP->get_storage_id($storageName, $sid);
                require_once("storage.php");
                $storage = New Storage($this->BP, $sid);
                return $storage->delete($storageID, $sid);
            } else {
                //Get alias list and check if there is an alias attached to client. If found aliases attached to client display false and display "This is alias for client xyz.
                //Remove alias and then try to delete client "

                $hostInfo=$this->BP->get_host_info($clientInfo['name'],$sid);

                $deleteClientResult=$this->BP->delete_client($which, $sid);
                if($deleteClientResult===false){
                    return false;
                }else{
                    if($hostInfo['ip']==="127.0.0.1"){
                        $newAliasList=null;
                        $aliasList=$hostInfo['aliases'];
                        foreach($aliasList as $key=>$val){
                            if($val!==$clientInfo['name']){
                                $newAliasList[]=$val;
                            }
                        }
                        $hostInfo['original_ip'] = $hostInfo['ip'];
                        $hostInfo['aliases']=$newAliasList;
                        $saveHost=$this->BP->save_host_info($hostInfo,$sid);
                        if($saveHost===false){
                            global $Log;
                            $msg= "could not update host file".$this->BP->getError();
                            $Log->writeError($msg, true);
                        }
                    }
                    else{
                        $removeHost=$this->BP->remove_host_info($clientInfo['name'],$sid);
                        if($removeHost===false){
                            global $Log;
                            $msg = 'Cannot remove host ' . $clientInfo['name'] . ' ' . $this->BP->getError();
                            $Log->writeError($msg, true);
                        }
                    }
                }
            }
        }
    }



    function addClient($which, $inputArray, $sid){
        if($sid===false) {
            $sid = $this->BP->get_local_system_id();
        }
        $hostInfo=$inputArray['host_info'];
        $credentialsInfo=$inputArray['credentials'];
        $isValid=$this->isValid($inputArray,$sid);
        if($isValid['valid']) {
            if ($inputArray['os_type'] == "VMware") {
                $vcenterExists=false;
                // VMware path:  add vCenter on a create.
                $bValid = $this->addHostAlias($this->BP, 'localhost', VCENTER_RRC, $sid);
                $clients=$this->BP->get_client_list($sid);
                foreach($clients as $key=>$value){
                    if($value===VCENTER_RRC){
                        $vcenterExists=true;
                    }
                }
                if(!$vcenterExists){
                    $inputArray['os_type']=VCENTER_RRC;
                    if($inputArray['os_type']===VCENTER_RRC){
                        $clientInfo['name']=VCENTER_RRC;
                        $clientInfo['is_enabled'] = true;
                        $clientInfo['is_synchable'] = false;
                        $clientInfo['is_encrypted'] = false;          //Setting "true" not allowed currently at VM/instance level. Check later for modifications.
                        $clientInfo['priority'] = 500;
                        $clientInfo['no_credentials'] =true;
                        $result = $this->BP->save_client_info($clientInfo, $sid);
                        if($result===false){
                            return false;
                        }
                    }
                }

                if($credentialsInfo === NULL){
                    return "Please enter username and password";
                }
                $vcenterInfo['address']=$hostInfo['ip'];
                $vcenterInfo['login']=$credentialsInfo['username'];
                $vcenterInfo['password']=$credentialsInfo['password'];
                $vcenter=$this->BP->save_vcenter_info($vcenterInfo, $sid);
                if($vcenter===false){
                    return false;
                }
                return $result;
            }
            else {
                if ($hostInfo['ip']!="") {
                    // Regular client.  First add to host file.
                    $clientToHost = $this->addClientToHostFile($inputArray, $sid);
                    if ($clientToHost === false) {
                        $status['error'] = 500;
                        $status['message'] = $this->BP->getError();
                        return $status;
                    }
                    else{
                        $credentialsResult  = $this->setCredentials($inputArray,$sid);
                        if($credentialsResult === false){
                            $status['error'] = 500;
                            $status['message'] = $this->BP->getError();
                            return $status;
                        }
                        else{
                            if($inputArray["install_agent"]){
                                $pushAgent = $this -> pushSupportedAgents($inputArray, $sid);
                                if($pushAgent === false){
                                    $status['error'] = 500;
                                    $status['message'] = $this->BP->getError();
                                    return $status;
                                }
                            }
                        }
                    }
                }
            }
        }
        else{
            $status['error'] = 500;
            $status['message'] = $isValid['message'];
            return $status;
        }

        if (isset($inputArray['name'])) {
            $name = $inputArray['name'];
            if(strlen($name)<=31)
                $clientInfo = array ("name" => $name);
        }

        if (isset($inputArray['defaultschedule'])) {
            $addToDefault = $inputArray['defaultschedule'];
        }

        if (isset($inputArray['credential_id'])) {
            $crid = $inputArray['credential_id'];
            if ($crid == -1 or $crid == "-1") {
                $clientInfo["no_credentials"] = true;
            } else {
                $clientInfo["credential_id"] = $crid;
            }
        }
        else{
            if($credentialsInfo['username']==="" and $credentialsInfo['password']===""){
                $clientInfo["no_credentials"] = true;
            }
        }

        $clientInfo['is_enabled'] = $inputArray['is_enabled'];
        $clientInfo['is_synchable'] = $inputArray['is_synchable'];
        $clientInfo['is_encrypted'] = $inputArray['is_encrypted'];
        $clientInfo['priority'] = isset($inputArray['priority']) ? $inputArray['priority'] : 500 ;

        if ( isset ($inputArray['oid']) && $inputArray['oid'] == 40 && isset($inputArray['generic_property']) ) {
            if ( $inputArray['generic_property'] === Constants::GENERIC_PROPERTY_XEN ) {
                $useRaeSave = false;
            } else {
                $useRaeSave = true;
            }
            $clientInfo['os'] = $inputArray['oid'];
            $clientInfo['os_type_id'] = isset($inputArray['oid']) ? strval($inputArray['oid']) : false;
            $clientInfo['generic_property']=$inputArray['generic_property'];
        } else {
            $useRaeSave = false;
        }

        global $Log;

        // How do I get port?

        $Log->writeVariable( "Setting the value for the port" );
        if ( isset( $inputArray['port'] )) {
            $clientInfo['remote_address'] = $inputArray['remote_address'];
            $useRaeSave = false;
            $clientInfo['remote_port'] = isset($inputArray['port']) ? (int)$inputArray['port'] : null;
            $Log->writeVariable( "The port is ".$clientInfo['port'] );

            $Log->writeVariable( $useRaeSave ? "Saving with RAE" : "Saving old way" );
        }  elseif ( isset( $inputArray['remote_address'] ))  {
            $clientInfo['remote_address'] = $inputArray['remote_address'];
            $Log->writeVariable( $useRaeSave ? "Saving with RAE - with remote_address" : "Saving old way - with remote_address" );
        }  else {
            $Log->writeVariable( $useRaeSave ? "Saving with RAE - no port" : "Saving old way - no port" );
        }

        //Hostinfo array has IP information
        $Log->writeVariable( "Setting value for the IP" );
        if ( isset( $hostInfo['ip'] )) {
            $useRAESave = false;
            $Log->writeVariable( "The ip is ".$clientInfo['remote_address'] );

            $Log->writeVariable( $useRaeSave ? "Saving with RAE" : "Saving old way" );
        } else {
            $Log->writeVariable( $useRaeSave ? "Saving with RAE - no ip" : "Saving old way - no ip" );
        }

        if (isset($inputArray['ssl'])) {
            $clientInfo['use_ssl'] = $inputArray['ssl'] == '1' ? true : false;
        }

        if ($useRaeSave) {
            $result = $this->BP->save_rae_client_info($clientInfo, $sid);
        } else {
            $result = $this->BP->save_client_info($clientInfo, $sid);
        }

        if ($result !== false) {

            $result = array();
            if ( $name !== VCENTER_RRC) {
                // If adding a client ($clientInfoID == NULL) and not client vCenter-RRC, create a schedule.
                // Let the user knows if this step is not successful.
                if ($addToDefault) {
                    // Comment this out.  Do not add to default global schedule any more.  Create client schedule.
                    //$result = $BP->add_client_to_default_schedule($clientInfo['name'], $dpuID);
                    $scheduleCreateResult = $this->createClientSchedule($clientInfo, $sid);
                    if ($scheduleCreateResult === false) {
                        $msg = null;
                        $msg .= "Successfully added asset " . $clientInfo['name'] . ", but a backup schedule was not " .
                            "automatically created: " . $this->BP->getError();
                        $result['message'] = $msg;
                    }
                }
            }
            $clientID=$this->BP->get_client_id($clientInfo['name'], $sid);
            $output['id'] = $clientID;

            // Do an inventory sync on the newly added client (done automatically for VMware)
            $syncStartStatus = $this->BP->put_inventory(array('client_id'=>$clientID), $sid);
            if ( $syncStartStatus !== true )
            {
                if ( array_key_exists('message', $result) )
                {
                    $Log->writeVariable( "The message is ".$result['message'] );
                    $result['message'] .= '.  An inventory sync was not automatically started for '.$clientInfo['name'].': ' . $this->BP->getError();
                }
                else
                {
                    $result['message'] = 'An inventory sync was not automatically started for '.$clientInfo['name'].': ' . $this->BP->getError();
                }
            }

            $result['result'] = $output ;
            return $result;
        } else {
            // on failure, return error to user.
            // At one time, we were deleting the associated credentialID, but should not as we have a credentials manager.

            $status['message'] = $this->BP->getError();
            $status['error'] = 500;
            $tempHostInfo = $this->BP->get_host_info($clientInfo['name'], $sid);
            if($tempHostInfo['ip']==="127.0.0.1"){
                $newAliasList=null;
                $aliasList=$tempHostInfo['aliases'];
                foreach($aliasList as $key=>$val){
                    if($val!==$clientInfo['name']){
                        $newAliasList[]=$val;
                    }
                }
                $tempHostInfo['original_ip'] = $tempHostInfo['ip'];
                $tempHostInfo['aliases']=$newAliasList;
                $saveHost=$this->BP->save_host_info($tempHostInfo,$sid);
                if($saveHost===false){
                    global $Log;
                    $msg= "could not update host file".$this->BP->getError();
                    $Log->writeError($msg, true);
                }
            }
            else{
                $removeHost=$this->BP->remove_host_info($clientInfo['name'],$sid);
                if($removeHost===false){
                    global $Log;
                    $msg = 'Cannot remove host ' . $clientInfo['name'] . ' ' . $this->BP->getError();
                    $Log->writeError($msg, true);
                }
            }
            return $status;
        }
    }

    function addClientToHostFile(&$input, $sid) {
        $hostInfo = $this->BP->get_host_info($input['name'], $sid);
        $hostInfoFromUser = $input['host_info'];
        $ipchange = false;
        if($hostInfo===false and  $input['os_type']  !== PROTECT_GENERIC_NAS){
            $newhostInfo=$input['host_info'];
        }
        else{
            $newhostInfo=$hostInfo;
            if($hostInfo['aliases']!==""){
                $aliases[]=$hostInfo['aliases'];
            }
            if($hostInfoFromUser['ip'] !== $hostInfo['ip']){
               $ipchange = true;
                $newhostInfo['original_ip'] = $hostInfo['ip'];
                $newhostInfo['ip'] = $hostInfoFromUser['ip'];
            }
        }
        if($aliases!==null){
            $newname=strtolower($input['name']);
            foreach($aliases as $alias){
                if(strtolower($alias)===$newname){
                    return "Alias already exists";
                }
            }
        }

        if ($this -> isGenericOS($input['os_type'])) {

            $genericID=0;
            switch($input['os_type']){
                case PROTECT_GENERIC_UCSM:
                    $genericID=0;
                    break;
                case PROTECT_GENERIC_UCSC:
                    $genericID=1;
                    break;
                case PROTECT_GENERIC_NAS:
                    $genericID=4;
                    break;
                case Constants::ADD_CLIENT_DISPLAY_NAME_XEN:
                    $genericID=Constants::GENERIC_PROPERTY_XEN;
                    break;
                default:
                    $genericID=40;
                    break;
            }
            $input['oid']=40;
            $input['generic_property']=$genericID;

            if ( $input['generic_property'] === 4 or $input['generic_property'] == Constants::GENERIC_PROPERTY_XEN ) {
                $hostInfo=$input['host_info'];
                $input['remote_address']=$hostInfo['ip'];
                $result = $this->addHostAlias($this->BP, 'localhost', $input['name'], $sid);
            }

        }

        if($ipchange === false and $hostInfo !== false){
            $result = true;                                     //This means hostinfo for this client is already present and ip is not changed. So continue with the process of addition of client.
        }
        else{
            $newhostInfo['name']=$input['name'];
            $result = $this->BP->save_host_info($newhostInfo, $sid);
        }

        return $result;
    }

    function setCredentials(&$input,$sid){
        if (!$input['is_auth_enabled']) {
            return true;
        }
        $credInfo=$input['credentials'];
        $hostInfo=$input['host_info'];
        if(isset($input['credential_id']) and $input['credential_id']!==null){
            $saveCred = $input['credential_id'];
        }
        else{
            if($credInfo['display_name'] === "" or $credInfo['display_name'] === null){
                $newCredInfo['display_name'] = $input['name'] . "-New-Credential";
            }
            else {
                $newCredInfo['display_name'] = $credInfo['display_name'];
            }
            if( $credInfo['is_default'] === "" or $credInfo['is_default'] === NULL){
                $newCredInfo['is_default'] = false;
            } else {
                $newCredInfo['is_default'] = $credInfo['is_default'];
            }
            $newCredInfo['username'] = $credInfo['username'];
            $newCredInfo['password'] = $credInfo['password'];
            $newCredInfo['domain'] = $credInfo['domain'];
            if($newCredInfo['username'] !== "" and $newCredInfo['password'] !== "" ){
                $saveCred = $this->BP->save_credentials($newCredInfo,$sid);
                if($saveCred === false){
                    return false;//$this->BP->getError();
                }
                else{
                    $input['credential_id'] = $saveCred;
                    return true;
                }
            }
        }
    }


    function pushSupportedAgents(&$input,$sid){
        $credInfo=$input['credentials'];
        $hostInfo=$input['host_info'];

        $cid=isset($input['cid']) ? $input['cid'] : "-1";
        $ip=isset($hostInfo['ip']) ? $hostInfo['ip'] : "";
        $ostype=isset($input['os_type']) ? $input['os_type'] : "";
        $credid=isset( $input['credential_id']) ?  $input['credential_id'] : "-1";

        if($input['os_type'] === "Windows"){
            $result = $this->BP->push_agent($cid, $ip, $ostype, $credid, $sid);
            if($result === false){
                $result = $this->BP->getError();
            }
        }
        return $result;

    }

    function isGenericOS($osType){
        if ($osType === PROTECT_GENERIC_UCSM
            or $osType === PROTECT_GENERIC_UCSC
            or $osType === PROTECT_GENERIC_NAS
            or $osType === Constants::ADD_CLIENT_DISPLAY_NAME_XEN
            or $osType === ALL_OTHER_OS) {
            return true;
        }
        else{
            return false;
        }
    }



    function addHostAlias($BP, $hostName, $aliasName, $dpuID) {
        $bValid = true;
        $hostInfo = $this->BP->get_host_info($aliasName, $dpuID);
        if ($hostInfo !== false) {
            ; // the alias name is already present, don't need to add it.
        } else {
            // The alias is not there, add to the host entry for host name.
            $hostInfo = $BP->get_host_info($hostName, $dpuID);
            if ($hostInfo !== false) {
                // Update existing entry, add the alias if not already present.
                $hostInfo['original_ip'] = $hostInfo['ip'];
                $aliases = isset($hostInfo['aliases']) ? $hostInfo['aliases'] : array();
                if (!in_array($aliasName, $aliases)) {
                    $aliases[] = $aliasName;
                    $hostInfo['aliases'] = $aliases;
                    $result = $BP->save_host_info($hostInfo, $dpuID);
                    if ($result === false) {
                        global $Log;
                        $Log->writeError($BP->getError(), true);
                    }
                    $bValid = $result;
                } else {
                    $bValid = true;
                }
            } else {
                $bValid = false;
            }
        }
        return $bValid;
    }

    function removeHostAlias($BP, $hostName, $aliasName, $dpuID) {
        global $Log;
        $Log->writeVariable( "RemoveHost alias ".$aliasName) ;
        $bValid = true;
        $newHostInfo = $this->BP->get_host_info($hostName, $dpuID);
        if ($newHostInfo !== false) {
            $newAliasList=null;
            $aliasList=$newHostInfo['aliases'];
            foreach($aliasList as $key=>$val){
                if($val!==$aliasName){
                    $newAliasList[]=$val;
                }
            }
            $newHostInfo['original_ip'] = $newHostInfo['ip'];
            $newHostInfo['aliases']=$newAliasList;
            $saveHost=$this->BP->save_host_info($newHostInfo,$dpuID);
            if($saveHost===false){
                global $Log;
                $msg= "could not update host file".$this->BP->getError();
                $Log->writeError($msg, true);
            }
            $bValid = $saveHost;
        } else {
            $bValid = false;
        }
        return $bValid;
    }


    function createClientSchedule($clientInfo, $dpuID) {
        global $BP;

        // Use Friday Master, Daily Incremental as the default.
        $iCal = <<< EOM
BEGIN:VCALENDAR
BEGIN:VEVENT
SUMMARY:Master
DESCRIPTION:weekly master
DTSTART:20120603T030000
DTEND:20120603T040000
RRULE:FREQ=WEEKLY;BYDAY=SU
END:VEVENT
BEGIN:VEVENT
SUMMARY:Incremental
DESCRIPTION:daily incremental
DTSTART:20120604T030000
DTEND:20120604T040000
RRULE:FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR
END:VEVENT
END:VCALENDAR
EOM;

        $clientID = $BP->get_client_id($clientInfo['name'], $dpuID);

        $defaultName = $clientInfo['name'] . ' Schedule';
        $description = 'Automatically created backup schedule for ' . $clientInfo['name'];

        $backupOptions = 	array(	'verify_level' => 3);			// set verify level to inline by default.
        $scheduleOptions = 	array(	'client_id' => $clientID,
            'include_new' => false,			// cannot be true for file-level schedule
            'email_report' => true,
            'failure_report' => true);

        $scheduleInfo = 	array(	'name' => $defaultName,
            'description' => $description,
            'enabled' => true,				// create as enabled.
            'app_id' => 1,					// app_id 1 is file-level
            'calendar' => $iCal,
            'backup_options' => $backupOptions,
            'schedule_options' => $scheduleOptions);

        $result = $BP->save_app_schedule_info($scheduleInfo, $dpuID);

        return $result;
    }


    function isValid(&$input,$sid){
        $error['valid'] = true;
        $server = $input['name'];
        $os = $input['os_type'];
        $hostinfo = $input['host_info'];
        if($os === "VMware") {
            if ($hostinfo['ip'] === "" && ($server === "" or $server === null)) {
                $error['message'] = "Either VMware server name or IP address is required.";
                $error['valid'] = false;
            }
        }
            else{
                if($server === ""){
                    $error['message'] = "Host Name is required.";
                    $error['valid'] = false;
                }
                if($os === "Windows" and (strlen($server) > 15)){
                    $error['message'] = "Host Name is too long.  For Windows, the maximum allowed is 15 characters.";
                    $error['valid'] = false;
                }
                elseif(strlen($server)> 31){
                    $error['message'] = "Host Name is too long.  The maximum allowed is 31 characters.";
                    $error['valid'] = false;
                }
            }

        //Currently using "is_auth_enabled" as input parameter to check if user wants to establish trust

        if ($error['valid'] && $input['is_auth_enabled'] == true) {
            $errorString = $this-> validateCredentials($input, $sid);
            if ($errorString != null) {
                $error['message'] = $errorString;
                $error['valid'] = false;
                return $errorString;
            }
        }
        return $error;
    }

    function validateCredentials(&$input, $sid) {
        $s = NULL;
        if (isset($input['credentials'])) {
            $credInfo = $input['credentials'];
            $credlist = $this->BP->get_credentials_list($sid);
            $exists = false;
            foreach($credlist as $credential){
                if($credential['display_name'] === $credInfo['display_name'] or $credential['display_name'] === $input['name'] . "-New-Credential"){
                    $exists = true;
                    $input['credential_id'] = $credential['credential_id'];
                    $credInfo['display_name'] = $credential['display_name'];
                    $input['credentials'] = $credInfo;
                }
            }

            if(!$exists){
                if (!$credInfo['is_default']) {
                    if($input['credential_id'] === -1 or $input['credential_id'] === "-1"){
                        $s = "No credentials have been selected";
                    }
                    else if($credInfo['username'] === null or $credInfo['username'] === "" ){
                        $s = "Enter username";
                    }
                    else if($credInfo['password'] === null or $credInfo['password'] === "" ){
                        $s = "Enter password";
                    }
                }
                else{
                    $defaultcred = $this->BP->get_default_credentials($sid);
                    $input['credential_id'] = $defaultcred['credential_id'];
                    if($defaultcred === null or $defaultcred === ""){
                        $s="No default credentials exist.  Please create a new credential or select an existing one.";
                    }
                    else if($defaultcred['credential_id'] !== $credInfo['credential_id']){
                        $s = "Not able to use default credentials.  Please enter username and password.";
                    }
                }
            }
        } else if (isset($input['credential_id'])) {
            $id = $input['credential_id'];
            if ($id < 0) {
                $s = "Invalid credential id.";
            }
        } else {
            $s = "No credentials were specified.";
        }
        return $s;
    }

    function agentPush($data) {
        $allClients = array();
        foreach($data['clients'] as $client) {
            $clientReturn = array();

            if(isset($client['client_id'])) {
                $clientID = $client['client_id'];
            } else {
                //log that client id is needed for this client
                $clientReturn['client'] = "";
                $clientReturn['message'] = "Client ID is required.";
                $allClients[] = $clientReturn;
                continue;
            }
            if(isset($client['system_id'])) {
                $sid = $client['system_id'];
            } else {
                //log that system id is needed for this client
                $clientReturn['client'] = $clientID;
                $clientReturn['message'] = "A system id for this client is required.";
                $allClients[] = $clientReturn;
                continue;
            }
            if(isset($client['credential_id'])) {
                $credentialID = $client['credential_id'];
            } else {
                $credentialID = null;
            }
            $clientInfo = $this->BP->get_client_info($clientID, $sid);
            if($clientInfo !== false) {
                $clientName = $clientInfo['name'];
                $osFamily = $this->functions->getOSFamily($clientInfo['os_type_id']);
                if($osFamily == Constants::OS_FAMILY_WINDOWS) {
                    $updateAvailable = false;
                    $updateInfo = $this->BP->agent_update_available($clientID, $sid);
                    if($updateInfo !== false) {
                        foreach($updateInfo as $update) {
                            if($clientID == $update['clientID']) {
                                $updateAvailable = $updateInfo['isUpdateAvailable'];
                            }
                        }
                        if($updateAvailable !== false) {
                            $ip = $this->getIP($clientName, $sid);
                            $agentPushResult = $this->BP->push_agent($clientID, $ip, $osFamily, $credentialID, $sid);
                            if($agentPushResult !== false) {
                                $clientReturn['client'] = $clientName;
                                $clientReturn['message'] = "Update successful.";
                            } else {
                                $clientReturn['client'] = $clientName;
                                $clientReturn['message'] = $this->BP->getError();
                            }
                        } else {
                            //log message that agent is up to date
                            $clientReturn['client'] = $clientID;
                            $clientReturn['message'] = "Latest available agent version is installed";
                        }
                    } else {
                        //log message from update available
                        $clientReturn['client'] = $clientID;
                        $clientReturn['message'] = $this->BP->getError();
                    }
                } else {
                    //log error that this is not Window client. Nothing to do
                    $clientReturn['client'] = $clientName;
                    $clientReturn['message'] = "This client is not a Windows client. Agents can only be installed or updated on Windows clients.";
                }
            } else {
                //log unable to retrieve the client information
                $clientReturn['client'] = $clientID;
                $clientReturn['message'] = $this->BP->getError();
            }

            $allClients[] = $clientReturn;
        }
        return $allClients;
    }

/*    function getClientName($clientID, $sid) {
        $clientInfo = $this->BP->get_client_info($clientID, $sid);
        if($clientInfo !== false) {
            return $clientInfo['name'];
        } else {
            return false;
        }
    } */

    function getIP($clientName, $sid) {
        $hostInfo = $this->BP->get_host_info($clientName, $sid);
        if($hostInfo !== false) {
            if(isset($hostInfo['ip'])) {
                return $hostInfo['ip'];
            } else {
                return $clientName;
            }
        } else {
            return $clientName;
        }
    }
}

?>
