<?php

class Jobs
{
//in progress
    private $BP;

    const HYPER_V = "Hyper-V";
    const VMWARE = "VMware";
    const XEN = "Xen";

    public function __construct($BP, $sid)
    {
        $this->BP = $BP;
        $this->now = time();
		$this->sid = $sid;
	    $this->itemCount = 0;

        require_once('function.lib.php');
        $this->functions = new Functions($this->BP);

        require_once('jobs/jobs.functions.php');
        $this->jobsFunctions = new JobsFunctions($this->BP);
    }


    public function get($which, $filter, $sid, $systems)
    {

// TODO: first parameters, $which, can be a job category, e.g., "active", "history",

// The 2nd paramater, filter, gives a type of job, e.g., active/backup returns all active backup jobs
//														 active returns all active jobs (no filter
//														 active/restore returns all restore jobs.
//														 active/replication returns all active replication "jobs"
//														 (so logic from the operations APIs will need to be used)
//														 active/archive will return all archive jobs

// For a $which of "history", need to build the job information from bp_get_backup_status output.
        $allJobs = array();

        // True if we need to add the "data" array wrapper to the results and false if not.
        $add_data_wrapper = true;
        $special_job_type = explode(".", $which);
        $limits = array();

        switch ($which) {
            case "history" :
                require_once('jobs/jobs-history.php');
                $jobsHistory = new JobsHistory($this->BP, $sid, $this->functions);
                $allJobs = $jobsHistory->get_jobs_history($which, $filter, $sid, $limits);

                if (isset($_GET['format']) && ($_GET['format'] == 'csv')) {
                    $jobs['data'] = $allJobs;
                    $allJobs = $this->jobsFunctions->buildCSVExport($jobs);
                    $add_data_wrapper = false;
                }
                break;

            case "system":
                $filter = "history";
                require_once('audit-history.php');
                $audit = new AuditHistory($this->BP, $sid);
                $allJobs = $audit->get($filter);

                if (isset($_GET['format']) && ($_GET['format'] == 'csv')) {
                    $allJobs = $this->jobsFunctions->buildCSVExport($allJobs);
                }
                $add_data_wrapper = false;
                break;

            case "active" :
            case is_int($which) :
            case is_array($special_job_type) && count($special_job_type) > 1:
                require_once('jobs/jobs-active.php');
                $jobsActive = new JobsActive($this->BP, $sid, $this->functions);
                $allJobs = $jobsActive->get_active_jobs($which, $filter, $sid, $systems);
                break;
        }

        if ($add_data_wrapper) {
            $data = array('data' => $allJobs);
            if (count($limits)> 0) {
                $data['count'] = $limits['count'];
                $data['limited'] = $limits['limited'];
            }
            return $data;
        } else {
            return($allJobs);
        }

    }

    public function delete($which){
        if ($which) {
            if (is_string($which)) {
                if (strpos($which, '.ir') !== false) {
                    $id = str_replace('.ir', '', $which);
                    $result = $this->deleteIRJobs((int)$id, $this->sid);
                } elseif (strpos($which, '.flr') !== false) {
                    $id = str_replace('.flr', "", $which);
                    if($id == "mount"){
                        $result = $this->BP->backup_unmount($this->sid);
                    } else {
                        $result = $this->deleteFLRJobs((int)$id, $this->sid);
                    }
                } elseif (strpos($which, '.rep') !== false) {
                    $id = str_replace('.rep', '', $which);
                    $result = $this->deleteReplicatedBackup($id, $this->sid);
                } else {
                    $jobs = explode(',', $which);
                    foreach ($jobs as $job) {
                        $result = $this->BP->cancel_job($job, $this->sid);
                    }
                }
            }
        }

        return $result;
    }

    function deleteReplicatedBackup($jobID, $sid)
    {
        $repJob = $this->BP->get_replication_active_job_info(array('system_id' => $sid));
        if ($repJob !== false) {
            require_once('replication.php');
            $replication = new Replication($this->BP);
            $queue = $replication->get("queue", null, null, $sid);

            if (array_key_exists('data', $queue)) {
                $data = $queue['data'];
                foreach ($data as $queuedJobs){
                    //are there queued jobs?
                    if(is_array($queuedJobs['inactive']) && !empty($queuedJobs['inactive'])) {
                        $inactive = $queuedJobs['inactive'];
                        foreach ($inactive as $index => $queueItem) {
                            if ($queueItem['queue_position'] == $jobID) {
                                $backupID = $queueItem['backup_id'];
                                $target = $queueItem['target'];
                                $filter[] = array(
                                    'backup_no' => $backupID,
                                    'target_name' => $target
                                );

                                $deleteFilter=array(
                                    "backup_targets"=>$filter,
                                    "action"=>"abort");
                                $result = $this->BP->delete_from_replication_queue($deleteFilter,$sid);
                            }
                        }
                    }

                    if(is_array($queuedJobs['active'])) {
                        $active = $queuedJobs['active'];
                        foreach ($active as $index => $queueItem) {
                            if ($queueItem['queue_position'] == $jobID) {
                                $backupID = $queueItem['backup_id'];
                                $target = $queueItem['target'];
                                $filter[] = array(
                                    'backup_no' => $backupID,
                                    'target_name' => $target
                                );

                                $deleteFilter=array(
                                    "backup_targets"=>$filter,
                                    "action"=>"abort");
                                $result = $this->BP->delete_from_replication_queue($deleteFilter,$sid);
                            }
                        }
                    }
                }
            }
        }

        return $result;

    }


    public function suspend_resume($action, $which){
        if (is_string($which)) {
            $jobs = explode(',', $which);
            foreach($jobs as $job){

                switch ($action) {
                    case "suspend":
                        $result = $this->BP->suspend_job($job, $this->sid);
                        break;
                    case "resume":
                        $result = $this->BP->resume_job($job, $this->sid);
                        break;
                    default:
                        $result = "Bad action specified.  Jobs can only be suspended or resumed.";
                        break;
                }
            }
        }
        return $result;
    }

    function deleteIRJobs($id, $sid)
    {
        $appInfo = $this->BP->get_appinst_info($id, $sid);
        if($appInfo !== false) {
            foreach($appInfo as $app) {
                $appName = $app['app_name'];

                //We pass in only one instanceID and are concerned only with that
                break;
            }
        }
        if($appName == 'VMware') {
            $vmwareStatus = $this->BP->get_vm_ir_status($sid);
            if($vmwareStatus !== false) {
                foreach($vmwareStatus as $status) {
                    if($status['id'] === $id) {
                        $audit = isset($status['audit']) ? $status['audit'] : 0;
                    }
                }
            }
            $force = $audit;
            $result = $this->BP->vmware_ir_destroy($id, $force, $sid);
        } else {
            $hypervStatus = $this->BP->get_hyperv_ir_status($sid);
            if($hypervStatus !== false) {
                foreach($hypervStatus as $status) {
                    if($status['id'] === $id) {
                        $audit = isset($status['audit']) ? $status['audit'] : 0;
                    }
                }
            }
            $force = $audit;
            $result = $this->BP->hyperv_ir_destroy($id, $force, $sid);
        }
        return $result;
    }

    function deleteFLRJobs($id, $sid){
        $instInfo = $this->functions->getInstanceNames($id, $sid);

        if ($instInfo !== false){
            if ($instInfo['app_type'] == Jobs::HYPER_V or $instInfo['app_type'] === JOBS::VMWARE or $instInfo['app_type'] === Jobs::XEN) {
                $result = $this->BP->destroy_disk_image($id, $sid);
            } else {
                $result = $this->BP->delete_application_share($id, $sid);
            }
        }

        return $result;
    }
} //end Jobs class


