<?php

class JobsHistory
{
    private $BP;

    public function __construct($BP, $sid, $Functions)
    {
        $this->BP = $BP;
        $this->now = time();
		$this->sid = $sid;
	    $this->itemCount = 0;

        $this->functions = $Functions;

        require_once('jobs.functions.php');
        $this->jobsFunctions = new JobsFunctions($this->BP);
    }


    public function get_jobs_history($which, $filter, $sid, &$limits)
    {
        $allJobs = array();
        $startDate = isset($_GET['start_date']) ? strtotime($_GET['start_date']) :
                     isset($_GET['days']) ? strtotime('today -' . $_GET['days'] - 1 . ' days') : strtotime(Constants::DATE_ONE_WEEK_AGO);
        $endDate = isset($_GET['end_date']) ? $this->functions->formatEndDate($_GET['end_date']) : strtotime('now');
        $failed = isset($_GET['result']) ? $_GET['result'] : null;
        $includeOnDemand = (isset($_GET['include_on_demand']) and $_GET['include_on_demand'] === 'false') ? false : true;
        $includeDisabled = (isset($_GET['include_disabled']) and $_GET['include_disabled'] === 'false') ? false : true;
        $includeSystemMetadata = (isset($_GET['include_metadata']) and $_GET['include_metadata'] === 'true') ? true : false;
        $showFlat = (isset($_GET['showFlat']) and $_GET['showFlat'] === 'true') ? true : false;
        $sort = (isset($_GET['sort'])) ? $_GET['sort'] : false;
        $limit = (isset($_GET['limit']) && (int)$_GET['limit'] > 0) ? (int)$_GET['limit'] : false;
        $limitHit = false;

        //did they pass in sid, cid, or iid?
        if ($sid !== false) {
            $historyFilter = array(
                'start_time' => $startDate,
                'end_time' => $endDate,
                'include_on_demand' => $includeOnDemand,
                'include_disabled' => $includeDisabled
            );

            $name = $this->functions->getSystemNameFromID($sid);
            $jobHistory = $this->BP->get_schedule_history($historyFilter, $sid);
            $replicationJobHistory = $this->BP->get_replication_job_history_info(array_merge($historyFilter, array('system_id' => $sid)));

            if ($jobHistory !== false) {
                foreach ($jobHistory as $history) {
                    $data = $this->buildHistoryOutput($history, $name, $sid, null, $which, $startDate, $failed, $showFlat);
                    $filtered = $this->buildReturn($data, $filter);
                    if ($filtered != NULL) {
                        if ($showFlat) {
                            $allJobs = array_merge($allJobs, $filtered);
                        } else {
                            $allJobs[] = $filtered;
                        }
                    }
                }
            } else {
                // This will fail for any un-managed source.  Need to filter them out of $systems list, then this can be uncommented.
                // global $Log;
                //$Log->writeError("job history:" . $this->BP->getError(), true);
            }

            $replicationHistoryArray = array();
            if ($replicationJobHistory !== false) {
                foreach  ($replicationJobHistory as $replicationHistory){
                    if (!$includeSystemMetadata && isset($replicationHistory['instance_name']) &&
                        $replicationHistory['instance_name'] == "System Metadata") {
                        continue;
                    }
                    $data = $this->buildReplicationHistoryOutput($replicationHistory, $sid, $startDate, $failed);
                    if (!empty($data)) {
                        $replicationHistoryArray[] = $data;
                    }
                }
            } else {
                // This will fail for any un-managed source.  Need to filter them out of $systems list, then this can be uncommented.
                // global $Log;
                //$Log->writeError("replication job history:" . $this->BP->getError(), true);
            }

            $allJobs = array_merge($allJobs, $replicationHistoryArray);

        } else {
            $systems = $this->functions->selectSystems();
            foreach ($systems as $sid => $name) {
                $historyFilter = array(
                    'start_time' => $startDate,
                    'end_time' => $endDate,
                    'include_on_demand' => $includeOnDemand,
                    'include_disabled' => $includeDisabled
                );

                $jobHistory = $this->BP->get_schedule_history($historyFilter, $sid);
                $replicationJobHistory = $this->BP->get_replication_job_history_info(array_merge($historyFilter, array('system_id' => $sid)));

                $systemHistory = array();
                if ($jobHistory !== false) {
                    foreach ($jobHistory as $history) {
                        $data = $this->buildHistoryOutput($history, $name, $sid, null, $which, $startDate, $failed, $showFlat);
                        if (!empty($data)) {
                            if ($showFlat) {
                                $systemHistory = array_merge($systemHistory, $data);
                            } else {
                                $systemHistory[] = $data;
                            }
                        }
                    }
                } else {
                    // This will fail for any un-managed source.  Need to filter them out of $systems list, then this can be uncommented.
                    // global $Log;
                    //$Log->writeError("job history:" . $this->BP->getError(), true);
                }
                $allJobs = array_merge($allJobs, $systemHistory);

                $replicationHistoryArray = array();
                if ($replicationJobHistory !== false) {
                    foreach  ($replicationJobHistory as $replicationHistory){
                        if (!$includeSystemMetadata && isset($replicationHistory['instance_name']) &&
                            $replicationHistory['instance_name'] == "System Metadata") {
                            continue;
                        }
                        $data = $this->buildReplicationHistoryOutput($replicationHistory, $sid, $startDate, $failed);
                        if (!empty($data)) {
                            $replicationHistoryArray[] = $data;
                        }
                    }
                } else {
                    // This will fail for any un-managed source.  Need to filter them out of $systems list, then this can be uncommented.
                    // global $Log;
                    //$Log->writeError("replication job history:" . $this->BP->getError(), true);
                }

                $allJobs = array_merge($allJobs, $replicationHistoryArray);
            }
        }
        if ($showFlat && $sort !== false) {
            $this->sort($allJobs, $sort);
            if ($limit > 0 && count($allJobs) > $limit) {
                $limitHit = true;
                array_splice($allJobs, $limit, count($allJobs) - $limit);
            }
        }
        $limits['count'] = count($allJobs);
        $limits['limited'] = $limitHit;

        return $allJobs;
    }

    public function buildHistoryOutput($jobData, $sysName, $sid, $clientName,$which, $historyStartDate = null, $failed = null, $showFlat = false)
    {
        $data = array();
        $hasRun = false;

        $scheduleName = isset($jobData['schedule_name']) ? $jobData['schedule_name'] : null;
        // Do this as the core is not returning duration.  Format duration as a string.
        $duration = isset($jobData['start_time']) ? $this->functions->formatTimeDelta($this->now - $jobData['start_time']) : "n/a";
        $appName = isset($jobData['application_name']) ? $jobData['application_name'] : "";
        $appType = isset($jobData['app_type']) ? $jobData['app_type'] : "";
        $scheduleDescription = isset($jobData['schedule_description']) ? $jobData['schedule_description'] : "";
        $failures = isset($jobData['failures']) ? $jobData['failures'] : "";
        $status = ((int)($failures) > 0) ? 'Error' : 'Successful';
        $historyStartDate = $this->functions->formatDateTime($historyStartDate);
        $backups = isset($jobData['backups']) ? $this->jobsFunctions->buildBackupsOutput($jobData['backups'], $failed, $sid) : "";
        if($backups){
            $hasRun = true;
        }

        if ($failed == null or ($failed == "failed" and $failures !== 0)) {
            if ($appName == "on_demand") {
                $clientName = "";
            } else {
                $clientName = isset($jobData['client_name']) ? $jobData['client_name'] : $this->jobsFunctions->getClientNameFromBackup($backups);
            }

            if ($hasRun) {
                if ($hasRun) {
                    $item = array(
                        'client_name' => $clientName,
                        'app_name' => $appName == "Archive" ? "Backup Copy" : $appName,
                        'description' => $scheduleDescription,
                        'duration' => $duration,
                        'type' => $appType == "Archive" ? "Backup Copy" : "Backup",
                        'status' => $status,
                        'failures' => $failures,
                        'name' => $scheduleName,
                        'system_name' => $sysName,
                        'system_id' => $sid);

                    if ($showFlat) {
                        $data = array();
                        foreach ($backups as $backup) {
                            unset($item['start_date']);
                            if ($backup['id'] == 'N/A') {
                                $backup['id'] = $sid . '.' . $backup['sort_start_time'];
                            }
                            $data[] = array_merge($item, $backup);
                        }
                    } else {
                        $data = $item;
                        $data['start_date'] = $this->jobsFunctions->getStartDateFromBackup($backups, $historyStartDate);
                        $data['id'] = $sid . '.' . $this->itemCount++;
                        $data['backups'] = $backups;
                    }
                }
            }
        }

        return $data;
    }

    function buildReplicationHistoryOutput($replicationHistory, $sid, $startDate, $failed){
        $data = array();

      //  var_dump($replicationHistory);
        $target = isset($replicationHistory['target']) ? $replicationHistory['target'] : "n/a";
        $status = isset($replicationHistory['status']) ? $this->getReplicationStatusName($replicationHistory['status']) : "n/a";
        $appName = isset($replicationHistory['instance_name']) ? $replicationHistory['instance_name'] : "";
        $clientNameFromAPI = isset($replicationHistory['client_name']) ? $replicationHistory['client_name'] : "";
        $primaryName = isset($replicationHistory['primary_name']) ? $replicationHistory['primary_name'] : "n/a";
        $secondaryName = isset($replicationHistory['secondary_name']) ? $replicationHistory['secondary_name'] : "n/a";

        switch($appName){
            case Constants::APPLICATION_TYPE_NAME_VMWARE:
                $clientName = $primaryName;
                $assetName = $secondaryName;
                break;

            case Constants::APPLICATION_TYPE_NAME_HYPER_V:
            case Constants::APPLICATION_TYPE_NAME_EXCHANGE:
            case Constants::APPLICATION_TYPE_NAME_FILE_LEVEL:
            case Constants::APPLICATION_TYPE_NAME_ORACLE:
            case Constants::APPLICATION_TYPE_NAME_SHAREPOINT:
            case Constants::APPLICATION_TYPE_NAME_NDMP_DEVICE:
                $clientName = $clientNameFromAPI;
                $assetName = $primaryName;
                break;

            case Constants::APPLICATION_TYPE_NAME_SQL_SERVER:
            case Constants::APPLICATION_TYPE_NAME_XEN:
                $clientName = $clientNameFromAPI;
                $assetName = $secondaryName;
                break;

        }

        $message = 'Job cancelled by user.';
        $data = array(
            'type' => isset($replicationHistory['type']) ? $replicationHistory['type'] : "",
            'mode' => isset($replicationHistory['mode']) ? $this->functions->getBackupTypeDisplayName($replicationHistory['mode']) : "",
            'client_name' => $clientName,
            'app_name' => $appName,
            'backup_id' => isset($replicationHistory['backup_id']) ? $replicationHistory['backup_id'] : "n/a",
            'id' => isset($replicationHistory['id']) ? $replicationHistory['id'] : "N/A",
            'duration' => isset($replicationHistory['start_time']) ? $this->functions->formatTimeDelta($this->now - $replicationHistory['start_time']) : "n/a",
            'status' => $status,
            'start_time' => isset($replicationHistory['start_time']) ? $this->functions->formatDatetime($replicationHistory['start_time']) : "n/a",
            'system_name' => isset($replicationHistory['system_name']) ? $replicationHistory['system_name'] : "n/a",
            'system_id' => $sid,
            'target' => $target,
            'asset_name' => $assetName,
            'name' => isset($replicationHistory['schedule_name']) ? $replicationHistory['schedule_name'] : "n/a",
            'sort_start_time' => $replicationHistory['start_time']
        );

        if ($status == 'Cancelled') {
            $data['message'] = $message;
        } else {
            $data['message'] = $status;
        }

    return $data;
}


    //this needs to be moved job functions as it's used in both active and history
    function buildReturn($data, $filter){
        $skip = false;

        if ($filter != "") {
            switch ($filter) {

                // get them all from jobs list.
                case 'active':
                    break;
                // see if type matches user selection.  If not, skip it.
                default:
                    if ($data['type'] != $filter) {
                        $skip = true;
                    }
                    break;
            }
        }
        if ($skip) {
            $data = NULL;
        }

        return $data;
    }

    function getReplicationStatusName($status) {
        $value = $status;
        switch ($status) {
            case 'completed':
                $value = 'Success';
                break;
            case 'failed':
            case 'terminated':
                $value = 'Error';
                break;
            case 'aborted':
                $value = 'Cancelled';
                break;
            case 'active':
                $value = 'Active';
                break;
            case 'queued':
                $value = 'Queued';
                break;
        }
        return $value;
    }

    // Sorts the Array by timestamp
    protected function sort(&$Array, $dir) {
        $direction = ($dir == "asc") ? SORT_ASC : SORT_DESC;
        $sortKey = array();
        foreach ($Array as $row) {
            $sortKey[] = $row['sort_start_time'];
        }
        array_multisort($sortKey, $direction, $Array);
    }

} //end JobsHistory class


