<?php
/**
 * Created by PhpStorm.
 * User: Sonja12
 * Date: 1/16/2015
 * Time: 6:47 PM
 */

class JobsFunctions {
    private $BP;

    public function __construct($BP = NULL){
        $this->BP = $BP;

        $rootdir = $_SERVER['DOCUMENT_ROOT'] . "/api/";
        $include_dir = $rootdir . "includes/";

        require_once( $include_dir . 'function.lib.php');
        $this->functions = new Functions($this->BP);
    }



    public function buildCSVExport($allJobs)
    {
        if ($allJobs != null) {
            require_once('./includes/csv.php');
            $CSV = new CSV();
            $allJobs = $allJobs['data'];
            $header = $allJobs[0];
            $csv = $CSV->toCSV($header, $allJobs);
            $allJobs['data'] = $csv;
            $allJobs['csv'] = true;
            return $allJobs;
        }
    }

    // convert detailed type name to "backup", "restore", "archive".
    public function getJobType($type)
    {
        $userType = "backup";
        if (strstr($type, 'restore') != false) {
            $userType = "restore";
        }
        if (strstr($type, 'archive') != false) {
            $userType = "archive";
        }
        if (strstr($type, 'replication') != false) {
            $userType = "replication";
        }


        return $userType;
    }

    public function getStartDateFromBackup($backups, $originalDate){
        $startDate = $originalDate;
        foreach($backups as $backup){
            if (isset($backup['start_time'])) {
                $startDate = $backup['start_time'];
                break;
            }
        }
        return $startDate;
    }

    public function getClientNameFromBackup($backups){
        $clientName = "";
        foreach($backups as $backup){
             if(isset($backup['primary_name'])){
                 $clientName = $backup['primary_name'];
                 break;
             }
        }
        return $clientName;
    }


    public function buildBackupsOutput($backups, $failed, $sid){
        $data = array();
        foreach ($backups as $runStart => $backupArray) {
            $formattedStart = strftime('%m/%d/%Y %r', $runStart);
            $backupsForSchedule = array();

            foreach ($backupArray as $backup) {

                if (isset($backup['instance_id'])) {
                    $iNames = $this->functions->getInstanceNames($backup['instance_id'], $sid);
                    if ($iNames !== false) {
                        $backupsForSchedule['asset_name'] = $iNames['asset_name'];
                        $backupsForSchedule['client_name'] = $iNames['client_name'];

                    }
                }

                $backupsForSchedule['instance_id'] = isset($backup['instance_id']) ? $backup['instance_id'] : "";
                $backupsForSchedule['message'] = isset($backup['description']) ? $backup['description'] : "";
                $backupsForSchedule['primary_name'] = isset($backup['primary_name']) ? $backup['primary_name'] : "";
                $backupsForSchedule['secondary_name'] = isset($backup['secondary_name']) ? $backup['secondary_name'] : "";
                $backupsForSchedule['start_time'] = $formattedStart;
                $backupsForSchedule['sort_start_time'] = $runStart;
                $backupsForSchedule['failed'] = isset($backup['failed']) ? $backup['failed'] : "";
                $backupsForSchedule['mode'] = isset($backup['type']) ? $this->functions->getBackupTypeDisplayName($backup['type']) : "";
                $backupsForSchedule['files'] = isset($backup['files']) ? $backup['files'] : "";
                $backupsForSchedule['size'] = isset($backup['size']) ? $backup['size'] : "";
                $backupsForSchedule['id'] = $backupsForSchedule['backup_id'] = isset($backup['backup_id']) ? $backup['backup_id'] : "N/A";
                $backupsForSchedule['status'] = $this->getBackupStatusName($backupsForSchedule['failed'], $backupsForSchedule['message']);

                if($failed !== null and $backupsForSchedule['failed'] !== true){
                    continue;
                }
                $data[] = $backupsForSchedule;
            }

        }
        return $data;
    }

    public function calcEndDate($endDateString){
        $endDate = $endDateString;
        $pieces = explode("/", $endDateString);
        if (count($pieces) > 3){
            //user specified h:m:s
            $endDate = strtotime($endDate);
        } else {
            $endDate = $this->functions->formatEndDate($_GET['end_date']);
        }
        return $endDate;
    }


    function getBackupStatusName($failed, $message) {

        $value = "Success";
        if ($failed != "") {
            if ($failed === true) {
                $value = "Error";
            }
        }
        if ($message != "") {
            $message = strtolower($message);
            if (strstr($message, "fail")) {
                $value = "Error";
            } else if (strstr($message, "warning")) {
                $value = "Warning";
            } else if (strstr($message, "active")) {
                $value = "Active";
            } else if (strstr($message, "queued")) {
                $value = "Queued";
            }
        }
        return $value;
    }
}