<?php

class Assets
{
    private $BP;

    public function __construct($BP)
    {
        $this->BP = $BP;

        require_once('function.lib.php');
        $this->functions = new Functions($this->BP);
        $this->localID = $this->BP->get_local_system_id();
        $this->grandClientView = false;
    }

    public function get($which)
    {
        $bValid = true;

        $sid = isset($_GET['sid']) ? (int)$_GET['sid'] : false;
        $grandClientView = isset($_GET['grandclient']) && $_GET['grandclient'] === "true";
        $this->grandClientView = $grandClientView;
        $includeVCenter = isset($_GET['include_vmware_client']) && $_GET['include_vmware_client'] === "true";
        $includeRetention = isset($_GET['include_retention']) && $_GET['include_retention'] === "true";

        // Only get non-managed systems if we are looking at copied assets.
        $systems = $grandClientView ? $this->functions->selectSystems($sid) : $this->functions->selectSystems($sid, false);

        $allClients = array();
        if ($sid !== false) {
            foreach ($systems as $id => $name) {
                if ($id === $sid) {
                    $system_id = $id;
                    $system_name = $name;
                    break;
                }
            }
            $allClients = $this->getClientsForSystem($which, $system_id, $system_name, $grandClientView, $includeVCenter, $includeRetention);
        } else {
            foreach ($systems as $id => $name) {
                $clients = $this->getClientsForSystem($which, $id, $name, $grandClientView, $includeVCenter, $includeRetention);
                $allClients = array_merge($allClients, $clients);
            }
        }
        $allClients = $this->sort($allClients);
        $allClients = array('data' => $allClients);
        return $allClients;
    }

    function getClientsForSystem($which, $system_id, $system_name, $grandClientView, $includeVCenter, $includeRetention)
    {

        global $Log;
        $allClients = array();
        $dpuID = $grandClientView ? $this->localID : $system_id;

        if($which !== -1 and $which == "archive") {
            $clients = $grandClientView ? $this->BP->get_archive_grandclients($system_id) : $this->BP->get_archive_client($system_id);
            if($clients !== false) {
                //convert the array to the same format as the get_client_list format to leverage code already written
                $tempClients = array();
                foreach($clients as $client) {
                    if($grandClientView) {
                        $tempClients[$client['client_id']] = $client['gcname'];
                    } else {
                        $tempClients[$client['id']] = $client['name'];
                    }
                }
                $clients = $tempClients;
            }
        } else {
            $clients = $grandClientView ? $this->BP->get_grandclient_list($system_id) : $this->BP->get_client_list($system_id);
        }
        if ($clients !== false) {
            foreach ($clients as $id => $name) {
                if ($which > 0 and $id != $which) {
                    // The user specified an ID, but not this one.
                    continue;
                }
                if ($system_name == $name) {
                    // Skip the "System Client".
                    continue;
                }
                if (!$includeVCenter && $name === VCENTER_RRC) {
                    // Skip the VCenter-RRC client.
                    if (!$grandClientView) {
                        continue;
                    }
                }
                $clientInfo = $this->BP->get_client_info($id, $dpuID);

                if ($clientInfo !== false) {
                    $hostInfo = $grandClientView ? array("ip" => "N/A") : $this->BP->get_host_info($clientInfo['name'], $system_id);
                    $clientName = $clientInfo['name'];
                    $associations = $this->getRetention(array('instance_id' => $clientInfo['file_level_instance_id']), $dpuID);
                    $this->getRetentionFromAssociations($associations, $clientInfo['file_level_instance_id'],
                                                        $clientInfo['retention'], $clientInfo['gfs_policy'], $clientInfo['min_max_policy']);;
                    $credentials = isset($clientInfo['credentials']) ? $clientInfo['credentials'] : null;
                    $clientInfo['credential_display'] = $this->getCredentialsDisplay($credentials, true);
                    if ($this->functions->isClientWindows($clientInfo['os_type_id'])) {
                        $clientInfo['supports_agent_push'] = true;
                    } else {
                        $clientInfo['supports_agent_push'] = false;
                    }
                    if($grandClientView && $clientInfo['name'] === Constants::CLIENT_NAME_VCENTER_RRC){
                        $clientInfo['asset_type'] = Constants::ASSET_TYPE_VMWARE_HOST;
                        $clientInfo['type'] = $clientInfo['os_family'];
                    }
                    else{
                        if($clientInfo['os_type'] === Constants::ADD_CLIENT_DISPLAY_NAME_XEN){
                            $clientInfo['asset_type'] = Constants::ADD_CLIENT_DISPLAY_NAME_XEN;
                            $clientInfo['type'] = $clientInfo['os_family'];
                        }
                        else{
                            $clientInfo['asset_type'] = Constants::ASSET_TYPE_PHYSICAL;
                            $clientInfo['type'] = $clientInfo['os_family'];
                        }
                    }
                    $clientInfo['system_id'] = $system_id;
                    $clientInfo['system_name'] = $system_name;
                    if(isset($clientInfo['remote_address']) && $clientInfo['remote_address'] !== null){
                        $clientInfo['ip'] = $clientInfo['remote_address'];
                    }
                    else{
                        $clientInfo['ip'] = $hostInfo['ip'];
                    }

                    $applications = $clientInfo['applications'];
                    if (count($applications) > 0) {
                        foreach ($applications as $appID => $appInfo) {
                            if ($appInfo['name'] == Constants::APPLICATION_NAME_FILE_LEVEL) {
                                // skip file level
                                continue;
                            }
                            if ($grandClientView && $appInfo['name'] == Constants::APPLICATION_NAME_VMWARE) {
                                if (isset($appInfo['servers'])) {
                                    $servers = $appInfo['servers'];
                                    $server = $clientInfo;
                                    $server['asset_type'] = Constants::ASSET_TYPE_VMWARE_HOST;
                                    if (isset($server['os_type'])) {
                                        unset($server['os_type']);
                                    }
                                    foreach ($servers as $host) {
                                        $results = $this->BP->get_grandclient_vm_info($host, $clientInfo['id']);
                                        if ($results !== false) {
                                            $vms = $this->gather($results, $includeRetention, $system_id, $system_name, $clientInfo['id'], $clientInfo['name'],
                                                Constants::APPLICATION_ID_VMWARE, Constants::ASSET_TYPE_VMWARE_VM);
                                            $server['children'] = $vms;
                                        }
                                        $server['type'] = Constants::APPLICATION_TYPE_NAME_VMWARE;
                                        $server['name'] = $host;
                                        $allClients[] = $server;
                                    }
                                }
                            } else {
                                $assets = $this->getAssets($id, $clientName, $appID, $appInfo, $grandClientView, $includeRetention, $system_id, $system_name);
                                $appInfo = array ('id' => $appID, 'name' => $appInfo['name'], 'description' => $appInfo['name'], 'type' => $appInfo['type'], 'system_id' => $system_id);
                                $appInfo['children'] = $assets;
                                $clientInfo['children'][] = $appInfo;
                            }
                        }
                    }

                    $isNAS = (($temp = strlen($clientInfo['name']) - strlen(Constants::NAS_POSTFIX)) >= 0 && strpos($clientInfo['name'], Constants::NAS_POSTFIX, $temp) !== FALSE);
                    if ($isNAS) {
                        if (!$grandClientView) {
                            $storageName = substr($clientInfo['name'], 0, $temp);
                            $storageID = $this->BP->get_storage_id($storageName, $system_id);
                            $storageInfo = $this->BP->rest_get_storage_info($storageID, $system_id);
                            if ($storageInfo === false) {
                                $bValid = false;
                                continue;
                            } else {
                                $clientInfo['ip'] = $storageInfo['properties']['hostname'];
                                $clientInfo['nas_properties'] = $storageInfo['properties'];
                                $clientInfo['nas_properties']['nas_id'] = $storageInfo['id'];
                            }
                        } else {
                            // This information is not on the target, so fill out a dummy template.
                            $clientInfo['ip'] = 'N/A';
                            $clientInfo['nas_properties'] = array('hostname' => 'N/A', 'nas_id' => -1, 'port' => 0,
                                                                  'protocol' => 'N/A', 'share_name' => "", 'username' => "");
                        }
                    }
                    if ($grandClientView && $appInfo['name'] == Constants::APPLICATION_NAME_VMWARE) {
                        ;
                    } else {
                        $allClients[] = $clientInfo;
                    }
                }
            }

            // add vCenters if present

            if (!$grandClientView) {
                $result = $this->BP->get_vcenter_list($system_id);
                if ($result !== false) {
                    foreach ($result as $vcenter_id => $name) {
                        if ($vcenter_id != $which and $which > 0) {
                            continue;
                        }
                        $allVMs = array();
                        $appInfo = $this->BP->get_vcenter_info($vcenter_id, $system_id);
                        $appInfo['asset_type'] = Constants::ASSET_TYPE_VMWARE_HOST;
                        $appInfo['system_id'] = $system_id;
                        $appInfo['system_name'] = $system_name;
                        $appInfo['credential_display'] = '(Unnamed)';
                        $appInfo['type'] = Constants::APPLICATION_TYPE_NAME_VMWARE;
                        $vmList = $this->BP->get_vm_info($vcenter_id, NULL, true, false, $system_id);
                        if ($vmList === false) {
                            global $Log;
                            $Log->writeVariable("error: " . $this->BP->getError(), true);
                            continue;
                        }
                        $vmList = $this->sort($vmList);
                        if (!empty($vmList)) {
                            $associations = $this->getRetention(array('uuid' => $vcenter_id), $system_id);
                            foreach ($vmList as $vm) {
                                $val = array('id' => $vm['instance_id'], 'name' => $vm['name'], 'is_encrypted' => $vm['is_encrypted']);
                                $this->getRetentionFromAssociations($associations, $val['id'], $val['retention'], $val['gfs_policy'], $val['min_max_policy']);;
                                $val['parent_id'] = $vcenter_id;
                                $val['parent_name'] = $name;
                                $val['asset_type'] = $val['description'] = Constants::ASSET_TYPE_VMWARE_VM;
                                $val['type'] = Constants::ASSET_TYPE_VMWARE_VM . ' Instance';
                                $val['system_id'] = $system_id;
                                $val['system_name'] = $system_name;
                                $val['version'] = "None";
                                $credentials = isset($vm['credentials']) ? $vm['credentials'] : null;
                                $val['credential_display'] = $this->getCredentialsDisplay($credentials, false);
                                $val['credentials'] = $credentials;
                                $val['app_aware'] = $vm['app_aware'];
                                $allVMs[] = $val;
                            }
                        }
                        $appInfo['children'] = $allVMs;
                        $allClients[] = $appInfo;
                    }
                } else {
                    $Log->writeError("Cannot get vcenter list", true);
                }
            }
        } else {
            $Log->writeError("Cannot get client list", true);
        }

        return $allClients;
    }

    function getAssets($clientID, $clientName, $appID, $appInfo, $grandClientView, $includeRetention, $system_id, $system_name)
    {
        $assets = array();
        global $Log;
        switch ($appID) {
            case Constants::APPLICATION_ID_FILE_LEVEL:
                break;

            case Constants::APPLICATION_ID_EXCHANGE_2003:
            case Constants::APPLICATION_ID_EXCHANGE_2007:
            case Constants::APPLICATION_ID_EXCHANGE_2010:
            case Constants::APPLICATION_ID_EXCHANGE_2013:
                $results = !$grandClientView ? $this->BP->get_exchange_info($clientID, true, $system_id) : $this->BP->get_grandclient_exchange_info($clientID);
                if ($results !== false) {
                    $assets = $this->gather($results, $includeRetention, $system_id, $system_name, $clientID, $clientName, $appID, Constants::APPLICATION_TYPE_NAME_EXCHANGE);
                } else {
                    $Log->writeError("Cannot load exchange info", true);
                }
                break;

            case Constants::APPLICATION_ID_SQL_SERVER_2005:
            case Constants::APPLICATION_ID_SQL_SERVER_2008:
            case Constants::APPLICATION_ID_SQL_SERVER_2008_R2:
            case Constants::APPLICATION_ID_SQL_SERVER_2012:
            case Constants::APPLICATION_ID_SQL_SERVER_2014:
                $results = !$grandClientView ? $this->BP->get_sql_info($clientID, $appID, true, $system_id) : $this->BP->get_grandclient_sql_info($clientID, $appID);
                if ($results !== false) {
                    $assets = $this->gather($results, $includeRetention, $system_id, $system_name, $clientID, $clientName, $appID, Constants::APPLICATION_TYPE_NAME_SQL_SERVER);
                } else {
                    $Log->writeError("Cannot load hyperv info", true);
                }
                break;

            case Constants::APPLICATION_ID_VMWARE:
                if ($grandClientView) {
                    if (isset($appInfo['servers'])) {
                        $servers = $appInfo['servers'];
                        foreach ($servers as $host) {
                            $results = $this->BP->get_grandclient_vm_info($host, $clientID);
                            if ($results !== false) {
                                $assets = array_merge($assets,
                                    $this->gather($results, $includeRetention, $system_id, $system_name, $clientID, $clientName, $appID, Constants::ASSET_TYPE_VMWARE_VM));
                            } else {
                                $Log->writeError("Cannot load vm info", true);
                            }
                        }
                    }
                }
                break;

            case Constants::APPLICATION_ID_HYPER_V_2008_R2:
            case Constants::APPLICATION_ID_HYPER_V_2012:
                $results = !$grandClientView ? $this->BP->get_hyperv_info($clientID, true, $system_id) : $this->BP->get_grandclient_hyperv_info($clientID);
                if ($results !== false) {
                    $assets = $this->gather($results, $includeRetention, $system_id, $system_name, $clientID, $clientName, $appID, Constants::ASSET_TYPE_HYPER_V_VM);
                } else {
                    $Log->writeError("Cannot load hyperv info", true);
                }
                break;

            case Constants::APPLICATION_ID_ORACLE_10:
            case Constants::APPLICATION_ID_ORACLE_11:
            case Constants::APPLICATION_ID_ORACLE_12:
                $results = !$grandClientView ? $this->BP->get_oracle_info($clientID, $appID, true, $system_id) : $this->BP->get_grandclient_oracle_info($clientID, $appID);
                if ($results !== false) {
                    $assets = $this->gather($results, $includeRetention, $system_id, $system_name, $clientID, $clientName, $appID, Constants::APPLICATION_TYPE_NAME_ORACLE);
                } else {
                    $Log->writeError("Cannot load oracle info", true);
                }
                break;

            case Constants::APPLICATION_ID_SHAREPOINT_2007:
            case Constants::APPLICATION_ID_SHAREPOINT_2010:
            case Constants::APPLICATION_ID_SHAREPOINT_2013:
                $results = !$grandClientView ? $this->BP->get_sharepoint_info($clientID, $appID, true, $system_id) : $this->BP->get_grandclient_sharepoint_info($clientID, $appID);
                if ($results !== false) {
                    $assets = $this->gather($results, $includeRetention, $system_id, $system_name, $clientID, $clientName, $appID, Constants::APPLICATION_TYPE_NAME_SHAREPOINT);
                } else {
                    $Log->writeError("Cannot load sharepoint info", true);
                }
                break;

            case Constants::APPLICATION_ID_UCS_SERVICE_PROFILE:
                $results = !$grandClientView ? $this->BP->get_ucssp_info($clientID, $appID, true, $system_id) : false;
                if ($results !== false) {
                    $assets = $this->gather($results, $includeRetention, $system_id, $system_name, $clientID, $clientName, $appID, Constants::APPLICATION_TYPE_NAME_UCS_SERVICE_PROFILE);
                } else {
                    $Log->writeError("Cannot load ucs info", true);
                }
                break;

            case Constants::APPLICATION_ID_VOLUME:
                $results = !$grandClientView ? $this->BP->get_ndmpvolume_info($clientID, true, $system_id) : false;
                if ($results !== false) {
                    $assets = $this->gather($results, $includeRetention, $system_id, $system_name, $clientID, $clientName, $appID, Constants::APPLICATION_TYPE_NAME_VOLUME);
                } else {
                    $Log->writeError("Cannot load ucs info", true);
                }
                break;

            case Constants::APPLICATION_ID_XEN:
                $results = !$grandClientView ? $this->BP->get_xen_vm_info($clientID, true, true, $system_id) : $this->BP->get_grandclient_xen_vm_info($clientID);
                if ($results !== false) {
                    $assets = $this->gather($results, $includeRetention, $system_id, $system_name, $clientID, $clientName, $appID, Constants::ASSET_TYPE_XEN_VM);
                } else {
                    $Log->writeError("Cannot load xen info", true);
                }
                break;
        }

        return $assets;
    }

    function gather($results, $includeRetention, $system_id, $system_name, $client_id, $clientName, $app_id, $asset_type)
    {
        $dpuID = $this->grandClientView ? $this->localID : $system_id;
        $assets = array();
        $associations = $this->getRetention(array('client_id' => $client_id, 'app_id' => $app_id), $dpuID);
        $results = $this->sort($results);
        foreach ($results as $result) {
            $name = isset($result['name']) ? $result['name'] :
            		(isset($result['instance']) && isset($result['database']) ? $result['instance'] . '\\' . $result['database'] : 'unknown');
            $val = array('id' => $result['instance_id'], 'name' => $name, 'is_encrypted' => $result['is_encrypted']);
            $this->getRetentionFromAssociations($associations, $val['id'], $val['retention'], $val['gfs_policy'], $val['min_max_policy']);

            $val['parent_id'] = $client_id;
            $val['parent_name'] = $clientName;
            $val['asset_type'] = $val['description'] = $asset_type;
            $val['type'] = $asset_type . ' Instance';
            $val['system_id'] = $system_id;
            $val['system_name'] = $system_name;
            $val['version'] = "None";
            $credentials = isset($result['credentials']) ? $result['credentials'] : null;
            $val['credential_display'] = $this->getCredentialsDisplay($credentials, false);
            $val['credentials'] = $credentials;
            $assets[] = $val;
        }
        return $assets;
    }

    function getRetention($filter, $system_id)
    {
        global $Log;
        $allAssociations = array();

        $associations = $this->BP->get_gfs_retention($filter, $system_id);
        if ($associations === false) {
            $Log->writeError("cannot get GFS retention settings: " . $this->BP->getError(), true);
            $associations = array();
        }
        $allAssociations['gfs'] = $associations;

        $retention = $this->BP->get_retention_settings($filter, $system_id);
        if ($retention === false) {
            $Log->writeError("cannot get min-max retention settings: " . $this->BP->getError(), true);
            $retention = array();
        }

        $allAssociations['min_max'] = $retention;

        return $allAssociations;
    }

    function getRetentionFromAssociations($associations, $instance_id, &$retention_name, &$gfs_policy, &$min_max_policy)
    {
        $retention_name = 'None';
        $gfs_policy = array();
        if ($associations !== false) {
            $found = false;
            foreach ($associations['min_max'] as $association) {
                if ($association['instance_id'] == $instance_id) {
                    $retention_name = $this->buildMinMaxPolicyName($association);
                    $min_max_policy = $association;
                    $found = true;
                    break;
                }
            }
            if (!$found) {
                foreach ($associations['gfs'] as $association) {
                    if ($association['instance_id'] == $instance_id) {
                        $retention_name = $association['policy_name'];
                        $gfs_policy = $association;
                        break;
                    }
                }
            }
        }
    }

    private function buildMinMaxPolicyName($setting) {
        $name = 'None';
        $a = array();
        if ($setting['retention_min'] > 0) {
            $a[] = 'Min ' . $setting['retention_min'];
        }
        if ($setting['retention_max'] > 0) {
            $a[] .= 'Max ' . $setting['retention_max'];
        }
        if ($setting['legal_hold'] > 0) {
            $a[] .= 'Hold ' . $setting['legal_hold'];
        } else if ($setting['legal_hold'] === -1) {
            $a[] .= 'Hold: Infinite';
        }
        if (count($a) > 0) {
            $name = implode(',', $a);
            $name .= ' Days';
        }
        return $name;
    }

    function getCredentialsDisplay($credentials, $applicableIfNotSet)
    {
        $displayName = "None";

        if ($credentials !== null) {
            if (!empty($credentials)) {
                if (isset($credentials['display_name'])) {
                    $displayName = $credentials['display_name'];
                } else {
                    $displayName = "(Unnamed)";
                }
            }
        } else if (!$applicableIfNotSet) {
            $displayName = "N/A";
        }

        return $displayName;
    }

    // Sorts the Array alphabetically
    private function sort($Array) {
        $Info = $Array;
        if (count($Array) > 0) {
            $orderByName = array();
            foreach ($Info as $key => $row) {
                $orderByName[$key] = isset($row['name']) ? strtolower($row['name']) : "";
            }
            array_multisort($orderByName, SORT_STRING, $Info);
        }
        return $Info;
    }
}
?>
