<?php

class Restores
{
    private $BP;

    public function __construct($BP)
    {
        $this->BP = $BP;

        require_once('function.lib.php');
        $this->functions = new Functions($this->BP);
    }

    public function get($which, $data, $sid, $systems)
    {
        if (is_string($which[0]))
        {
            switch ($which[0])
            {
                case 'targets':
                    return $this->get_restore_targets($data, $sid);
                    break;
            }
        }
    }

    public function get_restore_targets($data, $sid)
    {
        $targets = array();
        if ( $sid === false )
        {
            $sid = $this->BP->get_local_system_id();
        }

        $application_type = false;
        $instance_id = false;
        $return_all_servers = false;
        $backup_type = '';

        $replicated = (isset($data['replicated']) && $data['replicated'] == 1) ? true : false;

        // If both an instance_id and backup_id are passed in, ignore the backup_id
        if ( array_key_exists('instance_id', $data) )
        {
            $instance_id = (int)$data['instance_id'];
            if ( $instance_id === 0 )
            {
                $return_all_servers = true;
            }
            else
            {
                $appinst_info_array = $this->BP->get_appinst_info( $instance_id, $sid );
                if ( $appinst_info_array !== false and count($appinst_info_array) > 0 )
                {
                    foreach ( $appinst_info_array as $appinst_info )
                    {
                        $application_type = $appinst_info['app_type'];

                        //  There should only be one instance id, so no reason to go through a loop.  Throw out additional results from the array.
                        break;
                    }
                }
            }
        }
        else if ( array_key_exists('backup_id', $data) )
        {
            $backup_types = $this->getBackupType( array( (int)$data['backup_id'] ), $sid );
            if ( $backup_types !== false and count($backup_types) > 0 )
            {
                foreach ( $backup_types as $backup_type_array )
                {
                    $application_type = $this->functions->getApplicationTypeFromApplictionID( $backup_type_array['app_id'] );
                    $instance_id = $backup_type_array['instance_id'];
                    $backup_type = $backup_type_array['type'];
                    $instance_name = $backup_type_array['instance_name'] !== false ? $backup_type_array['instance_name'] : "";

                    //  There should only be one backup id, so no reason to go through a loop.  Throw out additional results from the array.
                    break;

                }
            }
        }
        else
        {
            $return_all_servers = true;
        }

        if ( $return_all_servers === true )
        {
            $hyper_v_recovery_clients = array();

            // For Hyper-V, get all of the servers using bp_get_hyperv_servers_for_wir, then pass in those results to get_Hyper_V_restore_targets_array
            // Using DOS as a fill in
            $hyper_v_servers = $this->BP->get_hyperv_servers_for_wir( Constants::OS_DOS, $sid );

            if ( $hyper_v_servers !== false and count($hyper_v_servers) > 0 )
            {
                foreach ( $hyper_v_servers as $hyper_v_server )
                {
                    $hyper_v_recovery_clients[$hyper_v_server['client_id']] = $hyper_v_server['name'];
                }
            }

            //Get the Hyper-V targets
            $targets = $this->get_Hyper_V_restore_targets_array( $sid, false, $hyper_v_recovery_clients, $hyper_v_servers );

            // Now merge in the VMware results
            $targets = array_merge( $targets, $this->get_VM_restore_targets_array( 0, $sid ) );
        }
        elseif ( $application_type !== false and $instance_id !== false )
        {
            switch ( $application_type )
            {
                case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                    $targets = $this->get_Hyper_V_restore_targets_array( $sid, $instance_id, false, false, $replicated, $data );
                    break;
                case Constants::APPLICATION_TYPE_NAME_VMWARE:
                    $targets = $this->get_VM_restore_targets_array( $instance_id, $sid );
                    break;
                case Constants::APPLICATION_TYPE_NAME_EXCHANGE:
                    if(isset($data['client_id'])) {
                       $clientID = (int)$data['client_id'];
                    } else {
                        $clientID = 0;
                        $clients = $this->getClientIDFromBackup(array((int)$data['backup_id']), $sid);
                        if ($clients !== false) {
                            foreach ($clients as $client) {
                                $clientID = $client;
                                break;
                            }
                        }
                    }

                    $db = $this->getExchangeDBFromBackup(array((int)$data['backup_id']), $sid);

                    $targets = $this->get_exchange_restore_targets_array( $clientID, $db, 1, $sid );
                    break;
                case Constants::APPLICATION_TYPE_NAME_ORACLE:
                    if ($replicated === true) {
                        $targets = array();
                    } else {
                        // restore to original server
                        $clientID = 0;
                        $clients = $this->getClientIDFromBackup(array((int)$data['backup_id']), $sid);
                        if ($clients !== false) {
                            foreach ($clients as $client) {
                                $clientID = $client;
                                break;
                            }
                        }
                        $targets = $this->get_oracle_restore_targets((int)$instance_id, $clientID, $sid);
                    }
                    break;
                case Constants::APPLICATION_TYPE_NAME_SHAREPOINT:
                    if ($replicated === true) {
                        $targets = array();
                    } else {
                        // restore to original server
                        $targets = $this->get_sharepoint_restore_targets((int)$instance_id, $sid);
                    }
                    break;
                case Constants::APPLICATION_TYPE_NAME_SQL_SERVER:
                    $targets = $this->get_sql_restore_targets((int)$data['backup_id'], (int)$instance_id, $instance_name, $replicated, $sid);
                    break;
                case Constants::APPLICATION_TYPE_NAME_XEN:
                    $targets = $this->get_xen_restore_targets($sid);
                    break;

            }
        }
        elseif ($backup_type !== '' && $this->functions->isBackupFileLevel($backup_type))
        {
            $systemName = $this->functions->getSystemNameFromID($sid);
            $targets = $this->get_file_level_restore_targets($sid, $systemName);
        }
        else
        {
            $targets = false;
        }


        return ( $targets !== false ? array( 'targets' => $targets) : false );
    }

    private function get_Hyper_V_restore_targets_array( $sid, $instance_id = false, $hyper_v_recovery_clients = false, $hyper_v_servers = false, $replicated, $data = null )
    {
        $hyper_v_targets_array = array();

        // If $instance_id is false, then $hyper_v_recovery_clients must not be equal to false
        if ( $instance_id !== false )
        {
            // this returns the original client on which we are trying to restore
            $hyper_v_application_clients = array();
            $temp_application_clients = $this->BP->get_application_clients( $instance_id, $sid );
            if ($temp_application_clients !== false) {
                if (array_key_exists('clients', $temp_application_clients)) {
                    $temp_clients = $temp_application_clients['clients'];
                    foreach ($temp_clients as $temp_client_id => $temp_client_name) {
                        $hyper_v_application_clients[$temp_client_id] = $temp_client_name;
                    }
                }
            }
            $recovery_clients = $this->BP->get_hyperv_recovery_clients( $instance_id, $sid ) ;
            $hyper_v_recovery_clients =  $hyper_v_application_clients + $recovery_clients;
            if (array_key_exists('backup_id', $data) && array_key_exists('target', $data)) {
                if ($data['target'] === 'original') {
                    $hyper_v_recovery_clients = array();
                    $clientID = 0;
                    $clients = $this->getClientIDFromBackup(array((int)$data['backup_id']), $sid);
                    if ($clients !== false) {
                        foreach ($clients as $client) {
                            $clientID = $client;
                            break;
                        }
                    }
                    foreach ($hyper_v_application_clients as $client_id => $client_name) {
                        if ($client_id == $clientID) {
                            $hyper_v_recovery_clients[$client_id] = $client_name;
                        }
                    }
                }
            } elseif (array_key_exists('type', $data)) {
                // Return only the servers that are eligible for Instant Recovery
                if ($data['type'] === 'ir') {
                    $hyper_v_recovery_clients = array();
                    $hyper_v_ir_clients = $this->BP->get_hyperv_servers_for_ir($instance_id, $sid);
                    if ($hyper_v_ir_clients !== false) {
                        foreach ($hyper_v_ir_clients as $hyper_v_ir_client) {
                            $hyper_v_recovery_clients[$hyper_v_ir_client['client_id']] = $hyper_v_ir_client['name'];
                        }
                    }
                } elseif ($data['type'] === 'restore') {
                    $hyper_v_recovery_clients = array();
                    $hyper_v_original_client = array();
                    $clientID = 0;
                    $clients = $this->getClientIDFromBackup(array((int)$data['backup_id']), $sid);
                    if ($clients !== false) {
                        foreach ($clients as $client) {
                            $clientID = $client;
                            break;
                        }
                    }
                    foreach ($hyper_v_application_clients as $client_id => $client_name) {
                        if ($client_id == $clientID) {
                            $hyper_v_original_client[$client_id] = $client_name;
                        }
                    }
                    // for replicated backups, restore to original server isn't supported
                    if ($replicated === true) {
                        $hyper_v_recovery_clients = $recovery_clients;
                    } else {
                        $hyper_v_recovery_clients = $hyper_v_original_client + $recovery_clients;
                    }
                }
            }
        }
        // This function call might have already been made, so no need to make it twice
        if ( $hyper_v_servers === false )
        {
            // Using DOS as a fill in
            $hyper_v_servers = $this->BP->get_hyperv_servers_for_wir( Constants::OS_DOS, $sid );
        }

        if ( $hyper_v_recovery_clients !== false and count($hyper_v_recovery_clients) > 0 )
        {
            $hyper_v_servers_info_array = array();

            if ( $hyper_v_servers !== false and count($hyper_v_servers) > 0 )
            {
                foreach ( $hyper_v_servers as $hyper_v_server )
                {
                    $hyper_v_servers_info_array[$hyper_v_server['client_id']] = $hyper_v_server['capabilities'];
                }
            }

            foreach ( $hyper_v_recovery_clients as $hyper_v_client_id => $hyper_v_client_name )
            {
                $temp_restore_target = array();
                $temp_restore_target['id'] = $hyper_v_client_id;
                $temp_restore_target['name'] = $hyper_v_client_name;

                $hyper_v_storage = $this->BP->get_hyperv_storage( $hyper_v_client_id, $sid );
                if ( $hyper_v_storage !== false and count($hyper_v_storage) > 0 )
                {
                    foreach ( $hyper_v_storage as $hyperv_storage )
                    {
                        $temp_storage = array();
                        $temp_storage['name'] = $hyperv_storage['name'];
                        // converting sizes in MB to GB
                        $temp_storage['total_size_gb'] = $this->functions->formatNumber($hyperv_storage['mb_size'], 1024, 1);
                        $temp_storage['free_size_gb'] = $this->functions->formatNumber($hyperv_storage['mb_free'], 1024, 1);

                        $temp_restore_target['datastores'][] = $temp_storage;
                    }
                }

                if ( array_key_exists( $hyper_v_client_id, $hyper_v_servers_info_array ) )
                {
                    $temp_restore_target['capabilities'] = $hyper_v_servers_info_array[$hyper_v_client_id];
                }

                $hyper_v_targets_array[] = $temp_restore_target;
            }
        }
        return $hyper_v_targets_array;
    }

    private function get_VM_restore_targets_array( $instance_id, $sid )
    {
        $vm_targets_array = array();
        $vm_restore_targets = $this->BP->get_vm_restore_targets( $instance_id, $sid );
        if ( $vm_restore_targets !== false and count($vm_restore_targets) > 0 )
        {
            $vm_servers = $vm_restore_targets['servers'];
            foreach ( $vm_servers as $vm_restore_target )
            {
                $temp_restore_target = array();
                $temp_restore_target['id'] = $vm_restore_target['uuid'];
                $temp_restore_target['name'] = $vm_restore_target['name'];

                if ( array_key_exists('datastores', $vm_restore_target) and count($vm_restore_target['datastores']) > 0 )
                {
                    foreach ( $vm_restore_target['datastores'] as $vm_data_store )
                    {
                        $temp_storage = array();
                        $temp_storage['name'] = $vm_data_store['name'];
                        // converting sizes in MB to GB
                        $temp_storage['total_size_gb'] = $this->functions->formatNumber($vm_data_store['mb_size'], 1024, 1);
                        $temp_storage['free_size_gb'] = $this->functions->formatNumber($vm_data_store['mb_free'], 1024, 1);

                        $temp_restore_target['datastores'][] = $temp_storage;
                    }
                }
                $temp_restore_target['groups'] = $this->get_vm_groups( $vm_restore_target['uuid'], $vm_restore_target['name'], $sid);
                $temp_restore_target['capabilities'] = $vm_restore_target['capabilities'];
                $vm_targets_array[] = $temp_restore_target;
            }
        }
        return $vm_targets_array;
    }

    private function get_exchange_restore_targets_array( $node_id, $dbname, $ball, $sid )
    {
        $vm_targets_array = array();

        $vm_restore_targets = $this->BP->get_exchange_restore_targets( $node_id, $dbname, $ball, $sid );
        if ( $vm_restore_targets !== false and count($vm_restore_targets) > 0 )
        {
            foreach ( $vm_restore_targets as $vm )
            {
                $temp_restore_target = array();
                if ($vm['name'] === $dbname) {
                   if (!$vm['mounted'] && $vm['writable']) {
                       $temp_restore_target['mounted'] = $vm['mounted'];
                       $temp_restore_target['name'] = $vm['name'];
                       $temp_restore_target['writable'] = $vm['writable'];
                       $temp_restore_target['recovery'] = $vm['recovery'];
                       $vm_targets_array[] = $temp_restore_target;
                   }
                } else {
                    if ($vm['recovery'] && !$vm['mounted'] && $vm['writable']) {
                        $temp_restore_target['mounted'] = $vm['mounted'];
                        $temp_restore_target['name'] = $vm['name'];
                        $temp_restore_target['writable'] = $vm['writable'];
                        $temp_restore_target['recovery'] = $vm['recovery'];
                        $vm_targets_array[] = $temp_restore_target;
                    }
                }
            }
        }
        return $vm_targets_array;
    }

    private function get_vm_groups( $uuid, $name, $sid )
    {
        $groups_info = array();
        $resourcePools = $this->BP->get_resource_pool_info( $uuid, $sid );
        $vapps         = $this->BP->get_vApp_info( $uuid, $sid );

        $include_vGroups = ( $resourcePools!== false || $vapps!== false  );

        if ( $include_vGroups ) {
            $groups_info['Name'] = $name;
        }

        if ( $resourcePools!== false && $resourcePools != NULL ) {
            foreach( $resourcePools as $rp ){
                $rpKey = $rp['key'];
                $rpName = $rp['name'];
                $rpParentType = $rp['parentType'];
                $rpParentKey = $rp['parentKey'];
                $temp_groups_info = array( 'Key' => $rpKey,
                    'Name' => $rpName,
                    'ParentType' => $rpParentType,
                    'ParentKey' => $rpParentKey,
                    'Type'=>"0" );
                $groups_info['VG'][] = $temp_groups_info;
            }
        }

        if ( $vapps!== false && $vapps != NULL ) {
            foreach( $vapps as $va ){
                $vaKey = $va['key'];
                $vaName = $va['name'];
                $vaParentType = $va['parentType'];
                $vaParentKey = $va['parentKey'];
                $temp_groups_info = array( 'Key' => $vaKey,
                    'Name' => $vaName,
                    'ParentType' => $vaParentType,
                    'ParentKey' => $vaParentKey,
                    'Type'=> "1" );
                $groups_info['VG'][] = $temp_groups_info;
            }
        }

        return $groups_info;
    }

    private function get_oracle_restore_targets($instance_id, $clientID, $sid)
    {
        $oracle_targets = array();
        $oracle_client_name = $this->getClientNameFromID($clientID, $sid);
        if ($oracle_client_name !== false) {
            $temp_oracle_targets['instance_id'] = $instance_id;
            $temp_oracle_targets['client_id'] = $clientID;
            $temp_oracle_targets['client_name'] = $oracle_client_name;

            $oracle_targets[] = $temp_oracle_targets;
        }
        return $oracle_targets;
    }

    private function get_sharepoint_restore_targets($instance_id, $sid)
    {
        $sharepoint_targets = array();
        $app_info = array();
        $sharepoint_app_info = $this->BP->get_appinst_info($instance_id, $sid);
        if ($sharepoint_app_info !== false) {
            foreach ($sharepoint_app_info as $inst_id => $inst_info) {
                $app_info['instance_id'] = $inst_id;
                $app_info['client_id'] = $inst_info['client_id'];
                $app_info['client_name'] = $inst_info['client_name'];
                $app_info['app_id'] = $inst_info['app_id'];
                $app_info['app_name'] = $inst_info['app_name'];

                break;
            }
        }
        $numOfServers = -1;
        $sharepoint_info = $this->BP->get_sharepoint_info($app_info['client_id'], $app_info['app_id'], true, $sid);
        if ($sharepoint_info !== false) {
            foreach($sharepoint_info as $sharepoint) {
                $numOfServers = $sharepoint['num_app_servers'];
            }
        }
        $app_info['num_servers'] = $numOfServers;
        $sharepoint_targets[] = $app_info;
        return $sharepoint_targets;
    }

    private function get_sql_restore_targets($backup_id, $instance_id, $instance_name, $replicated, $sid)
    {
        $sql_targets = array();
        $backup_type ='';
        $database = '';
        $backup_type_array = $this->getBackupType( array($backup_id), $sid );
        if ( $backup_type_array !== false and count($backup_type_array) > 0 )
        {
            $backup_type = $backup_type_array[0]['type'];
            $database = $backup_type_array[0]['database'];
            // there should just be one backup ID
        }
        $appinst_info = $this->BP->get_appinst_info($instance_id, $sid);
        foreach ($appinst_info as $inst_id => $inst_info) {
            $client_id = $inst_info['client_id'];
            $client_name = $inst_info['client_name'];

            break;
        }
        switch ($backup_type) {
            case Constants::BACKUP_TYPE_MSSQL_FULL:
            case Constants::BACKUP_TYPE_MSSQL_FULL_ALT:
                $isSystemDB = $this->isSystemDB($instance_id, $sid);
                if ($isSystemDB !== false && $replicated === false) {
                    // if system database, restore to original server
                    $temp_sql_targets['client_id'] = $client_id;
                    $temp_sql_targets['client_name'] = $client_name;
                    $temp_sql_targets['database'] = $database;
                    $temp_sql_targets['system_db'] = true;
                    // list of applicable instances on the server
                    $temp_sql_targets['instances'] = $this->get_sql_target_instances($client_id, $instance_id, $sid);
                } else {
                    // get applicable servers and instances
                    $temp_sql_targets = $this->get_sql_targets($instance_id, $database, $sid, $isSystemDB);
                }
                $sql_targets = $temp_sql_targets;
                break;
            case Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL:
            case Constants::BACKUP_TYPE_MSSQL_DIFFERENTIAL_ALT:
            case Constants::BACKUP_TYPE_MSSQL_TRANSACTION:
            case Constants::BACKUP_TYPE_MSSQL_TRANSACTION_ALT:
                // for replicated backups, cannot set target path
                if ($replicated === true) {
                    $temp_sql_targets = $this->get_sql_targets($instance_id, $database, $sid);
                    foreach ($temp_sql_targets as $temp_sql_target) {
                        $temp_sql_target['target_path'] = false;
                        $instance_array = $temp_sql_target['instances'];
                        foreach ($instance_array as $instance) {
                            if ($instance['instance_name'] === $instance_name) {
                                $sql_targets[] = $temp_sql_target;
                            }
                        }
                    }
                } else {
                    // differential and transaction backups allow restore to original server
                    $sql_targets['client_id'] = $client_id;
                    $sql_targets['client_name'] = $client_name;
                    $sql_targets['database'] = $database;
                    $sql_targets['system_db'] = $this->isSystemDB($instance_id, $sid);
                    // list of applicable instances on the server
                    $sql_targets['instances'] = $this->get_sql_target_instances($client_id, $instance_id, $sid);

                    // handling for PIT
                    if ($backup_type == Constants::BACKUP_TYPE_MSSQL_TRANSACTION){
                        $group = $this->BP->get_restore_group($backup_id, $sid);
                        if($group !== false and count($group) > 1){
                            $earliestBackup = $group[count($group) - 2]['start_time'];
                            $latestBackup = $group[count($group) - 1]['start_time'];
                            $sql_targets['pit_start_time'] = $earliestBackup;
                            $sql_targets['pit_end_time'] = $latestBackup;
                        }
                    }
                }
                break;
        }

        return $sql_targets;
    }

    private function isSystemDB($instance_id, $sid)
    {
        $isSysDB = $this->BP->is_sql_system_db($instance_id, $sid);
        if ($isSysDB == -1) {
            $isSysDB = true;
        }
        return $isSysDB;
    }

    private function get_sql_target_instances($client_id, $instance_id, $sid)
    {
        $target_instances = array();
        $recovery_targets = $this->BP->get_sql_server_recovery_targets($client_id, $instance_id, $sid);
        if ($recovery_targets !== false && count($recovery_targets) > 0) {
            foreach ($recovery_targets as $recovery_target) {
                $temp_target_instances = array();
                $temp_target_instances['instance_name'] = $recovery_target['name'];
                $temp_target_instances['is_running'] = $recovery_target['is_running'];

                $target_instances[] = $temp_target_instances;
            }
        }
        return $target_instances;
    }

    private function get_sql_targets($instance_id, $database, $sid, $isSystemDB = false)
    {
        $target_clients = array();
        $recovery_targets = $this->BP->get_sql_server_recovery_clients($instance_id, $sid);
        if ($recovery_targets !== false) {
            $target_clients = array();
            if (isset($recovery_targets['clients']) && count($recovery_targets['clients']) > 0) {
                foreach($recovery_targets['clients'] as $client_id => $client_name) {
                    $temp_target_clients = array();
                    $temp_target_clients['client_id'] = $client_id;
                    $temp_target_clients['client_name'] = $client_name;
                    $temp_target_clients['database'] = $database;
                    $temp_target_clients['system_db'] = $isSystemDB;

                    $temp_target_clients['instances'] = $this->get_sql_target_instances($client_id, $instance_id, $sid);

                    $target_clients[] = $temp_target_clients;
                }
            }
        }
        return $target_clients;
    }

    private function get_xen_restore_targets($sid)
    {
        $target_clients = array();
        $restore_targets = $this->BP->get_xen_restore_targets($sid);
        if ($restore_targets !== false) {
            if (isset($restore_targets['xen_hosts']) && count($restore_targets['xen_hosts']) > 0) {
                foreach($restore_targets['xen_hosts'] as $xen_host) {
                    $temp_target_clients = array();
                    $temp_target_clients['id'] = $xen_host['uuid'];
                    $temp_target_clients['name'] = $xen_host['name'];

                    if (isset($xen_host['storage_repositories']) && count($xen_host['storage_repositories']) > 0) {
                        foreach($xen_host['storage_repositories'] as $xen_storage) {
                            $temp_storage = array();
                            $temp_storage['id'] = $xen_storage['uuid'];
                            $temp_storage['name'] = $xen_storage['name'];
                            $temp_storage['total_size_gb'] = $xen_storage['gb_total'];
                            $temp_storage['free_size_gb'] = $xen_storage['gb_free'];

                            $temp_target_clients['datastores'][] = $temp_storage;
                        }
                    }
                    $target_clients[] = $temp_target_clients;
                }
            }
        }
        return $target_clients;
    }

    private function get_file_level_restore_targets($sid, $systemName)
    {
        $target_clients = array();
        $clients = $this->BP->get_client_list($sid);
        if ($clients !== false && count($clients) > 0) {
            foreach ($clients as $clientID => $clientName) {
                $temp_targets = array();
                $temp_targets['client_id'] = $clientID;
                $temp_targets['client_name'] = $clientName;

                // filtering out system client and vCenter-RRC
                if ($clientName !== Constants::CLIENT_NAME_VCENTER_RRC && $clientName !== $systemName){
                    $client_info = $this->BP->get_client_info($clientID, $sid);
                    $osTypeID = $client_info['os_type_id'];
                    $temp_targets['client_os'] = $client_info['os_type'];
                    //filtering out NDMP and CiscoUCS clients
                    if ($osTypeID !== Constants::OS_GENERIC) {
                        $target_clients[] = $temp_targets;
                    }
                }

            }
        }
        return $target_clients;
    }

    public function post($which, $data, $sid)
    {
        $sid = $sid !== false ? $sid : $this->BP->get_local_system_id();
        $result = array();
        if (is_string($which[0])) {
            switch ($which[0]) {
                case 'instant':
                    $backups = $this->getRestoreGroup($data['backup_id'], $sid);
                    if (array_key_exists('error', $backups)) {
                        $result = $backups['error'];
                    } else {
                        $backupArrStr = explode(",", $backups);
                        $backupArray = array();
                        $i = 0;
                        foreach ($backupArrStr as $backup) {
                            $backupArray[$i++] = (int)$backup;
                        }
                        $backupType = $this->getBackupType($backupArray, $sid);
                        if (array_key_exists('target', $data)) {
                            $target = $data['target'];
                            $host = $target['host'];
                            if (array_key_exists('datastore', $target)) {
                                $dataStore = $target['datastore'];
                            }
                            if (array_key_exists('directory', $target)) {
                                $directory = $target['directory'];
                            }
                            $name = $target['name'];
                            if (array_key_exists('group', $target)) {
                                $group = $target['group'] == "" ? NULL : $target['group'];
                            }
                            if (array_key_exists('switch', $target)) {
                                $switch = $target['switch'] == "" ? NULL : $target['switch'];
                            }
                        }
                        $address = $data['address'];
                        $audit = (int)$data['audit'];
                        $poweron = isset($data['poweron']) ? $data['poweron'] : 1;
                        if ($this->functions->isBackupVMWare($backupType[0]['type'])) {
                            $result = $this->BP->vmware_ir_start($host, $name, $dataStore, $address, $audit,
                                $backups, $poweron, $group, $sid);
                        } elseif ($this->functions->isBackupHyperV($backupType[0]['type'])) {
                            $result = $this->BP->hyperv_ir_start($host, $name, $directory, $address, $audit,
                                $backups, $poweron, $switch, $sid);

                        } else {
                            $result['error'] = 500;
                            $result['mesage'] = "Backup id " .$data['backup_id']. " is not supported.";
                        }
                    }
                    break;

                case 'full':
                    $backups = $this->getRestoreGroup($data['backup_id'], $sid);
                    if (array_key_exists('error', $backups)) {
                        $result = $backups['error'];
                    } else {
                        $backupArrStr = explode(",", $backups);
                        $backupArray = array();
                        $i = 0;
                        foreach ($backupArrStr as $backup) {
                            $backupArray[$i++] = (int)$backup;
                        }
                        $backupType = $this->getBackupType($backupArray, $sid);
                        $replicated = $backupType[0]['replicated'];
                        //if this is a vmware replicated backup, set client_id to non-replicating client
                        if ($replicated && $this->functions->isBackupVMWare($backupType[0]['type'])) {
                            $client_id = $this->getVMwareClient($sid);
                        } elseif (array_key_exists('client_id', $data)) {
                            $client_id = (int)$data['client_id'];
                        }
                        $includes = $data['includes'];
                        $includesArrStr = explode(",", $includes);
                        $includesArray = array();
                        if ($includes !== "") {
                            $i = 0;
                            foreach ($includesArrStr as $include) {
                                if (substr($include, strlen($include) - 1) == ':') {
                                    $include = $include . '/';
                                }
                                $includesArray[$i++] = $include;
                            }
                        }
                        $excludes = $data['excludes'];
                        $excludesArrStr = explode(",", $excludes);
                        $excludesArray = array();
                        if ($excludes !== "") {
                            $j = 0;
                            foreach ($excludesArrStr as $exclude) {
                                if (strpos($exclude, ':') === false) {
                                    foreach ($this->getVolumeNames($includesArray) as $volume) {
                                        $exclude = $volume . $exclude;
                                    }
                                } elseif (substr($exclude, strlen($exclude) - 1) == ':') {
                                    $exclude = $exclude . '/';
                                }
                                $excludesArray[$j++] = $exclude;
                            }
                        }
                        if (array_key_exists('synthesis', $data)) {
                            $synthesis = $data['synthesis'];
                        }
                        $target = $data['target'];
                        $database = "";
                        $instance = "";
                        $host = "";
                        $datastore = "";
                        $name = "";
                        $group = "";
                        $metadata = "";
                        $template = "";
                        $storageRepository = "";

                        if (array_key_exists('flat', $target)) {
                            $flat = $target['flat'];
                        }
                        if (array_key_exists('non_destructive', $target)) {
                            $non_destructive = $target['non_destructive'];
                        }
                        if (array_key_exists('newer', $target)) {
                            $newer = $target['newer'];
                        }
                        if (array_key_exists('today', $target)) {
                            $today = $target['today'];
                        }
                        if (array_key_exists('unix', $target)) {
                            $unix = $target['unix'];
                        }
                        if (array_key_exists('database', $target)) {
                            $database = $target['database'];
                        }
                        if (array_key_exists('instance', $target)) {
                            $instance = $target['instance'];
                        }
                        if (array_key_exists('point_in_time', $target)) {
                            $point_in_time = $target['point_in_time'];
                        }
                        if (array_key_exists('host', $target)) {
                            $host = $target['host'];
                        }
                        if (array_key_exists('datastore', $target)) {
                            $datastore = $target['datastore'];
                        }
                        if (array_key_exists('name', $target)) {
                            $name = $target['name'];
                        }
                        if (array_key_exists('group', $target)) {
                            $group = $target['group'];
                        }
                        if (array_key_exists('metadata', $target)) {
                            $metadata = $target['metadata'] ? 1 : 0;
                        }
                        if (array_key_exists('template', $target)) {
                            $template = $target['template'] ? 1 : 0;
                        }
                        if (array_key_exists('directory', $target) && $target['directory'] !== "") {
                            $directory = $target['directory'];
                        }
                        if (array_key_exists('storage_repository', $target)) {
                            $storageRepository = $target['storage_repository'];
                        }

                        $bcmd = $data['before_cmd'];
                        $acmd = $data['after_cmd'];

                        if (isset($flat) && isset($non_destructive) && isset($newer) && isset($today) && isset($unix)) {
                            $backup = (int)$backups;
                            $options = array();
                            if (isset($client_id)) {
                                $options['client_id'] = $client_id;
                            }
                            if (isset($directory) && $directory !== "") {
                                $options['directory'] = $directory;
                            }
                            $options['flat'] = $flat;
                            if ($bcmd !== "") {
                                $options['before_command'] = $bcmd;
                            }
                            if ($acmd !== "") {
                                $options['after_command'] = $acmd;
                            }
                            $options['non_destructive'] = $non_destructive;
                            $options['newer'] = $newer;
                            $options['today'] = $today;
                            $options['unix'] = $unix;

                            if (isset($synthesis) && $synthesis === true) {
                                $groupID = $this->BP->get_synthesis_group($data['backup_id'], $sid);
                                if ($groupID !== false) {
                                    $restore = $this->BP->restore_synthesized_files($groupID, $data['backup_id'], $includesArray, $excludesArray, $options, $sid);
                                    if ($restore !== false) {
                                        $result['id'] = $restore;
                                    } else {
                                        $result['error'] = 500;
                                        $result['message'] = $this->BP->getError();
                                    }
                                } else {
                                    $result['error'] = 500;
                                    $result['message'] = $this->BP->getError();
                                }
                            } else {
                                $restore = $this->BP->restore_files($backup, $includesArray, $excludesArray, $options, $sid);
                                if ($restore !== false) {
                                    $result['id'] = $restore;
                                } else {
                                    $result['error'] = 500;
                                    $result['message'] = $this->BP->getError();
                                }
                            }
                        } else {
                            if ($this->functions->isBackupHyperV($backupType[0]['type']) || $this->functions->isBackupVMWare($backupType[0]['type']) ||
                                $this->functions->isBackupExchange($backupType[0]['type']) || $this->functions->isBackupSQL($backupType[0]['type']) ||
                                $this->functions->isBackupXen($backupType[0]['type'])) {
                                $destination = "";
                                if ($this->functions->isBackupExchange($backupType[0]['type'])) {
                                    $destination = $database;
                                } elseif ($this->functions->isBackupSQL($backupType[0]['type'])) {
                                    $destination = $instance . '|' . $database;
                                } elseif ($this->functions->isBackupVMWare($backupType[0]['type'])) {
                                    $destination = $host . '|' . $datastore . '|' . $name . '|' . $group . '|' . $metadata . '|' . $template;
                                } elseif ($this->functions->isBackupXen($backupType[0]['type'])) {
                                    $destination = $host . '|' . $storageRepository . '|' . $name . '|' . $metadata;
                                }
                                $options = array();
                                if (isset($client_id)) {
                                    $options['client_id'] = $client_id;
                                }
                                if (isset($target)) {
                                    $options['target'] = $destination;
                                }
                                if (isset($directory)) {
                                    $options['directory'] = $directory;
                                }
                                if (isset($point_in_time)) {
                                    $options['point_in_time'] = $point_in_time;
                                }
                                if ($bcmd !== "") {
                                    $options['before_command'] = $bcmd;
                                }
                                if ($acmd !== "") {
                                    $options['after_command'] = $acmd;
                                }
                                $resultArray = $this->BP->restore_application($backups, $options, $sid);
                                if ($resultArray !== false) {
                                    $tempRes = "";
                                    $delim = "";
                                    $msg = "";
                                    $msg_delim = "";
                                    foreach ($resultArray as $resArr) {
                                        $tempRes = $tempRes . $delim . $resArr['job_id'];
                                        $delim = ",";

                                        if (array_key_exists('msg', $resArr)) {
                                            $backupType =  isset($resArr['type']) ? $this->functions->getBackupTypeString($resArr['type']) : "";
                                            $msg = $msg . $msg_delim . $backupType . " restore failed. " . $resArr['msg'];
                                            $msg_delim = "\n";
                                        }
                                    }
                                    if ($msg !== "") {
                                        $result['error'] = 500;
                                        $result['message'] = $msg;
                                    } else {
                                        $result['id'] = $tempRes;
                                    }
                                } else {
                                    $result = $resultArray;
                                }
                            } elseif ($this->isBackupSharePoint($backupType[0]['type']) || $this->isBackupOracle($backupType[0]['type'])) {
                                $options = array();
                                if (isset($client_id)) {
                                    $options['client_id'] = $client_id;
                                }
                                $resultArray = $this->BP->rae_restore_application($backups, $options, $sid);
                                if ($resultArray !== false) {
                                    $tempRes = "";
                                    $delim = "";
                                    $msg = "";
                                    $msg_delim = "";
                                    foreach ($resultArray as $resArr) {
                                        $tempRes = $tempRes . $delim . $resArr['job_id'];
                                        $delim = ",";

                                        if (array_key_exists('msg', $resArr)) {
                                            $backupType =  isset($resArr['type']) ? $this->functions->getBackupTypeString($resArr['type']) : "";
                                            $msg = $msg . $msg_delim . $backupType . " restore failed. " . $resArr['msg'];
                                            $msg_delim = "\n";
                                        }
                                    }
                                    if ($msg !== "") {
                                        $result['error'] = 500;
                                        $result['message'] = $msg;
                                    } else {
                                        $result['id'] = $tempRes;
                                    }
                                } else {
                                    $result = $resultArray;
                                }
                            } else {
                                $result['error'] = 500;
                                $result['message'] = "Backup id " .$data['backup_id']. " is not supported.";
                            }
                        }
                    }
                    break;

                case 'files':
                    if (isset($data['related'])) {
                        $related = $data['related'];
                    } else {
                        $related = 1;
                    }
                    if ($related === 1) {
                        $backups = $this->getRestoreGroup($data['backup_id'], $sid);
                    } else {
                        $backups = $data['backup_id'];
                    }
                    if (array_key_exists('error', $backups)) {
                        $result = $backups['error'];
                    } else {
                        $backupArrStr = explode(",", $backups);
                        $backupArray = array();
                        $i = 0;
                        foreach ($backupArrStr as $backup) {
                            $backupArray[$i++] = (int)$backup;
                        }
                        $backupType = $this->getBackupType($backupArray, $sid);
                        $path = "";
                        if (array_key_exists('path', $data)) {
                            $path = $data['path'];
                        }
                        if ($this->functions->isBackupExchange($backupType[0]['type'])) {
                            $result = $this->BP->backup_mount($backups, $sid);
                        } elseif ($this->functions->isBackupVMWare($backupType[0]['type']) || $this->functions->isBackupHyperV($backupType[0]['type']) || $this->functions->isBackupXen($backupType[0]['type'])) {
                            $result = $this->BP->create_disk_image($backups, $sid);
                        } elseif ($this->isBackupOracle($backupType[0]['type']) || $this->isBackupSharePoint($backupType[0]['type'])) {
                            $result = $this->BP->create_application_share($backups, $path, $sid);
                        } else {
                            $result['error'] = 500;
                            $result['message'] = "Backup id " .$data['backup_id']. " is not supported.";
                        }
                    }
                    break;
            }
        }
        return $result;
    }

    /*
     * Passing a string of backupIDs will return an array of backup IDs and type of backups
     */
    private function getBackupType($backupIDs, $sid)
    {
        $backupType = array();
        $result_format = array();
        $result_format['system_id'] = $sid;
        $result_format['backup_ids'] = $backupIDs;

        $resultArray = $this->BP->get_backup_status($result_format);
        if ($resultArray !== false) {
            foreach ($resultArray as $result) {
                $backupType[] = array( 'id' => $result['id'],
                                    'type' => $result['type'],
                                    'app_id' => ( array_key_exists('app_id', $result) ? $result['app_id'] : false ),
                                    'instance_id' => ( array_key_exists('instance_id', $result) ? $result['instance_id'] : false ),
                                    'instance_name' => ( array_key_exists('server_instance_name', $result) ? $result['server_instance_name'] : false ),
                                    'database' => ( array_key_exists('database_name', $result) ? $result['database_name'] : ''),
                                    'replicated' => ( array_key_exists('replicated', $result) ? $result['replicated'] : ''));
            }
        }
        return $backupType;
    }

    private function getRestoreGroup($backupID, $sid)
    {
        $backups = "";
        $delim = "";
        $backupType = $this->getBackupType(array((int)$backupID), $sid);
        foreach ($backupType as $type) {
            if ($type['type'] === Constants::BACKUP_TYPE_INCREMENTAL || $type['type'] === Constants::BACKUP_TYPE_DIFFERENTIAL) {
                $groupArray = $this->BP->get_synthesis_group($backupID, $sid);
                if ($groupArray !== false) {
                    $backups = $groupArray;
                } else {
                    $backups['error'] = $groupArray;
                }
            } else {
                $groupArray = $this->BP->get_restore_group($backupID, $sid);
                if ($groupArray !== false) {
                    foreach ($groupArray as $group) {
                        $backups = $backups . $delim . $group['id'];
                        $delim = ",";
                    }
                } else {
                    $backups['error'] = $groupArray;
                }
            }
            break; //Only one backup array is returned
        }
        return $backups;
    }

    private function isBackupSharePoint($backupType)
    {
        $isSharePoint = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_SHAREPOINT_FULL:
            case Constants::BACKUP_TYPE_SHAREPOINT_DIFF:

                $isSharePoint = true;
                break;
        }
        return $isSharePoint;
    }

    private function isBackupOracle($backupType)
    {
        $isOracle = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_ORACLE_FULL:
            case Constants::BACKUP_TYPE_ORACLE_INCR:

                $isOracle = true;
                break;
        }
        return $isOracle;
    }

    private function getClientIDFromBackup($backupID, $sid)
    {
        $clientID = array();
        $result_format = array();
        $result_format['system_id'] = $sid;
        $result_format['backup_ids'] = $backupID;

        $resultArray = $this->BP->get_backup_status($result_format);
        if ($resultArray !== false) {
            foreach ($resultArray as $result) {
                $clientID[] = $result['client_id'];
            }
        }
        return $clientID;
    }

    private function getExchangeDBFromBackup($backupID, $sid)
    {
        $dbNames = array();
        $result_format = array();
        $result_format['system_id'] = $sid;
        $result_format['backup_ids'] = $backupID;

        $resultArray = $this->BP->get_backup_status($result_format);
        if ($resultArray !== false) {
            foreach ($resultArray as $result) {
                $dbNames[] = $result['database_name'];
            }
        }
        foreach ($dbNames as $db) {
            $dbNameRet = $db;
            break;
	}
        return $dbNameRet;
    }

    private function getClientNameFromID($clientID, $sid)
    {
        $clientName = false;
        $clientInfo = $this->BP->get_client_info($clientID, $sid);
        if ($clientInfo !== false) {
            $clientName = isset($clientInfo['name']) ? $clientInfo['name'] : false;
        }
        return $clientName;
    }

    private function getVolumeNames($includesArray)
    {
        $volumeNames = array();
        $i = 0;
        foreach ($includesArray as $includes) {
            $index = strpos($includes, ':');
            $volume = substr($includes, 0, $index + 1);
            $volumeNames[$i++] = $volume;
        }
        return $volumeNames;
    }

    function getVMwareClient($sid) {
        $clientID = -1;
        global $BP;
        $cid = $BP->get_client_id(Constants::CLIENT_NAME_VCENTER_RRC, $sid);
        if ($cid !== false) {
            $clientID = $cid;
        }
        return $clientID;
    }
}













?>
