<?php

class Retention
{
    private $BP;

    const MIN_MAX = "MinMax";

    public function __construct($BP)
    {
        $this->BP = $BP;
        require_once('function.lib.php');
        $this->functions = new Functions($this->BP);
    }

    public function get_retention($which, $data, $sid)
    {
        $result = array();
        if ($which !== -1) {
            switch($which[0]) {
                case "policy":
                    if (!isset($which[1])) {
                        $which[1] = -1;
                    }
                    $result = $this->get_retention_policy($which[1], $sid);
                    break;
                case "strategy":
                    $result = $this->get_retention_strategy($sid);
                    break;
            }
        } else {
            $result = $this->get_gfs_retention($data, $sid);
        }
        return $result;
    }

    private function get_retention_policy($which, $sid)
    {
        $policy_list = array();
        $policyID = NULL;
        $sid = $sid !== false ? $sid : $this->BP->get_local_system_id();
        if ($which !== -1) {
            $policyID = (int)$which;
        }
        $strategy = $this->BP->get_retention_strategy($sid);
        if ($strategy !== false) {
            if ($strategy !== Retention::MIN_MAX) {
                $policies = $this->BP->get_gfs_policy($policyID, $sid);
                if ($policies !== false) {
                    foreach ($policies as $policy) {
                        $temp_policy_list = array();
                        $temp_policy_list['id'] = $policy['policy_id'];
                        $temp_policy_list['name'] = $policy['policy_name'];
                        $temp_policy_list['days'] = $policy['days'] == -1 ? 'Forever' : $policy['days'];
                        $temp_policy_list['weeks'] = $policy['weeks'] == -1 ? 'Forever' : $policy['weeks'];
                        $temp_policy_list['months'] = $policy['months'] == -1 ? 'Forever' : $policy['months'];
                        $temp_policy_list['years'] = $policy['years'] == -1 ? 'Forever' : $policy['years'];
                        $temp_policy_list['is_default'] = $policy['is_default'];

                        $policy_list['data'][] = $temp_policy_list;
                    }
                } else {
                    $policy_list = $policies;
                }
            } else {
                $policy_list['error'] = 500;
                $policy_list['message'] = "Retention policies cannot be listed as the retention strategy of the system is not GFS.";
            }
        } else {
            $policy_list = false;
        }
        return $policy_list;
    }

    private function get_retention_strategy($sid)
    {
        $result = false;
        if ($sid === false) {
            $sid = $this->BP->get_local_system_id();
        }
        $strategy = $this->BP->get_retention_strategy($sid);
        if ($strategy !== false) {
            $result['data']['strategy'] = $strategy;
        } else {
            $result = $strategy;
        }
        return $result;
    }

    private function get_gfs_retention($data, $sid)
    {
        $retention = false;
        $filter = array();
        $sid = $sid !== false ? $sid : $this->BP->get_local_system_id();
        if (array_key_exists('client_id', $data)) {
            $filter['client_id'] = (int)$data['client_id'];
        }
        if (array_key_exists('app_id', $data)) {
            $filter['app_id'] = (int)$data['app_id'];
        }
        if (array_key_exists('uuid', $data)) {
            $filter['uuid'] = $data['uuid'];
        }
        if (array_key_exists('instance_id', $data)) {
            $filter['instance_id'] = (int)$data['instance_id'];
        }
        $strategy = $this->BP->get_retention_strategy($sid);
        if ($strategy !== false && $strategy !== Retention::MIN_MAX) {
            $associations = $this->BP->get_gfs_retention($filter, $sid);
            if ($associations !== false) {
                foreach ($associations as $association) {
                    $temp_retention = array();
                    $temp_retention['client_id'] = $association['client_id'];
                    $temp_retention['app_id'] = $association['app_id'];
                    $temp_retention['app_name'] = $association['app_name'];
                    $temp_retention['instance_id'] = $association['instance_id'];
                    $temp_retention['instance_name'] = $association['instance_name'];
                    $temp_retention['policy_id'] = $association['policy_id'];
                    $temp_retention['policy_name'] = $association['policy_name'];
                    $temp_retention['days'] = $association['days'];
                    $temp_retention['weeks'] = $association['weeks'];
                    $temp_retention['months'] = $association['months'];
                    $temp_retention['years'] = $association['years'];

                    $retention['data'][] = $temp_retention;
                }
            } else {
                $retention = $associations;
            }
        } else {
            $retention['error'] = 500;
            $retention['message'] = "GFS Retention settings cannot be fetched as the retention strategy of the system is not GFS.";
        }
        return $retention;
    }

    public function post_retention($which, $data, $sid)
    {
        $result = array();
        if ($which !== -1) {
            switch($which) {
                case "policy":
                    $result = $this->post_retention_policy($data, $sid);
                    break;
                case "strategy":
                    $result = $this->post_retention_strategy($data, $sid);
                    break;
                case "affected-backups":
                    $result = $this->get_affected_backups($data, $sid);
                    break;
            }
        } else {
            $result = $this->post_gfs_retention($data, $sid);
        }
        return $result;
    }

    private function post_retention_policy($data, $sid)
    {
        $result = false;
        $gfsSettings = array();
        if (array_key_exists('name', $data)) {
            $gfsSettings['policy_name'] = $data['name'];
        }
        $gfsSettings['policy_description'] = 'description';
        if (array_key_exists('years', $data)) {
            $gfsSettings['years'] = $data['years'];
        }
        if (array_key_exists('months', $data)) {
            $gfsSettings['months'] = $data['months'];
        }
        if (array_key_exists('weeks', $data)) {
            $gfsSettings['weeks'] = $data['weeks'];
        }
        if (array_key_exists('days', $data)) {
            $gfsSettings['days'] = $data['days'];
        }
        if (array_key_exists('is_default', $data)) {
            $gfsSettings['is_default'] = (int)$data['is_default'];
        }
        $strategy = $this->BP->get_retention_strategy($sid);
        if ($strategy !== false && $strategy !== Retention::MIN_MAX) {
            $result = $this->BP->save_gfs_policy($gfsSettings, $sid);
        } else {
            $result['error'] = 500;
            $result['message'] = "Retention policy cannot be created as the retention strategy of the system is not GFS.";
        }
        return $result;
    }

    private function post_retention_strategy($data, $sid)
    {
        $result = false;
        $strategy = "";
        if (array_key_exists('strategy', $data)) {
            $strategy = $data['strategy'];
        }
        $result = $this->BP->set_retention_strategy($strategy, $sid);
        return $result;
    }

    private function get_affected_backups($data, $sid)
    {
        $settings = array();
        $sid = $sid !== false ? $sid : $this->BP->get_local_system_id();
        if (array_key_exists('retention', $data)) {
            foreach ($data['retention'] as $retention) {
                $settings[] = $retention;
            }
            $backups = $this->BP->get_gfs_affected_backups($settings, $sid);
            $affected_backups = array();
            foreach ($backups as $backup) {
                $result_format = array();
                $backupIDs = explode(",", $backup['backup_ids']);
                foreach ($backupIDs as $backupID) {
                    $result_format['backup_ids'][] = (int)$backupID;
                }
                $backup_status = $this->BP->get_backup_status($result_format);
                foreach ($backup_status as $status) {
                    $temp_backup_array = array();
                    $temp_backup_array['id'] = $status['id'];
                    $temp_backup_array['instance_id']  = isset($status['instance_id']) ? $status['instance_id'] : null;
                    $temp_backup_array['client_id'] = isset($status['client_id']) ? $status['client_id'] : null;
                    $temp_backup_array['client_name'] = isset($status['client_id']) ? $this->getClientName($status['client_id'], $sid) : null;
                    $temp_backup_array['type'] = isset($status['type']) ? $this->functions->getBackupTypeString($status['type']) : null;
                    $temp_backup_array['start_time'] = isset($status['start_time']) ? $this->functions->formatDateTime($status['start_time']) : null;
                    if (isset($status['database_name'])) {
                        $temp_backup_array['database_name'] = $status['database_name'];
                    }
                    if (isset($status['vm_name'])) {
                        $temp_backup_array['vm_name'] = $status['vm_name'];
                    }
                    if (isset($status['server_name'])) {
                        $temp_backup_array['server_name'] = $status['server_name'];
                    }
                    $temp_backup_array['replicated'] = $status['replicated'];

                    $affected_backups['backups'][] = $temp_backup_array;
                }
            }
        } else {
            $affected_backups['error'] = 500;
            $affected_backups['message'] = "Specify the GFS Retention settings for getting list of backups that might get deleted
                            if retention policy is associated with assets.";
        }
        return $affected_backups;
    }

    private function post_gfs_retention($data, $sid)
    {
        $settings = array();
        $sid = $sid !== false ? $sid : $this->BP->get_local_system_id();
        $strategy = $this->BP->get_retention_strategy($sid);
        if ($strategy !== false && $strategy !== Retention::MIN_MAX) {
            if (array_key_exists('retention', $data)) {
                foreach ($data['retention'] as $retention) {
                    $settings[] = $retention;
                }
                $result = $this->BP->apply_gfs_retention($settings, $sid);
            } else {
                $result['error'] = 500;
                $result['message'] = "Specify the GFS Retention settings for associating retention policies with assets.";
            }
        } else {
            $result['error'] = 500;
            $result['message'] = "GFS Retention Policies cannot be associated with assets as the retention strategy of the system is not GFS.";
        }
        return $result;
    }

    public function put_retention($which, $data, $sid)
    {
        $result = array();
        switch($which[0]) {
            case "policy":
                if (!isset($which[1])){
                    $which[1] = -1;
                }
                $result = $this->put_retention_policy($which[1], $data, $sid);
                break;
            case "min-max":
                if(isset($data['client_id'])){
                    switch($data['type']){
                        case Constants::APPLICATION_TYPE_NAME_VMWARE:
                            $vmList = $this->BP->get_vm_info($data['client_id'], NULL, true, false, $sid);
                            $result = $this -> addRetentionForAllInstances($vmList, $data, $sid);
                            break;
                        case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                            $instances = $this->BP->get_hyperv_info($data['client_id'], true, $sid);
                            $result = $this -> addRetentionForAllInstances($instances, $data, $sid);
                            break;
                        case Constants::APPLICATION_TYPE_NAME_SQL_SERVER:
                            $instances = $this->BP->get_sql_info($data['client_id'], $data['id'], true, $sid);
                            $result = $this -> addRetentionForAllInstances($instances, $data, $sid);
                            break;
                        case Constants::APPLICATION_TYPE_NAME_EXCHANGE:
                            $instances = $this->BP->get_exchange_info($data['client_id'], true, $sid);
                            $result = $this -> addRetentionForAllInstances($instances, $data, $sid);
                            break;
                        case Constants::APPLICATION_TYPE_NAME_ORACLE:
                            $instances = $this->BP->get_oracle_info($data['client_id'], $data['id'], true, $sid);
                            $result = $this -> addRetentionForAllInstances($instances, $data, $sid);
                            break;
                        case Constants::APPLICATION_TYPE_NAME_SHAREPOINT:
                            $instances = $this->BP->get_sharepoint_info($data['client_id'], $data['id'], true, $sid);
                            $result = $this -> addRetentionForAllInstances($instances, $data, $sid);
                            break;
                        case Constants::APPLICATION_TYPE_NAME_XEN:
                            $instances = $this->BP->get_xen_vm_info($data['client_id'], true, false, $sid);
                            $result = $this -> addRetentionForAllInstances($instances, $data, $sid);
                            break;
                        case Constants::APPLICATION_TYPE_NAME_UCS_SERVICE_PROFILE:
                            $instances = $this->BP->get_ucssp_info($data['client_id'], $data['id'], true, $sid);
                            $result = $this -> addRetentionForAllInstances($instances, $data, $sid);
                            break;
                        case Constants::APPLICATION_TYPE_NAME_NDMP_DEVICE:
                            $instances = $this->BP->$this->BP->get_ndmpvolume_info($data['client_id'], true, $sid);
                            $result = $this -> addRetentionForAllInstances($instances, $data, $sid);
                            break;
                    }
                }
                else{
                    $result = $this->BP-> save_retention_settings($data, $sid);
                }
                break;
        }
        return $result;
    }

    function addRetentionForAllInstances($instances, $data, $sid){
        foreach ($instances as $instance) {
            $val['instance_id'] = $instance['instance_id'];
            $val['legal_hold'] = $data['retentionInfo']['legal_hold'];
            $val['retention_min'] = $data['retentionInfo']['retention_min'];
            $val['retention_max'] = $data['retentionInfo']['retention_max'];
            $allInstances[] = $val;
        }
        if(!empty($allInstances)){
            $result = $this->BP-> save_retention_settings($allInstances, $sid);
        }
        else{
            $result['error'] = 500;
            $result['message'] = "No instances were found!";
        }
        return $result;
    }

    private function put_retention_policy($which, $data, $sid)
    {
        $result = false;
        $gfsSettings = array();
        if ($which !== -1) {
            $policyID = (int)$which;
            $gfsSettings['policy_id'] = $policyID;
            if (array_key_exists('name', $data)) {
                $gfsSettings['policy_name'] = $data['name'];
            }
            $gfsSettings['policy_description'] = 'description';
            if (array_key_exists('years', $data)) {
                $gfsSettings['years'] = $data['years'];
            }
            if (array_key_exists('months', $data)) {
                $gfsSettings['months'] = $data['months'];
            }
            if (array_key_exists('weeks', $data)) {
                $gfsSettings['weeks'] = $data['weeks'];
            }
            if (array_key_exists('days', $data)) {
                $gfsSettings['days'] = $data['days'];
            }
            if (array_key_exists('is_default', $data)) {
                $gfsSettings['is_default'] = (int)$data['is_default'];
            }
            $strategy = $this->BP->get_retention_strategy($sid);
            if ($strategy !== false && $strategy !== Retention::MIN_MAX) {
                $result = $this->BP->save_gfs_policy($gfsSettings, $sid);
            } else {
                $result['error'] = 500;
                $result['message'] = "Retention policy cannot be modified as the retention strategy of the system is not GFS.";
            }
        } else {
            $result['error'] = 500;
            $result['message'] = "Policy ID must be specified.";
        }
        return $result;
    }

    public function delete_retention($which, $sid)
    {
        $result = array();
        if ($which !== -1) {
            switch ($which[0]) {
                case "policy":
                    if (!isset($which[1])) {
                        $which[1] = -1;
                    }
                    $result = $this->delete_retention_policy($which[1], $sid);
                    break;
                default:
                    $result = $this->delete_gfs_retention($which[0], $sid);
                    break;
            }
        } else {
            $result = $this->delete_gfs_retention($which, $sid);
        }
        return $result;
    }

    private function delete_retention_policy($which, $sid)
    {
        $result = false;
        if ($which !== -1) {
            $policyID = (int)$which;
            $sid = $sid !== false ? $sid : $this->BP->get_local_system_id();
            $strategy = $this->BP->get_retention_strategy($sid);
            if ($strategy !== false && $strategy !== Retention::MIN_MAX) {
                $result = $this->BP->delete_gfs_policy($policyID, $sid);
            } else {
                $result['error'] = 500;
                $result['message'] = "Retention policy cannot be deleted as the retention strategy of the system is not GFS.";
            }
        } else {
            $result['error'] = 500;
            $result['message'] = "Policy ID must be specified.";
        }
        return $result;
    }

    private function delete_gfs_retention($which, $sid)
    {
        $result = false;
        if ($which !== -1) {
            $instanceID = (int)$which;
            $policyID = 0;
            $sid = $sid !== false ? $sid : $this->BP->get_local_system_id();
            $strategy = $this->BP->get_retention_strategy($sid);
            if ($strategy !== false && $strategy !== Retention::MIN_MAX) {
                $settings[] = array('instance_id'=>$instanceID,
                                  'policy_id'=>$policyID);
                $result = $this->BP->apply_gfs_retention($settings, $sid);
            } else {
                $result['error'] = 500;
                $result['message'] = "Policy association cannot be removed as the retention strategy of the system is not GFS.";
            }
        } else {
            $result['error'] = 500;
            $result['message'] = "Asset ID must be specified.";
        }
        return $result;
    }

    private function getClientName($cid, $sid)
    {
        $clientInfo = $this->BP->get_client_info($cid, $sid);
        $clientName = isset($clientInfo['name']) ? $clientInfo['name'] : null;

        return $clientName;
    }
}
?>
