<?php

if (!function_exists('json_encode')) {
    // This is an older, unsupported php version.  Build an error and exit.
    require_once('./includes/api.lib.php');
    require_once('./includes/logger.php');
    $not_supported_message = "The REST APIs are not supported on this appliance OS version.";
    $json_message = '{"result": [{"code": 500, ' .
        '"message": "' . $not_supported_message . '"}]}';
    define("LOG_DIRECTORY", "/usr/bp/logs.dir");
    $Log = new Logger(3, LOG_DIRECTORY);
    $Log->writeError($not_supported_message, false, true);
    Controller::respond(500, $json_message, 'application/json');
    exit();
}

require_once('./includes/api.lib.php');
require_once('./includes/constants.lib.php');
require_once('./includes/function.lib.php');

require_once('./includes/appliance.php');
require_once('./includes/bp.php');
require_once('./includes/logger.php');
require_once('./includes/inventory.php');
require_once('./includes/notifications.php');
require_once('./includes/reports.php');
require_once('./includes/schedules.php');
require_once('./includes/configuration.php');
require_once('./includes/credentials.php');
require_once('./includes/archive.php');
require_once('./includes/virtual-clients.php');
require_once('./includes/encryption.php');
require_once('./includes/replication.php');
require_once('./includes/capabilities.php');
require_once('./includes/forum.php');
require_once('./includes/catalogs.php');
require_once('./includes/uiadvisor.php');
require_once('./includes/activedirectory.php');
require_once('./includes/hypervisor.php');

define("LOG_DIRECTORY", "/usr/bp/logs.dir");

global $status;
global $message;
// get arguments -- auth (authentication) required for remote access, optional for local access!

// Get authentication cookie. Must be present for all requests except login and localhost initiated ones
$request_headers = getallheaders();
$authentication_cookie = isset($request_headers['AuthToken']) ? $request_headers['AuthToken'] : "";

// Get Logging options
$logLevel = isset($_REQUEST['lvl']) ? (int)$_REQUEST['lvl'] : 3;
$logDirectory = isset($_REQUEST['logdir']) ? $_REQUEST['logdir'] : LOG_DIRECTORY;

// Create standard objects for use in all BPL functions.
$Log = new Logger($logLevel, $logDirectory);
$BP = new BP($authentication_cookie);
$Constants = new Constants();

// create our object
$controller = new Controller();

// retrieve the request url path
$request = $controller->getRequest();

// validate the request
if (isset($request[0]) && empty($request[0])) {
    Controller::respond(404);
}

// Check access scheme
$method = $request[0];

if (!$BP->isBypassCookie() && $method != 'login') {
    // If authentication cookie not passed in, return message and error.
    if ($authentication_cookie == "") {
        $errorMessage = 'Error: An empty authentication string was passed from your browser to the system.';
        $status = 500;
        $return = buildResult($status, $errorMessage);
        if ($controller->getHttpAccept() == 'json') {
            Controller::respond($return['status'], json_encode($return['body']), 'application/json');
        } elseif ($controller->getHttpAccept() == 'xml') {
            Controller::respond($return['status'], array2xml($return['body']));
        } else {
            Controller::respond(500);
        }
        exit;
    }

    // If authentication cookie passed in is not valid, return message and error.
    if (!$BP->validCookie()) {
        $errorMessage = 'Error: Your session has expired or is invalid.  Please log in again to continue.';
        $status = 500;
        $return = buildResult($status, $errorMessage);
        if ($controller->getHttpAccept() == 'json') {
            Controller::respond($return['status'], json_encode($return['body']), 'application/json');
        } elseif ($controller->getHttpAccept() == 'xml') {
            Controller::respond($return['status'], array2xml($return['body']));
        } else {
            Controller::respond(500);
        }
        exit;
    }
}

// appliance object
$appliance = new Appliance($BP);

// functions object
$functions = new Functions($BP);

// inventory object
$inventory = new Inventory($BP);

//notification object
$notifications = new Notifications($BP);

//report object
$reports = new Reports($BP);

//schedule object
$schedules = new Schedule($BP);

//configuration object
$configuration = new Configuration($BP);

//credential object
$credentials = new Credentials($BP);

//replication object
$replication = new Replication($BP);

//archive object
$archive = new Archive($BP);

//forum object
$forum = new Forum($BP);

//catalog object
$catalogs = new Catalogs($BP);

// retrieve data sent with the request if any
$data = $controller->getData();

// get system ID if specified.
$sid = $controller->get_sid();
$systems = $functions->selectSystems($sid);

// get developer mode if specified.
$developer = $controller->get_developer();

//Capabilities object
$capabilities = new Capabilities($BP);

//satori switch advisor
$uiadvisor = new UIAdvisor($BP);

//AD configuration
$AD = new ActiveDirectory($BP);

// Hypervisor
$Hypervisor = new Hypervisor($BP);

// set initial status to not found (404).

$status = 404;

// set initial error message to empty

$message = "";

// run through your application methods making use of $data and $request easily where needed
switch ($controller->getMethod()) {
    case 'get':
        $status = 200;
        $Log->enterMethod("GET", $request[0], $data, $sid);
        $body = execute_get($request, $data, $sid, $systems);
        break;
    case 'post':
        // adding item returns Created (201)
        $status = 201;
        $Log->enterMethod("POST", $request[0], $data, $sid);
        $body = execute_post($request, $data, $sid, $systems);
        break;
    case 'put':
        $status = 200;
        $Log->enterMethod("PUT", $request[0], $data, $sid);
        $body = execute_put($request, $data, $sid, $systems);
        break;
    case 'delete':
        $status = 200;
        $Log->enterMethod("DELETE", $request[0], $data, $sid);
        $body = execute_delete($request, $data, $sid, $systems);
        break;
    default:
        $Log->enterMethod("UNKNOWN", $request[0], $data, $sid);
        $status = 400;
        $body = array();
        break;
}

if ($body === false) {
    // Server Failed.  This forces buildResult to get the error message.
    $status = 500;
}
$return = buildResult($status, $body, $developer);

if ($controller->getHttpAccept() == 'json') {
    Controller::respond($return['status'], json_encode($return['body']), 'application/json');
} elseif ($controller->getHttpAccept() == 'xml') {
    Controller::respond($return['status'], array2xml($return['body']));
} elseif ($controller->getHttpAccept() == 'csv') {
    Controller::respond($return['status'], $return['body']['data'], 'text/csv');
} elseif ($controller->getHttpAccept() == 'pdf') {
    Controller::respond($return['status'], $return['body']['data'], 'application/pdf');
} else {
    Controller::respond(400);
}


function execute_get($request, $data, $sid, $systems)
{
    global $status;
    global $message;

    global $appliance;
    global $notifications;
    global $reports;
    global $schedules;
    global $body;
    global $configuration;
    global $inventory;
    global $credentials;
    global $archive;
    global $replication;
    global $capabilities;
    global $uiadvisor;
    global $forum;
    global $catalogs;
    global $AD;
    global $Log;

    $method = $request[0];
    switch($method) {
        case 'users':
            $which = -1;
            if (isset($request[1])) {
                $which = (int)$request[1];
            }
            $body = $appliance->get_users($which, $sid);
            break;
        case 'clients':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            if(isset($request[2])){
                $filter = (int)$request[2];
            }
            $body = $appliance->get_clients($which, $filter, $data, $sid, $systems);
            break;
        case 'alerts':
            $which = -1;
            if (isset($request[1])) {
                if (is_numeric($request[1])) {
                    $which = (int)$request[1];
                } else {
                    $which = $request[1];
                }
            }
            $filter = "";
            if (isset($request[2])) {
                $filter = $request[2];
            }
            $body = $notifications->get_alerts($which,$filter, $sid);
            break;
        case 'audits':
            $which = -1;
            if (isset($request[1])) {
                if (is_numeric($request[1])){
                    $which = (int)$request[1];
                } else {
                    $which = $request[1];
                }
            }
            $body = $notifications->get_audit_history($which, $sid);
            break;
        //this is ugly - needs to be written better
        case 'traps':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
                if ($which == "destinations") {
                    $id = isset($request[2]) ? (int)$request[2] : -1;
                    $body = $notifications->get_snmp_config($id, $sid);
                } else {
                    $which = $request[1];
                    $body = $notifications->get_snmp_history($which, $sid);
                }
            } else {
                $body = $notifications->get_snmp_history($which, $sid);
            }
            break;
        case 'backups':
            $which = -1;
            if (isset($request[1])) {
                if (is_numeric($request[1])){
                    $which = (int)$request[1];
                } else {
                    $which = $request[1];
                }
            }
            $filter = "";
            if (isset($request[2])) {
                $filter = $request[2];
            }
            $body = $appliance->get_backups($which, $filter, $sid);
            break;
        case 'jobs':
            $which = -1;
            if (isset($request[1])) {
                if (is_numeric($request[1])){
                    $which = (int)$request[1];
                } else {
                    $which = $request[1];
                }
            }
            $filter = "";
            if (isset($request[2])) {
                $filter = $request[2];
            }
            $body = $notifications->get_jobs($which, $filter, $sid, $systems);
            break;
        case 'psa':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $notifications->get_psa_info($which);
            break;
        case 'reports':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $reports->get($which, $data, $sid, $systems);
            break;
        case 'joborders':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $schedules->get($which, $systems);
            break;
        case 'networks':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $filter = $request;
            $body = $appliance->get_networks($which, $filter, $data, $sid);
            break;
        case 'summary' :
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            if (isset($request[2])) {
                $filter = $request[2];
                //does $filter begin with ?  ?
                if ($filter[0] == '?'){
                    $filter = null;
                }
            } else {
                $filter = null;
            }

            $body = $appliance->get_summary($which, $filter, $sid);
            break;
        case 'storage':
            $which = null;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $appliance->get_storage($which, $data, $sid, $systems);
            break;
        case 'preferences':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            if (isset($request[2])) {
                $filter = $request[2];
                //does $filter begin with ?  ?
                if ($filter[0] == '?'){
                    $filter = null;
                }
            } else {
                $filter = null;
            }
            $body = $configuration->get_preferences($which, $filter, $sid);
            break;
        case 'inventory':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $inventory->get($which, $data, $sid, $systems);
            break;
        case 'license':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->get_license($which);
            break;
        case 'updates':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->get_updates($which);
            break;
        case 'credentials':
            $which = -1;
            if(isset($request[1])) {
                $which = $request[1];
            }
            $body = $credentials->get($which, $data, $sid, $systems);
            break;
        case 'date-time':
            $which = -1;
            if(isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->get_datetime($which,$sid);
            break;
        case 'mail-config':
            $which = -1;
            if(isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->get_mailconfig($which, $sid);
            break;
        case 'virtual_clients':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $appliance->get_virtual_clients($which, $data, $sid, $systems);
            break;
        case 'systems':
            $which = -1;
            if (isset($request[1])) {
                if (is_numeric($request[1])) {
                    $which = (int)$request[1];
                } else {
                    $which = $request[1];
                }
            }
            $filter = "";
            if (isset($request[2])) {
                $filter = $request[2];
            }
            $body = $appliance->get_systems($which, $filter, $data, $sid);
            break;
        case 'encryption':
            $which = -1;
            if(isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->get_encryption($which, $systems);
            break;
        case 'archive':
            $which = array_splice($request, 1);
            $body = $archive->get($which, $data, $sid, $systems);
            break;
        case 'replication':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $filter = "";
            if (isset($request[2])) {
                $filter = $request[2];
            }
            $body = $replication->get($which, $filter, $data, $sid);
            break;
        case 'hostname':
            $which = -1;
            if(isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->get_hostname($which, $sid);
            break;
        case 'restore':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $appliance->get_restore($which, $data, $sid, $systems);
            break;
        case 'capabilities':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $capabilities->get($which, $sid);
            break;
        case 'forum':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $forum->get_forum($which, $data, $sid, $systems);
            break;
        case 'catalog':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $catalogs->get_catalog($which, $data, $sid, $systems);
            break;
        case 'backup-copy':
            $which = -1;
            if(isset($request[1])) {
                $which = $request[1];
            }
            $body = $replication->get_backup_copy($which, $sid, $systems);
            break;
        case 'retention':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $configuration->get_retention($which, $data, $sid);
            break;
        case 'assets':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->get_protected_assets($which, $sid);
            break;
        case 'advisor':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $uiadvisor->get_advice($which, $data, $sid);
            break;
        case 'active-directory':
            $body = $AD->get($sid);
            break;
        case 'applications':
            $body = $configuration->get_applications($data, $sid);
            break;
        default:
            break;
    }
    //check to see if an error was returned from the webservice layer
    if(is_array($body) && isset($body['error'])) {
        $status = $body['error'];
        unset($body['error']);
        if (isset($body['message'])) {
            $message = $body['message'];
            unset($body['message']);
        } else {
            $message = "";
        }
    }
    // want to return object with body and status
    return $body;
}

function execute_delete($request, $data, $sid, $systems)
{
    global $status;
    global $message;

    global $appliance;
    global $schedules;
    global $notifications;
    global $credentials;
    global $body;
    global $configuration;
    global $archive;
    global $forum;
    global $replication;
    global $AD;

    $method = $request[0];
    switch($method) {
        case 'clients':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
                $body = $appliance->delete_clients($which,$sid);
            }
            break;
        case 'joborders':
            if (isset($request[1])) {
                $which = $request[1];
                $body = $schedules->delete($which, $sid);
            }
            break;
        case 'jobs':
            if (isset($request[1])) {
                $which = $request[1];
                $body = $notifications->delete_job($which, $sid);
            }
            break;
        case 'storage':
            if (isset($request[1])) {
                $which = $request[1];
                $body = $appliance->delete_storage($which, $sid);
            }
            break;
        case 'backups':
            if (isset($request[1])) {
                $which = $request[1];
                $body = $appliance->delete_backups($which, $sid);
            }
            break;
        case 'alerts':
            if (isset($request[1])) {
                $which = $request[1];
                $body = $notifications->close_alert($which, $sid);
            }
            break;
        case 'credentials':
            if(isset($request[1])) {
                if(is_numeric($request[1])) {
                    $which = (int)$request[1];
                    $body = $credentials->delete_credential($which, $sid);
                } else if($request[1] == "bind") {
                    $which = array_splice($request, 2);
                    $body = $credentials->unbind_credential($which, $data, $sid);
                }
            }
            break;
        case 'date-time':
            if (isset($request[1])) {
                $which = (int)$request[1];
                $body = $configuration->delete_datetime($which, $data, $sid);
            }
            break;
        case 'mail-config':
            if (isset($request[1])) {
                $which = $request[1];
                $body = $configuration->delete_mailconfig($which, $sid);
            }
            break;
        case 'networks':
            if (isset($request[1])) {
                $which = $request[1];
                $body = $appliance->delete_networks($which, $request, $sid);
            }
            break;
        case 'virtual_clients':
            $which = null;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $appliance->delete_virtual_client($which, $data, $sid);
            break;
        case 'systems':
            if (isset($request[1])) {
                if(is_numeric($request[1])){
                    $which = (int)$request[1];
                    $body = $appliance->delete_systems($which);
                }
            }
            break;
        case'archive':
            $which = array_splice($request, 1);
            $body = $archive->delete($which, $sid);
            break;
        case 'users':
            $which = -1;
            if (isset($request[1])) {
                $which = (int)$request[1];
                $body = $appliance->delete_users($which);
            }
            break;
        case 'forum':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $forum->delete_forum($which, $data, $sid);
            break;
        case 'retention':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $configuration->delete_retention($which, $sid);
            break;
        case 'replication':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $replication->delete($which, $sid);
            break;
        case 'backup-copy':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $replication->delete_backup_copy($which, $sid);
            break;
        case 'traps':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
                if ($which == 'destinations') {
                    $id = isset($request[2]) ? (int)$request[2] : -1;
                    $body = $notifications->delete_snmp_config($id, $sid);
                }
            }
            break;
        case 'active-directory':
            $body = $AD->delete($sid);
            break;
        default:
            break;
    }
    //check to see if an error was returned from the webservice layer
    if(is_array($body) && isset($body['error'])) {
        $status = $body['error'];
        unset($body['error']);
        if (isset($body['message'])) {
            $message = $body['message'];
            unset($body['message']);
        } else {
            $message = "";
        }
    }
    return $body;
}

/*
 * Updates an existing resource.
 * Temporarily, pass through to POST
  */
function execute_put($request, $data, $sid, $systems)
{
    global $status;
    global $message;

    global $appliance;
    global $schedules;
    global $inventory;
    global $notifications;
    global $configuration;
    global $credentials;
    global $archive;
    global $replication;
    global $forum;
    global $AD;
    global $Hypervisor;
    global $uiadvisor;

    $method = $request[0];

    switch($method) {
        case 'inventory':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $inventory->put($which, $data);
            break;
        case 'joborders':
            $which[] = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $schedules->put($which, $data, $sid);
            break;
        case 'networks':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $filter = "";
            if (isset($request[2])) {
                $filter = $request[2];
            }
            $body = $appliance->update_networks($which, $filter, $data, $sid);
            break;
        case 'jobs':
            $action = "";
            if (isset($request[1])) {
                $action = $request[1];
            }
            $which = -1;
            if (isset($request[2])) {
                $which = $request[2];
            }
            $body = $notifications->suspend_resume($action, $which, $sid);
            break;
        case 'preferences':
            $which = null;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->update_preferences($which, $data, $sid);
            break;
        case 'license':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->update_license($which, $data);
            break;
        case 'credentials':
            if (isset($request[1])) {
                if (is_numeric($request[1])) {
                    $which = (int)$request[1];
                    $body = $credentials->modify_credential($which, $data, $sid);
                } else if ($request[1] == "bind") {
                    $which = array_splice($request, 2);
                    $body = $credentials->bind_credential($which, $data, $sid);
                }
            }
            break;
        case 'users':
            $which = -1;
            if (isset($request[1])) {
                $which = (int)$request[1];
                $body = $appliance->update_users($which, $data, $sid);
            }
            break;

        case 'date-time':
            $which = -1;
            if (isset($request[1])) {
                $which = (int)$request[1];
            }
            $body = $configuration->put_datetime($which, $data, $sid);
            break;
        case 'mail-config':
            $which = -1;
            if (isset($request[1])) {
                $which = (int)$request[1];
            }
            $body = $configuration->put_mailconfig($which, $data, $sid);
            break;

        case 'backups':
            $action = "";
            if (isset($request[1])) {
                $action = $request[1];
            }
            $which = -1;
            if (isset($request[2])) {
                $which = $request[2];
            }
            $filter = "";
            if (isset($request[3])) {
                $filter = $request[3];
            }
            if ($action == "hold") {
                $body = $appliance->hold_backup($which, $filter, $sid);
            } else if($action == "") {
                $body = $schedules->backup_now($data, $sid);
            }
            break;
        case 'virtual_clients':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $appliance->modify_virtual_client($which, $data, $sid);
            break;
        case 'systems':
            $action = "";
            if (isset($request[1])) {
                $action = $request[1];
            }
            $which = -1;
            if (isset($request[2])) {
                $which = $request[2];
            }
            $body = $appliance->update_system($action, $which, $data, $sid);
            break;
        case 'archive':
            $which = "";
            if(isset($request[1])) {
                if($request[1] == "check") {
                    $which = "check";
                } else {
                    $which = array_splice($request, 1);
                }
            }
            $body = $archive->put($which, $data, $sid);
            break;
        case 'storage':
            if (isset($request[1])) {
                if (is_numeric($request[1])){
                    $which = (int)$request[1];
                    $body = $appliance->update_storage($which, $data, $sid);
                } else {
                    $action = $request[1];
                    $which = -1;
                    if (isset($request[2])) {
                        $which = $request[2];
                    }
                    switch($action){
                        case "online":
                        case "offline":
                            $body = $appliance->online_offline($action, $which, $data, $sid);
                            break;
                        case "ir":
                            $body = $appliance->allocate_ir_space($data, $sid);
                            break;
                        case "d2d":
                            $body = $appliance->allocate_d2d_space($data, $sid);
                            break;
                        case "vc":
                            $body = $appliance->allocate_vc_space($data, $sid);
                            break;
                    }
                }
            }
            break;
        case 'encryption':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $configuration->put_encryption($which, $data, $sid);
            break;
        case 'replication':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $filter = "";
            $body = $replication->put($which,$filter, $data, $sid);
            break;
        case 'clients':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $appliance->put_clients( $which, $data, $sid);
            break;
        case 'hostname':
            $which = -1;
            if (isset($request[1])) {
                $which = (int)$request[1];
            }
            $body = $configuration->put_hostname($which, $data, $sid);
            break;
        case 'forum':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $forum->put_forum($which, $data, $sid);
            break;
        case 'retention':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $configuration->put_retention($which, $data, $sid);
            break;
        case 'traps':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
                if ($which == 'test') {
                    $body = $notifications->send_snmp_trap($sid);
                } else if ($which == 'destinations') {
                    $id = isset($request[2]) ? (int)$request[2] : -1;
                    $body = $notifications->put_snmp_config($id, $data, $sid);
                }
            }
            break;
        case 'active-directory':
            $body = $AD->put($data, $sid);
            break;
        case 'hypervisor':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $Hypervisor->put($which, $data, $sid);
            break;
        case 'advisor':
            $which = -1;
            $ui = -1;
            if (isset($request[1])) {
                $which = $request[1];
                if (isset($request[2])) {
                    $ui = $request[2];
                }
            }
            $body = $uiadvisor->put($which, $ui, $sid);
            break;
        case 'summary' :
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $appliance->put_summary($which, $data, $sid);
            break;
    }
    //check to see if an error was returned from the webservice layer
    if(is_array($body) && isset($body['error'])) {
        $status = $body['error'];
        unset($body['error']);
        if (isset($body['message'])) {
            $message = $body['message'];
            unset($body['message']);
        } else {
            $message = "";
        }
    }
    return $body;
}

function execute_post($request, $data, $sid, $systems)
{
    global $status;
    global $message;

    global $appliance;
    global $schedules;
    global $credentials;
    global $body;
    global $archive;
    global $configuration;
    global $forum;
    global $replication;
    global $notifications;

    $method = $request[0];
    switch($method) {
        case 'clients':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $appliance->add_clients($which, $data, $sid);
            break;
        case 'joborders':
            $which = -1;
            if (isset($request[2]) and $request[1] == "copy") {
                $which = $request[2];
            }
            $body = $schedules->save_schedule($which, $data, $sid);
            break;
        case 'login':
            $body = $appliance->login($data);
            break;
        case 'logout':
            $body = $appliance->logout($data);
            break;
        case 'credentials':
            $body = $credentials->save_credential($data, $sid);
            break;
        case 'networks':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $appliance->save_network($which, $data, $sid);
            break;
        case 'backups':
            if (isset($request[1])) {
                $action = $request[1];
                if ($action == "search") {
                    $body = $appliance->search_backup($data, $sid);
                }
            } else {
                $body = $appliance->list_backups($data, $sid);
            }
            break;
        case 'storage':
            $body = $appliance->add_storage($data, $sid);
            break;
        case 'restore':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            if (is_array($which) and $which[0] == "archive") {
                $which = array_splice($which, 1);
                $body = $archive->restore($which, $data, $sid);
            } else {
                $body = $appliance->post_restore($which, $data, $sid);
            }
            break;
        case 'virtual_clients':
            $body = $appliance->save_virtual_client($data, $sid);
            break;
        case 'archive':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $archive->post($which, $data, $sid);
            break;
        case 'updates':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->post_updates($which, $data);
            break;
        case 'encryption':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->post_encryption($which, $data, $sid);
            break;
        case 'systems':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            if ( $which === 'make_target' )
            {
                $body = $appliance->make_local_system_into_a_target($data, $sid);
            } else {
                $body = $appliance->add_system($data, $sid);
            }
            break;
        case 'commands':
            $body = $appliance->post_commands($data, $sid);
            break;
        case 'feedback':
            $body = $appliance->post_feedback($data, $sid);
            break;
        case 'users':
            $body = $appliance->add_users($data);
            break;
        case 'forum':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
            }
            $body = $forum->post_forum($which, $data, $sid);
            break;
        case 'replication':
            $which = -1;
            if (isset($request[1])) {
                $which = array_splice($request, 1);
            }
            $body = $replication->post($which, $data, $sid);
            break;
        case 'retention':
            $which = -1;
            if(isset($request[1])) {
                $which = $request[1];
            }
            $body = $configuration->post_retention($which, $data, $sid);
            break;
        case 'traps':
            $which = -1;
            if (isset($request[1])) {
                $which = $request[1];
                if ($which == 'destinations') {
                    $body = $notifications->post_snmp_config($data, $sid);
                }
            }
            break;
    }
    //check to see if an error was returned from the webservice layer
    if(is_array($body) && isset($body['error'])) {
        $status = $body['error'];
        unset($body['error']);
        if (isset($body['message'])) {
            $message = $body['message'];
            unset($body['message']);
        } else {
            $message = "";
        }
    }
    return $body;
}

function array2xml($body)
{
    $xml = new SimpleXMLElement('<root/>');
    array_walk_recursive($body, array($xml, 'addChild'));
    return $xml->asXML();
}


/*
 * This function builds the result for the user.
 * The incoming body can be:
 * an array, a good result
 * false, an error result
 * string, return the message, setting the status as provided. This is typically errors detected at the REST server level.
 */
function buildResult($status, $body, $developer = false)
{
    global $message;
    global $Log;

    $result = array();
    $result['status'] = $status;

    $errorFromBpl = true;           //Most of the errors are from bpl layer(assumption: might change in future)
    $errorFromWebServices = false;
    if(isset($body['errorFromWebServices'])){
        $errorFromWebServices = true;
        $errorFromBpl = false;       //If customised error at rest layer is passed in
    }


    if (is_array($body)) {
        $body['timestamp'] = time();
        if ($status != 200 && $status != 201) {
            if($message) {
                $errorMessage = $message;
            } else {
                $errorMessage = 'HTTP request error';
            }
            if ( array_key_exists('error_code', $body) )
            {
                $element['error_code'] = $body['error_code'];
                unset($body['error_code']);
            }
            $element['message'] = $errorMessage;
            $body['result'] = array($element);
            $Log -> writeError($body, $errorFromBpl, $errorFromWebServices);

        } else {
            // partial success message - failures, if any, are returned as a part of result.
            if (isset($body['warning'])) {
                $warnings = $body['warning'];
                foreach ($warnings as $warning) {
                    if (!isset($warning['message'])) {
                        $warning['message'] = Controller::get_error_codes($warning['code']);
                    }
                    // if developer is set, display additional details if available
                    // otherwise, do not display any additional details
                    if ($developer) {
                        if (!isset($warning['details'])) {
                            $warning['details'] = "";
                        }
                    } else {
                        if (isset($warning['details'])) {
                            unset($warning['details']);
                        }
                    }
                    $warningsArray[] = $warning;
                }
                $body['warning'] = $warningsArray;
            }
        }

    } else if (is_string($body)) {
        $result['status'] = 403;
        // The current string is the message we want to display.  Structure it.
        $errorMessage = $body;

        $body = array();

        $element = array(
            'code' => '0',
            'link' => 'http://kb.unitrends.com',	// placeholder
        );
        $element['message'] = $errorMessage;
        $body['result'] = array($element);
        $Log -> writeError($errorMessage, $errorFromBpl, $errorFromWebServices);

    } else {
        $body = array();
        // Basic element to return, code 0 (success);
        $element = array('code' => '0');

        if ($status == 500) {
            // Status code 500 is a system error.  Put the message and link.
            global $BP;
            $element['message'] = $BP->getError();
            $element['link'] = 'http://kb.unitrends.com';	// placeholder, bp_error could return link or new link for API.
            $Log -> writeError($element['message'], $errorFromBpl, $errorFromWebServices);
        } else if ($status == 403 || $status == 404 || $status == 405) {
            $element['message'] = $message;
            $element['link'] = 'http://kb.unitrends.com';
            $Log -> writeError($element['message'], $errorFromBpl, $errorFromWebServices);
        } else if ($status != 200 && $status != 201) {
            // PUT, POST, DELETE will return 200 or 201.
            $element['message'] = 'HTTP request error';		// placeholder until better message is constructed.
            $element['link'] = 'http://kb.unitrends.com';	// placeholder, need general KB for HTTP errors.
            $Log -> writeError($element['message'], $errorFromBpl, $errorFromWebServices);
        }

        // put in array as there are times we may have more than one returned item.
        $body['result'] = array($element);
    }

    $result['body'] = $body;
    // convert to CSV if needed
    if (isset($body['csv'])) {
        global $controller;
        $controller->setHttpCSV();
    } else if (isset($body['pdf'])) {
        global $controller;
        $controller->setHttpPDF($body['data']);
    }
    return $result;
}

?>
