<?php
// archive functions
require 'arfunctions.php';

// converts xml string to archive profile
require 'arxml.php';

// defines BP, xml, action, cookie
require '../header.php';

// list base functions
require  $rootdir . 'bpl/listsbase.php';
$ListBase = new ListBase();

// general library functions
require $rootdir . 'bpl/library.php';

$lib = new Library();

// Initialize variables to be used in save.
$scheduleInfo = array();
$bValid = true;

$localID = $BP->get_local_system_id();
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $localID;

// figure out which sysID to used based on view.
$sysID = $grandClientView ? $localID : $dpuID;

switch ($action) {

	case "list":
			listSchedules($BP, $xml, $sysID, false);
			echo($xml->getXml());
			break;
			
	case "load":
			$id = isset($_GET['id']) ? (int)$_GET['id'] : -1;
			loadSchedule($BP, $xml, $id, $sysID);
			echo($xml->getXml());
			break;
					
	case "save":
			$id = isset($_GET['id']) ? (int)$_GET['id'] : -1;
			if ($id == -1) {
				if (isset($_GET['name'])) {
					$scheduleInfo = array_merge($scheduleInfo, array("name" => $_GET['name']));
				} else {
					$BP->buildResult($xml, false, "Error: must specify either name or id of schedule to save.");
					$bValid = false;
				}
			} else {
				$scheduleInfo = array_merge($scheduleInfo, array("id" => $id));
				if (isset($_GET['name'])) {
					$scheduleInfo = array_merge($scheduleInfo, array("name" => $_GET['name']));
				}
			}
			if ($bValid) {
				if (isset($_GET['desc'])) {
					$scheduleInfo['description'] = $_GET['desc'];
				}
				if (isset($_GET['enabled'])) {
					$enabled = $_GET['enabled'] == "1" ? true : false;
					$scheduleInfo['enabled'] = $enabled;
				}
				// Pass calendar data via request - can be a lot of text.
				if (isset($_REQUEST['calendar'])) {
					$scheduleInfo['calendar'] = $_REQUEST['calendar'];
				}
				// Build the archive profile.
				if (isset($_REQUEST['profile'])) {
					$profileXML = $_REQUEST['profile'];
					$profile = xmlToProfile($profileXML, $lib);
				} else {
					$profile = getProfileFromURL($lib);
				}
				$scheduleInfo['profile'] = $profile;
				// make schedule report setting the same as for the profile.
				$scheduleInfo['email_report'] = $profile['options']['email_report'];
				$scheduleInfo['failure_report'] = $profile['options']['email_report'];
	
				//$ListBase->buildResult($xml, $BP->save_archive_schedule_info($scheduleInfo, $sysID));
				buildArchiveScheduleResult($xml, $BP->save_archive_schedule_info($scheduleInfo, $sysID));

			}
			echo($xml->getXml());			
			break;
				
	case "delete":
			//$id = (int)$_GET['id'];
			//$BP->buildResult($xml, $BP->delete_archive_schedule($id, $sysID));
			if (($id = $ListBase->getID($BP, 'get_archive_schedule_list', $sysID)) == -1) {
				$BP->buildResult($xml, false, "Error: cannot find identifier for deleting schedule.");
			} else {
				$BP->buildResult($xml, $BP->delete_archive_schedule($id, $sysID));
			}
			echo($xml->getXml());
			break;
			
	case "enable":
			$id = (int)$_GET['id'];
			$BP->buildResult($xml, $BP->enable_archive_schedule($id, $sysID));
			echo($xml->getXml());
			break;
			
	case "disable":
			$id = (int)$_GET['id'];
			$BP->buildResult($xml, $BP->disable_archive_schedule($id, $sysID));
			echo($xml->getXml());
			break;
}

//
// This function lists the schedules for DPU.
//
function listSchedules($BP, $xml, $dpuID) {
	global $ListBase;
	
	$scheduleList = $BP->get_archive_schedule_list($dpuID);
	if ($scheduleList !== false) {
		$xml->push("root");
		foreach ($scheduleList as $schedule) {
		  $attributes = $ListBase->getAttributes($schedule);	
			if (array_key_exists('enabled', $schedule)) {
				$enabled = $schedule['enabled'] == true ? "1" : "0";
				$attributes['Enabled'] = $enabled;
			}
			$xml->push("Schedule", $attributes);
			$xml->pop(); // schedule
		}
		$xml->pop(); // root
	} else {
		$BP->buildResult($xml, false);
	}
}

//
// This function loads the schedule given its ID.  It optionally wraps the schedule
// information with a root XML tag.
//
function loadSchedule($BP, $xml, $id, $dpuID, $bShowRoot = true)
{
	$scheduleInfo = $BP->get_archive_schedule_info($id, $dpuID);
	if ($scheduleInfo !== false) {
		if ($bShowRoot) {
			$xml->push("root");
		}
		createScheduleXML($xml, $scheduleInfo);	
		if ($bShowRoot) {
			$xml->pop(); // root
		}
	} else {
		if ($bShowRoot) {
			$BP->buildResult($xml, false);
		} else {
			// Add a warning that this schedule could not be loaded.
			;
		}
	}
}

//
// This function processes the detailed messages returned when archive schedules are saved.
//
//	Return: true - if saved successfully and list already exists
//			scalar - if saved successfully and new item created
//			false - if save failed
//			array - if save succeeded (partially) with additional information about item.
//
function buildArchiveScheduleResult($xml, $returnValue, $isWarning = false)
{
	global $Log;
	
	$xml->push("root");
	if ($returnValue === false) {
		$xml->element("Result", 0);
		$errorString = bp_error();
		$xml->element("ErrorString", $errorString);
		$Log->writeError($errorString, true);
	} else if (is_array($returnValue)) {
		if (array_key_exists('id', $returnValue)) {
			$xml->element("Result", $returnValue['id']);						
		}
		if (array_key_exists('messages', $returnValue)) {
			$messages = $returnValue['messages'];
			if (is_array($messages) && (count($messages) > 0)) {
				$xml->push("ResultArray");
				$xml->push("Result");
				foreach ($messages as $message) {
					$xml->element("Message", $message);
				}
				$xml->pop(); // Result
				$xml->pop(); // ResultArray
			}
		}
	} else if ($returnValue === true) {
		$xml->element("Result", 1);
	} else {
		$xml->element("Result", $returnValue);
	}
	$xml->pop();
}

$BP->destroyCookie();
?>