<?php
 
 // general library functions
//require $rootdir . 'bpl/library.php';
/*
require '../library.php';
 
$xml = "<root> \
	<Desc>archive description</Desc> \
	<Target>archive target</Target> \
	<RangeEnd>21384950</RangeEnd> \
	<RangeSize>86400</RangeSize> \
	<Clients>2,3</Clients> \
	<Instances>1,2,3,4,5</Instances> \
	<Types>Master,Incremental</Types> \
	<Dirs>dir1,dir2</Dirs> \
	<Options> \
		<Append>1</Append> \
		<Compress>1</Compress> \
		<Encrypt>0</Encrypt> \
		<EMail>0</EMail> \
		<TargetSlots>2</TargetSlots> \
	</Options> \
	</root>";
 
$profile = xmlToProfile($xml, $lib);

var_dump($profile);
*/

//
// Convert an XML document for the archive profile into a profile array.
//
function xmlToProfile($xml, $lib)
{
	$parser = xml_parser_create();
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
	xml_parse_into_struct($parser, $xml, $values, $tags);
	xml_parser_free($parser);

	$profile = array();

	$stack = array();
	$i = 0;
	foreach ($values as $val) {
		if ($val['type'] == "open") {
			if ($val['tag'] != "root") {
				$field = buildFieldName($val['tag']);
				array_push($stack, $field);
			}
		} else if ($val['type'] == "close") {
			if ($val['tag'] != "root") {
				array_pop($stack);
			}
		} else if ($val['type'] == "complete") {
			$field = buildFieldName($val['tag']);
			array_push($stack, $field);
			$bInteger = isInteger($val['tag']);
			$bBoolean = isBoolean($val['tag']);
			setArrayValue($profile, $stack, $val['value'], $bInteger, $bBoolean);
			array_pop($stack);
		}
	}

	if (array_key_exists('localdirs', $profile)) {
		$profile['localdirs'] = $lib->getStringArray($profile['localdirs']);
	}
	if (array_key_exists('types', $profile)) {
		$profile['types'] = $lib->getStringArray($profile['types']);
	}
	if (array_key_exists('instances', $profile)) {
		$profile['instances'] = $lib->getIDs($profile['instances']);
	}
	if (array_key_exists('clients', $profile)) {
		$profile['clients'] = $lib->getIDs($profile['clients']);
	}
	
	//
	// Promote target slots to top-level of profile.
	//
	if (array_key_exists('options', $profile)) {
		$options = $profile['options'];
		if (isset($options['target_slots'])) {
			$profile['target_slots'] = $options['target_slots'];
			unset($options['target_slots']);
		}
	}
	return $profile;
	
}

function setArrayValue(&$array, $stack, $value, $bInteger, $bBoolean) {
	if ($stack) {
		$key = array_shift($stack);
		setArrayValue($array[$key], $stack, $value, $bInteger, $bBoolean);
	} else {
		if ($bInteger) {
			$array = (int)$value;
		} else if ($bBoolean) {
			$array = (bool)$value;
		} else {
			$array = $value;
		}
	}
}

function buildFieldName($tag)
{
	$field = "";

	switch ($tag) {
		case "Desc":
			$field = "description";
			break;
		case "Target":
			$field = "target";
			break;
		case "RangeEnd":
			$field = "range_end";
			break;
		case "RangeSize":
			$field = "range_size";
			break;
		case "Instances":
			$field = "instances";
			break;
		case "Clients":
			$field = "clients";
			break;
		case "Dirs":
			$field = "localdirs";
			break;
		case "Types":
			$field = "types";
			break;
		case "Options":
			$field = "options";
			break;
		case "Append":
			$field = "append";
			break;
		case "Purge":
			$field = "purge";
			break;
		case "Compress":
			$field = "compress";
			break;
		case "Encrypt":
			$field = "encrypt";
			break;
		case "Deduplicate":
			$field = "dedup";
			break;	
		case "FastSeed":
			$field = "fastseed";
			break;	
		case "EMail":
			$field = "email_report";
			break;
		case "Days":
			$field = "retention_days";
			break;
		case "Verify":
			$field = "verify";
			break;
		case "Slots":
			$field = "target_slots";
			break;
		default:
			$field = $tag;
			break;
	}

	return $field;
}

function isInteger($tag)
{
	$bInteger = false;
	switch ($tag) {
		case "RangeEnd":
		case "RangeSize":
		case "Days":
			$bInteger = true;
			break;
	}
	return $bInteger;
}

function isBoolean($tag)
{
	$bBoolean = false;
	switch ($tag) {
		case "Append":
		case "Purge":
		case "Compress":
		case "Encrypt":
		case "Deduplicate":
		case "FastSeed":
		case "EMail":
		case "Verify":
			$bBoolean = true;
			break;
	}
	return $bBoolean;
}
?>