<?php
// archive functions
require 'arfunctions.php';

// defines BP, xml, action, cookie
require '../header.php';

// general library functions
require $rootdir . 'bpl/library.php';
$lib = new Library();

$localID = $BP->get_local_system_id();
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $localID;

// figure out which sysID to used based on view.
$sysID = $grandClientView ? $localID : $dpuID;

// Defined used during restore.
define("FILE_SEPARATOR", ",");
// Media types.
define("DISK", "disk");
define("TAPE", "tape");
define("CHANGER", "changer");

if (isset($_REQUEST['ids'])) {
	$ids = $lib->getIDs($_GET['ids']);
	$options = array('association' => 'assoc');
	$bOptions = array();
	$iOptions = array('device_id' => 'dev',
					  'association_id' => 'aid');
	$arOptions = array();
	$arOptions = buildOptions($arOptions, $options, $bOptions);
	$arOptions = addIntegerOptions($arOptions, $iOptions);
	
	$inclusionList = array();
	if (isset($_REQUEST['inclusion'])) {
		$inclusions = $_REQUEST['inclusion'];
		$inclusionNames = explode(FILE_SEPARATOR, $inclusions);
		for ($i = 0; $i < count($inclusionNames); $i++) {
			$inclusionNames[$i] = urldecode($inclusionNames[$i]);
		}
		$inclusionList = $inclusionNames;
		if (count($inclusionList) > 0) {
			$arOptions['inclusions'] = $inclusionList;
		}
	}
	$exclusionList = array();
	if (isset($_REQUEST['exclusion'])) {
		$exclusions = $_REQUEST['exclusion'];
		$exclusionNames = explode(FILE_SEPARATOR, $exclusions);
		for ($i = 0; $i < count($exclusionNames); $i++) {
			$exclusionNames[$i] = urldecode($exclusionNames[$i]);
		}
		$exclusionList = $exclusionNames;
		if (count($exclusionList) > 0) {
			$arOptions['exclusions'] = $exclusionList;
		}
	}
	
	// If we are checking the media, check first.  Otherwise, assume Found and continue.
	$media = isset($_GET['media']) ? checkMedia($BP, $ids, $sysID) : array("FOUND" => true);
	if ($media) {
		if ($media['FOUND'] == true) {
			$xml->push("root");
			$bSingleRestoreError = false;
			foreach ($ids as $id) {
				$returnValue = $BP->restore_archived_files($id, $arOptions, $sysID);
				if ($returnValue !== false) {
					if (is_array($returnValue)) {
						foreach ($returnValue as $result) {
							$cid = isset($result['client_id']) ? $result['client_id'] : "";
							$attributes = array("ClientID" => $cid,
												"JobID" => $result['job_id']);			
							if (array_key_exists('type', $result)) {
								$attributes = array_merge($attributes, array("Type" => $result['type']));
							}
							if (array_key_exists('msg', $result)) {
								$attributes = array_merge($attributes, array("Message" => $result['msg']));
							}
							$xml->emptyelement("Restore", $attributes);
						}
					} else {
						$xml->element("JobID", $returnValue);
					}
				} else {
					if (count($ids) > 1) {
						$xml->element("RestoreError", $BP->getError());
					} else {
						$bSingleRestoreError = true;
					}
				}
			}
			$xml->pop(); // root
			
			// if more than one entry, issue RestoreError, else, build error result.
			if ($bSingleRestoreError) {
				$xml->clear(false);
				$BP->buildResult($xml, false);
			}
		} else {
			$errorString = "Cannot start the archive restore.  Please ensure that media serial " . $media['media_serials'] . " is connected";
			if (isset($media['target']) && $media['target'] != "") {
				$errorString .= " and that the target media, " . $media['target'] . ", is online and connected";
			}
			$errorString .= " to the system.";
			$BP->buildResult($xml, false, $errorString);
		}
	} else {
		$BP->buildResult($xml, false, "Cannot restore until this archive's media is connected to the system.");
	}
} else {
	$BP->buildResult($xml, false, "must specify 'ids' as comma-separated list of backups to be restored");
}
echo($xml->getXml());

//
// See if the media that has these archives is connected.
//
function checkMedia($BP, $ids, $dpuID) {

	$matchMedia = false;
	
	// See what media was used for this archive.  Pick the first id.
	$archiveIDs = array($ids[0]);
	$result_format = array('archive_ids' => $archiveIDs);
	$statusArray = $BP->get_archive_status($result_format, $dpuID);
	if (is_array($statusArray) && count($statusArray) > 0) {
		$status = $statusArray[0];
		$setID = $status['archive_set_id'];
		
		// Get the set information, then see if the serials are connected.
		// If a tape, don't check serials.
		$setInfo = $BP->get_archive_set_info($setID, $dpuID);
		if ($setInfo !== false) {
			$setSerials = $setInfo['media_serials'];
			$mediaTarget = isset($setInfo['profile']) && isset($setInfo['profile']['target']) ? $setInfo['profile']['target'] : "";
			$matchMedia = array('media_serials' => $setSerials,
								'FOUND' => false,
								'target' => $mediaTarget);
			
			$mediaList = $BP->get_connected_archive_media($dpuID);
			if ($mediaList !== false) {
				if (mediaSetOnTape($setInfo, $mediaList)) {
					// If on tape, assume found.
					$matchMedia = array("FOUND" => true);
				} else {
					foreach ($mediaList as $media) {
						if (serialsMatch($media['media_serials'], $setSerials)) {
							$matchMedia = $media;
							if ($media['is_mounted'] == false) {
								// The media is not mounted, so attempt to mount it before the restore.
								$result = $BP->mount_archive_media($media['name'], $dpuID);
								if ($result !== true) {
									global $Log;
									$Log->writeError("mount of media " . $media['name'] . " failed: " . $BP->getError(), true);
								}
							}
							$matchMedia['FOUND'] = true;
							break;
						}
					}
				}
			}
		}
	}
	return $matchMedia;
}

//
// Returns true if the media serial numbers match that of the archive set, otherwise returns false.
//
// lmc test with comma-separated serials that match, that are in diff order, that do not match.
//	$mediaSerials = "123456789,987654321,111111111,222222222";
//	$setSerials = "111111111,222222222,987654321,123456789";
function serialsMatch($mediaSerials, $setSerials) {
	$bFound = false;
	if ($mediaSerials == $setSerials) {
		$bFound = true;
	} else {
		// See if we have a series of comma-separated serials.  Split and check each serial for a match.
		if (strstr($mediaSerials, ',') && strstr($setSerials, ',')) {
			$mediaSerialArray = explode(',', $mediaSerials);
			$setSerialArray = explode(',', $setSerials);
			// Make sure we have the same number of serials number tokens first, otherwise it is not a match.
			if (count($mediaSerialArray) == count($setSerialArray)) {
				$bFound = true;
				foreach ($mediaSerialArray as $serial) {
					if (!in_array($serial, $setSerialArray)) {
						$bFound = false;
						break;
					}
				}
			}
		}
	}
	return $bFound;
}

//
// This function returns true if the media is connected and is a tape, false if not.
//
function mediaSetOnTape($setInfo, $mediaList) {
	$bTape = false;
	global $Log;
	
	$setProfile = isset($setInfo['profile']) ? $setInfo['profile'] : array();
	if (isset($setProfile['target'])) {
		// Get archive profile target media name.
		$mediaName = $setProfile['target'];
		if (strpos($mediaName,CHANGER) !== false) {
			$bTape = true;
		}
		if (strpos($mediaName,TAPE) !== false) {
			$bTape = true;
		}
		foreach ($mediaList as $media) {
			// Search the connected media for this name and when found, see if tape or changer.
			if ($media['name'] == $mediaName) {
				if ($media['type'] == TAPE || $media['type'] == CHANGER) {
					$bTape = true;
				}
				break;
			}
		}
	}
	$sMessage = 'is tape: ' . (($bTape == true ) ? "true" : "false");
	$Log->writeVariable($sMessage);
	return $bTape;
}

$BP->destroyCookie();
?>
