<?xml version="1.0" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

	<xsl:param name="report_type" select="'short'"/>

	<xsl:template match="root">
		<html>

			<head>

				<style type="text/css">
					h1 {
						margin: auto;
						text-align: center;
					}

					table {
						border: 2px solid black;
						border-collapse: collapse;
						margin: auto;
					}

					th {
						background-color: #DCDCDC;
					}

					th, td {
						border: 1px solid black;
						padding: 0 1em;
					}

					td.right {
						text-align: right;
					}
				</style>

				<title>Retention Settings Report</title>

			</head>

			<body>

				<h1>Retention Settings</h1>

				<table>

					<tr>
						<th>ID</th>
						<th>Client</th>
						<th>Min</th>
						<th>Max</th>
						<th>Legal Hold</th>
						<th>Days</th>
					</tr>

					<xsl:if test="$report_type = 'full'">
						<xsl:for-each select="Client">
							<xsl:sort select="@Name"/>
							<xsl:variable name="highlight">
								<xsl:choose>
									<xsl:when test="@Days &lt; @Min">#FF0000</xsl:when>
									<xsl:otherwise></xsl:otherwise>
								</xsl:choose>
							</xsl:variable>

							<tr bgcolor="{$highlight}">
								<td class="right"><xsl:value-of select="@ID"/></td>
								<td><xsl:value-of select="@Name"/></td>
								<td class="right"><xsl:value-of select="@Min"/></td>
								<td class="right"><xsl:value-of select="@Max"/></td>
								<td class="right"><xsl:value-of select="@LegalHold"/></td>
								<td class="right"><xsl:value-of select="@Days"/></td>
							</tr>

						</xsl:for-each>
					</xsl:if>

					<xsl:if test="$report_type = 'sorted'">
						<xsl:for-each select="Client">
							<xsl:sort select="@Days - @Min" data-type="number"/>
							<xsl:sort select="@Name"/>
							<xsl:variable name="highlight">
								<xsl:choose>
									<xsl:when test="@Days &lt; @Min">#FF0000</xsl:when>
									<xsl:otherwise></xsl:otherwise>
								</xsl:choose>
							</xsl:variable>

							<tr bgcolor="{$highlight}">
								<td class="right"><xsl:value-of select="@ID"/></td>
								<td><xsl:value-of select="@Name"/></td>
								<td class="right"><xsl:value-of select="@Min"/></td>
								<td class="right"><xsl:value-of select="@Max"/></td>
								<td class="right"><xsl:value-of select="@LegalHold"/></td>
								<td class="right"><xsl:value-of select="@Days"/></td>
							</tr>

						</xsl:for-each>
					</xsl:if>

					<xsl:if test="$report_type = 'short'">
						<xsl:for-each select="Client">
							<xsl:sort select="@Name"/>
							<xsl:if test="@Days &lt; @Min">

								<tr>
									<td class="right"><xsl:value-of select="@ID"/></td>
									<td><xsl:value-of select="@Name"/></td>
									<td class="right"><xsl:value-of select="@Min"/></td>
									<td class="right"><xsl:value-of select="@Max"/></td>
									<td class="right"><xsl:value-of select="@LegalHold"/></td>
									<td class="right"><xsl:value-of select="@Days"/></td>
								</tr>

							</xsl:if>
						</xsl:for-each>
					</xsl:if>

				</table>

			</body>

		</html>

	</xsl:template>

</xsl:stylesheet>
