<?php
// defines BP, xml, action, cookie
require 'header.php';

// defines functions for URL parsing.
require('urloptions.php');

// general library functions
require 'library.php';
$lib = new Library();

// VCenter-RRC is a special client that will not have last file-based backups.
define("VCENTER_RRC", "vCenter-RRC");
define("FAILURE_STATUS", 2);

$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
$cid = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;
				
// Loop through all systems.  Will be only one in the list for a DPU, multiple systems on a Vault or a Manager.
$systems = $BP->get_system_list();	
$localID = $BP->get_local_system_id();
$bVault = $BP->local_system_is_vault();
$bManager = $BP->local_system_is_mgr();
$bMDPU = isset($_GET['mdpu']) ? true : false;

$showReport = (isset($_GET['report']) && (int)$_GET['report'] == 1 ) ? true : false;

if ( isset($_GET['iids']) ) {
	// If iids is set and the $showReport variable is false, we want to show last backups for a list of instances
	$perInstance = !$showReport;
	$instanceArray = $lib->getIDs($_GET['iids']);	
} else {
	$perInstance = false;
	$instanceArray = false;
}

if ($systems === false) {
	$BP->buildResult($xml, false);
	echo($xml->getXml());
	break;
} elseif ($perInstance == true) {  //Used mainly for the integration with RDR
	// get last backups for an instance list
	$detail = isset($_GET['detail']);
	$lastHours = isset($_GET['hours']) ? (int)$_GET['hours'] : 0;
	$lastBackups = $BP->get_last_backup_per_instance($instanceArray, $dpuID);
	if ($lastBackups !== false) {
		$xml->push('Backups');
		foreach ($lastBackups as $backup) {
			$xml->push('Backup');
			listInstanceBackups($BP, $xml, $backup, $detail, $lastHours, $dpuID);
			$xml->pop();
		}
		$xml->pop();
	} else {
		$BP->buildResult($xml, false);
	}
	echo($xml->getXml());
} else {
	
	$doApplicationReport = isset($_GET['aid']);  //If true, shows the report for the application, otherwise shows it for clients
	$aid = $doApplicationReport ? (int)$_GET['aid'] : -1;
	
	$xml->push('root');

	foreach ($systems as $systemID => $systemName) {
		// Either we've specified one dpu ID or all.
		if ($systemID == $dpuID || $dpuID == $localID) {				
			// If this system is a vault and we are looking at this system, skip it and go to the next.
			if ($bVault && ($systemID == $localID)) {
				;
			// If this system is a manager and we are not running on the MDPU node nor are
			// not looking at the selected system, skip it.
			} else if (!$bVault && $bManager && !$bMDPU && ($systemID != $dpuID)) {
				;
			} else {
				// Build the client and device lists once per system.
				//$clientList = $BP->get_client_list($systemID);
				$xml->push("DPU", array("Name" => $systemName, "ID" => $systemID));
				
				$clientIDList = array();
				$clients = $BP->get_client_list($systemID);
				if ($cid != -1) {
					$clientIDList[] = $cid;	
				} else {
					if ($clients !== false) {
						foreach ($clients as $tempClientID => $tempClientName) {
							$clientIDList[] = $tempClientID;
						}
					}
				}
				
				$allLastBackups = $BP->get_all_last_backups($clientIDList, $systemID);
				if ($allLastBackups !== false) {
					if (count($allLastBackups) > 0) {
						
						$backupIDs = array();
						$clientsWithApplicationBackups = array();
						
						foreach ($allLastBackups as $lastBackupInfoArray) {
							$groupsArray = $lastBackupInfoArray['groups'];
							foreach ($groupsArray as $group) {
								if ( $doApplicationReport ) {
									if ( $group['app_id'] == $aid ) {
										$backupIDs = array_merge( $backupIDs, $group['backup_ids'] );
									}									
								} else {
									if ( $group['app_id'] == $app_id_file_level ) {
										$backupIDs = array_merge( $backupIDs, $group['backup_ids'] );
									} else if ( $group['app_id'] != $app_id_archive && $group['app_id'] != $app_id_block_level && $group['app_id'] != $app_id_system_metadata) {
										$clientsWithApplicationBackups[] = $lastBackupInfoArray['client_id'];
									}
								}
							}
						}
						
						
	
						if ( !$doApplicationReport ) {
							$backupTypes = array( "master", "differential", "incremental", "baremetal", "selective" );
						} else {
							$backupTypes = array( 	"mssql full", 
													"mssql differential", 
													"mssql transaction", 
													"exchange full", 
													"exchange differential", 
													"exchange incremental",
													//"legacy mssql full",
													//"legacy mssql diff",
													//"legacy mssql trans",
													"vmware full",
													"vmware differential",
													"vmware incremental",
													"hyperv full",
													"hyperv incremental",
													"hyperv differential",
													"oracle full",
													"oracle incr",
													"sharepoint full",
													"sharepoint diff",
													"ucs service profile full",
													"ndmp full",
													"ndmp incr",
													"ndmp diff");
						}	
							
						$backupStatusArray = $BP->get_backup_status( array('backup_ids'=>$backupIDs, 'type'=>$backupTypes) );
						$clientsBackupArray = array();
						foreach ($clientIDList as $cidArrayKey) {
							$clientsBackupArray[$cidArrayKey] = array();
						}
						$clientBackupInfo = array();
						$globalBackupTypesUnderscored 	= array (
																	"master" 					=> "Master",
																	"differential"				=> "Differential",
																	"incremental"				=> "Incremental",
																	"baremetal" 				=> "BareMetal",
																	"selective" 				=> "Selective",
																	"mssql full" 				=> "Mssql_Full",
																	"mssql differential" 		=> "Mssql_Differential",
																	"mssql transaction" 		=> "Mssql_Transaction",
																	"exchange full" 			=> "Exchange_Full",
																	"exchange differential" 	=> "Exchange_Differential",
																	"exchange incremental" 		=> "Exchange_Incremental",
																	//"legacy mssql full" 		=> "Legacy_Mssql_Full",
																	//"legacy mssql diff" 		=> "Legacy_Mssql_Diff",
																	//"legacy mssql trans" 		=> "Legacy_Mssql_Trans",
																	"vmware full"				=> "Vmware_Full",
																	"vmware differential" 		=> "Vmware_Differential",
																	"vmware incremental" 		=> "Vmware_Incremental",
																	"hyperv full" 				=> "Hyperv_Full",
																	"hyperv incremental" 		=> "Hyperv_Incremental",
																	"hyperv differential" 		=> "Hyperv_Differential",
																	"oracle full"				=> "Oracle_Full",
																	"oracle incr" 				=> "Oracle_Incr",
																	"sharepoint full" 			=> "SharePoint_Full",
																	"sharepoint diff"			=> "SharePoint_Diff",
																	"ucs service profile full" 	=> "UCS_Service_Profile_Full",
																	"ndmp full"					=> "NDMP_Full",
																	"ndmp incr"					=> "NDMP_Incremental",
																	"ndmp diff"					=> "NDMP_Differential");
						foreach ( $backupStatusArray as $backupStatus ) {
							if ( $instanceArray === false || in_array($backupStatus['instance_id'], $instanceArray)) {
								$clientBackupInfo = $clientsBackupArray[$backupStatus['client_id']];
								$clientBackupInfo[$globalBackupTypesUnderscored[$backupStatus['type']]] = array('ID' => $backupStatus['id'], 'Date' => $backupStatus['start_time']);
								$clientsBackupArray[$backupStatus['client_id']] = $clientBackupInfo;
							}
						}
						foreach ( $clientsBackupArray as $clientID => $backupArrayPerClient ) {														
							if (is_array($clients) && array_key_exists($clientID, $clients)) {
								$clientName = $clients[$clientID];
							} else {
								$clientName = NULL;
							}
							$xml->push("Client", array("Name" => $clientName));
							createLastBackupXML($xml, $backupArrayPerClient, $doApplicationReport);
							$xml->element("HasAppBackups", (in_array($clientID, $clientsWithApplicationBackups) ? 1 : 0) );
							$xml->pop(); // Client
						}
					}
				}
				$xml->pop();  //DPU
			}
		}
	}
	$xml->pop(); //root
	echo($xml->getXml());
}


//
// Creates the XML for the last backups for a given client.
//
//
	
function createLastBackupXML($xml, $backup, $doApplicationTypes)
{
	if( !$doApplicationTypes ) {
		lastBackupOfType($xml, $backup, 'Master');
		lastBackupOfType($xml, $backup, 'Differential');	
		lastBackupOfType($xml, $backup, 'Incremental');	
		lastBackupOfType($xml, $backup, 'BareMetal');
		lastBackupOfType($xml, $backup, 'Selective');
	} else {
		lastBackupOfType($xml, $backup, 'Mssql_Full');
		lastBackupOfType($xml, $backup, 'Mssql_Differential');
		lastBackupOfType($xml, $backup, 'Mssql_Transaction');
		lastBackupOfType($xml, $backup, 'Exchange_Full');
		lastBackupOfType($xml, $backup, 'Exchange_Differential');
		lastBackupOfType($xml, $backup, 'Exchange_Incremental');
		//lastBackupOfType($xml, $backup, 'Legacy_Mssql_Full');
		//lastBackupOfType($xml, $backup, 'Legacy_Mssql_Diff');	
		//lastBackupOfType($xml, $backup, 'Legacy_Mssql_Trans');
		lastBackupOfType($xml, $backup, 'Vmware_Full');
		lastBackupOfType($xml, $backup, 'Vmware_Differential');
		lastBackupOfType($xml, $backup, 'Vmware_Incremental');
		lastBackupOfType($xml, $backup, 'Hyperv_Full');
		lastBackupOfType($xml, $backup, 'Hyperv_Incremental');
		lastBackupOfType($xml, $backup, 'Hyperv_Differential');
		lastBackupOfType($xml, $backup, 'Oracle_Full');
		lastBackupOfType($xml, $backup, 'Oracle_Incr');
		lastBackupOfType($xml, $backup, 'SharePoint_Full');
		lastBackupOfType($xml, $backup, 'SharePoint_Diff');
		lastBackupOfType($xml, $backup, 'UCS_Service_Profile_Full');
		lastBackupofType($xml, $backup, 'NDMP_Full');
		lastBackupofType($xml, $backup, 'NDMP_Incremental');
		lastBackupofType($xml, $backup, 'NDMP_Differential');
	}
} 

//
// Creates xml of the specified backup type, with ID and date.
//
function lastBackupOfType($xml, $backup, $typeKey)
{
	if (array_key_exists($typeKey, $backup)) {
		$backupInfo = $backup[$typeKey];
		$id = $backupInfo['ID'] !== 0 ? $backupInfo['ID'] : 'None';
		$date = $backupInfo['Date'] !== 0 ? date('m/d/Y   h:i:sA', $backupInfo['Date']) : 'Never';		
	} else {
		$id = 'None';
		$date = 'Never';
	}
	$xml->emptyElement($typeKey, array('ID' => $id, 'Date' => $date));
}


//
// Create xml for the given $backup, showing details if $detail is true
// if $lastHours is a positive number, gets backups over the last specified hour earlier.
//
function listInstanceBackups($BP, $xml, $backup, $detail, $lastHours, $dpuID)
{
	$instanceID = $backup['instance_id'];
	$xml->element('Instance_ID', $instanceID);
	$xml->element('Backup_ID', $backup['backup_id']);
	if ($detail) {
		// Get properties of the backup.
		$result_format = array('backup_ids' => array($backup['backup_id']), 'system_id' => $dpuID);
		$statusArray = $BP->get_backup_status($result_format);
		if ($lastHours == 0) {
			if (is_array($statusArray) && count($statusArray) > 0) {
				createBackupStatusXML($xml, "Details", $statusArray[0], false, false);
			}
		} else {
			// Get all backups over the last specified hours snce the last backup.
			$latestStartTime = $statusArray[0]['start_time'];
			$earliestStartTime = $latestStartTime - (3600 * $lastHours);
			$result_format = array('start_time' => $earliestStartTime,
					       'end_time' => $latestStartTime,
						'system_id' => $dpuID);
			$statusArray = $BP->get_backup_status($result_format);
			if (is_array($statusArray) && count($statusArray) > 0) {
				foreach ($statusArray as $status) {
					// must not be failed backup, must be for this instance,
					// and not replication of a backup.
					if (isset($status['instance_id']) &&
						$status['status'] != FAILURE_STATUS &&
				    		$status['instance_id'] == $instanceID &&
				    		strstr($status['type'], 'securesync') == false) {
						createBackupStatusXML($xml, "Details", $status, false, false);
					}
				}
			}
		}
	}
}

$BP->destroyCookie();
?>
