<?php
// defines BP, xml, action, cookie
require '../header.php';

define("DEFAULT_STRATEGY", "Recency");
define("DEFAULT_WORKERS", 2);

define("MAX_WORKERS", "MaxConcurrent");
define("PREVIOUS_WORKERS", "prevMaxCon");

switch ($action) {
	case "list":
		// check replication configuration status.			
		$systemID = (int)$_GET['id'];
		checkReplicationFromTarget($xml, $BP, $systemID);
		echo($xml->getXml());
		break;
						
	case "add":
		// specify source ID, potentially user name.			
		$id = (int)$_GET['id'];
		
		// see if target supports replication
		$canReplicate = $BP->replication_supported($id);
		if ($canReplicate == true) {
			// see if replication on source configured to a different target and if so, don't let them add this new target
			$replicating = $BP->is_replication_configured($id);
			$currentTarget = $BP->get_ini_value('Replication', 'SyncTo', $id);
			if (!$replicating && $currentTarget !== false && $currentTarget != "") {
				// see if the target is in the hosts file.
				$hostInfo = $BP->get_host_info($currentTarget, $id);
				if ($hostInfo !== false) {
					$replicating = true;
				}
			}
			
			if (!$replicating) {
				// see if user passed in or assign one.
				if (isset($_GET['user'])) {
					$user = $_GET['user'];
				} else {
					$user = "";
					$system = $BP->get_system_info($id);
					if ($system) {
						$user = $system['name'];
						$asset = $BP->get_asset_tag($id);
						if ($asset != false) {
							$user .= '-' . $asset;
						}
					}
				}			
				
				// get password or assign one.
				$password = isset($_GET['password']) ? $_GET['password'] : $user;
				$deviceID = isset($_GET['dev_id']) ? (int)$_GET['dev_id'] : -1;
				
				$result = configureReplicationOnTarget($xml, $BP, $id, $user, $password, $deviceID);
				$warning = NULL;
				if ($result !== false) {
					if (is_string($result)) {
						$warning = $result;
					}
					$hostName = $BP->get_hostname();
					if ($hostName !== false && isset($hostName['name'])) {
						$result = configureReplicationOnSource($xml, $BP, $hostName['name'], $id);
						if (is_string($result)) {
							$warning .= $result;
						}
                        migrate_legacy_replication($BP, $id);
					} else {
						$warning .= 'Error getting hostname to configure replication on source: ' . $BP->getError() . '.';
					}
				}
				$BP->buildResult($xml, $result, NULL, $warning);
			}
			else {
				$BP->buildResult($xml, false, "This system can only be managed. Replication to this target cannot be configured because this source system is already replicating to target \"$currentTarget.\" \n\n If you want this system to replicate to a different target, you must first deconfigure the current target. While logged into target \"$currentTarget\", select Replication>Systems Management and uncheck the \"Configured For Replication\" checkbox to deconfigure replication for that target. Then proceed with replication configuration on this target.");
			}
		} else if ($canReplicate === false) {
			$BP->buildResult($xml, false, 'Cannot configure replication on a system that does not support it.');
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
					
	case "delete":
		// specify source ID.			
		$id = (int)$_GET['id'];
		
		// delete the user if set.
		$uid = isset($_GET['uid']) ? (int)$_GET['uid'] : -1;
		$username = isset($_GET['user']) ? $_GET['user'] : "";
		
		// disable it first?
		$disableFirst = isset($_GET['disable']) && $_GET['disable'] == '1';
		
		$result = deleteReplicationFromTarget($xml, $BP, $id, $uid, $username, $disableFirst);
		$warning = NULL;
		if ($result !== false) {
			if (is_string($result)) {
				$warning = $result;
			}
			$hostName = $BP->get_hostname();
			if ($hostName !== false && isset($hostName['name'])) {
				$warning = deleteReplicationFromSource($xml, $BP, $hostName['name'], $id);
			} else {
				$warning .= ' Error getting hostname to delete replication configuration on source: ' . $BP->getError() . '.';
			}
		}
		$BP->buildResult($xml, $result, NULL, $warning);
		echo($xml->getXml());
		break;
		
	case "status":
		// check replication/vaulting status.  If an id is specified, see if local or remote.
		$local = false;	
		$result = false;
		$replicating = false;					// system is replicating
		$replicationSupported = false;			// system supports replication
		$vaulting = false;						// system is vaulting
		$target = false;						// replication or vaulting target
		if (isset($_GET['id'])) {
			$systemID = (int)$_GET['id'];
			$localID = $BP->get_local_system_id();
			// if the id is the same as the local system, it's the local.
			if ($systemID == $localID) {
				$local = true;
			} else {
				$systemInfo = $BP->get_system_info($systemID);
				// If the remote systemID a virtual DPU, handle as a local system.
				if ($systemInfo !== false && $systemInfo['host'] == 'localhost') {
					$local = true;
				}
			}
			if ($local) {
				$replicationSupported = $BP->replication_supported();
				if ($replicationSupported === true) {
					$replicating = $BP->local_system_is_replicating();
					if (is_string($replicating)) {
						$target = $replicating;
						$replicating = true;
					}
				}
				if ($replicating == false) {
					$vaulting = $BP->local_system_is_vaulting();
					if (is_string($vaulting)) {
						$target = $vaulting;
						$vaulting = true;
					}
				}
			} else {
				// it's a remote, see if replicating or vaulting.
				$replicationSupported = $BP->replication_supported($systemID);
				if ($replicationSupported === true) {
					$replicating = $BP->is_replication_configured($systemID);
					// one more case.  The remote system may be vaulting somewhere else, so read
					// the variables directly to see if vaulting configured or not.
					$target = $BP->get_ini_value('Replication', 'SyncTo', $systemID);
					if (!$replicating && $target !== false && $target != "") {
						// see if the target is in the hosts file.
						$hostInfo = $BP->get_host_info($target, $systemID);
						if ($hostInfo !== false) {
							$replicating = true;
						}
					}
				}
				if ($replicating == false) {
					$vaulting = $BP->is_vaulting_configured($systemID);
					$target = $BP->get_ini_value('Securesync', 'SyncTo', $systemID);
					if (!$vaulting && $target !== false && $target != "") {
						$vaulting = true;
					}
				}
			}
			buildReplicationXML($xml, $BP, $vaulting, $replicating, $replicationSupported, $target);
		} else {
			// see if the local system is vaulting.  Will either return a string with the name of the vault,
			// an empty string if vaulting but we can't tell the name (both strings), or false if not vaulting
			// or -1 on error.  If not a string, pass down the result (false or -1) to build the XML.
			$replicationSupported = $BP->replication_supported();
			if ($replicationSupported === true) {
				$replicating = $BP->local_system_is_replicating();
				if (is_string($replicating)) {
					$target = $replicating;
					$replicating = true;
				}
			}
			if ($replicating == false) {
				$vaulting = $BP->local_system_is_vaulting();
				if (is_string($vaulting)) {
					$target = $vaulting;
					$vaulting = true;
				}
			}
			buildReplicationXML($xml, $BP, $vaulting, $replicating, $replicationSupported, $target);
		}
		echo($xml->getXml());
		break;
		
	case "enable":
		// set the workers to a non-zero value and re-enable replication (in case disabled).
		$id = (int)$_GET['id'];
		$result = setWorkers($BP, DEFAULT_WORKERS, $id);
		$BP->buildResult($xml, $BP->enable_replication(true, $id));
		echo($xml->getXml());
		break;
		
	case "disable":
		// Do not disable replication if configured!  Sync-needed will not get set on backups!
		$id = (int)$_GET['id'];
		$result = setWorkers($BP, 0, $id);
		$BP->buildResult($xml, $result);
		echo($xml->getXml());
		break;
		
	case "stop":
		// Disable future replication and stop current replications by restarting the process.
		$id = (int)$_GET['id'];
		$result = setWorkers($BP, 0, $id);
		if ($result !== false) {
			$result = $BP->restart_replication($id);
		}
		$BP->buildResult($xml, $result);
		echo($xml->getXml());
		break;
		
	case "restart":
		$id = (int)$_GET['id'];
		$BP->buildResult($xml, $BP->restart_replication($id));
		echo($xml->getXml());
		break;
}

//
// Checks replication status from the target.
//
function checkReplicationFromTarget($xml, $BP, $id) {
	$xml->push('root');
	
	// see if replication is supported.
	$replicationSupported = $BP->replication_supported($id);
	$xml->element('Replication', $replicationSupported == true ? 1 : 0);
	
	$result = $BP->is_replication_configured($id);
	if ($result === true) {
		$xml->element('Replicating', 1);
		$xml->element('Vaulting', 0);
		// get the target system for display.
		$target = $BP->get_ini_value('Replication', 'SyncTo', $id);
		if ($target === false) {
			$target = "unknown";
		}
		// see if replication process (securesync) is enabled.
		$result = $BP->is_replication_enabled($id);
		$enabled = 0;
		if ($result !== -1) {
			if ($result === true) {
				$enabled = 1;
			}
		} else {
			$enabled = "unknown";
		}
		if ($enabled != 0) {
			$workers = $BP->get_ini_value('Replication', 'MaxConcurrent', $id);
			if ($workers !== false) {
				if ((int)$workers === 0) {
					$enabled = 0;
				}
			}
		}
		$xml->element('Enabled', $enabled);
		$xml->element("SyncTo", $target);
	} else if ($result === false) {
		$xml->element('Replicating', 0);
		$result = $BP->is_vaulting_configured($id);
		$xml->element('Vaulting', $result === true ? '1' : '0');
		if ($result === true) {
			// see if vaulting process (securesync) is active.
			$result = $BP->is_securesync_enabled($id);
			// get the securesync user for display.
			$dpuUser = $BP->get_ini_value('SecureSync', 'SyncUser', $id);
			if ($dpuUser === false) {
				$dpuUser = "unknown";
			}
			$enabled = 0;
			if ($result !== -1) {
				if ($result === true) {
					$enabled = 1;
				}
			} else {
				$enabled = "unknown";
			}
			$xml->element('Enabled', $enabled);
			$xml->element("DPUUser", $dpuUser);
		} else if ($result == -1) {
			global $Log;
			$msg = 'vault status error: ' . $BP->getError();
			$Log->writeVariable($msg);
		}
	} else {
		$BP->buildResult($xml, false);
	}
	$xml->pop();
}

//
// Configures replication on the target (local)
//
// - configures replication of system id $id
// - creates vault_user user/password
//
function configureReplicationOnTarget($xml, $BP, $id, $user, $password, $deviceID) {

	$result = $BP->configure_replication($id, $deviceID);
	if ($result !== false) {
		// System privileges, monitor on local, admin on source.
		$localID = $BP->get_local_system_id();
		
		$privTarget = array(	'id' => $localID,
								'privilege_level' => 1);
		$privSource = array(	'id' => $id,
								'privilege_level' => 3);
		
		$sysArray = array(	$privTarget, $privSource);
		
		$userInfo = array(	'name' => $user,
						  	'superuser' => false,
							'systems' => $sysArray,
							'password' => $password,
							'vault_user' => true,
							'source_id' => $id);

		$result = $BP->save_user_info($userInfo);
		if ($result === false) {
			$result = 'replication configured but not able to create the vault user: ' . $BP->getError();
		}
	}

	return $result;
}

//
// Configures replication on the source id $dpuID.  Returns warning message on failure.
//
//
function configureReplicationOnSource($xml, $BP, $target, $dpuID) {
	$result = $BP->configure_source_replication($target, DEFAULT_STRATEGY, $dpuID);
	if ($result === false) {
		$result = 'Error configuring replication on source: ' . $BP->getError() . '.';
		global $Log;
		$Log->writeError($result, true);
	}
	return $result;
}

function migrate_legacy_replication($BP, $dpuID)
{
    $BP->migrate_legacy_replication_config("Replication configured from ".FLASH_UI_NAME, $dpuID);
}

//
// On the replication target, stops replication and cleans up.
//
// - removes the vault_user
// - removes all the grand clients
// - if disableFirst set, disables replication
// - removes replication of source to target
//
function deleteReplicationFromTarget($xml, $BP, $id, $uid, $username, $disableFirst) {
	
	$result = true;
	if ($uid == -1 && $username != "") {
		$users = $BP->get_user_list();
		if ($users !== false) {
			$key = array_search($username, $users);
			if ($key !== false) {
				$uid = $key;
			}
		}
	}
	if ($uid != -1) {
		$result = $BP->remove_user($uid);
	}
	
	if ($result === false ) {
		// warn but keep going.  Someone could have deleted the user.
		global $Log;
		$warning = 'Could not automatically remove vault user for replication source: ' . $BP->getError() . '.';
		$Log->writeVariable($warning);
	}
			
	// delete the grand clients.
	$gclist = $BP->get_grandclient_list($id);
	if ($gclist !== false) {
		foreach ($gclist as $gid => $gname) {
			$result = $BP->delete_client($gid);
			if ($result === false) {
				break;
			}
		}
	}
			
	if ($result !== false) {
		// If we've requested to stop replication first, do it.
		if ($disableFirst) {
			$result = $BP->enable_replication(false, $id);
		}
		if ($result === false) {
			// attempt to remove anyway, but log an error.
			global $Log;
			$message = " Error disabling before removing replication, continuing";
			$message = $BP->getError() . ":" . $message;
			$Log->writeError($message, true);
		}
		// remove replication
		$result = $BP->remove_replication($id);
	}
	
	if (isset($warning)) {
		$result = $warning;
	}
	
	return $result;
}

//
// On the replication source, stops replication and cleans up.
//
// - disables replication
// - sets SyncTo to ''
// - removes entries from the replication queue
//
function deleteReplicationFromSource($xml, $BP, $target, $dpuID) {

	$warning = NULL;
	
	$BP->enable_replication(false, $dpuID);
	$result = $BP->remove_source_replication($target, $dpuID);
	if ($result === false) {
		// warn but keep going.
		global $Log;
		$warning = 'Could not remove replication configuration from source: ' . $BP->getError() . '.';
		$Log->writeVariable($warning);
	}
	
	return $warning;
}

//
// Builds the result of checking the vaulting status in an XML string.
//
function buildReplicationXML($xml, $BP, $vaulting, $replicating, $replicationSupported, $target = "") {
	$xml->push('root');
	$xml->element('Replicating', $replicating === true ? 1 : 0);
	$xml->element('Vaulting', $vaulting === true ? 1 : 0);
	$xml->element('Replication', $replicationSupported === true ? 1 : 0);
	if ($target !== false && ($replicating || $vaulting)) {
		$xml->element('Target', $target);
	}
	$xml->pop();
}

//
// Sets the number of workers on system $id, to the value passed in.
//
function setWorkers($BP, $newWorkers, $id) {
	if ($newWorkers == 0) {
		// Disable: Get the current value and save it as the "previous" value.
		$currentWorkers = $BP->get_ini_value('Replication', MAX_WORKERS, $id);
		if ($currentWorkers !== false) {
			$result = $BP->set_ini_value('Replication', PREVIOUS_WORKERS, $currentWorkers, $id);
			if ($result === false) {
				global $Log;
				$msg = 'error setting previous workers: ' . $BP->getError();
				$Log->writeError($msg, true);
			}
		}
	} else {
		// Enable: try to get the previous workers value, use value passed in if unattainable.
		$priorWorkers = $BP->get_ini_value('Replication', PREVIOUS_WORKERS, $id);
		if ($priorWorkers !== false) {
			$priorWorkers = (int)$priorWorkers;
			if ($priorWorkers > 0) {
				$newWorkers = $priorWorkers;
			}
		}	
	}
	
	// set replication to the new value.
	$result = $BP->set_ini_value('Replication', MAX_WORKERS, $newWorkers, $id);
	return $result;
}

$BP->destroyCookie();
?>
