<?php
// defines BP, xml, action, cookie
require '../header.php';

// defines functions for backup status processing.
require $rootdir."bpl/basebackupstatus.php";

// general library functions
require $rootdir . 'bpl/library.php';
$lib = new Library();

$localID = $BP->get_local_system_id();
$dpuID = $grandClientView || !isset($_GET['sid']) ? $localID : (int)$_GET['sid'];

$cid = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;
$aid = isset($_GET['aid']) ? (int)$_GET['aid'] : -1;

$appName = isset($_GET['appName']) ? $_GET['appName'] : NULL;

$instanceIDs = isset($_GET['iids']) ? $lib->getIDs($_GET['iids']) : NULL;

// used for creation of summaries of VM instance statuses
$days = isset($_GET['days']) ? (int)$_GET['days'] : 7;
$endTime = isset($_GET['end']) ? (int)$_GET['end'] : time();

// true if I want to log a lot of crap to the error logs for debugging
$heavyLog = isset($_GET['hvy_log']) ? $_GET['hvy_log'] == "1" : false;

$daysFromBeforeEndTime = generateIntervals( $days, $endTime );

// VMware app ID.
define("VMWARE_ID", 40);

//Hyper-V app ID 
//7/18/11 - this is not yet defined
define("HYPERV_ID", 50);
define("HYPERV_2012_ID", 51);

$VMWARE_TYPES = array(
		"vmware full",
		"vmware differential",
		"vmware incremental" );

$HYPERV_TYPES = array(
		"hyperv full",
		"hyperv incremental",
		"hyperv differential" );
							
// Build filter array for backups.  Include all types (GLOBAL_BACKUP_TYPES) and RRC will flag the ones not restorable.
$result_format = array();
$result_format['start_time'] = (int)$_GET['start'];
$result_format['end_time'] = $endTime;
$result_format['system_id'] = $dpuID;
if ($cid != -1) {
	$result_format['client_id'] = $cid;
}

switch ($action) {
	case "list":
		$result_format['type'] = $GLOBAL_BACKUP_TYPES;
		break;
	case "summary":
		switch( $aid ) {
			case HYPERV_2012_ID:
			case HYPERV_ID:
				$typeArray = $HYPERV_TYPES;
				break;
			case VMWARE_ID:
				$typeArray = $VMWARE_TYPES;
				break;
			default:
				$typeArray = $GLOBAL_BACKUP_TYPES;
		}
		
		$result_format['type'] = $typeArray;
		break;
}

$resultArray = $BP->get_backup_status($result_format);

if ($resultArray !== false) {
	$xml->push('root');
	
	$log = $heavyLog ? $Log : NULL;
	
	if ( $action == "list" ) {
		
		foreach ( $resultArray as $result ) {
			$isNotFailed = isValid( $result );
			$validForApp = isValidForApp( $aid, $appName, $instanceIDs, $result, $log );
			
			if ( $isNotFailed && $validForApp ) {
				createBackupStatusXML($xml, 'Backup', $result, true);
			}
		}
	} else {
		buildSummaryXML( $BP, $dpuID, $aid, $appName, $instanceIDs, $xml, $daysFromBeforeEndTime, $resultArray, $log );
	}
	$xml->pop();
} else {
	$BP->buildResult($xml, false);
}
echo($xml->getXml());

function generateIntervals($totalDays, $endTime) {
	$intervals = array();
	for ($i = 0, $days = $totalDays - 1; $i < $totalDays; $i++, $days--) {
		$seconds = 60 * 60 * 24 * $days;
		$start = strtotime(date("Y-m-d", $endTime - $seconds));
		$intervals[$i] = array("start_time" => $start);
		if ($i != 0) {
			$intervals[$i - 1]["end_time"] = $start - 1;
		}
	}
	$intervals[$totalDays - 1]["end_time"] = $endTime;
	return($intervals);
}


function isValid( $result ) {
	return $result['complete'] == true && $result['status'] !== STATUS_FAILURE;
}

// returns true is the result passed in is for a client or 
// if the result is an app instance and the instance matches the app type and name passed in.
// if this returns false it is not a valid instance
function isValidForApp( $aid, $appName, $instanceIDs, $result, $log = null ) {
	$isValid = $aid == -1;
	if ( $log ) $log->writeVariable("-------------------------------- Precheck - isValid = $isValid" );
	
	if ( !$isValid ) {
		if ( $log ) $log->writeVariable("aid is not -1 :: $aid" );
		
		// needs to have the app set, or we should skip it.
		$resultHasAppID = isset($result['app_id']);
		
		if ( $log ) $log->writeVariable("resultHasAppID :: $resultHasAppID" );
		
		// make sure the app ID matches.
		$resultAppIDMatches = $resultHasAppID ? $result['app_id'] == $aid : false;
		
		if ( $log ) $log->writeVariable("resultAppIDMatches :: $resultAppIDMatches" );
		
		// is this one of the types we are wanting to specify backups by vm name?
		$isHyperV = $aid == HYPERV_ID || $aid == HYPERV_2012_ID;
		$isVMWare = $aid == VMWARE_ID;
		
		if ( $log ) $log->writeVariable("isHyperV :: $isHyperV, isVMWare :: $isVMWare" );
		if ( $log ) $log->writeVariable("aid=$aid, appName=$appName, instanceIDs=$instanceIDs" );
		
		// do the IDs match - and is there an app name - and is the app types one where that is important?
		if ( $resultAppIDMatches && $appName != NULL && ( $isHyperV || $isVMWare ) ) {
			
			// Looking to see if this is the server or app that contains the vm we are looking for
			$serverAppNameOrInstanceIDMatches = $isVMWare ? ( $result['server_name'] == $appName ) : ( $isHyperV ? ( $result['app_name'] == $appName ) : false );
			
			if ( $log ) $log->writeVariable( "serverAppNameOrInstanceIDMatches :: $serverAppNameOrInstanceIDMatches" );
			if ( $log ) $log->writeVariable( $instanceIDs );
			
			// If the virtualization provider name matches - then look to see if the $instanceIDs match the instance_id of the VM
			if ( $serverAppNameOrInstanceIDMatches && $instanceIDs != NULL ) {
				
				if ( $log ) $log->writeVariable( "**** server name matches and the instances are not null " );
				// Does the result contain a vm name we can look up?
				if ( isset($result['instance_id'] )  ) {
					
					if ( $log ) $log->writeVariable( "Instances exist " );
					
					// pulling out the result for debugging
					$currentInstanceID = $result['instance_id'];
					
					if ( $log ) $log->writeVariable( "Current VM instance key is $currentInstanceID" );
					
					//If the names match this is true
					$serverAppNameOrInstanceIDMatches = in_array( $currentInstanceID, $instanceIDs );
					
					
					if ( $log ) {
						if ( $serverAppNameOrInstanceIDMatches ) {
							$log->writeVariable( "The instance  $currentInstanceID was found!" );
						} else {
							$log->writeVariable( "The instance  $currentInstanceID was NOT found!" );
						}
					}
					
				} else {
					// no VM instance ID - we passed in the vm name but the result has none - so no match
					$serverAppNameOrInstanceIDMatches = false;
				}
			}
		} else {
			
			// If this is not a hyperV or VMware result - I'll mark the server names as passing - we might want to 
			// do moe here later if we want to drill down into the other types of clients.
			$serverAppNameOrInstanceIDMatches = true;
		}
		// IF its the same app ID, and or same instance and or same vm name - this is true and the 
		// result is true.
		$isValid = $resultAppIDMatches && $serverAppNameOrInstanceIDMatches;
	}
	if ( $log ) $log->writeVariable("---------------------------------- PostCheck - isValid :: $isValid" );
	return $isValid;
}

function buildSummaryXML( $BP, $dpuID, $aid, $appName, $instanceIDs, $xml, $days, $backupStatuses, $log = NULL ) {
	// Getting the clients list so I can name the nodes for a summary XML
	$clients = $BP->get_client_list($dpuID);
	
	// only need to look for this once
	$isVMWare = $aid == VMWARE_ID;
	
	foreach ( $days as $key => $day ) {
		// rolling through the days
		$dayStartTime = (int)$day[ "start_time" ];
		$dayEndTime   = (int)$day[ "end_time" ];
		
		// formatting the dates for discemination
		$startFormatedDate = date('M d Y', $dayStartTime );
		
		// Pushing in the day node entry
		$xml->push('Day', array('StartTime' => $dayStartTime, 'Date' => $startFormatedDate ) );
		
		// each day we gather up that day's clients in a summary.
		// this hash of clients and their various tallies will 
		// be used when looping through the statuses that occurred
		// on the current day in the loop
		$daysClients = array();
		
		if ( $log ) $log->writeVariable( "Getting summary for day: $startFormatedDate"  );
		
		foreach ( $backupStatuses as $key=>$backupStatus ) {
			// if the backup started during the day - we will add up the successes and failures
			// and remove the entry from the $backupStatuses array
			
			$statusStartTime = (int)$backupStatus[ "start_time" ];
			
			if ( $log ) $formatedStartTime = date('M d Y', $statusStartTime );
			if ( $log ) $log->writeVariable( "checking backups for date: $formatedStartTime"  );
			
			if ( $dayStartTime <= $statusStartTime &&  $dayEndTime >= $statusStartTime ) {
				$isValid = isValid( $backupStatus );
				
				// if this is a failed backup - just kick it out now and remove it from the list
				if ( !$isValid ) {
					unset( $backupStatuses[ $key ] );
					continue;
				}
				
				if ( $log ) $log->writeVariable( "ENTER: isValidForApp in buildSummaryXML"  );
				// Looking to see if this passes the criteria established for a valid
				// restore given the passed in variables.
				$validForApp = isValidForApp( $aid, $appName, $instanceIDs, $backupStatus, $log );
				
				if ( $log ) $log->writeVariable( "EXIT: isValidForApp in buildSummaryXML"  );
				
				
				// if it has not failed and passes the criteria given the passed in vars - then we tally the successes and failures
				if ( $isValid && $validForApp ) {
					
					// getting this guy out of this list since we are counting him and don't want to recount him.
					unset( $backupStatuses[ $key ] );
					
					if ( $log ) $buStatusCount = count( $backupStatuses );
					if ( $log ) $log->writeVariable( "Number of elements in backupStatuses = $buStatusCount"  );
					
					$clientID = $backupStatus[ "client_id" ];
					
					// Using the clientID to make a quick lookup hash to store the 
					// summaries for each client
					if ( !isset( $daysClients[ "__$clientID" ] ) ) {
						$daysClients[ "__$clientID" ] = array();
					}
					
					// using the current day's client summary
					$clientSummary = $daysClients[ "__$clientID" ];
					
					// Getting the name and display for the client summary - this only hast to be done once per client
					if ( !isset( $clientSummary[ "client_name" ] ) ) {
						$clientSummary[ "client_name" ] = getClientName( $BP, $clientID, $dpuID, $clients );
						$clientSummary[ "client_id" ] = $clientID;
					}
					
					// getting the status count for this backup
					$success = $backupStatus['status'] == STATUS_SUCCESS ? 1 : 0;
					$warning = $backupStatus['status'] == STATUS_WARNING ? 1 : 0;
					$failure = $backupStatus['status'] == STATUS_FAILURE ? 1 : 0;
					
					$verifySuccess = $backupStatus['verify_status'] == STATUS_SUCCESS ? 1 : 0;
					$verifyFailure = $backupStatus['verify_status'] == STATUS_FAILURE ? 1 : 0;
					
					$inProgress =  $backupStatus['complete'] ? 0 : 1;
					
					// adding the status counts to the sums in the summary for the client.
					$clientSummary[ "Successes" ] = !isSet( $clientSummary[ "Successes" ] ) ? $success : $clientSummary[ "Successes" ] + $success;
					$clientSummary[ "Warnings" ]  = !isSet( $clientSummary[ "Warnings"  ] ) ? $warning : $clientSummary[ "Warnings"  ] + $warning;
					$clientSummary[ "Failures" ]  = !isSet( $clientSummary[ "Failures"  ] ) ? $failure : $clientSummary[ "Failures"  ] + $failure;
					
					$clientSummary[ "VerifySuccesses" ] = !isSet( $clientSummary[ "VerifySuccesses" ] ) ? $verifySuccess : $clientSummary[ "VerifySuccesses" ] + $verifySuccess;
					$clientSummary[ "VerifyFailures" ]  = !isSet( $clientSummary[ "VerifyFailures"  ] ) ? $verifyFailure : $clientSummary[ "VerifyFailures"  ] + $verifyFailure;
					
					$clientSummary[ "InProgress" ]  = !isSet( $clientSummary[ "InProgress"  ] ) ? $inProgress : $clientSummary[ "InProgress"  ] + $inProgress;
					
					// pushing the client summary back onto the days summmary so it can be added to the remaining instances for the same
					// client this current iteration's day
					$daysClients[ "__$clientID" ] = $clientSummary;
				}
			}
		}
		// Looping through the day's client summaries to print out their xml as a summary
		foreach( $daysClients as $clientSummary ) {
			$xml->push('Client', array('Name' => $clientSummary["client_name"], 'ID' => $clientSummary['client_id'] ) );
			
			if ( isset($clientSummary['Successes']) ) {
				$xml->element('Successes', $clientSummary['Successes']);
			}
			if (isset($clientSummary['Warnings'])) {
				$xml->element('Warnings', $clientSummary['Warnings']);
			}
			if (isset($clientSummary['Failures'])) {
				$xml->element('Failures', $clientSummary['Failures']);
			}
			/*
			 * We don't really need these so we are not going to return them.  
			 * 
			 * 
			if (isset($clientSummary['InProgress'])) {
				$xml->element('InProgress', $clientSummary['InProgress']);
			}
			if (isset($clientSummary['VerifySuccesses'])) {
				$xml->element('VerifySuccesses', $clientSummary['VerifySuccesses']);
			}
			if (isset($clientSummary['VerifyFailures'])) {
				$xml->element('VerifyFailures', $clientSummary['VerifyFailures']);
			}
			*/
			$xml->pop(); // client
		}
		$xml->pop();
	}
}


$BP->destroyCookie();
?>