<?php
// defines BP, xml, action, cookie
require 'header.php';
// set of functions for user management
require 'userfunctions.php';

// set UserInfo ID to null (for update vs. add)
$userInfoID = NULL;

switch ($action) {

	case "list":
			$bValid = true;
			$users = $BP->get_user_list();
			if ($users === false) {
				$bValid = false;
			} else {
				$xml->push("root");
				foreach ($users as $id => $name) {
					$userInfo = $BP->get_user_info($id);
					if ($userInfo === false) {
						$bValid = false;
						break;
					} else {
						$xml->push("User", array("Name" => $name,
											 "ID" => $id,
											 "Administrator" => isset($userInfo['administrator']) ? ($userInfo['administrator'] == true ? "true": "false") : "",
											 "Superuser" => isset($userInfo['superuser']) ? ($userInfo['superuser'] == true ? "true" : "false") : ""));

						getXMLPrivileges($BP, $userInfo, $xml);
						$xml->pop(); // user
					}
				}
				$xml->pop(); //root
			}
			if (!$bValid) {
				$xml->clear(false);
				$BP->buildResult($xml, false);
			}
			echo($xml->getXml());
			break;
					
	case "update":
			// if updating, specify id
			if (isset($_REQUEST['id'])) {
				$id = (int)$_REQUEST['id'];
				$userInfoID = array("id" => $id);
			} else {
				$BP->buildResult($xml, false, "error: when updating a user, the user id must be specified.");
			}
			// fall through to add.
	
	case "add":
			// Get the user definition from arguments.
			$userInfo = array();
			if (isset($_REQUEST['name'])) {
				$name = $_REQUEST['name'];
				
				$name = base64_decode( $name );
				$userInfo = array ("name" => $name);
			}
			if (isset($_REQUEST['password'])) {
				$password = $_REQUEST['password'];
				$password = base64_decode ( $password );
				$userInfo = array_merge($userInfo, array("password" => $password));						
			}
			if (isset($_REQUEST['superuser'])) {
				$superuser = $_REQUEST['superuser'] == "true" ? true : false;
				$userInfo = array_merge($userInfo, array("superuser" => $superuser));						
			}
			if (isset($_REQUEST['customers'])) {
				$customersArray = buildprivilegeArray("customers", $_REQUEST['customers']);
				$userInfo = array_merge($userInfo, $customersArray);					
			}
			if (isset($_REQUEST['locations'])) {
				$locationsArray = buildprivilegeArray("locations", $_REQUEST['locations']);
				$userInfo = array_merge($userInfo, $locationsArray);					
			}
			if (isset($_REQUEST['systems'])) {
				$systemsArray = buildprivilegeArray("systems", $_REQUEST['systems']);
				$userInfo = array_merge($userInfo, $systemsArray);					
			}
			if ($userInfoID != NULL) {
				// If an ID was specified (for update), add it.
				$userInfo = array_merge($userInfoID, $userInfo);
			}
			
			if (isset($_REQUEST['name']) || isset($_REQUEST['id'])) {
				$BP->buildResult($xml, $BP->save_user_info($userInfo));
				echo($xml->getXml());
			} else {
				$BP->buildResult($xml, false,
						  "error: when adding or updating a user, the name or id must be specified.");
				echo($xml->getXml());
			}
			break;
					
	case "delete":
			$id = (int)$_REQUEST['id'];
			$BP->buildResult($xml, $BP->remove_user($id));
			echo($xml->getXml());
			break;
}

function buildPrivilegeArray($privilegeIndex, $privilegeString)
{
	$privilegeArray = NULL;
	
	// privilege string will be in the form <id>:<privilege_level>,<id>:<privilege_level>, ...
	
	// strip quotes
	//$privilegeString = trim($privilegeString, "\",");
	$privilegeString = trim($privilegeString, ",");

	$idlevelPairs = explode(",", $privilegeString);
	$nameValuePairs = array();
	$nameValuePairArray = array();
	$i = 0;
	foreach ($idlevelPairs as $idlevelPair) {
		//unique separator string.  Also in Flex user services file.
		$pieces = explode('zxywz', $idlevelPair);
		//var_dump($pieces);
		
		if ($pieces[0] != "") {
			$id = (int)$pieces[0];
			//printf("id is %d<br/>", $id);
			$strPrivilege = $pieces[1];
		
			$privilege_level = 0;
			// Convert privilege from string to numeric level
			switch ($strPrivilege) {
				case "none":		$privilege_level = 0; break;
				case "monitor":		$privilege_level = 1; break;
				case "manage":		$privilege_level = 2; break;
				case "administer":	$privilege_level = 3; break;
			}
			
			$nameValuePairs['id'] = $id;
			$nameValuePairs['privilege_level'] = $privilege_level;
			$nameValuePairArray[$i++] = $nameValuePairs;
		}
		
	}
	//printf("after for each, nvpair array is <br/>");
	//var_dump($nameValuePairs);
	
	//$privilegeArray = array($privilegeIndex => $nameValuePairs);
	$privilegeArray = array($privilegeIndex => $nameValuePairArray);
	//printf("var dump of n/v array is <br/>");
	//var_dump($privilegeArray);
	
	return $privilegeArray;
}


function getXMLPrivileges($BP, $userInfo, $xml)
{

	// get complete list of customers, locations and systems. (Need to preface with bp_set_cookie().
	$customers = $BP->get_customer_list();
	$locations = $BP->get_location_list();
	$systems = $BP->get_system_list();
	
	if (array_key_exists('customers', $userInfo)) {
		foreach ($userInfo['customers'] as $customer) {
			// Display customer-level privileges for this user.
			$xml->push("Customer", array("Name" => getCustomerName($customers, $customer['id']),
											"ID" => $customer['id']));
			$xml->element("Privilege", getPrivilegeName($customer['privilege_level']));		
			// Get locations for this customer.
			$customerLocations = $BP->get_location_list($customer['id']);
			if (array_key_exists('locations', $userInfo)) {
				foreach ($userInfo['locations'] as $location) {
			
					// If any specified location-level privileges matches this customer's locations,
					// display name, location, and  privileges.
					if (matchLocation($location, $customerLocations)) {
						$xml->push("Location", array("Name" => getLocationName($locations, $location['id']),
														"ID" => $location['id']));
						$xml->element("Privilege", getPrivilegeName($location['privilege_level']));
								
						// Get systems for this customer at this location.	
						$customerLocationSystems = $BP->get_system_list($customer['id'], $location['id']);
						if (array_key_exists('systems', $userInfo)) {
							foreach ($userInfo['systems'] as $system) {
					
								// If any specified system-level privileges matches this customer's locations/systems,
								// display name, location, system and privileges.
								if (matchSystem($system, $customerLocationSystems)) {
									$xml->push("System", array("Name" => getSystemNameFromList($systems, $system['id']),
																"ID" => $system['id']));
									$xml->element("Privilege", getPrivilegeName($system['privilege_level']));
									$xml->pop(); // system
								}
							}
						}
						$xml->pop(); // location
					}
				}
			}
			$xml->pop(); //customer
		}
	}
}

$BP->destroyCookie();
?>
